/* http://en.wikibooks.org/wiki/Embedded_Systems/Mixed_C_and_Assembly_Programming
 * Call-used registers (r18-r27, r30-r31) (volatile)
 * Call-saved registers (r2-r17, r28-r29) (non-volatile)
 * Arguments - allocated left to right, r25 to r8. All arguments are aligned
 * to start in even-numbered registers (odd-sized arguments, including char,
 * have one free register above them)
 * Return values: 8-bit in r24 (not r25!), 16-bit in r25:r24, up to 32 bits in r22-r25, up to 64 bits in r18-r25
 */
__SP_H__ = 0x3e
__SP_L__ = 0x3d

.comm sched_stack_p,2

.text
.globl StartTask
	.type StartTask, @function
StartTask:
	/* save non-volatile registers */
	push r2
	push r3
	push r4
	push r5
	push r6
	push r7
	push r8
	push r9
	push r10
	push r11
	push r12
	push r13
	push r14
	push r15
	push r16
	push r17
	/* frame pointer Y; needed?! */
	push r28
	push r29

  /* previous save sched_stack_p */
	lds r18, sched_stack_p
	lds r19, sched_stack_p+1
	push r18
	push r19

  /* set current sched_stack_p */
	in r18, __SP_L__
	sts sched_stack_p, r18
	in r18, __SP_H__
	sts sched_stack_p+1,r18

	/* start the task */
	mov r30, r24
	mov r31, r25
	sei
	icall
	cli

	/* restore */
	rjmp 0f

.globl ClearStackPart
	.type ClearStackPart, @function
ClearStackPart:
/* CAUTION: We do not restore the framepointer. The program MUST be compiled
   with -fomit-frame-pointer, otherwise the wrong ptask in the BCC scheduler
   will be loaded
*/

	/* restore previous stack partition */
	lds r18,sched_stack_p
	lds r19,sched_stack_p+1
	cli
	out __SP_L__,r18
  sei
	out __SP_H__,r19

0:
	/* restore previous sched_stack_p */
	pop r19
	pop r18
	sts sched_stack_p+1, r19
	sts sched_stack_p, r18

	/* restore non-volatile registers */
	/* frame pointer Y; needed?! */
	pop r29
	pop r28
	pop r17
	pop r16
	pop r15
	pop r14
	pop r13
	pop r12
	pop r11
	pop r10
	pop r9
	pop r8
	pop r7
	pop r6
	pop r5
	pop r4
	pop r3
	pop r2

	ret

