# /**(c)
# 
#   Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt
# 
#   This file is part of the KESO Java Runtime Environment.
# 
#   KESO is free software: you can redistribute it and/or modify it under the
#   terms of the Lesser GNU General Public License as published by the Free
#   Software Foundation, either version 3 of the License, or (at your option)
#   any later version.
# 
#   KESO is distributed in the hope that it will be useful, but WITHOUT ANY
#   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#   FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
#   more details. You should have received a copy of the GNU Lesser General
#   Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.
# 
#   Please contact keso@cs.fau.de for more info.
# 
#   (c)**/
my $h = {
	ext_escape_analysis => {
		sdesc => 'Enable new escape analysis code',
		desc  => <<EOF
	Enable new escape analysis as described in "Stack Allocation and
	Synchronization Optimizations for Java Using Escape Analysis" by Choi,
	Gupta, Serrano, Sreedhar and Midkiff (DOI 10.1145/945885.945892). BA
	Clemens Lang.
EOF
	},
	comp_cls_store=> {
		sdesc => 'Create a smaller class-store',
		desc  => <<EOF
	If this flag is enabled the number of references stored in an object
	is removed from the class-store. The information is provided by a function
	instead. (memory vs. runtime)
EOF
	},
	ssa=> {
		sdesc => 'Enable SSA-based code optimizations',
		desc  => <<EOF
EOF
	},
	ssa_alias_less=> {
		sdesc => 'Enable a less aggressive alias propagation',
		desc  => <<EOF
	
EOF
	},
	ssa_alias_prop=> {
		sdesc => 'Enable alias propagation (aggressive)',
		desc  => <<EOF
	Alias propargation is a powerful code transformation. If the value of an object field was
	previously stored in a local variable then all follow up readings to the same field are
	replaced by a read to the local variable.
EOF
	},
	ssa_astack => {
		sdesc => 'Enable escape analysis and allocating objects on stack where possible',
		desc  => <<EOF
EOF
	},
	ssa_astack_less => {
		sdesc => 'NOOP-switch. Currently does nothing',
		desc  => <<EOF
EOF
	},
	ssa_astack_noinline => {
		sdesc => 'Don\'t inline the code to allocate objects on the method stack',
		desc  => <<EOF
EOF
	},
	ssa_fold_cbranch=> {
		sdesc => 'Enable folding of conditional branches (buggy).',
		desc  => <<EOF
EOF
	},
	ssa_fold_switch=> {
		sdesc => 'Enable folding of switch statements (buggy).',
		desc  => <<EOF
EOF
	},
	ssa_melt=> {
		sdesc => 'Reduce the number basic blocks',
		desc  => <<EOF
EOF
	},
	_X_ssa_no_array_len=> {
		sdesc => 'Disable constant array length propagation',
		desc  => <<EOF
EOF
	},
	ssa_no_const_prop=> {
		sdesc => 'Disable constant propagation',
		desc  => <<EOF
	If the value of a local variable is constant then the the variable can be replaced
	by the constant value. 
EOF
	},
	ssa_no_copy_prop=> {
		sdesc => 'Disable copy propagation',
		desc  => <<EOF
	If the value of a local variable is just the copy of a previously defined variable then 
	in some cases the variable can just be replaced by the previously one.
EOF
	},
	ssa_no_omit_undef=> {
		sdesc => 'Don\'t replace undefined local variables by constant null values.',
		desc  => <<EOF
EOF
	},
	ssa_omit_null_init=> {
		sdesc => 'Remove explicit null initialisation of object fields if the object was just allocated.',
		desc  => <<EOF
		Remove explicit null initialisation of object fields if the object was just allocated 
		because all object fields are nulled anyway.
EOF
	},
	ssa_no_virtual_method=> {
		sdesc => 'Disable transformation of virtual method invocations into direct method calls',
		desc  => <<EOF
EOF
	},

	no_javac_debug => {
		sdesc => 'Don\'t create debug information in Java class files.',
		desc  => <<EOF
	Jino calls the JDK java compiler to create class files. If this
	flag is set the java compiler is called without "-g" option.
EOF
	},
	no_inline_alloc => {
		sdesc => 'Do not inline object size and number of object references into the code.',
		desc => <<EOF
	If this flag is set the allocation function (keso_allocObject or
	keso_alloc_stack) will lookup the object size and number of references from
	the class store, otherwise the information is inlined into the code and
	passed as arguments. (to KESO_ALLOC or keso_alloc_stack)
EOF
	},
	vt_comp => {
		sdesc => 'Compress virtual function table.',
		desc  => <<EOF
EOF
	},
	vt_full => {
		sdesc => 'Create a function table for every virtual function even if there is only one candidate.',
		desc  => <<EOF
EOF
	},
	vt_switch => {
		sdesc => 'Use switch statements instead of virtual function tables (pointers).',
		desc  => <<EOF
EOF
	},
	iface_store => {
		sdesc => 'Use a separate table for interface type information.',
		desc  => <<EOF
	This option enables the creation of a separate table interface type information
	and therefore will reduce the size of the class store. 
EOF
	},
	const_arg_prop => {
		sdesc => 'constant-argument propagation',
		desc  => <<EOF
	This option enables the propagation of constant method arguments.
EOF
	},
	compact_header => {
		sdesc => 'Use a smaller object header',
		desc  => <<EOF
	This option enables the use of the compact object header used by default
	only on 8-bit architectures for larger architectures. The system must
	contain at most 255 classes. The maximum size of an array is reduced to
	65535 elements. The resulting object header for regular objects/arrays
	will be 1/4 bytes if no GC is used, 2/4 bytes if a GC is used.
EOF
	},
	
	dbg_src => {
		sdesc => 'Annotate generated C code with the original Java code',
		desc  => <<EOF
	When this option is enabled, the generated C code will -- to the possible
	extent -- be interleaved with comments containing the Java code that a
	portion of C code was generated from. This option can greatly increase the
	compre comprehensibility of the generated code.
EOF
	},

	dump_class_statistics => {
		sdesc => 'Dump domain-reachability for classes',
		desc  => <<EOF
	Creates a file classes-<DOMAIN>.txt in the output directory for each domain,
	which contains a list of classes reachable from that domain, and marks which
	of these classes are instantiated in the domain.
EOF
	},

	dump_method_statistics => {
		sdesc => 'Dump domain-reachability for methods',
		desc  => <<EOF
	Creates a file methods-<DOMAIN>.txt in the output directory for each domain,
	which contains a list of methods reachable from that domain, and marks which
	of these methods are exclusively reachable from the domain.
EOF
	},

	dump_pass_graph => {
		sdesc => 'Dump the compiler pass dependency graph',
		desc  => <<EOF
	When this option is enabled, the dependency graph for each pass manager
	will be emitted into a .dot file.
EOF
	},

	dump_dominator_tree => {
		sdesc => 'Dump the dominator graphs',
		desc  => <<EOF
	When this option is enabled, the dominator tree for each method will be
	emitted into a .dot file.
EOF
	},

	enable_pcp_emu => {
		sdesc => 'Java Monitor emulation using the OSEK RES_SCHEDULER resource',
		desc  => <<EOF
	This option enables an emulation of Java monitors' lock and unlock
	primitives by mapping them to the aquisition and release of the special OSEK
	RES_SCHEDULER resource, which raises the priority of the holder to the
	highest in the system and prevents preemption. Preemption by category 1 ISRs
	is not prevented. ISR1 must not use Java monitors when this option is
	enabled.
EOF
	},

	exception_messages => {
		sdesc => 'Outputs a message on application-generated exceptions',
		desc  => <<EOF
	When this option is enabled, an error message will be generated upon the
	rise of an application-generated exceptions with details on the location
	at that the exception was thrown.
EOF
	},

	init_systemobjects_on_startup => {
		sdesc => 'Initialize system objects on startup',
		desc  => <<EOF
	Normally, system objects are statically initialized. For some OSEK
	implementations, this approach does not work since the system object
	identifiers used are no compile-time constants. This option replaces the
	static initialization of the system objects by startup code that initializes
	the system objects, which works even if the OSEK identifiers are no
	compile-time constants. OSEK backends that require this option will set it
	automatically.
EOF
	},

	imcode_checks => {
		sdesc => 'Move null-checks into the intermediate code',
		desc  => <<EOF
	Normally, runtime checks are emitted into the C code during the final
	translation phase. When this option is enabled, the checks are moved
	into the intermediate code at an early time as explicit instructions,
	permitting more aggressive optimizations.
EOF
	},

	int8 => {
		sdesc => 'Use 8-bit wide integers',
		desc  => <<EOF
	This option forces the use of 8-bit wide integers as opposed to the 32-bit
	mandated by the JVM spec. The JVM's long type will be 16-bit wide.
EOF
	},

	int16 => {
		sdesc => 'Use 16-bit wide integers',
		desc  => <<EOF
	This option forces the use of 16-bit wide integers as opposed to the 32-bit
	mandated by the JVM spec. The JVM's long type will be 32-bit wide. This is
	the default on 8-bit microcontrollers.
EOF
	},

	jdk_bootclasses => {
		sdesc => "Use the host JDK's bootclasses",
		desc  => <<EOF
	Normally, all KESO modules except the core module are compiled using KESO's
	core lib as the bootclasses. This option compiles all modules using the host
	JDK's bootclasses in the java source to bytecode precompilation phase, which
	enables the compilation of modules that reference classes not part of KESO's
	JDK with javac. Normally, jino will fail at a later stage since those classes
	cannot be loaded. Only if the referenced classes are already dead with respect
	to jino's early loading phase, jino will be able to complete successfully. This
	option is currently a workaround for CDj for which the above scenario is the
	case.
EOF
	},

	production => {
		sdesc => 'Enable Production Mode',
		desc  => <<EOF
	This option enables KESO's production mode with the aim to generate smaller
	and faster code. This is the equivalent to defining the KESO_PRODUCTION
	CPP macro when compiling the generated code. The current effect of this
	option is that runtime errors will not output an error message. Note that
	this option is ignored if the 'debug' option is also set.
EOF
	},

	rtcheck_opt => {
		sdesc => 'Optimize runtime checks',
		desc  => <<EOF
	This option enables the omission of redundant runtime checks during the
	translation phase.
EOF
	},

	simplify_cfg => {
		sdesc => 'Simplify the control flow graph',
		desc  => <<EOF
	When this option is enabled, the control-flow graph is simplified by
	removing empty basic blocks.
EOF
	},
	
	single_file => {
		sdesc => 'Create one C file per class',
		desc  => <<EOF
	With this option, jino creates one C module per Java class containing all
	the class' methods. The standard behavior is to create one C module per
	Java method.
EOF
	},
	
	task_rlookup_fkt => {
		sdesc => 'Switch-Function for reverse lookup of task ids',
		desc  => <<EOF
	Normally, KESO retrieves the Java system object for an OSEK task id by using
	the task ID as an index into an array (keso_task_index) containing references
	to the task objects. This only works for OSEK implementations that sequentially
	enumerate the tasks.
	This option causes jino to create a function that compares the ID value to
	the respective OSEK constants, which works in all cases but has linear lookup
	time. Note that this option is automatically set by backends for OSEK systems
	for that the default method does not work.
EOF
	},

	use_mathlib => {
		sdesc => 'Use C math library for performance-critical Math functions',
		desc  => <<EOF
	KESO's java.lang.Math class contains Java implementations for most of the
	Math class' methods. This option enables the use of the C library's
	implementation of some performance-critical functions where the Java
	version showed to perform significantly worse than the C version.
EOF
	},
	
	use_mathlib_all => {
		sdesc => 'Use C math library for all Math functions',
		desc  => <<EOF
	Like `use_mathlib`, but enables the use of the native Math functions for all
	methods of the Math class for that native pendants exist.
EOF
	},
	
	whole_program_opt => {
		sdesc => 'Enable whole-program optimizations',
		desc  => <<EOF
	This option enables a set of advanced optimizations based on a
	flow-sensitive whole-program analysis.
EOF
	},

	wide_char => {
		sdesc => 'Use 16-bit char type',
		desc  => <<EOF
	Deviating from the JVM spec, KESO normally uses 8-bit characters for the Java
	char type. This option enables the use of 16-bit characters to enforce
	standard-compliant behavior for applications that need it.
EOF
	},
	
	_X_llref_mark2 => {
		sdesc => "Marker value marks the end of a stack frame's reference list",
		desc  => <<EOF
	When used with a garbage collector, KESO uses a linked list of references
	to allow the GC to identify all reference values on the stack of a task.
	For this, local reference variables of a method are grouped in an array on
	the stack frame. To mark the end of this array, KESO normally uses a mangled
	pointer to the next method's reference list, where the least significant bit
	of the pointer is set to 1. This assumes that an object reference is always
	aligned to at least 2 bytes. It also requires that the reference array on
	each stack frame starts at a 2-byte-aligned address. These prerequisites
	are satisfied on pretty much any 16-bit or larger architecture.
	This option activates an alternate method to mark the end of the reference
	array by a special marker value. The size of the reference array is
	increased by one reference value on each frame when this option is used,
	however, it should be a safe choice for any architecture.
EOF
	},

	# Debugging/Benchmarking options
	count_checks => {
		sdesc => 'Count the executions of each runtime check',
		desc  => <<EOF
	With this option enabled jino records each execution of a runtime check in
	an array. Each check has a unique ID that is the index into this array. When
	the program has finished the array can be inspected by using a debugger to
	identify hot runtime checks in the application.
EOF
	},
	
	debug => {
		sdesc => 'Enables sanity checks and assertions to detect bugs',
		desc  => <<EOF
	This option enables assertions and also inserts runtime sanity checks in the
	generated code to provide an early detection of bugs. This option disables
	the `production` option.
EOF
	},
	
	ignore_critical => {
		sdesc => 'Turns critical internal compiler errors into warnings',
		desc  => <<EOF
	Enabling this option turns some critical internal error conditions that
	normally cause the compiler to abort into compiler warnings.
EOF
	},

	keep_final_fields => {
		sdesc => 'Keep primitive static final fields',
		desc  => <<EOF
	Normally, jino eliminated compile-time constant static final primitive
	fields to that all accessess were subject to constant propagation. This
	option causes these fields to be retained in the domains' static field sets.
EOF
	},

	omit_safechk => {
		sdesc => 'Disable all runtime safety checks',
		desc  => <<EOF
	This option disables all runtime safety checks (null, bounds, etc.). The
	effect is the same as defining the CPP macro `KESO_OMIT_SAFECHECKS` when
	compiling the generated C code.
EOF
	},

	rtcheckstat => {
		sdesc => 'Dump Statistics on the emitted runtime checks',
		desc  => <<EOF
	When this option is enabled, jino creates a file rtCheckStat.lst in the
	output directory that contains a categorized statistics of the number
	of runtime checks emitted and optimized at compile time.
EOF
	},
	show_costs => {
		sdesc => 'Add the heuristic value of the inline cost function to the comment in output file',
		desc  => <<EOF
EOF
	},
	todo => {
		sdesc => 'Print some developer TODO messages affecting the current application',
		desc  => <<EOF
	This option shows some TODOs, mainly affecting not yet implemented compiler
	optimizations that the currently compiled application would benefit from.
EOF
	},

	_X_all_CanBlock => {
		sdesc => 'Treat all methods as blocking',
		desc  => <<EOF
	This option causes jino to consider all methods as blocking methods. Normally,
	only methods (transitively) calling a blocking system service (WaitEvent) are
	considered as blocking. The main effect of this option is that the linked list
	of stack references will be generated and maintained in all methods.
EOF
	},
	
	
	gradmp => {
		sdesc => 'Enables gradual software-based memory protection',
		desc  => <<EOF
	This option enables gradual software-based memory protection (GradMP).
	Null- and bounds-checks will be omitted, according to the safety-levels specified
	for each domain.
	AllChecks: No checks will be omitted.
	HWOffloading:
	Omit all software-checks, that can be done in hardware;
	The program's behaviour will not change.
	Containment:
	Omit all checks, that are not required for memory-safety;
	The program's behaviour may change, in case of program-errors
	(i.e some exceptions won't be triggered).
EOF
	},
	
	gradmp_inactive => {
		sdesc => 'Disables GradMP (see gradmp)',
		desc  => <<EOF
	This option disables gradual software-based memory protection (default).
EOF
	},
	
	gradmp_no_omit_nullchecks => {
		sdesc => 'Prevents GradMP from omitting nullchecks (see gradmp)',
		desc  => <<EOF
	This option prevents GradMP from omitting nullchecks.
EOF
	},
	
	gradmp_no_omit_boundchecks => {
		sdesc => 'Prevents GradMP from omitting boundchecks (see gradmp)',
		desc  => <<EOF
	This option prevents GradMP from omitting boundchecks.
EOF
	},
	
	gradmp_allow_unexpected_nullrefs => {
		sdesc => 'Allows GradMP to omit checks even if it could result in unexpected null-references',
		desc  => <<EOF
	When a reference is read from an object,
	GradMP (see gradmp) could remove the nullcheck before this read-operation,
	if the reference that would be read from a null-reference is also null (safety="containment").
	However, other optimizations could have flagged this reference-field as "never null" (see final)
	and subsequent accesses on this never-null-reference (which now could be null!) might be unchecked.
	This could result in UNSAFE code!
	This option allows GradMP to omit checks even if it could result in unexpected null-references.
EOF
	},
	
	gradmp_read_has_global_impact => {
		sdesc => 'GradMP assumes that read-operations have global impact',
		desc  => <<EOF
	Reading invalid primitive data (int,short,...) has just local impact.
	Therefore, checks before such operations could be omitted (if safety="containment").
	However, on some architectures read-operations could read a memory-mapped register and change its value (global impact).
	This option prevents GradMP from omitting such checks, if there are blocks with "unspec" read-behaviour.
	(default; see gradmp_read_has_local_impact)
EOF
	},

	gradmp_read_has_local_impact => {
		sdesc => 'GradMP assumes that read-operations have local impact',
		desc  => <<EOF
	Reading invalid primitive data (int,short,...) has just local impact,
	no matter whether there are Blocks with "unspec" read-behaviour or not.
	(see gradmp_read_has_global_impact)
EOF
	},

};

