#!/bin/bash

# include this script from the KESO rootpath, e.g.
# cd ~/keso
# source bin/setup.bash

function KESOSETUP_addpath {
  [ -d "$2" ] || return
  if eval test -z "\"\$$1\""; then
		eval "export $1=$2"

	elif ! eval test -z "\"\${$1##*:$2:*}\"" -o -z "\"\${$1%%*:$2}\"" -o -z "\"\${$1##$2:*}\"" -o -z "\"\${$1##$2}\""; then
    eval "export $1=\$$1:$2"
  fi
}

function KESOSETUP_setenv {
	if eval test -z "\"\$$1\"" -o ! -x "\"\$$1\""; then
		if [ -n "$2" -a -f "$2" ]; then
			eval "export $1=$2"
		elif [ -n "$3" ]; then
			echo "Warning: could not determine a sensible setting for $1. Please set manually." 1>&2
		fi
	fi
}

if [ ! -d "$PWD/src/builder" ]; then
	echo "the current directory ($PWD) does not appear to be the KESO root path" 1>&2
	echo "please source this script from the KESO root directory (contains the src and bin subdirs)" 1>&2
else

	export KESOROOTPATH=$PWD
	export KESOSRCPATH="$KESOROOTPATH/src"

	# default JINO compile options
	[ -z "$JINOFLAGS" ] && export JINOFLAGS="-nobuild -X:dbg_src:ssa:const_arg_prop:ssa_alias_prop:simplify_cfg:rtcheck_opt:whole_program_opt:imcode_checks"

	KESOSETUP_addpath PATH "${KESOROOTPATH}/bin"

	# JDK setup
	# JDK tools.jar is needed since jino uses the javac Sun API, which is not
	# part of the public API
	KESOSETUP_setenv JAVACC "`which javacc`" WARN

	if [ -n $JDK ] && [ -f ${JDK}/lib/tools.jar ]; then
		echo "Using preconfigured JDK at $JDK"

		######         Mac OS X 10.6
	elif [ `uname` = 'Darwin' ]; then
		if [ -d "/System/Library/Frameworks/JavaVM.framework/Versions/1.6.0" ]; then
			export JDK=/System/Library/Frameworks/JavaVM.framework/Versions/1.6.0
			export JDKTOOLS=$JDK/Classes/classes.jar
			KESOSETUP_setenv JAVA "${JDK}/Commands/java" WARN
			KESOSETUP_setenv JAVAC "${JDK}/Commands/javac" WARN
		fi

		if [ -z "$JDK" -o ! -f ${JDK}/Classes/classes.jar ]; then
			echo "cannot find a valid JDK" 1>&2
			echo "please correct the setting of the JDK environment variable" 1>&2
			echo "the JDK path should contain the Classes/classes.jar file" 1>&2
			unset JDK
		fi

	else
		for jdkpath in \
			/usr/lib/icedtea6 \
			/usr/lib/jvm/java-6-sun \
			/usr/lib/jvm/java-6-openjdk \
			; do

			if [ -f "$jdkpath/lib/tools.jar" ]; then
				export JDK="$jdkpath"
				break
			fi
		done

		if [ -z "$JDK" -o ! -f "${JDK}/lib/tools.jar" ]; then
			echo "cannot find a valid JDK $JDK" 1>&2
			echo "please correct the setting of the JDK environment variable" 1>&2
			echo "the JDK path should contain the lib/tools.jar file" 1>&2
			unset JDK
		else
			KESOSETUP_setenv JAVA "${JDK}/bin/java"
			KESOSETUP_setenv JAVAC "${JDK}/bin/javac"
			export JDKTOOLS=${JDK}/lib/tools.jar
		fi
	fi

	# Path to tricore-gcc toolchain
	#if [ -d "/proj/i4ciao/tools/trigcc/" ] ; then
	#	export TOOLPATH=/proj/i4ciao/tools/trigcc/
	# KESOSETUP_addpath PATH "${TOOLPATH}bin"
	#fi

	# setup for JOsek
	KESOSETUP_setenv JOSEK "${KESOROOTPATH}/josek.jar"
	if [ -n "$JOSEK" -a -f "$JOSEK" ]; then
		alias josek="$JAVA -jar $JOSEK"
		export JOSEKRULES="josek/code/arch/"
	fi

	# setup alias for avrora
	KESOSETUP_setenv AVRORA_JAR "${KESOROOTPATH}/avrora.jar"
	[ -n "$AVRORA_JAR" ] && alias avrora="$JAVA -jar ${AVRORA_JAR}"

	# setup for ProOSEK
	#if [ -d "/usr/local/ProOSEK" ] ; then
	#	export OSEK_BASE=/usr/local/ProOSEK
	# KESOSETUP_addpath PATH "${OSEK_BASE}/bin"
	#elif [ -d "$HOME/ProOSEK" ] ; then
	#	export OSEK_BASE=$HOME/ProOSEK
	# KESOSETUP_addpath PATH "${OSEK_BASE}/bin"
	#fi

	# setup for Lauterbach t32 debugging tools
	#export TRACE32PATH="/proj/i4ciao/tools/bin"
	#if [ -d "$TRACE32PATH" ] ; then
	# KESOSETUP_addpath PATH "$TRACE32PATH"
	#fi

	# optional binaries
	KESOSETUP_setenv CTAGBIN "`which ctags`"
	KESOSETUP_setenv AVRDUDE "`which avrdude`"
fi

unset -f KESOSETUP_addpath
unset -f KESOSETUP_setenv

