/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.classfile;

import java.io.*;
import keso.classfile.constantpool.*;

/**
    All data about a exception handler, that can be found
    in a class file. This is a rather passive class.
    It only reads the data from the class file and
    stores it.
*/
public class ExceptionHandlerData {

	private int startBCIndex;
	private int endBCIndex;
	private int handlerBCIndex;
	private int catchTypeCPIndex;
	private ClassCPEntry catchTypeCPEntry;

	public ExceptionHandlerData() {}

	public ExceptionHandlerData(DataInput input, ConstantPool cPool) throws IOException {
		readFromClassFile(input, cPool);
	}

	//constructor for copy
	public ExceptionHandlerData(int startBCIndex, int endBCIndex,
			int handlerBCIndex, int catchTypeCPIndex) {
		this.startBCIndex = startBCIndex;
		this.endBCIndex = endBCIndex;
		this.handlerBCIndex = handlerBCIndex;
		this.catchTypeCPIndex = catchTypeCPIndex;
	}

	public ExceptionHandlerData copy() {
		return new ExceptionHandlerData(startBCIndex, endBCIndex, handlerBCIndex,
				catchTypeCPIndex);
	}

	public int getStartBCIndex() {return startBCIndex;}
	public int getEndBCIndex() {return endBCIndex; }
	public int getHandlerBCIndex() {return handlerBCIndex;}
	public int getCatchTypeCPIndex() {return catchTypeCPIndex; }

	public String getCatchTypeCPEntry() {
		if (catchTypeCPEntry==null) return null;
		return catchTypeCPEntry.getClassName();
	}

	/**
	 * read exception_table
	 * @see <a http://java.sun.com/docs/books/jvms/second_edition/html/ClassFile.doc.html#3129>
	 *      JVM Spec 4.7.4 The Exceptions Attribute</a>
	 */
	public void readFromClassFile(DataInput input, ConstantPool cPool) throws IOException {
		startBCIndex = input.readUnsignedShort();
		endBCIndex = input.readUnsignedShort();
		handlerBCIndex = input.readUnsignedShort();
		catchTypeCPIndex = input.readUnsignedShort();
		if (catchTypeCPIndex!=0) {
			catchTypeCPEntry = (ClassCPEntry)cPool.entryAt(catchTypeCPIndex);
		}
	}

	public String toString() {
		return new String(startBCIndex+" - "+endBCIndex+" handler "+handlerBCIndex+" type "+catchTypeCPIndex);
	}
}
