/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.classfile;

/**
    Adapter for sources of the data of
    bytecode methods.
    See subclasses
*/

abstract public class MethodSource {

  abstract public ClassSource getDeclaringClassSource();
  abstract public String getMethodName();
  abstract public String getMethodType();

  abstract public boolean isPublic();
  abstract public boolean isPrivate();
  abstract public boolean isProtected();
  abstract public boolean isStatic();
  abstract public boolean isFinal();
  abstract public boolean isAbstract();
  abstract public boolean isNative();

  public boolean isOverrideable() {
     return !(isPrivate() || isStatic() || isFinal() || getDeclaringClassSource().isFinal());
  }

  abstract public byte[] getBytecode();
  abstract public int getNumInstr();
  abstract public int getNumLocalVariables();
  abstract public int getNumStackSlots();
  abstract public ExceptionHandlerData[] getExceptionHandlers();
  abstract public LineAttributeData[] getLineNumberTable();

}
