/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.classfile.constantpool;

import java.io.*;

public class ClassCPEntry extends ConstantPoolEntry {

	private int classNameCPIndex;
	private UTF8CPEntry classNameCPEntry;

	public ClassCPEntry() {}

	public ClassCPEntry(int classNameCPIndex) {
		this.classNameCPIndex = classNameCPIndex;
	}

	public int getTag() {return CONSTANT_CLASS;}

	void readFromClassFile(DataInput input) throws IOException {
		classNameCPIndex = input.readUnsignedShort();
	}

	public void linkCPEntries(ConstantPool cPool) {
		classNameCPEntry = (UTF8CPEntry)cPool.entryAt(classNameCPIndex);
	}

	public String getSimpleDescription() {
		return String.valueOf(classNameCPIndex);
	}

	public String getClassName() {
		return classNameCPEntry.value();
	}

	public String getDescription(ConstantPool cPool, boolean withIndex) {
		if (withIndex)
			return getIndexDescString(cPool, classNameCPIndex);
		else
			return classNameCPEntry.getDescription(cPool,true);
	}

	public String getDescription() {
		return getClassName();
	}

	int getStaticDataAddress() {return 0;}
}
