/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.classfile.constantpool;

import java.io.*;

abstract public class ClassMemberCPEntry extends ConstantPoolEntry {

	protected int classCPIndex;
	protected int nameAndTypeCPIndex;

	protected ClassCPEntry classCPEntry;
	protected NameAndTypeCPEntry nameAndTypeCPEntry;

	public ClassMemberCPEntry() {}

	ClassMemberCPEntry(int classCPIndex, int nameAndTypeCPIndex) {
		this.classCPIndex = classCPIndex;
		this.nameAndTypeCPIndex = nameAndTypeCPIndex;
	}

	public ClassMemberCPEntry(ClassCPEntry classCPEntry,
			NameAndTypeCPEntry nameAndTypeCPEntry) {
		this.classCPEntry = classCPEntry;
		this.nameAndTypeCPEntry = nameAndTypeCPEntry;
		classCPIndex = -1;
		nameAndTypeCPIndex = -1;
	}

	@Override
	void readFromClassFile(DataInput input) throws IOException {
		classCPIndex = input.readUnsignedShort();
		nameAndTypeCPIndex = input.readUnsignedShort();
	}

	@Override
	public void linkCPEntries(ConstantPool cPool) {
		classCPEntry = (ClassCPEntry)cPool.entryAt(classCPIndex);
		nameAndTypeCPEntry = (NameAndTypeCPEntry)cPool.entryAt(nameAndTypeCPIndex);
	}

	@Override
	public String getSimpleDescription() {
		return String.valueOf(classCPIndex)+", "+
			String.valueOf(nameAndTypeCPIndex);
	}

	public String getClassName() {
		return classCPEntry.getClassName();
	}

	public ClassCPEntry getClassCPEntry() {
		return classCPEntry;
	}

	public String getMemberName() {
		return nameAndTypeCPEntry.getName();
	}

	public String getMemberTypeDesc() {
		return nameAndTypeCPEntry.getTypeDesc();
	}

	public boolean isMember(String className,String memberName,String typeDesc) {
		return
			(classCPEntry.getClassName().equals(className) &&
			 nameAndTypeCPEntry.getName().equals(memberName) &&
			 nameAndTypeCPEntry.getTypeDesc().equals(typeDesc));
	}

	@Override
	public String getDescription(ConstantPool cPool, boolean withIndex) {
		return "class=" + getIndexDescString(cPool, classCPIndex) + ", " +
			"name_and_type=(" + nameAndTypeCPIndex + "), " +
			nameAndTypeCPEntry.getDescription(cPool, true);
	}

	@Override
	public String getDescription() {
		return getClassName() + "." + getMemberName() + getMemberTypeDesc();
	}

	public final int getNameAndTypeIndex() { return nameAndTypeCPIndex; }

	public final int getClassIndex() { return classCPIndex; }
}
