/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.classfile.constantpool;

/**
 * Fake method reference constant pool entry.
 * The normal {@link MethodRefCPEntry} contains a set of references to the
 * actual constant pool entries for class name, member name and type descriptor.
 * Such a descriptor may not exist for each method, but it may be necessary to
 * create one on-the-fly in order to replace the method descriptor in an invoke
 * statement.
 */
public final class FakeMethodRefCPEntry extends MethodRefCPEntry {

	private final String className;
	private final String memberName;
	private final String memberTypeDesc;

	public FakeMethodRefCPEntry(String className, String memberName,
			String memberTypeDesc) {
		this.className      = className;
		this.memberName     = memberName;
		this.memberTypeDesc = memberTypeDesc;
	}

	@Override
	public String getClassName() {
		return className;
	}

	@Override
	public String getMemberName() {
		return memberName;
	}

	@Override
	public String getMemberTypeDesc() {
		return memberTypeDesc;
	}

}
