/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.classfile.constantpool;
import java.io.*;

public class NameAndTypeCPEntry extends ConstantPoolEntry {
	private int nameCPIndex;
	private int typeCPIndex;

	private UTF8CPEntry nameCPEntry;
	private UTF8CPEntry typeCPEntry;

	public NameAndTypeCPEntry() {}

	NameAndTypeCPEntry(int nameCPIndex, int typeCPIndex) {
		this.nameCPIndex = nameCPIndex;
		this.typeCPIndex = typeCPIndex;
	}

	public int getTag() {return CONSTANT_NAMEANDTYPE;}

	void readFromClassFile(DataInput input) throws IOException {
		nameCPIndex = input.readUnsignedShort();
		typeCPIndex = input.readUnsignedShort();
	}

	void linkCPEntries(ConstantPool cPool) {
		nameCPEntry = (UTF8CPEntry)cPool.entryAt(nameCPIndex);
		typeCPEntry = (UTF8CPEntry)cPool.entryAt(typeCPIndex);
	}

	public String getSimpleDescription() {
		return String.valueOf(nameCPIndex)+", "+
			String.valueOf(typeCPIndex);
	}

	public String getName() {
		return nameCPEntry.value();
	}

	public String getTypeDesc() {
		return typeCPEntry.value();
	}

	public String getDescription(ConstantPool cPool, boolean withIndex) {
		return "name=" + getIndexDescString(cPool, nameCPIndex) + ", " +
			"type=" + getIndexDescString(cPool, typeCPIndex);
	}

	public String getDescription() {
		return getName() + " (" + getTypeDesc() + ") ";
	}
}
