/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/


package keso.classfile.constantpool;
import java.io.*;
import keso.classfile.datatypes.*;
/**
    This class is used for all CP entries that represent
    a numeric value. (i.e. int, long, float, double)
*/

public class NumericCPEntry extends ConstantPoolEntry {
  private BCBasicDatatype value;
  private int tag;

  public NumericCPEntry() {}

  // die folgenden konstruktoren sollten auch noch argumente fuer die
  // adressen bekommen

  // the dummy argument is required to distinguish the constructors
  public NumericCPEntry(int value, int dummy) {
    this.value = new BCInteger(value);
    tag = CONSTANT_INTEGER;
  }

  public NumericCPEntry(float value) {
    this.value = new BCFloat(value);
    tag = CONSTANT_FLOAT;
  }

  public NumericCPEntry(double value) {
    this.value = new BCDouble(value);
    tag = CONSTANT_DOUBLE;
  }
  public NumericCPEntry(long value) {
    this.value = new BCLong(value);
    tag = CONSTANT_LONG;
  }

  public NumericCPEntry(int tag) {
    this.tag = tag;
  }

  public int getTag() {return tag;}

  void readFromClassFile(DataInput input) throws IOException {
    switch(tag) {
    case CONSTANT_INTEGER:
      value = new BCInteger(input.readInt());
      break;
    case CONSTANT_FLOAT:
      value = new BCFloat(input.readFloat());
      break;
    case CONSTANT_LONG:
      value = new BCLong(input.readLong());
      break;
    case CONSTANT_DOUBLE:
      value = new BCDouble(input.readDouble());
      break;
    }
  }

    void readDummyValue(DataInput input) throws IOException {
	switch(tag) {
	case CONSTANT_INTEGER:
	    value = new BCInteger(0);
	    input.skipBytes(4);
	    break;
	case CONSTANT_FLOAT:
	    value = new BCFloat(0);
	    input.skipBytes(4);
	    break;
	case CONSTANT_LONG:
	    value = new BCLong(0);
	    input.skipBytes(8);
	    break;
	case CONSTANT_DOUBLE:
	    value = new BCDouble(0);
	    input.skipBytes(8);
	    break;
	}
    }

  public BCBasicDatatype value() {return value;}

  public int valueInt() {
     return ((BCInteger)value).value();
  }

  public String getSimpleDescription() {
    return value.toString();
  }

  public String getDescription() {
    return value.toString();
  }

  int getNumericValueAddress() { return 0;}
}
