/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/


package keso.classfile.constantpool;
import java.io.*;

public class StringCPEntry extends ConstantPoolEntry {

	private int stringCPIndex;
	private UTF8CPEntry stringCPEntry;

	public StringCPEntry() {}

	public int getTag() {return CONSTANT_STRING;}

	void readFromClassFile(DataInput input) throws IOException {
		stringCPIndex = input.readUnsignedShort();
	}

	void linkCPEntries(ConstantPool cPool) {
		stringCPEntry = (UTF8CPEntry)cPool.entryAt(stringCPIndex);
	}

	public String getSimpleDescription() {
		return String.valueOf(stringCPIndex);
	}

	public String getStringObject(ConstantPool cPool) {
		return stringCPEntry.value();
	}

	public String value() {
		return stringCPEntry.value();
	}

	public String getDescription(ConstantPool cPool, boolean withIndex) {
		if (withIndex)
			return getIndexDescString(cPool, stringCPIndex);
		else
			return stringCPEntry.getDescription(cPool,true);
	}

	public String getDescription() {
		return value();
	}
}
