/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler;

import keso.compiler.imcode.*;

/**
 * Domain pass class.
 * This is a base class for passes that analyze all domains of a repository.
 * Note that passes derived from this class are not thread-safe!
 */
public abstract class DomainPass extends Pass {

	/** Currently used class store. */
	protected ClassStore classStore;

	/**
	 * Runs the pass for a specific repository.
	 * @param repository Class store.
	 * @return <code>true</code> if the pass has made any modifications,
	 *         <code>false</code> if it has left the code untouched.
	 * @throws CompileException
	 */
	public boolean runOnRepository(ClassStore repository)
	throws CompileException {
		boolean modified = false;
		classStore = repository;
		for (IMDomain domain : repository.getOptions().getDomains()) {
			modified |= runOnDomain(domain);
		}
		return modified;
	}

	/**
	 * Runs the pass for a specific domain.
	 * @param domain The domain.
	 * @return <code>true</code> if the pass has made any modifications,
	 *         <code>false</code> if it has left the code untouched.
	 */
	public abstract boolean runOnDomain(IMDomain domain)
	throws CompileException;

}
