/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler;

import java.io.*;

final public class Main {
	static class NoExit extends SecurityManager {
		public NoExit() {

		}
		public void checkCreateClassLoader() { }
		public void checkAccess(Thread g) { }
		public void checkAccess(ThreadGroup g) { }
		public void checkExec(String cmd) { }
		public void checkLink(String lib) { }
		public void checkRead(FileDescriptor fd) { }
		public void checkRead(String file) { }
		public void checkRead(String file, Object context) { }
		public void checkWrite(FileDescriptor fd) { }
		public void checkWrite(String file) { }
		public void checkDelete(String file) { }
		public void checkConnect(String host, int port) { }
		public void checkConnect(String host, int port, Object context) { }
		public void checkListen(int port) { }
		public void checkAccept(String host, int port) { }
		public void checkMulticast(java.net.InetAddress maddr) { }
		public void checkPropertiesAccess() { }
		public void checkPropertyAccess(String key) { }
		public boolean checkTopLevelWindow(Object window) { return true; }
		public void checkPrintJobAccess() { }
		public void checkSystemClipboardAccess() { }
		public void checkAwtEventQueueAccess() { }
		public void checkPackageAccess(String pkg) { }
		public void checkPackageDefinition(String pkg) { }
		public void checkSetFactory() { }
		public void checkMemberAccess(Class<?> clazz, int which) { }
		public void checkSecurityAccess(String provider) { }

		public void checkExit(int status) {
			throw new NoExitException(status);
		}

		public void checkPermission(java.security.Permission perm) {
		}
	}

	public static void main(String[] argv) {
		BuilderOptions opts = null;

		try {
			opts = new BuilderOptions(argv);
			System.setSecurityManager(new NoExit());
			Builder builder = new Builder(opts);

			if (opts.getPreCompile()) {
				// compile all source files into class files
				builder.preCompileModules();
			}

			if (opts.getTranslate())
				builder.computeServices();

			opts.parseFirstSystemDefinition();
			do {
				if (opts.getPreCompile()) {
					// copy the source files of all modules to the src/ subdir of the output directory
					// purpose?
					String[] modules = builder.getRequiredModules(opts.getModulePaths());
					builder.copySourceFiles(opts.getOutputPath()+"/src", modules);
				}

				if (opts.getTranslate()) {
					// read class files from disk and prune the class tree
					builder.loadModules();

					// pre-execute class constructors and parse method data
					builder.loadClasses();

					// Run analysis and transformation passes
					builder.runPasses();

					builder.writeRTCheckStats();
				}

			} while (opts.parseNextSystemDefinition());

			if (opts.doFirstLevelMakefile())
				builder.writeFirstLevelMakefile();

			if (opts.doBuildImage())
				builder.callMake();

		} catch (CompileException e) {
			if (opts != null && opts.hasVerbose())
				e.printStackTrace();
			else
				System.err.println("Error: " + e.getMessage());
			exit(1);
		} catch (Exception e) {
			e.printStackTrace();
			exit(1);
		}
	}

	private static void exit(int status) {
		System.setSecurityManager(new SecurityManager());
		System.exit(status);
	}

}
