/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler;

import keso.compiler.imcode.*;

public abstract class MethodPass extends ClassPass {

	/**
	 * Runs the pass for a specific class.
	 * @param domain The class.
	 * @return <code>true</code> if the pass has made any modifications,
	 *         <code>false</code> if it has left the code untouched.
	 */
	@Override
	public boolean runOnClass(IMClass imClass)
	throws CompileException {
		if (skipClass(imClass))
			return false;
		IMMethod[] methods = imClass.getMethods();
		if (methods == null)
			return false;
		boolean modified = false;
		for (IMMethod method : methods) {
			modified |= runOnMethod(method);
		}
		return modified;
	}

	/**
	 * Runs the pass for a specific method.
	 * @param domain The method.
	 * @return <code>true</code> if the pass has made any modifications,
	 *         <code>false</code> if it has left the code untouched.
	 */
	public abstract boolean runOnMethod(IMMethod method)
	throws CompileException;

	protected boolean skipClass(IMClass imClass) {
		return false;
	}

}
