/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.analysis;

import keso.classfile.datatypes.*;
import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.transform.*;

public final class EscapeAnalysis extends MethodPass {

	@Override
	public String getName() {
		return "Escape analysis";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.hasOption("ssa_astack");
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		info.setIsAnalysis(false); // FIXME: Make some pass depend on this
		info.addDependency(ConstantFolding.class);
		info.addDependency(SSAConstruction.class);
		info.addDependency(VTableComputation.class);
		info.addAntiDependency(SSADeconstruction.class);
	}

	@Override
	public boolean runOnMethod(IMMethod method) throws CompileException {

		BuilderOptions opts = method.getOptions();
		IMEscapeVisitor pv = new IMEscapeVisitor(opts, IMEscapeVisitor.PREPARE);
		for (IMBasicBlock bb : method) {
			bb.visitNodes(pv);
		}
		IMEscapeVisitor av = new IMEscapeVisitor(opts, IMEscapeVisitor.ANALYSE);
		for (IMBasicBlock bb : method) {
			bb.visitNodes(av);
		}

		// Analyze if method arguments escape and save the result
		IMSlot args[] = method.getMethodFrame().getOriginalArguments();
		method.initEscapeInfo();
		if (opts.hasOption("_X_ssa_less_astack") == false) {
			for (int i = 0; i < args.length; ++i) {
				opts.vverbose("## test escape " + method.getAlias() + " "
						+ args[i]);
				if (args[i].getType() == BCBasicDatatype.REFERENCE)
					method.setEscapeInfo(i, args[i].doesEscape(true));
			}
		}
		return false;
	}

	@Override
	protected boolean skipClass(IMClass imClass) {
		return (imClass instanceof IMArrayClass);
	}

}
