/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.analysis;

import java.util.*;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.util.*;

/** Path information about a specific stack slot. */
public final class SlotInfo {

	private static class SlotEntry {
		private final Map<IMBasicBlock, IMDataFlow> pathCells
		= new HashMap<IMBasicBlock, IMDataFlow>();
		private IMDataFlow bbCell; // Cached intersection of all pathCells
	}

	private final SplitKeyMap<IMSlot, IMBasicBlock, SlotEntry> entries
	= new SplitKeyHashMap<IMSlot, IMBasicBlock, SlotEntry>();

	boolean setPathCell(IMSlot slot, IMBasicBlock bb,
			IMBasicBlock dominator, IMDataFlow cell) throws CompileException {

		// If the cell is null, remove it
		boolean   changed = true;
		SlotEntry entry   = entries.get(slot, bb);
		if (cell == null) {
			if (entry == null)
				return false;
			entries.remove(slot, bb);
		} else {
			if (entry == null) {
				entry = new SlotEntry();
				entries.put(slot, bb, entry);
			}
			IMDataFlow oldCell = entry.pathCells.put(dominator, cell);
			changed = !cell.equals(oldCell);
		}

		// Compute the intersection of the pathCells and cache it
		IMDataFlow bbCell = IMDataFlow.ANY;
		for (IMDataFlow pathCell : entry.pathCells.values()) {
			bbCell = bbCell.intersect(pathCell);
		}
		entry.bbCell = bbCell;
		return changed;
	}

	IMDataFlow getBBCell(IMSlot slot, IMBasicBlock bb) {
		SlotEntry entry = entries.get(slot, bb);
		if (entry == null)
			return null;
		return entry.bbCell;
	}

}
