/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.analysis.connectiongraph;

import keso.compiler.CompileException;
import keso.compiler.imcode.IMField;
import keso.util.DotWriterException;

/**
 * A field reference in the connection graph, i.e. non-static fields in objects that are object references
 */
public class CGFieldReferenceNode extends CGReferenceNode {
	/**
	 * The object node for which this field reference is valid, i.e. the object which contains the referenced field
	 */
	private final CGObjectNode	object;

	/**
	 * The field referenced inside the object
	 */
	private final IMField		field;

	/**
	 * Creates a new field reference node in the connection graph for the field described by {@code field} in the object
	 * {@code object}.
	 *
	 * @param object
	 *            the object that contains the field
	 * @param field
	 *            the field inside the object
	 */
	protected CGFieldReferenceNode(final CGObjectNode object, final IMField field) {
		this.object = object;
		this.field = field;
		// initialize the escape state of this field reference with the escape state of the object we're attaching to
		updateEscapeState(object.getEscapeState());
		referenceNodeList.add(this);
	}

	/**
	 * Returns the object node that contains the referenced field
	 *
	 * @return the object node
	 */
	public CGObjectNode getObject() {
		return object;
	}

	/**
	 * Returns the field that is referenced inside the object
	 *
	 * @return the {@code IMField}
	 */
	public IMField getField() {
		return field;
	}

	@Override
	public void pathCompression() {
		// handle pathCompression() on field reference nodes different, because ww never want to mark these nodes as
		// stale which would prevent them from showing up in the graph dump and
		super.pathCompression();
		setStale(false);
	}

	@Override
	public void accept(final CGDotWriter visitor) throws DotWriterException, CompileException {
		visitor.visit(this);
	}
}
