/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.backend;

import keso.compiler.*;
import java.io.*;
import java.util.HashSet;

public class CoderAVR_JOSEK extends CoderJOSEK {
	public CoderAVR_JOSEK(BuilderOptions opts, MethodTableFactory table, ClassStore store) throws CompileException {
		super(opts,table,store);

		HashSet<String> supAttrs = supportedAttributes.get("OS");
		supAttrs.add("MICROCONTROLLER");
	}

	protected String extraAllTargets() { return "keso_main.od"; }
	protected void writeMakefile(PrintStream out) throws CompileException {
		super.writeMakefile(out);
		out.println("\nkeso_main.od: keso_main_g.elf");
		out.println("\t@echo Generating keso_main.od");
		out.println("\t@$(OBJCPY) -R .stab -R .stabstr keso_main_g.elf keso_main.tmp");
		out.println("\t@$(OBJDUMP) -zhD keso_main.tmp > keso_main.od");

		out.println("\nprogram: keso_main.hex");
		out.println("\tsudo avrdude -p atmega32 -P usb -c avrisp2 -U flash:w:keso_main.hex\n");
	}

	protected String architecture() { return "avr"; }

	protected void writePrintCharFunction(PrintStream out) throws IOException {
		// utilizes the avrora c-print monitor
		out.println("char keso_avrora_cprint_buf[64] __attribute__ ((externally_visible));");
		out.println("int __keso_printc( jchar c ) {");
		out.println("\tstatic unsigned char pos=1;");

		// we never overwrite the last byte of the buffer, which should be
		// initialized with 0 on startup -> we never explicitely terminate
		// the string with \0
		out.println("\tif(c=='\\0' || pos >= (sizeof(keso_avrora_cprint_buf)-2)) {");
		out.println("\t\tpos=1;");
		out.println("\t\tkeso_avrora_cprint_buf[0] = 2;"); // code for C-String
		out.println("\t\treturn 0;");
		out.println("\t}");

		out.println("\tkeso_avrora_cprint_buf[pos++] = (char) c;");
		out.println("\treturn 1;");
		out.println("}\n");
	}
}

