/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.backend;

import keso.compiler.*;
import java.io.*;


public class CoderAVR_Trampoline extends CoderTrampoline {
	public CoderAVR_Trampoline(BuilderOptions opts, MethodTableFactory table, ClassStore store) throws CompileException {
		super(opts,table,store);
	}

	protected String architecture() { return "avr"; }

	protected void writePrintCharFunction(PrintStream out) throws IOException {
		// utilizes the avrora c-print monitor
		out.println("char keso_avrora_cprint_buf[64] __attribute__ ((externally_visible));");
		out.println("int __keso_printc( jchar c ) {");
		out.println("\tstatic unsigned char pos=1;");

		// we never overwrite the last byte of the buffer, which should be
		// initialized with 0 on startup -> we never explicitely terminate
		// the string with \0
		out.println("\tif(c=='\\0' || pos >= (sizeof(keso_avrora_cprint_buf)-2)) {");
		out.println("\t\tpos=1;");
		out.println("\t\tkeso_avrora_cprint_buf[0] = 2;"); // code for C-String
		out.println("\t\treturn 0;");
		out.println("\t}");

		out.println("\tkeso_avrora_cprint_buf[pos++] = (char) c;");
		out.println("\treturn 1;");
		out.println("}\n");
	}

	protected String getGOILArchitecture() {
		return "avr";
	}
}

