/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.backend;

import keso.compiler.*;
import keso.compiler.config.*;
import keso.compiler.config.domain.*;
import keso.compiler.config.task.*;
import keso.compiler.imcode.*;

import java.io.*;


public class CoderCiAO extends CoderOSEK {

	public CoderCiAO(BuilderOptions opts, MethodTableFactory table, ClassStore store) throws CompileException {
		super(opts,table,store);

		opts.doFirstLevelMakefile(false);
	}

	protected String ISRMacro(int ISRCategory) {
		if(ISRCategory==2)
			return "ISR2";
		return "ISR";
	}

	protected void writeMainfileIncludes(PrintStream out) {
		out.println("#include \"ciao_appcode.h\"");
	}

	protected void writeOSEKIncludes(PrintStream out) {
		out.println("/************ CiAO compatibility ***************/");
		out.println("#ifndef KESO_CiAO_COMPAT");
		out.println("#define KESO_CiAO_COMPAT");
		out.println("#define TASK(TNAME) void functionTask##TNAME()");

		out.println("#include \"as/osek_compat.h\"");
		out.println("#include \"objects.h\"");

//		out.println("#include \"hw/triboard1796.h\"");
//		out.println("#include \"stdio.h\"");
//		out.println("#include \"unistd.h\"");

		out.println("#endif");
		out.println("/***********************************************/");
	}

	public void writeInfoFile() { /* CiAO build system doesn't like additional C-files */ }

	protected void addTaskHeader(PrintStream out, TaskDefinition task) throws CompileException {
		out.print("void ");
		out.print(task.getComponent());
		out.print("::functionTask");
		out.print(task.getIdentifier());
		out.print("() {\n");
	}

	protected void addISRHeader(PrintStream out, ISRDefinition isr) throws CompileException {
		out.print("void ");
		out.print(isr.getComponent());
		out.print("::functionISR");
		out.print(isr.getIdentifier());
		out.println("() {");
	}

	public void add_portal_enter(StringBuffer tgt, IMClass task_clazz, DomainDefinition<?> target_domain) throws CompileException {
		// CiAO, domain switching is implemented through aspects
	}

	public void add_portal_leave(StringBuffer tgt, IMClass task_clazz) throws CompileException {
	}

	public String extraProxyHelperArgString() throws CompileException {
		add_class("os/krn/Application");
		return ", os::krn::Application::Id CALLEE_APP";
	}

	public String extraProxyHelperArgs(ServiceDefinition srv) throws CompileException {
		add_class("os/mp/UCInfo");
		return ", os::UCInfo<"+srv.getDomain().getComponent()+">::APP";
	}

	public String getDomainSymbolPrefix(DomainDefinition<?> dom) throws CompileException {
		return "__CIAO_APPDATA_" + dom.getIdentifier() + "_";
	}

	/**
	 * Writes the CiAO config.xml file instead of an OIL file.
	 */
	protected void writeOILFile(String OILprefix) throws CompileException {
		String filename = opts.getOutputPath()+"/config.xml";

		StringBuffer buf = new StringBuffer(1000);

		buf.append("<?xml version=\"1.0\"?>\n");

		opts.getSysDef().toCiAOConfig(buf,"");

		System.err.println("writing CiAO configuration '" + filename + "'");
		try {
			File file = new File(filename);
			PrintStream out = new PrintStream(new FileOutputStream(file));
			out.print(buf.toString());
			out.close();
		} catch(IOException e) {
			throw new CompileException(e.getMessage());
		}
	}

	protected void writeMakefile(PrintStream out) throws CompileException {
		out.print("EXTRACXXFLAGS+=-I");
		out.print(opts.getOutputPath());
		out.print(" ");
		out.println(opts.getCFlags());
		
		out.print("LIBC_CXXFLAGS+=-I");
		out.print(opts.getOutputPath());
		out.print(" ");
		out.println(opts.getCFlags());

		out.println("\n\n# Dieses Target loescht in den von KESO erzeugten Quellen alle Sourcefiles von Funktionen,");
		out.println("# die in der Anwendung nirgends aufgerufen werden (i.d.R. weil diese an allen Stellen von");
		out.println("# jino geinlined wurden)");
		out.println(".PHONY: cleankeso");
		out.println("cleankeso:");
		out.println("\tcd Keso_$(APP) ; \\");
		out.println("\t\tfor file in c[0-9]*.cpp; do \\");
		out.println("\t\t\tfn=`basename $$file .cpp`; \\");
		// sucht nach direkten Aufrufen der Funktion in allen C Dateien
		out.println("\t\t\tif [ -z \"`grep -l $$fn\\( *.cpp | grep -v $$fn.cpp`\" ] \\");
		// sucht nach Eintraegen der Funktion in der dispatch table
		out.println("\t\t\t  && [ -z \"`grep $$fn\\)\\, global.cpp`\" ] \\");
		out.println("\t\t\t; then \\");
		out.println("\t\t\t\trm $$file; \\");
		out.println("\t\t\tfi; \\");
		out.println("\t\tdone\n");
	}

	private void writeCiAOComponent(DomainDefinition<? extends TaskDefinition> dom, PrintStream out) throws CompileException {
		out.print("\nCIAO_COMPONENT (");
		out.print(dom.getComponent());
		out.println(")\n\tpublic:");
		for(TaskDefinition td: dom.getTasks()) {
			out.print("\t\tTASKDECL(");
			out.print(td.getIdentifier());
			out.println(");");
		}
		for(ISRDefinition id: dom.getISRs()) {
			out.print("\t\tstatic void functionISR");
			out.print(id.getIdentifier());
			out.println("();");
		}
		out.println("};");
	}

	public void writeMainFile() throws CompileException {
		super.writeMainFile();

		// generate an additional ciao_appcode.h file that contains component definitions
		try {
			String fn = opts.getOutputPath()+"/ciao_appcode.h";
			PrintStream out = new PrintStream(new FileOutputStream(fn));

			out.println("/* THIS FILE IS AUTO-GENERATED BY KESO! DON'T EDIT */\n");
			out.println("#ifndef CIAO_APPCODE_H\n");
			out.println("#define CIAO_APPCODE_H\n");
			out.println("#include \"app/componentheader.h\"");

			// hook prototypes
			out.println("void StartupHook();");
			out.println("void PreTaskHook();");

			// component definitions
			SystemDefinition sdef = opts.getSysDef();

			for(JavaDomain dom: sdef.getJavaDomains()) {
				writeCiAOComponent(dom,out);
			}
			writeCiAOComponent(sdef.getDomainZero(), out);

			out.println("\n#endif\n");
		} catch (IOException ex) {
			throw new CompileException(ex.toString());
		}

		// generate an keso_pointcuts.ah, which contains pointcut expression for
		// trusted and untrusted parts of KESO
//	try {
//		String fn = opts.getOutputPath()+"/keso_pointcuts.ah";
//		PrintStream out = new PrintStream(new FileOutputStream(fn));
//
//		out.println("/* THIS FILE IS AUTO-GENERATED BY KESO! DON'T EDIT */\n");
//		out.println("#ifndef KESO_POINTCUTS_AH\n");
//		out.println("#define KESO_POINTCUTS_AH\n");
//		out.println("#include \"os/mp/common_pointcuts.ah\"");
//
//		// FIXME this is hardcoded until we have an advanced reachability analysis
//		out.println("namespace os { namespace mp {");
//		out.println("\tpointcut pcKesoInfrastructure() = \"% keso_%(...)\";");
//		out.println("\tpointcut pcKesoApplication() = \"% c%_%_m%_%(...)\";");
//		out.println("\tpointcut appTrusted() = pcKesoInfrastructure() || pcKesoApplication();");
//		out.println("\tpointcut appNonTrusted() = \"\";");
//		out.println("}}");
//
//		out.println("\n#endif\n");
//	} catch (IOException ex) {
//		throw new CompileException(ex.toString());
//	}
	}

	protected void writePrintCharFunction(PrintStream out) throws IOException {
		out.println("#define __keso_printc(c) __ciao_stdio_putc(c)");
	}

	public void emitPrintf() throws CompileException {
		global_header_add_class("unistd");
		global_header_add("#define KESO_WRITE(msg,len) write(1,msg,len)\n");
		global_header_add_class("stdio");
		global_header_add("#define KESO_PRINTF(...) printf(__VA_ARGS__)\n");
	}

	protected void allocSharedMemoryObjects() throws CompileException {
		if(opts.getSysDef().getSharedMemories().size() == 0)
			return;

		global_add("#include \"sharedmemory_objects.h\"\n");
		for(SharedMemoryDefinition shm: opts.getSysDef().getSharedMemories()) {
			if(!shm.isAccessedFromJavaDomain()) {
				continue;
			}

			IMClass mclass = repository.getClass(shm.getResourceClass());
			String objname = getResObjSymbol(shm);

			global_add_allocConstObject(mclass, objname);
			global_add_init_field("", "addr", "&" + shm.ident + "_SharedMemory");
			global_add_init_field("", "size", "sizeof(" + shm.getCType() + ")");
			global_add_init_end();
		}
	}

	protected void writeRuntimeData(PrintStream out) throws CompileException {
		super.writeRuntimeData(out);
		allocSharedMemoryObjects();
	}

	public void emitSystemTimeNanos() throws CompileException {
		add_class("hw/hal/SystemClock");
		// this only works correctly if freq is an even MHz value (can be divided by 1 mio)
		addln("\tjulong freq = (julong) hw::hal::SystemClock::Inst().freq() / 1000000ULL;");
		addln("\tjulong ticks = (julong) hw::hal::SystemClock::Inst().value();");
		addln("\treturn (jlong) (ticks * 1000ULL / freq);");
	}

	// tricore specific
	public String lock_sysreg() throws CompileException {
		return "hw::_tc::lock_wdtcon()";
	}
	public String unlock_sysreg() throws CompileException {
		return "hw::_tc::unlock_wdtcon()";
	}

	protected String codeFileSuffix() { return ".cpp"; }
}
