/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.backend;

import keso.compiler.*;
import java.io.*;
import java.util.HashSet;


public abstract class CoderJOSEK extends CoderOSEK {

	public CoderJOSEK(BuilderOptions opts, MethodTableFactory table, ClassStore store) throws CompileException {
		super(opts,table,store);
		HashSet<String> supAttrs = supportedAttributes.get("OS");
		supAttrs.add("FREQ");

		supAttrs = supportedAttributes.get("COUNTER");
		supAttrs.add("TIMER_BASE");
		supAttrs.add("TIME_IN_NS");
	}

	protected String extraObjectFiles() { return "os.o dispatch.o"; }
	protected String extraSourceFiles() { return "os.c dispatch.s"; }
	protected String extraGeneratedFiles() { return "os.h os.c dispatch.s"; }

	protected abstract String architecture();

	protected String extraMakeMacros() throws CompileException {
		StringBuffer fg = new StringBuffer("JOSEKFLAGS=-h ");
		fg.append(architecture());
		return fg.toString();
	}

	protected void writeOSEKIncludes(PrintStream out) {
		super.writeOSEKIncludes(out);
		// E_NOT_OK is non standard and not defined by JOSEK
		out.print("#ifndef E_NOT_OK\n");
		out.print("#define E_NOT_OK 10\n");
		out.print("#endif\n");
	}

	protected void writeMakefile(PrintStream out) throws CompileException {
		super.writeMakefile(out);
		out.println("\ndispatch.o: os.h");
		out.println("\t@echo OBJ dispatch.o");
		out.println("\t@$(CC) $(CFLAGS) -c -o dispatch.o dispatch.s");

		out.println("\nos.h os.c: keso_main.oil");
		out.println("\t@echo JOSEK os.c os.h");
		out.println("\t@$(JAVA) -jar ${JOSEK} ${JOSEKFLAGS} -c ${JOSEKRULES} -o ./ -f keso_main.oil");

		out.println("\nos.o: os.c os.h");
		out.println("\t@echo OBJ os.o");
		out.print("\t$(CC) $(CFLAGS) -c -o os.o os.c\n");
	}

	protected String ISRMacro(int ISRCategory) {
		if(ISRCategory==2)
			return "ISR2";
		return "ISR";
	}

	public boolean emptyObjectsNeedBraces() {
		return true;
	}
}

