/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.backend;

import keso.compiler.*;
import java.io.*;


public class CoderPOSIX_Trampoline extends CoderTrampoline {
	public CoderPOSIX_Trampoline(BuilderOptions opts, MethodTableFactory table, ClassStore store) throws CompileException {
		super(opts,table,store);
	}

	protected String architecture() { return "x86_32"; }

	protected void writePrintCharFunction(PrintStream out) throws IOException {
		out.println("#include <stdio.h>");
		out.println("int __keso_printc( jchar c ) {");
		out.println("\tif(c=='\\0') {");
		out.println("\t\treturn 0;");
		out.println("\t}");

		out.println("\tputchar(c);");
		out.println("\treturn 1;");
		out.println("}\n");
	}

	protected String getGOILArchitecture() {
		return "posix";
	}

	public void emitSystemTimeNanos() throws CompileException {
		add_class("stdlib");
		add_class("time");
		addln("\t#ifdef __APPLE__");
		addln("\t\tuint64_t time = mach_absolute_time();");
		addln("\t\tmach_timebase_info_data_t info;");
		addln("\t\tmach_timebase_info(&info);");
		addln("\t\treturn time * (info.numer / info.denom);");
		addln("#else");
		addln("\t\tstruct timespec res;");
		addln("\t\tif(-1==(clock_gettime(CLOCK_MONOTONIC, &res))) {");
		addln("\t\t\texit(1);");
		addln("\t\t}");
		addln("\t\treturn (jlong) res.tv_sec * (jlong) 1000000000 + (jlong) res.tv_nsec;");
		addln("\t#endif");
	}

}
