/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.backend;

import keso.compiler.*;
import java.io.*;
import java.util.HashSet;

public abstract class CoderProOSEK extends CoderOSEK {

	public CoderProOSEK(BuilderOptions opts, MethodTableFactory table, ClassStore store) throws CompileException {
		super(opts,table,store);

		HashSet<String> supAttrs = supportedAttributes.get("OS");
		supAttrs.add("CC");
		supAttrs.add("SCHEDULE");
		supAttrs.add("MICROCONTROLLER");
		supAttrs.add("USERMAIN");
		supAttrs.add("STACKCHECK");
		supAttrs.add("TRACEBUFFER");
		supAttrs.add("TRICORE_RT_CLOCK");
		supAttrs.add("TRICORE_NUM_CSA");
		supAttrs.add("EXTRA_RUNTIME_CHECKS");
		supAttrs.add("SERVICETRACE");
		supAttrs.add("USELASTERROR");
		supAttrs.add("PREISRHOOK");
		supAttrs.add("POSTISRHOOK");

		supAttrs = supportedAttributes.get("TASK");
		supAttrs.add("TYPE");
		supAttrs.add("STACKSIZE");
		supAttrs.add("CALLSCHEDULER");

		supAttrs = supportedAttributes.get("COUNTER");
		supAttrs.add("TRICORE_TIMER");
		supAttrs.add("TRICORE_IRQLEVEL");
		supAttrs.add("TIME_IN_NS");

		supAttrs = supportedAttributes.get("ISR");
		supAttrs.add("TRICORE_VECTOR");
		supAttrs.add("TRICORE_IRQLEVEL");
		supAttrs.add("TRICORE_NUM_CSA");
		supAttrs.add("STACKSIZE");
	}

	protected void writeCurrTaskFkt() throws CompileException {
		global_header_add("#define keso_curr_task_fkt() (keso_task_index[OSEKOStidact])");
	}

	protected abstract String objDir();

	protected void writeMakefile(PrintStream out) throws CompileException {
		try {
            out.println("# THIS FILE IS AUTO-GENERATED BY KESO! DON'T EDIT\n\n");
            out.println(".PHONY: clean all recreate tags\n");
            out.print("CC=");
            out.println(opts.getCC());
            out.print("STRIP=");
            out.println(opts.getStrip());
            out.print("SIZE=");
            out.println(opts.getObjSize());

            if (opts.getLinkerOptions()!=null) {
                    out.print("LDSCR=");
                    out.println(opts.getLinkerOptions());
            }

            out.print(opts.getCFlagsVarName() );
            out.print( "=");
            out.println(opts.getCFlags());
            out.println();

            out.print(opts.getObjVarName() );
            out.print( " = ");
            out.print("global.o ");
            for (MakeRule mr : rules) {
                out.print(mr.getTarget());
                out.print(" ");
            }

			String od=objDir();
			out.println("\n\n# default is the target that the shipped ProOSEK Makefiles build");
			out.println("# we will use the all target to append commands to the default build process");
			out.println("all: kesolib default");

			out.println("\t@ln -fs " +od+ "/keso_main.elf");
			out.println("\t@cd "+od+" && cp keso_main.elf keso_main_g.elf");
			out.println("\t@ln -fs "+od+"/keso_main_g.elf");
			out.print("\t@cd .. && ln -fs ");
			out.print(opts.getSystemName());
			out.print("/keso_main_g.elf && ln -fs ");
			out.print(opts.getSystemName());
			out.println("/"+od+"/os.orti");
			out.println("\t@$(STRIP) keso_main.elf");
			out.println("\t@tricore-objcopy -R .version_info keso_main.elf");
			out.println("\t@echo $(KESORC) : $(JINOFLAGS)");
			out.println("\t@$(SIZE) keso_main.elf");
			out.println();

			out.println("clean:");
			out.println("\t@rm -rf keso_main.elf keso_main_g.elf "+od);
			out.println();

			out.println("recreate:");
			out.println("\tcd ../.. ; make\n");

			out.println("tags:");
			out.println("\t$(CTAGBIN) *.c *.h\n");

			out.println("\nkesoinfo: kesoinfo.c");
			out.println("\tgcc -O2 -ansi -Wall -o kesoinfo kesoinfo.c");

			out.println("\n\n# include template with shipped ProOSEK build process");
			out.println("include $(KESOSRCPATH)/templates/Makefile.tricore-osek");

			for (MakeRule rule : rules) {
				rule.writeDotRule(out);
			}

		} catch (IOException e) {
			throw new CompileException(e.getMessage());
		}
	}

	protected String ISRMacro(int ISRCategory) {
		if(ISRCategory==1)
			return "ISR1";
		return "ISR";
	}

	protected void writeOSEKIncludes(PrintStream out) {
		super.writeOSEKIncludes(out);
		// E_NOT_OK is non standard and not defined by ProOSEK
		out.print("#ifndef E_NOT_OK\n");
		out.print("#define E_NOT_OK 10\n");
		out.print("#endif\n");
	}
}

