/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.backend;

import keso.compiler.*;
import java.io.*;


public abstract class CoderRTAOSEK extends CoderOSEK {

	public CoderRTAOSEK(BuilderOptions opts, MethodTableFactory table, ClassStore store) throws CompileException {
		super(opts,table,store);

		// RTAOSEK does not sequentially enumerate task identifiers, we therefore
		// need the switch-function variant of the reverse lookup
		opts.setExtraOption("task_rlookup_fkt");

		// RTAOSEK does not use constant value system objects, therefore we cannot
		// statically initialize our system objects but need to postpone it to the
		// startup phase
		opts.setExtraOption("init_systemobjects_on_startup");
	}

	protected void writeOSEKIncludes(PrintStream out) {
		out.println("#include <osek.h>");
		out.println("#include <string.h>");
	}

	public void writeInfoFile() throws CompileException {
		// call super.writeInfoFile() if you need keso_info.c
		try {

			String filelist = opts.getOutputPath()+"/sourcefiles.txt";
			PrintStream nout = new PrintStream(new FileOutputStream(filelist));
			nout.print("global.c\n");
			for (MakeRule mr : rules) {
				nout.print(mr.getSource());
				nout.print("\n");
			}
			//nout.print(extraSourceFiles());
			nout.close();

		} catch (IOException ex) {
			opts.warn("Can't write sourcefiles.txt -- "+ex+"\n");
		}
	}

	protected void writeMakefile(PrintStream out) throws CompileException {
		// don't write generic makefile
	}
}

