/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.backend;

import keso.compiler.*;
import java.io.*;


public class CoderTricore_ProOSEK extends CoderProOSEK {

	private static final boolean tricore_PMI_workaround = true;

	public CoderTricore_ProOSEK(BuilderOptions opts, MethodTableFactory table, ClassStore store) throws CompileException {
		super(opts,table,store);
	}

	protected String objDir() { return "TriboardTC1796gnu"; }

	protected void maincHook(PrintStream out) {
		if (tricore_PMI_workaround) {
			out.print("\n\n/*\n * Workaround for TC1796 and Trace 32:");
			out.print("\n * Bug: Program Memory Interface (PMI) line buffer not invalidated during CPU halt.\n */\n");
			out.print("static short flush_data[9] __attribute__ ((unused));\n");
			out.print("void flush_PMI_buffer() {\n");
			out.print("\t__asm__ __volatile__ (\".align 4; debug; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;\");\n");
			out.print("}\n");
		}
	}

	protected String extraMakeMacros() throws CompileException {
		return "CCBUGS=-mtc13 -mcpu16 -mcpu009";
	}

	protected void writeGlobalFiles() throws CompileException {
		writeGlobalFiles("#include <TRICORE_OS.oil>\n");
	}

	// tricore specific
	private boolean _locksysreg_init=false;
	private final void lock_sysreg_init() throws CompileException {
		if(!_locksysreg_init) {
			_locksysreg_init=true;
			global_header_add("extern int endinit_status;\n");
			global_add("int endinit_status;\n");
		}
	}
	public String lock_sysreg() throws CompileException {
		lock_sysreg_init();
		return "TRICORE_setEndinit(endinit_status)";
	}
	public String unlock_sysreg() throws CompileException {
		lock_sysreg_init();
		return "TRICORE_clearEndinit(endinit_status)";
	}
}
