/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.backend;

import keso.compiler.BuilderOptions;

import java.io.*;


class MakeRule {
	BuilderOptions opts;
	String alias;
	String target;
	String source;
	String depends;
	boolean isClass;

	MakeRule(BuilderOptions opts, String alias, String depends, boolean isClass) {
		this.opts = opts;
		this.alias = alias;
		this.target = alias+".o";
		this.source = alias+".c";
		this.depends = depends;
		this.isClass = isClass;
	}

	MakeRule(BuilderOptions opts, String target, String source, String depends, boolean isClass) {
		this.opts = opts;
		this.target = target;
		this.source = source;
		this.depends = depends;
		this.isClass = isClass;
	}

	String getTarget() {
		return target;
	}

	String getSource() {
		return source;
	}

	private void writeDotRule(PrintStream out, String alias, String comment, String target_fmt) {
		if (!isClass) return;
		out.print("\n\t@echo ");
		out.print(comment);
		out.print(' ');
		out.print(alias);
		out.print('.');
		out.print(target_fmt);
		out.print(' ');
		out.print("\n\t@dot -T");
		out.print(target_fmt);
		out.print(" -o ");
		out.print(alias);
		out.print('.');
		out.print(target_fmt);
		out.print(' ');
		out.print(alias);
		out.print(".dot");
	}

	public void writeDotRule(PrintStream out) throws IOException {
		if (alias==null) return;
		if (!isClass) return;

		if (opts.createCFG()) {
			out.print(alias);
			out.print(".ps: ");
			out.print(source);
			out.print(" ");
			out.print(alias);
			out.print(".dot");
			writeDotRule(out,alias,"CFG","ps");
			out.println();
		}
		if (opts.hasOption("dump_dominator_tree")) {
			out.print("dom_");
			out.print(alias);
			out.print(".ps: ");
			out.print(source);
			out.print(" dom_");
			out.print(alias);
			out.print(".dot");
			writeDotRule(out,"dom_"+alias,"DOM","ps");
			out.println();
		}
	}

	/**
	 * Write Makefilerule to out stream
	 * <target>: <source>
	 * 	@echo OBJ <target>
	 * 	@$(CC) $(CFLAGS) -c -o <target> <source>
	 */
	void writeRule(PrintStream out) throws IOException {
		writeRule(out,"");
	}

	void writeRule(PrintStream out, String ext_depends) throws IOException {
		out.print(target);
		out.print(": ");
		out.print(source);
		out.print(" ");
		out.print(depends);
		if (!ext_depends.equals("")) {
			out.print(" ");
			out.print(ext_depends);
		}
		if (alias !=null && opts.createCFG()) {
			writeDotRule(out,alias,"CFG","ps");
		}
		if (alias !=null && opts.hasOption("dump_dominator_tree")) {
			writeDotRule(out,"dom_"+alias,"DOM","ps");
		}
		out.print("\n\t@echo OBJ ");
		out.print(target);
		out.print("\n\t@$(CC) $(" + opts.getCFlagsVarName() + ") -c -o ");
		out.print(target);
		out.print(" ");
		out.print(source);
	}
}

