/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
/*
	M.Strotz 2011
*/

package keso.compiler.backend.gradmp;

public final class CheckChecker extends Defs {

	/**
		Wert der null-Referenz.
		Bei uns immer 0.
	*/
	private final static long nullref = 0;

	/**
		Wert des ungueltigen Klassendescriptors.
		TODO
	*/
	private final static int the_invalid_classdesc = 0;
	private final static boolean has_the_invalid_classdesc = false;

	/**
		Speicherbeschreibung.
		Kennt Speicherereiche.
		Kennt Adressbusbreite.
	*/
	private MemoryDescription	memdesc;
	
	private boolean
		allow_unexpected_nullrefs,
		read_has_global_impact;

	
	private FieldInfo
		fi_arraylength = null,
		fi_classdesc = null;
	
	//private String	//TODO
		//rtoc_arraylength = null,
		//rtoc_classdesc = null;

	// ---- ----
	public CheckChecker(final MemoryDescription md, boolean allow_unexpected_nullrefs_, boolean read_has_global_impact_) {
		//int optlvl,
		//long nullref_,
		//long array_length_offset_, int array_length_size_,
		//long class_desc_offset_, int class_desc_size_,
		//boolean has_errorhandling_classdesc_, long errorhandling_classdesc_
		memdesc = md;
		allow_unexpected_nullrefs = allow_unexpected_nullrefs_;
		read_has_global_impact = read_has_global_impact_;
	}

	// ---- ----

	public void check(final FieldInfo fi) {
		check(fi,HWOFFLOADING);
		check(fi,CONTAINMENT);
	}
	private void check(final FieldInfo fi, int level) {
		fi.setReasonToOmitCheckOnGet(
			level,
			reasonToOmitCheckOnGetField(level, fi.offset(), fi.sizeof(), fi.isRef())
		);
		fi.setReasonToOmitCheckOnPut(
			level,
			reasonToOmitCheckOnPutField(level, fi.offset(), fi.sizeof(), fi.isRef())
		);
	}
	
	public void checkArrayLength( FieldInfo fi ) {
		int level;
		fi_arraylength = fi;
		level = HWOFFLOADING;
		fi.setReasonToOmitCheckOnGet(level, reasonToOmitCheckOnArrayLength(level, fi.offset(), fi.sizeof()));
		fi.setReasonToOmitCheckOnPut(level, null);
		level = CONTAINMENT;
		fi.setReasonToOmitCheckOnGet(level, reasonToOmitCheckOnArrayLength(level, fi.offset(), fi.sizeof()));
		fi.setReasonToOmitCheckOnPut(level, null);
	}
	public void checkClassDesc( FieldInfo fi ) {
		int level;
		fi_classdesc = fi;
		level = HWOFFLOADING;
		fi.setReasonToOmitCheckOnGet(level, reasonToOmitCheckOnGetClassDesc(level, fi.offset(), fi.sizeof()));
		fi.setReasonToOmitCheckOnPut(level, null );
		level = CONTAINMENT;
		fi.setReasonToOmitCheckOnGet(level, reasonToOmitCheckOnGetClassDesc(level, fi.offset(), fi.sizeof()));
		fi.setReasonToOmitCheckOnPut(level, null );
	}




	// ---- ----
	// load
	// ---- ----

	/**
		returns the reason to omit the nullcheck for loading an instance-field,
		or null if the check may not be omitted.
		@param offset	offset of the instance-field relative to this.
		@param size		sizeof the instance-field (ign?)
		@param ref		is the field a reference?
	*/
	public String reasonToOmitCheckOnGetField(int optimize, long offset, int size, boolean ref) {
		if( optimize == NOPT ) return null;
		if( optimize == KILLALL ) return "KILLALL";

		MemoryBlock mb = memdesc.getBlock( nullref + offset, size );
		if( mb.trapsOnRead() ) {
			return READ_TRAP; // reads on nullref will trap
		}
		if( ref ) {	// Referenz

			if( mb.constOnRead() ) {
				if( allow_unexpected_nullrefs
					&& optimize >= WEAK_DETECTION
					&& memdesc.readConstant(mb,offset,size) == memdesc.toAddress(nullref) //TODO einmal!
				)
				{
					return READ_NULL_NULL;	// read on nullref will read nullref
					//daduch koennten nullrefs dahin kommen, wo sie nicht erwartet --> nicht ueberprueft werden (notnull)
				} else {
					return null; // read on nullref could read wild-reference
				}
			} else {
				return null; // read on nullref could read wild-reference
			}

		} else {	// Primitive Daten

			if( optimize < CONTAINMENT || (read_has_global_impact && mb.unspecOnRead()) ) {
				return null; // error detection required / read might change the memory's state.
			} else {
				return READ_LOCAL; // reading invalid data from nullref has just local impact
			}

		}
	}

	/**
		returns the reason to omit the nullcheck for loading the length of an array,
		or null if the check may not be omitted.
		@param offset	offset of the instance-field relative to this.
		@param size		sizeof the instance-field
	*/
	public String reasonToOmitCheckOnArrayLength(int optimize, long offset, int size) {
		if( optimize == NOPT ) return null;
		if( optimize == KILLALL ) return "KILLALL";

		MemoryBlock mb = memdesc.getBlock( nullref + offset, size );
		if( mb.trapsOnRead() ) {
			return READ_TRAP;	// read on nullref will trap
		}
		if( mb.constOnRead() ) {
			if(	memdesc.readConstant(mb,offset,size) == 0 && optimize >= WEAK_DETECTION ) {
				return READ_EMPTY_ARRAY; // array at nullref will appear empty
			} else {
				return null;
			}
		}
		return null;
	}

	/**
		returns the reason to omit the nullcheck for loading the length of an array,
		or null if the check may not be omitted.
		checkArrayLength has to be called before.
	*/
	public String reasonToOmitCheckOnArrayLength(int optimize) {
		return fi_arraylength.reasonToOmitCheckOnGet(optimize);
		//return rtoc_arraylength;
	}

	/**
		returns the reason to omit the nullcheck for loading an array-component,
		or null if the check may not be omitted.
		@param ref		is the field a reference?
	*/
	public String reasonToOmitNullcheckOnArrayLoad(int optimize, boolean ref) {
		if( optimize == NOPT ) return null;
		if( optimize == KILLALL ) return "KILLALL";

		if( reasonToOmitBoundcheckOnArrayLoad(optimize,ref) == BOUND_READ_LOCAL ) {
			return READ_NOT_ARRAYLENGTH; //arraylength won't be read
		} else {
			return reasonToOmitCheckOnArrayLength(optimize);
		}
	}

	/**
		returns the reason to omit the bound-check for loading an array-component,
		or null if the check may not be omitted.
		@param ref		is the field a reference?
	*/
	public String reasonToOmitBoundcheckOnArrayLoad(int optimize, boolean ref) {
		if( optimize == NOPT ) return null;
		if( optimize == KILLALL ) return "KILLALL";

		if( !ref && optimize >= CONTAINMENT && !(read_has_global_impact && memdesc.hasUnspecReadBlock()) ) {
			// reading "wrong" primitive data from array has just local impact.
			return BOUND_READ_LOCAL;
		}
		return null;
	}

	// ---- ----
	// store
	// ---- ----

	/**
		returns the reason to omit the nullcheck for storing an instance-field,
		or null if the check may not be omitted.
		@param offset	offset of the instance-field relative to this.
		@param size		sizeof the instance-field (ign?)
		@param ref		is the field a reference?
	*/
	public String reasonToOmitCheckOnPutField(int optimize, long offset, int size, boolean ref) {
		if( optimize == NOPT ) return null;
		if( optimize == KILLALL ) return "KILLALL";

		MemoryBlock mb = memdesc.getBlock( nullref + offset, size );
		if( mb.trapsOnWrite() ) {
			return WRITE_TRAP;	// null-writes will trap
		} else if( mb.ignoresWrite() && optimize >= CONTAINMENT ) {
			return WRITE_IGNORE; // null-writes will be ignored
		} else {
			return null;
		}
	}

	/**
		returns the reason to omit the nullcheck for storing an array-component,
		or null if the check may not be omitted.
		@param ref		is the field a reference?
	*/
	public String reasonToOmitNullcheckOnArrayStore(int optimize, boolean ref) {
		return reasonToOmitCheckOnArrayLength(optimize);
	}

	/**
		returns the reason to omit the bound-check for storing an array-component,
		or null if the check may not be omitted.
		@param ref		is the field a reference?
	*/
	public String reasonToOmitBoundcheckOnArrayStore(int optimize, boolean ref) {
		return null;
	}

	// ---- ----
	// invoke
	// ---- ----

	/**

	*/
	public String reasonToOmitCheckOnGetClassDesc(int optimize, long offset, int size) {
		if( optimize == NOPT ) return null;
		if( optimize == KILLALL ) return "KILLALL";

		MemoryBlock mb = memdesc.getBlock( nullref + offset, size );
		if( mb.trapsOnRead() ) {
			return READ_TRAP;	// read on nullref will trap
		}
		if( mb.constOnRead() && has_the_invalid_classdesc && memdesc.readConstant(mb,offset,size) == the_invalid_classdesc ) {
			return READ_ERROR_CLASSDESC; // null-read will read errorhandling-classdescriptor
		}
		return null;
	}

	/**
		checkClassDesc has to be called before.
	*/
	public String reasonToOmitCheckOnGetClassDesc(int optimize) {
		return fi_classdesc.reasonToOmitCheckOnGet(optimize);
		//return rtoc_classdesc;
	}

	public String reasonToOmitCheckOnInvokeVirtual(int optimize) {
		return reasonToOmitCheckOnGetClassDesc(optimize);
	}

	public String reasonToOmitCheckOnInvokeInterface(int optimize) {
		return reasonToOmitCheckOnGetClassDesc(optimize);
	}

	public String reasonToOmitCheckOnInvokeSpecial(int optimize, boolean no_valid_this_ptr) {
		if( no_valid_this_ptr && optimize >= CONTAINMENT ) {
			return NO_VALID_THIS; //this-pointers don't have to be valid.
		} else {
			return null;
		}
	}

};
