/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
/*
	M.Strotz 2011
*/

package keso.compiler.backend.gradmp;

import keso.compiler.backend.gradmp.util.*;
import keso.compiler.imcode.*;



class FieldInfo {
	private final String m_name;
	private final IMField imfield;
	private Interval m_layout;

	public FieldInfo(IMField imf, String name, Interval i) {
		imfield = imf;
		m_name = name;
		m_layout = i;
		rtoc_hwoffloading = new RTOC();
		rtoc_containment = new RTOC();
	}
	public FieldInfo(String name, Interval i) { this( null, name, i ); }
	public FieldInfo(IMField imf, Interval i) { this( imf, imf.getFieldName(), i ); }

	public FieldInfo(Interval i) {
		this( "__unnamed_field", i );
	}

	// CHG vvv

	//! ReasonToOmitCheck
	private static class RTOC {
		String get, put;
		public String toString() {
			return "(" + rtos(get) + "," + rtos(put) + ")";
		}
	}
	private final RTOC rtoc_hwoffloading, rtoc_containment;
	private final static RTOC rtoc_inactive = new RTOC(); // (null,null) fuer inactive
	
	private final RTOC rtoc(final int level) {
		switch(level) {
		case Defs.HWOFFLOADING: return rtoc_hwoffloading;
		case Defs.CONTAINMENT: return rtoc_containment;
		//case Defs.INACTIVE:
		//case Defs.KILLALL:
		default: return rtoc_inactive;
		}
	}
	private final RTOC set_rtoc(final int level) {
		RTOC r = rtoc(level);
		return (r==rtoc_inactive)?null:r;
	}

	public final boolean omitCheckOnGet(int level) { return rtoc(level).get != null; }
	public final boolean omitCheckOnPut(int level) { return rtoc(level).put != null; }

	public final String reasonToOmitCheckOnGet(int level) { return rtoc(level).get; }
	public final String reasonToOmitCheckOnPut(int level) { return rtoc(level).put; }

	public final void setReasonToOmitCheckOnGet(int level, String reason) { set_rtoc(level).get = reason; }
	public final void setReasonToOmitCheckOnPut(int level, String reason) { set_rtoc(level).put = reason; }

	public final String toStringDesc() {
		return desc_begin() + ", " + rtoc_hwoffloading + ", " + rtoc_containment + " )";
	}

	// CHG ^^^

	public final long offset()	{ return m_layout.offset(); }
	public final int  sizeof()	{ return m_layout.sizeof(); }

	public final Interval layout() { return m_layout; }
	public final void setLayout( Interval i ) { m_layout = i; }
	public final void setLayout( long offs, int size ) { m_layout = new Interval( offs, size ); }

	public final String name() {
		return m_name;
	}

	public final boolean isRef() {
		return (imfield != null)? imfield.isReference() : (offset() < 0);
	}

	public String toString() {
		return desc_begin() + " )";
	}

	private final String desc_begin() {
		return "Field( " + name() + ", " + offset() + ", " + sizeof();
	}
	private static String rtos( String r ) {
		return (r == null)?"check required":r;
	}
};
