
%token TKIDENTIFIER
%token TKNUMBER
%token TKMEMORY
%token TKATTRIB

%start linkerscript

%{
package keso.compiler.backend.gradmp.ldparser;

import java.io.*;
import java.util.*;

import keso.compiler.backend.gradmp.ldparser.ast.*;
%}

%%

linkerscript:
	stmt
		{

			LinkerScript ls = new LinkerScript( (Stmt) $1, file, line );
			root = ls;
			$$ = ls;
		}
	| linkerscript stmt
		{
			LinkerScript ls = (LinkerScript) $1;
			ls.add( (Stmt) $2 );
			$$ = ls;
		}
	|
		{
			//TODO Riesiger Shift-Reduce-Konflikt um Fehler bei Scripts ohne "MEMORY" zu vermeiden.
			LinkerScript ls = new LinkerScript( file, line );
			root = ls;
			$$ = ls;
		}
	;

stmt:
	memory_stmt
		{
			$$ = $1;
		}
	/*| ';'
		{
			$$ = new EmptyStmt( file, line );
		}
		*/
	;

identifier:
	TKIDENTIFIER
		{
			$$ = new Identifier( lxtext, file, line );
		}
	;

memory_stmt:
	TKMEMORY '{' memory_block_list '}'
		{
			$$ = new MemoryStmt( (MemoryBlockList) $3, file, line );
		}
	;

memory_block_list:
	memory_block
		{
			$$ = new MemoryBlockList( (MemoryBlock) $1, file, line );
		}
	| memory_block_list memory_block
		{
			MemoryBlockList mbl = (MemoryBlockList) $1;
			mbl.add( (MemoryBlock) $2 );
			$$ = mbl;
		}
	;

memory_block:
	identifier ':' memory_interval
		{
			$$ = new MemoryBlock( (Identifier) $1, (MemoryInterval) $3, file, line );
		}
	| identifier mem_attrib ':' memory_interval
		{
			$$ = new MemoryBlock( (Identifier) $1, (MemoryAttributes) $2, (MemoryInterval) $4, file, line );
		}
	;

memory_interval:
	assgn_expr ',' assgn_expr
		{
			$$ = new MemoryInterval( (AssgnExpr) $1, (AssgnExpr) $3, file, line );
		}
	;

assgn_expr:
	identifier '=' arith_expr
		{
			$$ = new AssgnExpr( (Identifier) $1, (ArithExpr) $3, file, line );
		}
	;

arith_expr:
	TKNUMBER
		{
			keso.compiler.backend.gradmp.ldparser.ast.Number num = dummy_number;
			try {
				num = new keso.compiler.backend.gradmp.ldparser.ast.Number( lxtext, file, line );
			} catch( NumberFormatException nfe ) {
				yyerror( nfe.toString() );
				//throw nfe;
			}
			$$ = num;
		}
	;

mem_attrib:
	TKATTRIB
		{
			$$ = new MemoryAttributes( lxtext, file, line );
		}
	;

%%
private String lxtext	= "";
private Yylex lx		= null;
private Node root		= null;
private int line 		= 1;
private String file;
private int error_count = 0;
private static final keso.compiler.backend.gradmp.ldparser.ast.Number dummy_number = new keso.compiler.backend.gradmp.ldparser.ast.Number( -1, "???", 0 );

public Parser(final Reader reader) {
	lx = new Yylex(reader);
}

public final Node getRoot() {
	return root;
}

public final void start(String filename) {
	file = filename;
	yyparse();
	if( error_count > 0 ) {
		throw new RuntimeException("parsing failed: " + error_count + " error(s)!");
	}
	return;
}

private final void yyerror(final String s) {
	++error_count;
	System.err.println(file + ":" + line + " : parser-error: \"" + s + "\" at token \"" + lxtext + "\" (length: " + lxtext.length() + ")");
}

public final int getErrorCount() {
	return error_count;
}

public final int yylex() {
	try{
		yylval=lx.yylex();
		lxtext=((Yytoken) yylval).text();
		line = ((Yytoken) yylval).line();
		return ((Yytoken) yylval).type();
	} catch (Error e) {
		System.err.println(file + ":" + line + " : parser-error: \"" + e + "\" after token \"" + lxtext + "\"");
		throw new RuntimeException();
	} catch(Exception e) {
		return -1;
	}
}
