/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.backend.gradmp.ldparser;

/**
 * Token class of the Lexer
 *
 */
public final class Yytoken {

    private final int line;

    private final String text;

    private final int type;

    /**
     * Creates a new Token
     *
     * @param type
     *            set the type to this
     * @param text
     *            set the text to this
     * @param line
     *            set the line number of the token to this
     */
    public Yytoken(int type, String text, int line) {
        this.line = line;
        this.text = text;
        this.type = type;
    }

    /**
     * Returns the line number in which the token occured
     *
     * @return line number
     */
    public final int line() {
        return line;
    }

    /**
     * Returns the text of the token
     *
     * @return token text
     */
    public final String text() {
        return text;
    }

    /**
     * Returns the type of the token
     *
     * @return type of the token
     */
    public final int type() {
        return type;
    }
}
