/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
/*
	M.Strotz 2011
*/

package keso.compiler.backend.gradmp.ldparser.ast;

public final class Number extends ArithExpr {
	private final long m_val;

	public Number( long val, String file, int line ) {
		super( file, line );
		m_val = val;
	}
	public Number( String txt, String file, int line ) throws NumberFormatException {
		this( parseLong(txt), file, line );
	}

	public long value() {
		return m_val;
	}

	public static long parseLong( final String original_s ) throws NumberFormatException {
		String s = original_s;
		if( s.equals("") ) throw new NumberFormatException("Empty String");
		s = s.toLowerCase();

		boolean neg = false;
		int shift = 0;
		int radix = 10;

		if( s.charAt(0) == '-' ) {
			neg = true;
			s = s.substring(1);
			if( s.equals("") ) throw new NumberFormatException("Just a '-'");
		}
		if( s.charAt(0) == '0' ) {
			if( s.length() > 1 && s.charAt(1) == 'x' ) {
				s = s.substring(2);
				if( s.equals("") ) throw new NumberFormatException("Empty Hexnumber \"" + original_s + "\"");
				radix = 0x10;
			} else {
				radix = 8;
			}
		}
		{
			char back = s.charAt( s.length() -1 );
			final String old_s = s;
			s = s.substring( 0, s.length() - 1 );
			switch( back ) {
			case 'k': shift = 10; break;
			case 'm': shift = 20; break;
			case 'g': shift = 30; break;
			case 't': shift = 40; break;
			default : s = old_s;
			}
		}

		if( s.equals("") ) throw new NumberFormatException("Invalid Number: \"" + original_s + "\"");

		long value = Long.parseLong(s,radix) << shift;

		return neg? -value : value;
	}
};
