/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
/*
	M.Strotz 2011
*/

package keso.compiler.backend.gradmp.test;

import keso.compiler.backend.gradmp.*;

public final class ChecksTest {

	private static String str( Object obj ) {
		return (obj == null)?"null":obj.toString();
	}

	private static MemoryDescription gen_arch0() {
		MemoryDescription md = new MemoryDescription();
		md.blocks().add( new MemoryBlock( "flash_null", "rx!w", 0, 0x10 ) );
		md.blocks().add( new MemoryBlock( "flash", "rx!w", 0x10, ( 1 << 31 ) - 0x10 ) );
		md.blocks().add( new MemoryBlock( "ram", "wx", 1l << 31, 1l << 31 ) );
		return md;
	}

	private static MemoryDescription gen_arch1() {
		MemoryDescription md = new MemoryDescription();
		md.blocks().add( new MemoryBlock( "invalid", "!rxw", 0, 0x10 ) );
		md.blocks().add( new MemoryBlock( "flash", "rx!w", 0x10, ( 1 << 31 ) - 0x10 ) );
		md.blocks().add( new MemoryBlock( "ram", "wx", 1l << 31, 1l << 31 ) );
		return md;
	}

	private static MemoryDescription gen_arch2() {
		MemoryDescription md = new MemoryDescription();
		md.blocks().add( new MemoryBlock( "invalid", "!rxw", 0, 1l << 32 ) );
		return md;
	}

	private static MemoryDescription gen_arch3() {
		MemoryDescription md = new MemoryDescription();
		md.blocks().add( new MemoryBlock( MemoryBlock.READ_CONST, MemoryBlock.WRITE_IGNORE,  "flash_null", 0, 0x10 ) );
		md.blocks().add( new MemoryBlock( "ram", "rwx", 0x10, ( 1l << 32 ) - 0x90 ) );
		md.blocks().add( new MemoryBlock( MemoryBlock.READ_CONST, MemoryBlock.WRITE_IGNORE,  "prot_null", ( 1l << 32 ) - 0x80, 0x80 ) );
		//add( new MemoryBlock( MemoryBlock.READ_CONST, MemoryBlock.WRITE_IGNORE,  "prot_null", 0, ( 1l << 32 ) - 0x80, 0x80, (byte)0xff ) );
		return md;
	}

	private static void test0( MemoryDescription md, int level ) {
		CheckChecker chk = new CheckChecker(md,level);
		String ok;

		System.out.println("level     : " + level );
		System.out.println("err_detect: " + chk.preservesErrorDetection() );


		ok = chk.reasonToOmitCheckOnArrayLength();
		System.out.println("arrlen    : " + str(ok) );

		ok = chk.reasonToOmitCheckOnGetClassDesc();
		System.out.println("classdesc : " + str(ok) );


		ok = chk.reasonToOmitCheckOnGetField( 32, 4, false );
		System.out.println("get_data(32,4): " + str(ok) );

		ok = chk.reasonToOmitCheckOnGetField( -32, 4, true );
		System.out.println("get_ref(32,4) : " + str(ok) );


		ok = chk.reasonToOmitCheckOnPutField( 32, 4, false );
		System.out.println("put_data(32,4): " + str(ok) );

		ok = chk.reasonToOmitCheckOnPutField( -32, 4, true );
		System.out.println("put_ref(32,4) : " + str(ok) );


		ok = chk.reasonToOmitNullcheckOnArrayLoad( false );
		System.out.println("a_ld_data (null) : " + str(ok) );

		ok = chk.reasonToOmitNullcheckOnArrayLoad( true );
		System.out.println("a_ld_ref  (null) : " + str(ok) );

		ok = chk.reasonToOmitNullcheckOnArrayStore( false );
		System.out.println("a_st_data (null) : " + str(ok) );

		ok = chk.reasonToOmitNullcheckOnArrayStore( true );
		System.out.println("a_st_ref  (null) : " + str(ok) );


		ok = chk.reasonToOmitBoundcheckOnArrayLoad( false );
		System.out.println("a_ld_data (bound): " + str(ok) );

		ok = chk.reasonToOmitBoundcheckOnArrayLoad( true );
		System.out.println("a_ld_ref  (bound): " + str(ok) );

		ok = chk.reasonToOmitBoundcheckOnArrayStore( false );
		System.out.println("a_st_data (bound): " + str(ok) );

		ok = chk.reasonToOmitBoundcheckOnArrayStore( true );
		System.out.println("a_st_ref  (bound): " + str(ok) );

		ok = chk.reasonToOmitCheckOnInvokeInterface();
		System.out.println("invoke interfc: " + str(ok) );

		ok = chk.reasonToOmitCheckOnInvokeVirtual();
		System.out.println("invoke virtual: " + str(ok) );

		System.out.println();
	}

	private static void tests0( MemoryDescription md ) {
		System.out.println( md.getDescriptionString() );
		test0( md, 0 );
		test0( md, 1 );
		test0( md, 2 );
		test0( md, 3 );
	}

	public static void main( String []args ) {
		tests0( gen_arch3() );
	}
};
