/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
/*
	M.Strotz 2011
*/

package keso.compiler.backend.gradmp.util;

public final class Interval {

	final long m_origin, m_length;

	public Interval( long org, long len ) {
		m_origin = org;
		m_length = len;
	}
	public Interval( Object n, long org, long bey ) {
		m_origin = org;
		m_length = bey - org;
	}

	public static Interval new_org_len( final long org, final long len ) {
		return new Interval( org, len );
	}
	public static Interval new_org_bey( final long org, final long bey ) {
		return new Interval( null, org, bey );
	}


	public long origin() { return m_origin; }
	public long length() { return m_length; }
	public long beyond() { return m_origin + m_length; }

	public long begin() { return origin(); }
	public long size() { return length(); }
	public long end() { return beyond(); }

	public long offset() { return origin(); }
	public int sizeof() { return (int)m_length; }

	public boolean includes( final long x ) {
		return (origin() <= x) && (x < beyond());
	}
	public boolean includes( final Interval r ) {
		return includes_org_bey( r.origin(), r.beyond() );
	}
	public boolean includes_org_bey( final long org, final long bey ) {
		return (origin() <= org) && (bey <= beyond());
	}
	public boolean includes_org_len( final long org, final long len ) {
		return includes_org_bey( org, org + len );
	}


	public Interval intersection( final Interval r ) {
		if( !intersects(r) ) return null;
		return new_org_bey(
			Math.max( origin(), r.origin() ),
			Math.min( beyond(), r.beyond() )
		);
	}
	public boolean intersects( final Interval r ) {
		return origin() < r.beyond() && r.origin() < beyond();
	}

	public boolean equals( Interval r ) {
		return origin() == r.origin() && length() == r.length();
	}
};
