/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config;

import keso.compiler.*;
import keso.compiler.config.domain.*;
import keso.compiler.config.task.*;
import java.util.*;
import keso.compiler.backend.CoderOSEK;

public class AppmodeDefinition extends Set<SystemDefinition> {
	public AppmodeDefinition(SystemDefinition parent, String name, int line) {
		super(parent, name, line);
	}

	public String toString() { return "Appmode:"+ ident; }

	/**
	 * Generate an APPMODE block in OIL syntax out of this APPMODE Definition.
	 *
	 * Known and supported attributes
	 * OIL standard:
	 *   (none)
	 */
	public void toOIL(StringBuffer oil, String prefix, CoderOSEK coder) throws CompileException {
		oil.append(prefix+"APPMODE " + ident);
		if(coder.emptyObjectsNeedBraces())
			oil.append(" { }");
	 	oil.append(";\n\n");
	}

	public void toCiAOConfig(StringBuffer buf, String prefix) throws CompileException {
		buf.append(prefix+"<applicationmode name=\""+ident+"\">\n");
		buf.append(prefix+"\t<autostarttasks>\n");

		for(DomainDefinition dd: parent.getAllSysDomains()) {
			Collection<TaskDefinition> tasks = dd.getTasks();
			for(TaskDefinition td: tasks) {
				if(td.getAutoStart()) {
				  String as = td.getAutoStartAppmode();
					if(as!=null && as.compareToIgnoreCase(ident)==0) {
						buf.append(prefix+"\t\t<taskref name=\"");
						buf.append(td.getIdentifier());
						buf.append("\"/>\n");
					}
				}
			}
		}
		buf.append(prefix+"\t</autostarttasks>\n");
		buf.append(prefix+"</applicationmode>\n\n");
	}

	protected String typeID() { return "appmode_"; }
}
