/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config;

import keso.compiler.CompileException;
import keso.compiler.backend.CoderOSEK;

import java.util.Map;

abstract public class Attribute<ParentType extends Set> {
	public final ParentType parent;
	public final String    ident;
	protected int line;

	public Attribute(ParentType parent, String ident, int line) {
		this.parent = parent;
		this.ident  = ident;
		this.line   = line;
	}

	public Attribute(Attribute<ParentType> attr) {
		this(attr.parent, attr.ident, attr.line);
	}

	public String getIdentifier() {
		String ret = typeID()+ident;
		if (parent!=null)
			ret = parent.getIdentifier()+"_"+ret;
		return ret;
	}

	protected String typeID() { return ""; }

	public int getLine() { return line; }

	public void toOIL(StringBuffer oil, String prefix, CoderOSEK coder) throws CompileException {
		oil.append(prefix);
		oil.append(ident.toUpperCase());
		oil.append(" = ");
		oil.append(valueString()/*.toUpperCase()*/);
		oil.append(";\n");
	}

	public void toCiAOConfig(StringBuffer buf, String prefix, Map<String, String> idmap) throws CompileException {
		buf.append(prefix);
		String ident = null;
		if(idmap!=null)
			ident = idmap.get(this.ident.toLowerCase());
		if(ident == null)
			ident=this.ident;

		buf.append(ident.toLowerCase());
		buf.append("=\"");
		buf.append(valueString());
		buf.append("\" ");
	}

	public void registerFinalize(Set<?> set) { throw new Error("abstract"); }

	abstract public int valueInt();
	abstract public double valueDouble();
	abstract public String valueString();
	abstract public boolean valueBoolean();
}
