/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config;

import keso.compiler.*;
import keso.compiler.config.parser.ParseException;
import keso.compiler.backend.CoderOSEK;

public class ComplexAttribute<ParentType extends Set<?>> extends Set<ParentType> {
	public final String value;

	public ComplexAttribute(ParentType parent, String name, String setting, int line) throws ParseException {
		super(parent, name, line);
		this.value = setting;
	}

	public boolean enabled() {
		if (value.toUpperCase().equals("FALSE")) return false;
		return true;
	}

	public String toString() { return ident; }

	/**
	 * Return the information from this cattr in a
	 * format suitable for inclusion within an OIL file.
	 *
	 * If incSubAttrs is true, then Set.toOIL()
	 * will be used to generate the output for the subattributes.
	 */
	public void toOIL(StringBuffer oil, boolean incSubAttrs, String prefix, String subAttrCategory, CoderOSEK coder)
			throws CompileException {

		oil.append(prefix);
		oil.append(ident.toUpperCase());
		oil.append(" = ");
		oil.append(value.toUpperCase());

		if (!incSubAttrs) {
			oil.append(";\n");
			return;
		}

		// Dump the subattributes
		oil.append(" {\n");
		super.toOIL(oil, prefix + "\t", subAttrCategory, coder);

		oil.append(prefix + "};\n");
	}
}
