/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config;

import keso.compiler.*;
import keso.compiler.backend.CoderOSEK;

public class CounterDefinition extends Set {
	public CounterDefinition(Set<?> parent, String name, int line) {
		super(parent, name, line);
	}

	public String toString() { return "Counter:"+ ident; }

	/**
	 * Generate a COUNTER block in OIL syntax out of this COUNTER Definition.
	 *
	 * Known and supported attributes
	 * OIL standard:
	 *   MINCYCLE        (UINT32)
	 *   TICKSPERBASE    (UINT32)
	 *   MAXALLOWEDVALUE (UINT32)
	 *
	 * ProOSEK extensions:
	 *   TRICORE_TIMER    (GPTU{0,1}_T{0,1}, STM_T{0,1}, USERCOUNTER)
	 *   TRICORE_IRQLEVEL (UINT32)
	 *   TIME_IN_NS       (UINT32)
	 */
	public void toOIL(StringBuffer oil, String prefix, CoderOSEK coder) throws CompileException {
		// hack for trampoline
		if(getIdentifier().equals("SystemCounter"))
			return;

		oil.append(prefix+"COUNTER " + getIdentifier() + " {\n");
		super.toOIL(oil, prefix+"\t", "COUNTER", coder);
		oil.append(prefix+"};\n\n");
	}

	public void toCiAOConfig(StringBuffer buf, String prefix) throws CompileException {
		String[] supportedAttributes = { "mincycle", "ticksperbase", "maxallowedvalue"};

		buf.append(prefix+"<counter name=\"" + getIdentifier() + "\" ");

		for(String a : supportedAttributes) {
			Attribute<?> attr = getAttribute(a);
			if(attr == null) continue;

			buf.append(prefix);
			buf.append(a);
			buf.append("=\"");
			buf.append(attr.valueString());
			buf.append("\" ");
		}

		buf.append(" />\n\n");
	}

	protected String typeID() { return "c_"; }

	public String getIdentifier() { return ident; }
}
