/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config;

import java.util.TreeSet;
import java.util.Set;

import java.lang.Comparable;

/**
 * This class is used to transport information to the builder
 * after a configuration has entirely been read.
 * It contains things the builder might need and that are not statically
 * known.
 */
public final class FinalizeResult {
	/**
	 * Contains ProtectedMethod Objects for each protected Method.
	 */
	public final Set<ProtectedMethod> requiredMethods;

	FinalizeResult() {
		requiredMethods = new TreeSet<ProtectedMethod>();
	}

	public void requireMethod(String className, String method) {
		requiredMethods.add(new ProtectedMethod(className, method));
	}

	public final class ProtectedMethod implements Comparable<ProtectedMethod> {
		public final String className;
		public final String method;

		public ProtectedMethod(String c, String m) {
			className=c;
			method=m;
		}

		public int compareTo(ProtectedMethod p) {
			int result = p.className.compareTo(className);

			if(result!=0) return result;
			if(p.method==null && method==null) return 0;
			if(p.method==null) return 1;
			if(method==null) return -1;
			return p.method.compareTo(method);
		}

		public boolean equals(ProtectedMethod p) {
			return compareTo(p)==0;
		}
	}
}
