/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config;

import keso.compiler.*;
import keso.compiler.config.domain.*;
import keso.compiler.backend.CoderOSEK;

import java.util.*;

public class SharedMemoryDefinition extends SystemObject<SystemDefinition> {
	private List<DomainDefinition<?>> readers = new ArrayList<DomainDefinition<?>>();
	private List<DomainDefinition<?>> writers = new ArrayList<DomainDefinition<?>>();

	private boolean _isAccessedFromJavaDomain = false;

	public SharedMemoryDefinition(SystemDefinition parent, String name, int line) {
		super(parent, name, line);
	}

	public String toString() { return "SharedMemory: "+ ident; }

	public void toOIL(StringBuffer oil, String prefix, CoderOSEK coder) throws CompileException {
		throw new CompileException("Shared Memory is CiAO specific");
	}

	protected String typeID() { return "shm_"; }

	public boolean isAccessedFromJavaDomain() { return _isAccessedFromJavaDomain; }

	public void registerUser(DomainDefinition<?> ddef, boolean read, boolean write) {
		if(ddef instanceof JavaDomain)
			_isAccessedFromJavaDomain = true;

		if(read)
			readers.add(ddef);
		if(write)
			writers.add(ddef);
	}

	public void toCiAOConfig(StringBuffer buf, String prefix) throws CompileException {
		buf.append(prefix);
		buf.append("<sharedmemory name=\"");
		buf.append(ident);
		buf.append("\" ");
		super.toCiAOConfig(buf,prefix,null);
		buf.append(" />\n");
	}

	public String getResourceClass() {
		return "keso/core/Memory";
	}

	public String getCType() throws CompileException {
		String ctype;
		try {
			ctype = getAttribute("ctype").valueString();
		} catch(Exception e) {
			throw new CompileException("Shared Memory " + ident + " has no C type defined!");
		}
		return ctype;
	}

	public DomainDefinition<?> getDomain() throws CompileException {
		if(writers.size() != 1)
			throw new CompileException(toString() + " each shared memory object needs exactly one writer!");
		return writers.get(0);
	}
}
