/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config;

import keso.compiler.*;
import keso.compiler.config.parser.ParseException;
import java.util.*;

final public class WorldDefinition extends Set<Set> {
	private final String rcfile;
	private List<SystemDefinition> systems;
	private List<NetworkDefinition> networks;
	private int pos = 0;
	private long mtime = 0;

	public WorldDefinition(String name, int line, String rcfile) throws ParseException {
		super(name, line);
		this.rcfile = rcfile;
		systems = new ArrayList<SystemDefinition>();
		networks = new ArrayList<NetworkDefinition>();
	}

	public void setLastModified(long mt) {
		mtime = mt;
	}

	public long lastModified() {
		return mtime;
	}

	public String rcFile() {
		return rcfile;
	}

	public void addSystem(SystemDefinition system) throws ParseException {
		BuilderOptions.getOpts().verbose("system: "+system);
		systems.add(system);
		addAttribute(ident,system);
	}

	public void addNetwork(NetworkDefinition net) throws ParseException {
		BuilderOptions.getOpts().verbose("Network: " + net);
		networks.add(net);
		addAttribute(ident, net);
	}

	public void rewindSystemDef() { pos = 0; }

	public SystemDefinition nextSystemDef() {
		if(pos < systems.size())
			return systems.get(pos++);

		return null;
	}

	public List<SystemDefinition> getSystems() { return systems; }
	public List<NetworkDefinition> getNetworks() { return networks; }
}

