/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config.domain;

import keso.compiler.config.*;
import keso.compiler.config.task.*;
import keso.compiler.config.parser.ParseException;
import java.util.ArrayList;
import java.util.List;

public class JavaDomain extends DomainDefinition<JavaTask> {
	protected ComplexAttribute<?> heapdef;
	private List<ServiceDefinition> services;
	private List<ImportDefinition> imports;

	public JavaDomain(SystemDefinition system, String name, int line) {
		super(system, name, line);
		services = new ArrayList<ServiceDefinition>();
		imports = new ArrayList<ImportDefinition>();
	}

	final public void registerService(String name, ServiceDefinition srv) {
		services.add(srv);
	}

	final public void registerImport(String name, ImportDefinition imp) {
		imports.add(imp);
	}

	final public List<ImportDefinition> getImports() { return imports; }

	final public List<ServiceDefinition> getServices() { return services; }

	public boolean needGCRunEvent() {
		ComplexAttribute<?> heap = getHeap();
		if (heap==null) return false;

		Attribute<?> gcmode = ((Set<?>) heap).getAttribute("GCTMode");
		if (gcmode==null) return false;

		String value = gcmode.valueString().toUpperCase();
		if (value!=null && value.equals("ENFORCEONDEMAND")) return true;
		if (value!=null && value.equals("DEFERTASKSONDEMAND")) return true;

		return false;
	}

	public void finalizeCfg(FinalizeResult res) throws keso.compiler.config.parser.ParseException {
		if(getHeap() == null)
			throw new keso.compiler.config.parser.ParseException("Domain " + getIdentifier() + " was configured without a heap\n");

		// GCRun event needs to be registers with all tasks before finalizing the tasks
		if (needGCRunEvent()) {
			EventDefinition gcevent = getSystem().createGCRunEvent();

			for(TaskDefinition td: getTasks()) {
				td.addEvent(gcevent.ident);
			}
		}

		super.finalizeCfg(res);
	}

	public void setComplex(ComplexAttribute cattr) throws ParseException {
		if(cattr.ident.compareToIgnoreCase("Heap")==0) {
			heapdef = cattr;

			if (needGCRunEvent()) {
				getSystem().createGCRunEvent();
			}
		} else {
			super.setComplex(cattr);
		}
	}

	public ComplexAttribute<?> getHeap() {
		return heapdef;
	}
}

