/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config.messageport;

import java.util.*;
import keso.compiler.*;
import keso.compiler.config.*;
import keso.compiler.config.domain.*;
import keso.compiler.config.parser.ParseException;
import keso.compiler.backend.CoderOSEK;

public class SenderPortDefinition extends SystemObject<DomainDefinition<?>> {
	private ReceiverPortDefinition rpd;

	public SenderPortDefinition(DomainDefinition<?> dom, String name, int line) {
		super(dom, name, line);
	}

	public void toOIL(StringBuffer oil, String prefix, CoderOSEK coder) throws CompileException {
		// messageports are CiAO specific
	}

	protected String typeID() { return "sp_"; }

	protected void finalizeCfg(FinalizeResult res) throws ParseException {
		Collection<DomainDefinition<?>> doms = parent.getSystem().getAppDomains();
		for(DomainDefinition<?> ddef: doms) {
			Collection<ReceiverPortDefinition> rpds = ddef.getReceiverPorts();
			for(ReceiverPortDefinition _rpd: rpds) {
				if(_rpd.ident.compareToIgnoreCase(ident) == 0) {
					if(rpd != null)
						throw new ParseException("Multiple Receiver Ports for Sender Port" + ident);
					rpd = _rpd;
				}
			}
		}

		if(rpd==null)
			throw new ParseException("No Receiver Port for Sender Port " + ident);
		//res.requireMethod("keso/core/Resource", null);
	}

	public String getResourceClass() { return "keso/ciao/SenderPort"; }

	public void toCiAOConfig(StringBuffer buf, String prefix) throws CompileException {
		buf.append(prefix + "<messageport name=\"");
		buf.append(getIdentifier());
		buf.append("\" ");

		toCiAOConfig(buf, prefix, null);

		buf.append("/>\n");
	}

	public String getIdentifier() { return ident; }

	public String getMessageType() {
		return ((StringAttr<?>) getAttribute("msgctype")).valueString();
	}

	public DomainDefinition<?> getDomain() { return parent; }
}

