/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config.task;

import keso.compiler.config.*;
import keso.compiler.config.domain.*;
import keso.compiler.CompileException;

public class JavaTask extends TaskDefinition implements UserObject {
	public JavaTask(JavaDomain dom, String name, int line) {
		super(dom, name, line);
	}

	// TODO Maybe tasks could be used to automatically handle packets
	// without any user code.
	public String getServiceName() {
		Attribute<?> attr = getAttribute("ServiceName");
		if (attr == null) return null;
		return attr.valueString();
	}

	public String getNetworkName() {
		Attribute<?> attr = getAttribute("NetworkName");
		if (attr == null) return null;
		return attr.valueString();
	}

	public boolean isServiceHandlerTask() {
		return getServiceName() != null && getNetworkName() != null;
	}

	public String getMainMethodName() throws CompileException {
		if(getAttribute("MainMethod") != null) {
			throw new CompileException("MainMethod Task Attribute no longer supported. MainMethod is now run()V");
		}
		return "run()V";
	}

	public String getMainClassName() {
		Attribute<?> main = getAttribute("MainClass");
		if (main==null) return "Main";
		return main.valueString();
	}

	public void finalizeCfg(FinalizeResult res) throws keso.compiler.config.parser.ParseException {
		super.finalizeCfg(res);

		// mark run method as required
		res.requireMethod(getAttribute("MainClass").valueString(), "run()V");
		res.requireMethod(getAttribute("MainClass").valueString(), "<init>()V");
	}

	public String getUserResourceClass() { return getMainClassName(); }
}
