/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.config.task;

import keso.compiler.config.*;
import keso.compiler.config.parser.ParseException;
import keso.compiler.config.domain.*;
import keso.compiler.CompileException;
import keso.compiler.backend.CoderOSEK;

public final class NativeTask extends TaskDefinition {
	public NativeTask(NativeDomain dom, String name, int line) {
		super(dom, name, line);
	}

	public String getIdentifier() { return ident; }

	public void toOIL(StringBuffer oil, String prefix, CoderOSEK coder) throws CompileException {
		try {
			addAttribute("PRIORITY", new IntegerAttr(this, "PRIORITY", Integer.toString(priority), -1));
			addAttribute("SCHEDULE", new StringAttr(this, "SCHEDULE", preemptable?"FULL":"NON", -1));
			addAttribute("TYPE", new StringAttr(this, "TYPE", usedEvents.size()==0 ? "BASIC" : "EXTENDED", -1));
			addAttribute("CALLSCHEDULER", new StringAttr(this, "CALLSCHEDULER", callsched?"YES":"NO", -1));
			addAttribute("STACKSIZE", new IntegerAttr(this, "STACKSIZE", Integer.toString(stacksize), -1));
			addAttribute("ACTIVATION", new IntegerAttr(this, "ACTIVATION", Integer.toString(activation), -1));
		} catch (ParseException ex) {
			throw new CompileException(ex.getMessage());
		}

		super.toOIL(oil, prefix + "\t", coder);
	}
}
