/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.dump;

import java.io.*;
import keso.classfile.*;
import keso.compiler.*;
import keso.compiler.analysis.*;
import keso.compiler.imcode.*;

public class ByteCodeDump extends MethodPass {

	private final IMCodeWriter cw = createCodeWriter();

	@Override
	public String getName() {
		return "Bytecode dump";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.hasDumpBC();
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		info.setIsDump(true);
		info.addDependency(LivenessAnalysis.class);
	}

	@Override
	public boolean runOnMethod(IMMethod method) throws CompileException {
		String outPath = method.getOptions().getOutputPath();
		dumpMethod(method, outPath + '/' + method.getAlias() + ".dump");
		return false;
	}

	protected IMCodeWriter createCodeWriter() {
		return new IMCodeWriter();
	}

	protected final void dumpMethod(IMMethod method, String fileName)
	throws CompileException {

		if (method.unused() == true || method.hasCode() == false)
			return;
		MethodData source = method.getSource();
		if (source.getCode() == null)
			return;

		// Open output file
		PrintStream out;
		try {
			out = new PrintStream(fileName);
		} catch (IOException e) {
			throw new CompileException(e.getMessage());
		}
		out.println("class " + method.getClassName());
		out.println();

		// Dump raw bytecode
		byte[]              byteCode = source.getBytecode();
		BytecodeInputStream bcStream = new BytecodeInputStream(byteCode);
		while (bcStream.hasMore()) {
			int bc = bcStream.readUnsignedByte();
			out.print(" 0x");
			if (bc < 16)
				out.print('0');
			out.print(Integer.toHexString(bc));
		}
		out.println();
		out.println();

		// Dump instruction mnemonics
		cw.writeMethod(method, out);
		out.close();
	}

}
