/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.dump;

import java.io.*;
import keso.compiler.*;
import keso.compiler.imcode.*;

public final class CFGWriter extends MethodPass {

	@Override
	public String getName() {
		return "CFG writer";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.createCFG();
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		info.setIsDump(true);
	}

	@Override
	public boolean runOnMethod(IMMethod method) throws CompileException {

		if (method.hasCode() == false)
			return false;

		String outPath  = method.getOptions().getOutputPath();
		String fileName = outPath + '/' + method.getAlias() + ".dot";
		PrintStream out;
		try {
			out = new PrintStream(fileName);
		} catch (IOException e) {
			throw new CompileException(e.getMessage());
		}

		out.println("digraph Flowgraph {\n");
		//out.println("\tsize=\"7,10\"\n\tpage=\"8.25,11.75\"");
		out.println("\tnode [shape=box height=.5 width=.5 fontsize=10];\n");

		for (IMBasicBlock bb : method) {
			bb.writeCFGEdges(out);
		}

		boolean first = true;
		for (IMBasicBlock bb : method) {
			if (first == true || bb.getPred().length() > 0) {
				first = false;
				bb.writeCFGBoxes(out);
			}
		}

		out.println();
		out.println("}\ndigraph Flowgraph {\n");
		out.println("\tnode [shape=box];\n");
		String prev = null;
		int    i    = 0;
		for (IMSlot slot : method.getMethodFrame().getAllSlots()) {
			if (slot.hasNoUses() == false) {
				if (prev != null) {
					out.print("\t\t" + prev + "->");
					prev = slot.toString().replace('[', '_').replace(']', '_');
					out.print(prev);
					out.print(";\n");
				} else {
					prev = slot.toString().replace('[', '_').replace(']', '_');
				}
				out.print("\t\t" + prev + " [label=\"\\l" + slot + "\\ldef:\\l");
				IMNode def = slot.getDefiningStatement();
				if (def != null) {
					if (def instanceof IMStoreLocalVariable)
						def = ((IMStoreLocalVariable) def).getOperand();
					out.print(def.toReadableString());
				}
				out.print("\\luse:");
				//if (true) {
					out.print(" " + slot.numberOfUses());
				/*} else {
					for (IMNode use : slot.getUses()) {
						out.print("\\l");
						out.print(slot.getUseBB(use).toLabel());
						//out.print("\\l");
						//out.print(use.toReadableString());
					}
				}*/
				out.println("\" fontsize=6 ]");
				++i;
				if (i == 8) {
					prev = null;
					i    = 0;
				}
			}
		}

		out.println('}');
		out.close();
		return false;
	}

}
