/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.dump;

import keso.compiler.*;
import keso.compiler.analysis.*;
import keso.compiler.imcode.*;
import keso.util.*;

/** Call graph .dot writer class. */
public final class CallGraphWriter extends Pass {

	@Override
	public String getName() {
		return "Call graph writer";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.createCallGraph();
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		info.setIsDump(true);
		info.addDependency(CallGraphAnalysis.class);
	}

	@Override
	public boolean runOnRepository(ClassStore repository)
	throws CompileException {

		try {

			String outPath  = repository.getOptions().getOutputPath();
			String fileName = outPath + "/callGraph.dot";
			DotWriter<IMMethod> dw = new DotWriter<IMMethod>(fileName);
			dw.initGraph("CallGraph");
			dw.writeLine("ranksep=2;");
			dw.setNodeStyle("shape=box");

			// Iterate over all classes
			for (IMClass imClass : repository.getAllClasses()) {

				IMMethod[] methods = imClass.getMethods();
				if (methods == null)
					continue;

				// Cluster all methods of the current class
				dw.initSubGraph("cluster_" + imClass.getClassID());
				dw.writeLine("label=\"" + imClass.getClassName() + "\";");
				for (IMMethod method : imClass.getMethods()) {
					if (method.unused() == false)
						dw.addNode(method, method.getMethodNameAndType());
				}
				dw.finalizeSubGraph();

				// Write edges to all callees for each method of the current class
				for (IMMethod method : imClass.getMethods()) {
					if (method.unused() == true)
						continue;
					for (IMMethod callee : method.getCalledMethods()) {
						dw.addEdge(method, callee);
					}
				}
			}

			dw.finalizeGraph();

		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}

		return false;
	}

}
