/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.dump;

import java.io.*;

import keso.compiler.*;
import keso.compiler.imcode.*;

public final class ClassListDump extends ClassPass {

	private PrintStream out;

	@Override
	public String getName() {
		return "Class list dump";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.hasOption("classlist");
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		info.setIsDump(true);
	}

	@Override
	public boolean runOnRepository(ClassStore repository)
	throws CompileException {
		String outPath = repository.getOptions().getOutputPath();
		try {
			out = new PrintStream(outPath + "/classes.lst");
		} catch (IOException e) {
			throw new CompileException(e.getMessage());
		}
		super.runOnRepository(repository);
		out.close();
		return false;
	}

	@Override
	public boolean runOnClass(IMClass imClass) throws CompileException {
		out.println("classes/" + imClass.getClassName() + ".class");
		return false;
	}

}
