/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/


package keso.compiler.dump;

import java.io.*;
import java.util.*;

import keso.compiler.*;
import keso.compiler.analysis.*;
import keso.compiler.imcode.*;
import keso.compiler.transform.*;

public final class ClassStatisticsDump extends DomainPass {

	private FlowAnalysis fa;

	@Override
	public String getName() {
		return "Class statistics dump";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.hasOption("dump_class_statistics");
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		super.getPassInfo(info, opts);
		info.addDependency(FlowAnalysis.class);
		info.addDependency(SimpleReachabilityAnalysis.class);
		info.addAntiDependency(FlowBasedOptimization.class);
		info.addAntiDependency(SSADeconstruction.class);
	}

	@Override
	public boolean init() throws CompileException {
		fa = pm.getPass(FlowAnalysis.class);
		return true;
	}

	@Override
	public boolean runOnDomain(IMDomain domain) throws CompileException {

		PrintStream out;
		String      outPath = domain.getOptions().getOutputPath();
		try {
			out = new PrintStream(outPath + "/classes-" + domain +".txt");
		} catch (FileNotFoundException e) {
			throw new CompileException(e.getMessage());
		}

		out.println("List of classes reachable in Domain " + domain);
		out.println("[x] instances of the class are created in the code reachable from domain " + domain);
		out.println();
	
		ReachabilityData rData = domain.getReachabilityData();
		List<IMClass> rClasses = new ArrayList<IMClass>(rData.getClasses());
		Collections.sort(rClasses);
		FlowLatticeInfoStore flowInfoStore = fa.getFlowLatticeInfoStore();
		FlowLatticeInfo flowInfo = flowInfoStore.getForDomain(domain);

		for (IMClass imClass : rClasses) {
			if (flowInfo.isClassInstantiated(imClass))
				out.println("[x] " + imClass);
		}
		for (IMClass imClass : rClasses) {
			if (flowInfo.isClassInstantiated(imClass) == false)
				out.println("[ ] " + imClass);
		}

		out.close();
		return false;
	}

}
