/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.dump;

import java.util.*;

import keso.compiler.*;
import keso.compiler.analysis.*;
import keso.compiler.imcode.*;
import keso.compiler.transform.*;

public final class ReachabilityDump extends ByteCodeDump {

	private FlowAnalysis fa;

	@Override
	public String getName() {
		return "Reachability dump";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.hasOption("dump_reachability_info");
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		super.getPassInfo(info, opts);
		info.addDependency(FlowAnalysis.class);
		info.addDependency(SimpleReachabilityAnalysis.class);
		info.addAntiDependency(FlowBasedOptimization.class);
		info.addAntiDependency(SSADeconstruction.class);
	}

	@Override
	public boolean init() throws CompileException {
		fa = pm.getPass(FlowAnalysis.class);
		return true;
	}

	@Override
	public boolean runOnMethod(IMMethod method) throws CompileException {
		String outPath = method.getOptions().getOutputPath();
		dumpMethod(method, outPath + '/' + method.getAlias()
				+ "_reachability.dump");
		return false;
	}

	@Override
	protected IMCodeWriter createCodeWriter() {

		return new IMCodeWriter() {

			@Override
			protected String getExtraString(IMNode node) {
				IMBasicBlock bb = node.getMethod().getCurrentBasicBlock();
				String result = "\n              : [";
				Iterator<IMDomain> it = bb.getDomains().iterator();
				while (it.hasNext()) {
					IMDomain domain = it.next();
					result += domain + ": ";
					FlowLatticeInfo flowInfo
					= fa.getFlowLatticeInfoStore().getForDomain(domain);
					IMDataFlow cell = null;
					if (flowInfo != null)
						cell = flowInfo.getCell(node);
					if (cell == null)
						result += "(N/A)";
					else
						result += cell.toReadableString();
					if (it.hasNext())
						result += ", ";
				}
				return result + ']';
			}
		};
	}

}
