/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
/**
 * (c)
 *
 * Copyright (C) 2006-2011 Christian Wawersich, Michael Stilkerich, Christoph Erhardt
 *
 * This file is part of the KESO Java Runtime Environment.
 *
 * KESO is free software: you can redistribute it and/or modify it under the terms of the Lesser GNU General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * KESO is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with KESO. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Please contact keso@cs.fau.de for more info.
 *
 * (c)
 **/

package keso.compiler.imcode;

import keso.compiler.CompileException;
import keso.compiler.analysis.connectiongraph.EscapeState;

public abstract class IMAllocation extends IMNode {
	/**
	 * The escape state of the object allocated in this allocation according to {@code ExtendedEscapeAnalysis}. This is
	 * set by {@code keso.compiler.transform.StackAllocation} and thus obviously only set when this phase is being run.
	 * It is defined to be {@code null} otherwise. A value of {@code EscapeState.Local} marks stack-allocatable objects.
	 */
	protected EscapeState	escapeState		= null;

	/**
	 * This field is true, when the liveness period of objects created by this allocation overlap each other, which
	 * prevents this allocation from being statically stack-allocatable. Allocation sites with this field set to true
	 * can still be stack-allocated by means of the {@code alloca()} function, i.e. by moving the stack pointer at
	 * runtime. Beware of lots of memory usage on the stack if you do this, though!
	 *
	 * This field is set by {@code StackAllocation} and is only valid, if this pass was run.
	 */
	protected boolean		livenessOverlap	= false;

	protected IMAllocation() {
		// Nothing to do
	}

	protected IMAllocation(IMMethod method, int bc, int type, int bcPos) {
		super(method, bc, type, bcPos);
	}

	@Override
	public final boolean isNew() {
		return true;
	}

	@Override
	public void accept(IMTypedVisitor visitor) throws CompileException {
		visitor.visit(this);
	}

	/**
	 * Set the escape state for the object allocated in this allocation.
	 *
	 * @param escapeState
	 *            the new escape state for this object
	 */
	public void setEscapeState(final EscapeState escapeState) {
		this.escapeState = escapeState;
	}

	/**
	 * Set the liveness period overlap status.
	 *
	 * @param livenessOverlap
	 *            the new liveness overlap status
	 */
	public void setLivenessOverlap(boolean livenessOverlap) {
		this.livenessOverlap = livenessOverlap;
	}

}
