/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import java.util.*;

/**
 * Special Phi-function for inter-procedural argument passing.
 * Every inbound edge represents an actual argument coming from a method
 * invocation.
 */
public final class IMArgPhi extends IMNode {

	private Set<IMNode> inEdges = new HashSet<IMNode>();

	public IMArgPhi(IMMethod method, int type) {
		super(method, -1, type, -1);
	}

	public void addInEdge(IMNode node) {
		inEdges.add(node);
	}

	public Collection<IMNode> getInEdges() {
		return Collections.unmodifiableCollection(inEdges);
	}

	@Override
	public void visitNodes(IMVisitor visitor) {
		// Nothing to do
	}

}
