/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import keso.compiler.backend.*;
import java.io.*;
import java.util.*;

public class IMArrayClass extends IMClass {

	protected String element_type;
	protected boolean require_alloc = false;
	protected boolean require_zero_array = false;

	public IMArrayClass(BuilderOptions opts, ClassStore src_repository, String classname,
			String alias, String element_type, boolean isFinal) {
		super(opts,src_repository,classname,alias,isFinal);
		this.element_type=element_type;
	}

	public String getElementType () {
		return element_type;
	}

	public int getObjRefFieldCount() {
		return 0;
	}

	public void requireAlloc() {
		require_alloc=true;
	}

	public void requireZeroLengthArray() {
		require_zero_array=true;
	}

	public boolean hasZeroLengthArray() {
		return require_zero_array;
	}

	public void requireMultiAlloc() {
		throw new Error();
	}

	@Override
	public IMClass cloneClass(ClassStore src_repository) {
		return new IMArrayClass(opts,src_repository,name,alias,element_type,isFinal);
	}

	@Override
	public boolean isInterface() {
		return false;
	}

	@Override
	public IMClass[] getInterfaces() {
		return null;
	}

	@Override
	public boolean hasSourceMethod(String nameAndType) {
		return false;
	}

	@Override
	public IMMethod getMethod(String nameAndType) {
		return null;
	}

	@Override
	public IMMethod[] getMethods() {
		return null;
	}

	@Override
	public String getSuperClassName() {
		return "java/lang/Object";
	}

	@Override
	public String emitObjectSize() {
		return "sizeof("+element_type+")";
	}

	@Override
	public void global_header(Coder coder) throws CompileException {
		/*
		coder.global_add("#include \"");
		coder.global_add(getAlias());
		coder.global_add(".h\"\n");
		*/
	}

	@Override
	public void load() throws IOException, CompileException {
		return;
	}

	@Override
	public void resetCallGraph() throws CompileException {
		return;
	}

	@Override
	public boolean omitUnusedMethods() throws CompileException {
		return false;
	}

	public void emit_ZeroLengthArray(Coder coder) throws CompileException {
			coder.add("((object_t*)&");
			coder.add(alias);
			coder.add("_zero_length)");
	}

	@Override
	public void translate(Coder coder) throws CompileException {
		opts.verbose("###### create "+getClassName());

		coder.beginHeader(this);

		coder.header_add("#include \"keso_types.h\"\n");
		coder.header_add("\n#define ");
		coder.header_add(alias.toUpperCase());
		coder.header_add("_ID ((class_id_t)");
		coder.header_add(type_info.getClassTypeID());
		coder.header_add(")\n");

		coder.header_add("typedef struct {\n");
		coder.header_add("ARRAY_HEADER\n\t");
		coder.header_add(element_type);
		coder.header_add(" data[1];\n");
		coder.header_add("} ");
		coder.header_add(alias);
		coder.header_add("_t;\n");

		coder.local_add("#include \"");
		coder.local_add(alias);
		coder.local_add(".h\"\n\n");

		if (require_zero_array) {
			coder.header_add("extern ");
			coder.header_add(alias);
			coder.header_add("_t ");
			coder.header_add(alias);
			coder.header_add("_zero_length;\n");
			coder.local_add(alias);
			coder.local_add("_t ALIGN4 ");
			coder.local_add(alias);
			coder.local_add("_zero_length = {\n");
			coder.local_add(coder.add_initObjectHeader(this,true));
			coder.local_add("0 /* size */\n");
			coder.local_add("};\n\n");
		}

		if (require_alloc) {
			coder.header_add("\nobject_t *keso_alloc_");
			coder.header_add(alias);
			coder.header_add("(array_size_t size);\n");

			coder.beginClassMethod(this);

			coder.local_add("object_t	*keso_alloc_");
			coder.local_add(alias);
			coder.local_add("(array_size_t size) {\n");
			coder.local_add("\tarray_t* array;\n");
			coder.local_add("\tarray=(array_t*)KESO_ALLOC(");
			coder.local_add(alias.toUpperCase());
			coder.local_add("_ID,");
			coder.local_add("sizeof(array_t)+(size*");
			coder.local_add(emitObjectSize());
			coder.local_add("),0);\n");
			coder.local_add("\tarray->size=size;\n");
			coder.local_add("\treturn (object_t*)array;\n");
			coder.local_add("}");

			coder.endClassMethod();
		}


		coder.endHeader();
	}

	public void dumpBC() throws IOException { return ; }

	@Override
	public void collectMethodNames(int DomainID,
			Map<String, Vector<IMClass>> method_to_class) {
		return;
	}

	@Override
	protected void parseMethodData() { return; }
}
