/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import java.util.*;

final public class IMBasicBlockList implements Iterable<IMBasicBlock> {

	private final class Itr implements Iterator<IMBasicBlock> {

		private int cursor  = 0;
		private int lastRet = -1;

		@Override
		public boolean hasNext() {
			return (cursor != count);
		}

		@Override
		public IMBasicBlock next() {
			try {
				IMBasicBlock next = lst[cursor];
				lastRet = cursor++;
				return next;
		    } catch (IndexOutOfBoundsException e) {
				throw new NoSuchElementException();
		    }
		}

		@Override
		public void remove() {
			if (lastRet == -1)
				throw new IllegalStateException();
			IMBasicBlockList.this.remove(lst[lastRet]);
			if (lastRet < cursor)
			    --cursor;
			lastRet = -1;
		}

	}

    private IMBasicBlock[] lst;
    private int count;

    public IMBasicBlockList() { this(2); }

    public IMBasicBlockList(IMBasicBlock block) {
		this(2);
		add(block);
	}

    public IMBasicBlockList(int size) {
        lst=new IMBasicBlock[size];
        count=0;
    }

	public IMBasicBlockList copy(IMCopyVisitor visitor) throws CompileException {
		IMBasicBlockList nbl = new IMBasicBlockList(count);

		for (int i=0;i<count;i++) {
			nbl.lst[i]=visitor.updateBlock(lst[i]);
		}
		nbl.count=count;

		return nbl;
	}

    public void add(IMBasicBlockList blk) {
		for (int i=0;i<blk.length();i++) add(blk.at(i));
    }

    public void add(IMBasicBlock[] blk) {
		for (int i=0;i<blk.length;i++) add(blk[i]);
    }

	public void add(IMBasicBlock blk) {
	    if (count==lst.length) {
	        IMBasicBlock[] nlst = new IMBasicBlock[count+100];
	        System.arraycopy(lst,0,nlst,0,count);
	        lst=nlst;
	    }
	    lst[count]=blk;
	    count++;
	}

	public void remove(IMBasicBlock blk) {
		boolean move = false;
        for (int i=0;i<count;i++) {
			if (lst[i]==blk) move=true;
			if (move) {
				if (i+1<count) {
					lst[i] = lst[i+1];
				} else {
					lst[i] = null;
				}
			}
		}
		if (move) count--;
	}

	public void update(IMBasicBlock old_bb, IMBasicBlock new_bb) {
		for (int i=0;i<count;i++) {
			if (old_bb==lst[i]) lst[i] = new_bb;
		}
	}

	public void set(int i, IMBasicBlock bb) {
		if (i<0 || i>=count) throw new IndexOutOfBoundsException();
		lst[i] = bb;
	}

	public IMBasicBlock at(int i) {
		//if (i<0 || i>=count) throw new IndexOutOfBoundsException();
		return lst[i];
	}

	public void clear() {
		count = 0;
	}

	public int length() {
		return count;
	}

	@Override
	public String toString() {
	    String ret = "";
	    for (int i = 0; i < count; ++i) {
	    	//if (lst[i]!=null) ret+=" "+lst[i].toLabel();
	        if (lst[i] != null)
	        	ret += " " + lst[i].getBCPosition();
	    }
	    return ret;
	}

	@Override
	public Iterator<IMBasicBlock> iterator() {
		return new Itr();
	}

}

