/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.classfile.datatypes.*;

import keso.compiler.*;

public class IMBinaryNode extends IMNode {

	protected IMNode rOpr;
	protected IMNode lOpr;

	public IMBinaryNode() {
	}

	public IMBinaryNode(IMMethod method, int bc, int type, int bcpos) {
		super(method,bc,type,bcpos);
	}

	public int getDatatype() {
		if (datatype==BCBasicDatatype.INT) {
			if (opts.hasOption("stack8") ||
					opts.hasOption("stack16")) {
				int rtype = rOpr.getDatatype();
				int ltype = lOpr.getDatatype();
				if (rtype==ltype) return rtype;
				if (rtype==BCBasicDatatype.SHORT || rtype==BCBasicDatatype.BYTE) {
					if (ltype==BCBasicDatatype.SHORT || ltype==BCBasicDatatype.BYTE)
						return BCBasicDatatype.SHORT;
				}
			}
		}
		return datatype;
	}

	public boolean hasConstant() { return lOpr.isConstant() || rOpr.isConstant(); }

	public boolean hasSideEffect() { return lOpr.hasSideEffect() || rOpr.hasSideEffect(); }

	public boolean isPureNode() { return lOpr.isPureNode() && rOpr.isPureNode(); }

	final public IMNode getLeftNode() { return lOpr; }

	final public IMNode getRightNode() { return rOpr; }

	final public void setLeftNode(IMNode opr) { lOpr = opr; }

	final public void setRightNode(IMNode opr) { rOpr = opr; }

	final public void swapNodes() {
		IMNode swap = rOpr;
		rOpr = lOpr;
		lOpr = swap;
	}

	public IMNode processStack(VirtualOperandStack stack, IMNode extra_ops) throws CompileException {

		if (opts.hasOption("ssa")) {
			stack.store3(bcPosition, extra_ops, 2);
		}

		rOpr = stack.pop();
		lOpr = stack.pop();
		stack.push(this);

		return null;
	}

	public String dumpOperands() {
		return lOpr.getRegId() + ", " + rOpr.getRegId();
	}

}
