/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.util.*;

public class IMBranch extends IMNode {

	protected IMBasicBlock targets[];
	protected IMBasicBlock[] uniqTargets=null;

	protected IMBranch() {
	}

	public IMBranch(IMMethod method, int bc, int bcpos) {
		super(method,bc,-1,bcpos);
	}

	public IMBranch(IMMethod method, int bc, int type, int bcpos) {
		super(method,bc,type,bcpos);
	}

	@Override
	final public boolean isBranch() {
		return true;
	}

	final public IMBasicBlock[] getTargets() { return targets; }

	public IMBasicBlock[] getUniqTargets() {
		if (uniqTargets == null) {
			IntegerHashtable<IMBasicBlock> hash
			= new IntegerHashtable<IMBasicBlock>();
			for (int i = 0; i < targets.length; ++i) {
				hash.put(targets[i].getBCPosition(), targets[i]);
			}
			uniqTargets = new IMBasicBlock[hash.size()];
			int i = 0;
			for (IMBasicBlock target : hash.getElements()) {
				uniqTargets[i++] = target;
			}
		}
		return uniqTargets;
	}

	protected String getTargetsString() {
		String res = "[";
		for (int i = 0; i < targets.length; ++i) {
			if (i > 0)
				res += ", ";
			res += targets[i].getShortLabel();
		}
		return res + ']';
	}

}
