/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import keso.compiler.backend.Coder;

import java.util.*;

final public class IMCallerEntry {

	IMMethod from;
	IMDataFlow[] argsInfo;

	public IMCallerEntry(IMMethod from, IMDataFlow[] argsInfo) {
		this.from = from;
		this.argsInfo = argsInfo;
	}

	public String getAlias() {
		if (from!=null) return from.getAlias();
		return "<kore>";
	}

	public IMDataFlow elementAt(int i) {
		if (argsInfo==null) return null;
		return argsInfo[i];
	}

	public int length() {
		return (argsInfo != null) ? argsInfo.length : 0;
	}

	public IMConstant getValue(int i) throws CompileException {
		return argsInfo[i].getValue();
	}

	public static IMCallerEntry getFoldedCallerEntry(Collection<IMCallerEntry> entries) throws CompileException {
		if (entries==null) return null;
		return new IMCallerEntry(null, getFoldedArgs(entries));
	}

	public static IMDataFlow[] getFoldedArgs(Collection<IMCallerEntry> entries)
	throws CompileException {
		IMDataFlow[] nentry=null;
		for (IMCallerEntry m : entries) {
			IMDataFlow[] mf = m.argsInfo;
			if (mf==null) return null;
			if (nentry==null) {
				nentry=new IMDataFlow[mf.length];
				for (int i=0;i<mf.length;i++) {
					nentry[i]=mf[i].copy();
				}
			} else {
				for (int i=0;i<mf.length;i++) {
					nentry[i] = nentry[i].fold(mf[i].copy());
				}
			}
		}
		return nentry;
	}

	public static void emitCallerInfo(Coder coder, IMMethod method,
	        Collection<IMCallerEntry> called_from) throws CompileException {
		coder.local_add("/*\n * call analysis ");
		coder.local_add(called_from.size());
		coder.local_add(" ");
		coder.local_add(method.getAlias());
		coder.local_add("\n");
		for (IMCallerEntry m : called_from) {
			coder.local_add(" *\n * ");
			coder.local_add(m.getAlias());
			coder.local_add("\n * ");
			coder.local_add(m.showArgs());
			coder.local_add("\n");
		}
		IMCallerEntry m = IMCallerEntry.getFoldedCallerEntry(called_from);
		coder.local_add(" *\n * total ");
		coder.local_add(m.showArgs());
		coder.local_add("\n");
		if (method.getOptions().hasOption("show_costs")) {
			coder.local_add(" *\n * costs: ");
			coder.local_add(method.getCosts());
			coder.local_add("\n");
		}
		coder.local_add(" */\n\n");

	}

	public String showArgs() {
		if (argsInfo==null) return "()";
		StringBuffer nb = new StringBuffer();
		nb.append("(");
		for (int i=0;i<argsInfo.length;i++)  {
			if (i>0) nb.append(", ");
			nb.append(argsInfo[i].toReadableString());
		}
		nb.append(")");
		return nb.toString();
	}
}
