/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.imcode;

import java.util.*;

import keso.compiler.*;

/**
 * Code flattener class.
 * The intermediate code is stored as a list of tree expressions. This class
 * offers a way to flatten these trees into a linked list, which is needed by
 * some IMCode operations.
 */
public final class IMCodeFlattener extends IMBasicVisitor {

	private List<IMNode> list = new LinkedList<IMNode>();

	@Override
	public void visit(IMNode node) throws CompileException {

		// All IMNodes are visited before their operands, which are visited in
		// reverse order. The only exception are the subclasses of
		// IMStoreLocalVariable, which are visited after their operands (what
		// the heck, why?) - so we have to add them to the end of the list.
		if (node instanceof IMStoreLocalVariable)
			list.add(node);
		else
			list.add(0, node);
	}

	private List<IMNode> getList() {
		return list;
	}

	public static List<IMNode> getFlattenedCode(IMNode root)
	throws CompileException {
		IMCodeFlattener flattener = new IMCodeFlattener();
		root.visitNodes(flattener);
		return flattener.getList();
	}

}
