/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;

public class IMConditionalBranch extends IMBranch {

	protected IMNode rOpr;
	protected IMNode lOpr;

	public IMConditionalBranch() {
	}

	public IMConditionalBranch(IMMethod method, int bc, int bcpos, IMBasicBlock label, IMBasicBlock next) {
		super(method,bc,-1,bcpos);
		targets = new IMBasicBlock[2];
		targets[0]=next;
		targets[1]=label;
	}

	@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		if (lOpr!=null) lOpr.setEscapePath(IMNode.NONE);
		if (rOpr!=null) rOpr.setEscapePath(IMNode.NONE);
	}

	@Override
	public IMBasicBlock[] getUniqTargets() {
		if (uniqTargets==null) {
			if (targets[0].equals(targets[1])) {
				uniqTargets = new IMBasicBlock[1];
				uniqTargets[0] = targets[0];
			} else {
				uniqTargets = targets;
			}
		}
		return uniqTargets;
	}

	@Override
	public IMNode processStack(VirtualOperandStack stack, IMNode extra_ops) throws CompileException {
		if (opts.doInlineCalls())
			stack.store3(bcPosition, extra_ops, 2);

		if (bytecode<159) {
			rOpr = method.createIMIConstant(0,bcPosition);
		} else {
			if (bytecode==198 || bytecode==199) {
				rOpr = method.createIMNullConstant(bcPosition);
			} else {
				rOpr = stack.pop();
			}
		}
		lOpr = stack.pop();
		return this;
	}

	@Override
	public String dumpOperands() {
		return lOpr.getRegId() + ", " + rOpr.getRegId() + ", "
		+ getTargetsString();
	}

	@Override
	public IMNode getLeftNode() {
		return lOpr;
	}

	@Override
	public IMNode getRightNode() {
		return rOpr;
	}

	protected IMConstant getTargetFor(boolean condition) {
		int index = condition ? 1 : 0;
		return method.createIMIConstant(index, -1);
    }

}
