/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;

public final class IMDataFlowAMem extends IMDataFlow {

	private IMDataFlow object;
	private IMDataFlow address;
	private IMDataFlow size;

	public IMDataFlowAMem(IMDataFlow object, IMDataFlow address,
			IMDataFlow size) {
		super(object.opts);
		this.object  = object;
		this.address = address;
		this.size    = size;
	}

	@Override
	public boolean isValid() {
		return object.isValid();
	}

	@Override
	public boolean isConstant() {
		return object.isConstant()
		|| (address.isConstant() && size.isConstant());
	}

	@Override
	public String toReadableString() {
		String addrString = address.toReadableString();
		try {
			addrString = "0x" + Integer.toHexString(Integer.parseInt(addrString));
		} catch (NumberFormatException e) {
			// Nothing to do
		}
		return object.toReadableString() + " (" + addrString + ", "
		+ size.toReadableString() + ')';
	}

	@Override
	public IMDataFlow copy(IMClass type) throws CompileException {
		return new IMDataFlowAMem(object.copy(type), address, size);
	}

	@Override
	public IMDataFlow fold(IMDataFlow flow) throws CompileException {
		if (flow instanceof IMDataFlowAMem == false)
			return object.fold(flow);
		IMDataFlowAMem aMemFlow   = (IMDataFlowAMem) flow;
		IMDataFlow     newObject  = object.fold(aMemFlow.object);
		IMDataFlow     newAddress = address.fold(aMemFlow.address);
		IMDataFlow     newSize    = size.fold(aMemFlow.size);
		return new IMDataFlowAMem(newObject, newAddress, newSize);
	}

	@Override
	public IMDataFlow intersect(IMDataFlow flow) throws CompileException {
		IMDataFlow newObject;
		IMDataFlow newAddress;
		IMDataFlow newSize;
		if (flow instanceof IMDataFlowAMem == false) {
			newObject  = object.intersect(flow);
			newAddress = address;
			newSize    = size;
		} else {
			IMDataFlowAMem aMemFlow   = (IMDataFlowAMem) flow;
			newObject  = object.intersect(aMemFlow.object);
			newAddress = address.intersect(aMemFlow.address);
			newSize    = size.intersect(aMemFlow.size);
	}
		return new IMDataFlowAMem(newObject, newAddress, newSize);
	}

	@Override
	public IMConstant getValue() throws CompileException {

		if (object.isConstant())
			return object.getValue();
		if (address.isConstant() == false || size.isConstant() == false)
			throw new CompileException("Memory object is not constant");

		// If the object itself is not constant, but address and size are,
		// create a new surrogate memory object
		IMConstant addr = address.getValue();
		int addrVal = addr.getIntValue();
		int sizeVal = size.getValue().getIntValue();
		return addr.getMethod().createMemConstant(addr, addrVal, sizeVal);
	}

	@Override
	public IMClass getType() {
		return object.getType();
	}

	public IMDataFlow getAddress() {
		return address;
	}

	public IMDataFlow getSize() {
		return size;
	}

}
