/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;

public final class IMDataFlowArray extends IMDataFlow {

	private final IMDataFlow array;
	private final IMDataFlow length;

	public IMDataFlowArray(IMDataFlow array, IMDataFlow length) {
		super(array.opts);
		this.array  = array;
		this.length = length;
	}

	@Override
	public boolean isValid() {
		return array.isValid();
	}

	@Override
	public boolean isConstant() {
		return array.isConstant();
	}

	@Override
	public boolean isArray() {
		return true;
	}

	@Override
	public String toReadableString() {
		return array.toReadableString() + " (" + length.toReadableString() + ')';
	}

	@Override
	public IMDataFlow copy(IMClass type) throws CompileException {
		return new IMDataFlowArray(array.copy(type), length);
	}

	@Override
	public IMDataFlow fold(IMDataFlow flow) throws CompileException {
		if (flow.isArray() == false)
			return array.fold(flow);
		IMDataFlowArray arrayFlow = (IMDataFlowArray) flow;
		IMDataFlow      newArray  = array.fold(arrayFlow.array);
		IMDataFlow      newLength = length.fold(arrayFlow.length);
		return new IMDataFlowArray(newArray, newLength);
	}

	@Override
	public IMDataFlow intersect(IMDataFlow flow) throws CompileException {
		IMDataFlow newArray;
		IMDataFlow newLength;
		if (flow.isArray() == false) {
			newArray  = array.intersect(flow);
			newLength = length;
		} else {
			IMDataFlowArray arrayFlow = (IMDataFlowArray) flow;
			newArray  = array.intersect(arrayFlow.array);
			newLength = length.intersect(arrayFlow.length);
		}
		return new IMDataFlowArray(newArray, newLength);
	}

	@Override
	public IMConstant getValue() throws CompileException {
		return array.getValue();
	}

	@Override
	public IMClass getType() {
		return array.getType();
	}

	public IMDataFlow getLength() {
		return length;
	}

}
