/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;

final public class IMDataFlowConstObj extends IMDataFlowObj {

	private IMConstant alias;

	public IMDataFlowConstObj(IMConstant alias, IMClass type) {
		super(alias.getOptions(), type);
		this.alias = alias;
	}

	@Override
	public IMDataFlow fold(IMDataFlow flow) throws CompileException {
		if (flow.isConstant() == false)
			return flow.fold(this);
		IMClass foldedType = foldTypes(flow.getType());
		if (flow.getValue().compareTo(alias) == 0)
			return copy(foldedType);
		if (isValid() && flow.isValid())
			return new IMDataFlowValidObj(opts, foldedType);
		else
			return new IMDataFlowInvalidObj(opts, foldedType);
	}

	@Override
	public IMDataFlow intersect(IMDataFlow flow) throws CompileException {
		if (flow instanceof IMDataFlowConstObj) {
			if (alias.compareTo(flow.getValue()) == 0)
				return this;
		} else if (flow instanceof IMDataFlowValidObj) {
			if (alias.isNullValue() == false)
				return this;
		} else if (flow instanceof IMDataFlowInvalidObj) {
			return this;
		}
		return new IMDataFlowInvalidObj(opts, type);
	}

	@Override
	public boolean isValid() {
		return !alias.isNullValue();
	}

	@Override
	public boolean isConstant() {
		return true;
	}

	@Override
	public String toReadableString() {
		return alias.toReadableString() + ' ' + type.getClassName();
	}

	@Override
	public IMConstant getValue() {
		return alias;
	}

	@Override
	public IMConstant getConstValue(IMMethod method, int bcpos) throws CompileException {
		if (alias instanceof IMNullConstant) {
			return method.createIMNullConstant(bcpos);
		}
		return method.createIMAConstant(alias.getCPEntry(), bcpos);
	}

	@Override
	public IMDataFlow copy(IMClass type) {
		return new IMDataFlowConstObj(alias, type);
	}

}
