/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;

public final class IMDataFlowIntRange extends IMDataFlow {

	private final int lowerBound;
	private final int upperBound;
	private String valueString;

	public IMDataFlowIntRange(int lowerBound, int upperBound, BuilderOptions opts) {
		super(opts);
		this.lowerBound = lowerBound;
		this.upperBound = upperBound;
	}

	@Override
	public boolean hasFixedRange() {
		return true;
	}

	@Override
	public int maxValue() {
		return upperBound;
	}

	@Override
	public int minValue() {
		return lowerBound;
	}

	@Override
	public IMDataFlow fold(IMDataFlow flow) throws CompileException {
		if (flow instanceof IMDataFlowConstValue || flow.hasFixedRange())
			return addConstValue(flow);
		return IMDataFlow.ANY;
	}

	@Override
	public IMDataFlow intersect(IMDataFlow flow) throws CompileException {
		if (flow instanceof IMDataFlowConstValue
				|| flow instanceof IMDataFlowSet) {
			return flow.fold(this);
		} else if (flow instanceof IMDataFlowIntRange) {
			int newLower = Math.max(lowerBound, flow.minValue());
			int newUpper = Math.min(upperBound, flow.maxValue());
			if (newLower <= newUpper)
				return new IMDataFlowIntRange(newLower, newUpper, opts);
		} else if (flow instanceof IMDataFlowAny) {
			return this;
		}
		return IMDataFlow.ANY;
	}

	@Override
	public IMDataFlow addConstValue(IMDataFlow df) throws CompileException {
		int newLower = Math.min(lowerBound, df.minValue());
		int newUpper = Math.max(upperBound, df.maxValue());
		return new IMDataFlowIntRange(newLower, newUpper, opts);
	}

	@Override
	public String toReadableString() {
		if (valueString == null)
			valueString = "[" + lowerBound + " -> " + upperBound + "]";
		return valueString;
	}

}
