/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;

public final class IMDataFlowMessage extends IMDataFlowResource {

	public IMDataFlowMessage(IMDataFlow message, IMDataFlow name) {
		super(message, name);
	}

	@Override
	public String toReadableString() {
		return resource.toReadableString() + " (Message: "
		+ name.toReadableString() + ')';
	}

	@Override
	public IMDataFlow copy(IMClass type) throws CompileException {
		return new IMDataFlowMessage(resource.copy(type), name);
	}

	@Override
	public IMDataFlow fold(IMDataFlow flow) throws CompileException {
		if (flow instanceof IMDataFlowMessage == false)
			return resource.fold(flow);
		IMDataFlowMessage msgFlow = (IMDataFlowMessage) flow;
		IMDataFlow newMessage = resource.fold(msgFlow.resource);
		IMDataFlow newName    = name.fold(msgFlow.name);
		return new IMDataFlowMessage(newMessage, newName);
	}

	@Override
	public IMDataFlow intersect(IMDataFlow flow) throws CompileException {
		IMDataFlow newMessage;
		IMDataFlow newName;
		if (flow instanceof IMDataFlowMessage == false) {
			newMessage = resource.intersect(flow);
			newName    = name;
		} else {
			IMDataFlowMessage msgFlow = (IMDataFlowMessage) flow;
			newMessage = resource.intersect(msgFlow.resource);
			newName    = name.intersect(msgFlow.name);
		}
		return new IMDataFlowMessage(newMessage, newName);
	}

}
