/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;

public final class IMDataFlowSHM extends IMDataFlowResource {

	public IMDataFlowSHM(IMDataFlow memory, IMDataFlow name) {
		super(memory, name);
	}

	@Override
	public String toReadableString() {
		return resource.toReadableString() + " (SHM: " + name.toReadableString()
		+ ')';
	}

	@Override
	public IMDataFlow copy(IMClass type) throws CompileException {
		return new IMDataFlowSHM(resource.copy(type), name);
	}

	@Override
	public IMDataFlow fold(IMDataFlow flow) throws CompileException {
		if (flow instanceof IMDataFlowSHM == false)
			return resource.fold(flow);
		IMDataFlowSHM shmFlow   = (IMDataFlowSHM) flow;
		IMDataFlow    newMemory = resource.fold(shmFlow.resource);
		IMDataFlow    newName   = name.fold(shmFlow.name);
		return new IMDataFlowSHM(newMemory, newName);
	}

	@Override
	public IMDataFlow intersect(IMDataFlow flow) throws CompileException {
		IMDataFlow newMemory;
		IMDataFlow newName;
		if (flow instanceof IMDataFlowSHM == false) {
			newMemory = resource.intersect(flow);
			newName   = name;
		} else {
			IMDataFlowSHM shmFlow   = (IMDataFlowSHM) flow;
			newMemory = resource.intersect(shmFlow.resource);
			newName   = name.intersect(shmFlow.name);
		}
		return new IMDataFlowSHM(newMemory, newName);
	}

}
