/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;

final public class IMDataFlowValidObj extends IMDataFlowObj {

	public IMDataFlowValidObj(BuilderOptions opts, IMClass type) {
		super(opts, type);
	}

	@Override
	public IMDataFlow fold(IMDataFlow flow) throws CompileException {
		if (flow.isValid())
			return copy(foldTypes(flow.getType()));
		else if (flow.isConstant())
			return new IMDataFlowInvalidObj(opts, foldTypes(flow.getType()));
		return flow.fold(this);
	}

	@Override
	public IMDataFlow intersect(IMDataFlow flow) throws CompileException {
		if (flow instanceof IMDataFlowValidObj
				|| flow instanceof IMDataFlowInvalidObj)
			return this;
		return flow.intersect(this);
	}

	@Override
	public boolean isValid() {
		return true;
	}

	@Override
	public String toReadableString() {
		return "valid " + type.getClassName();
	}

	@Override
	public IMDataFlow copy(IMClass type) {
		return new IMDataFlowValidObj(opts, type);
	}

}
