/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;

final public class IMDeadCodeVisitor extends IMBasicVisitor {

	private BuilderOptions opts;
	private IMMethod method;

	public IMDeadCodeVisitor(IMMethod method) {
		this.opts = method.getOptions();
		this.method=method;
	}

	public void visit(IMNode node) throws CompileException {
		if (node.isVariable()) {
			IMSlot var = node.getIMSlot();
			if (var == null || var.hasNoUses())
				return;
			try {
				var.deleteUse(node);
			} catch (RuntimeException ex) {
				opts.warn(node.lineInfo()+": Can't delete use of "+node.toReadableString());
				throw ex;
			}
			if (var.hasNoUses()) method.doAnotherSSARun();
		}
	}
}
