/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import java.util.*;
import keso.util.IntegerHashtable;

final public class IMDefVisitor extends IMBasicVisitor {

	private IMMethod method;
	private Map<IMSlot, IntegerHashtable<IMBasicBlock>> defsites;
	private IntegerHashtable<Set<IMSlot>> bb_vars;

	public IMDefVisitor(IMMethod method,
			Map<IMSlot, IntegerHashtable<IMBasicBlock>> defsites,
			IntegerHashtable<Set<IMSlot>> bb_vars) throws CompileException {
		this.method = method;
		this.defsites = defsites;
		this.bb_vars = bb_vars;
	}

	public void visit(IMNode node) throws CompileException {
		if (node instanceof IMStoreLocalVariable) {
			IMBasicBlock bb = method.getCurrentBasicBlock();
			IMSlot cvar = node.getIMSlot();
			int n = bb.getDFNum();

			IntegerHashtable<IMBasicBlock> def_var;
			if ((def_var=defsites.get(cvar))==null) {
				def_var = new IntegerHashtable<IMBasicBlock>();
				defsites.put(cvar, def_var);
			}
			def_var.put(n,bb);

			Set<IMSlot> vars;
			if ((vars=bb_vars.get(n))==null) {
				vars = new HashSet<IMSlot>();
				bb_vars.put(n, vars);
			}
			vars.add(cvar);
		}
	}
}
