/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.classfile.*;

final public class IMExceptionHandler {

	private IMMethod method;
	private ExceptionHandlerData expData;
	private IMBasicBlock handler;
	private IMClass type;

	public IMExceptionHandler(IMMethod method, ExceptionHandlerData source) {
		this.method = method;
		this.expData=source;
	}

	public void setHandler(IMBasicBlock handler) {
		this.handler=handler;
	}

	public IMBasicBlock getHandler() {
		return this.handler;
	}

	public int getStartPos() {
		return expData.getStartBCIndex();
	}

	public int getEndPos() {
		return expData.getEndBCIndex();
	}

	public int getRangeID() {
		return (expData.getStartBCIndex() << 16) + expData.getEndBCIndex();
	}

	public boolean inRegion(int position) {
		return (expData.getStartBCIndex()<=position && position<expData.getEndBCIndex());
	}

	public String getClassName() {
		return expData.getCatchTypeCPEntry();
	}

	public IMClass getIMClass() {
		if (type==null) type=method.getIMClass(expData.getCatchTypeCPEntry());
		return type;
	}
}
