/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import keso.compiler.backend.*;
import keso.compiler.config.ComplexAttribute;
import java.io.PrintStream;

/**
 * Abstract superclass of all heap implementations.
 */
public abstract class IMHeap extends IMDescriptorContainer {

	/**
	 * Size of this heap.
	 */
	protected int heap_size;

	/**
	 * Domain this heap belongs to.
	 */
	protected final IMDomain domain;

	/**
	 * Definition of this heap from kesorc.
	 */
	protected final ComplexAttribute<?> heapdef;

	protected IMHeap(String name, String alias, int size, BuilderOptions opts, IMDomain domain, ComplexAttribute<?> heapdef) {
		super(name,alias,opts);

		this.domain = domain;
		this.heapdef = heapdef;

		heap_size = size;
	}

	/**
	 * Get the name of the allocator function belonging to this
	 * heap's type.
	 */
	public String getAllocatorName() { return "keso_"+name+"_alloc"; }

	/**
	 * Common tasks used by many if not all implementations.
	 */
	protected void translate(Coder coder) throws CompileException { }

	public void initHeap(Coder coder, PrintStream out) throws CompileException {}

	public int getSize() { return heap_size; }

	/**
	 * Initial value of the colorbit.
	 *
	 * Can be overridden by specific heap implementations, however, the
	 * least significant bit must always be set to support stack scanning
	 * and the invocation of finally() from the IRR GC.
	 */
	public int getInitialColor() { return 1; }
}
