/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import keso.compiler.backend.*;
import keso.compiler.config.ComplexAttribute;
import keso.compiler.config.SystemDefinition;

public class IMHeapRestrictedDomainScope extends IMHeap {

	/**
	 * Flag to indicate if one-time translation parts like the implementations
	 * functions have already been emitted or not.
	 */
	private static SystemDefinition once = null;
	public IMHeapRestrictedDomainScope(IMDomain dom, BuilderOptions opts, int size, ComplexAttribute<?> heapdef) throws CompileException {
		super("rds","rds",size,opts,dom,heapdef);

		addDescriptorField("KESO_CONST char*", "heap_end", "_heap+"+size, false, true, false);
		addDescriptorField("char*", "new_ptr", "_heap", false,true, false);
	}

	public void translate(Coder coder) throws CompileException {
		opts.verbose("#### create heap "+alias);

		super.translate(coder);

		// Write functions for this heap (allocator, etc.) only done once per system
		SystemDefinition sysDef = opts.getSysDef();
		if (once != sysDef) {
			once = sysDef;

			coder.beginHeader(this);
			coder.header_add("object_t* keso_");
			coder.header_add(alias);
			coder.header_add("_alloc(class_id_t class_id, obj_size_t size, obj_size_t roff);\n");

			coder.beginClassMethod(this);

			coder.local_add("#include \"");
			coder.local_add(alias);
			coder.local_add(".h\"\n\n");

			if (!opts.isAVR() && opts.targetAddrSizeShift()>0) {
				int tmp = opts.targetAddrSize()-1;
				coder.local_add("#define KESO_ALIGN_OBJ(_size_) ((_size_+");
				coder.local_add(tmp);
				coder.local_add(")&~");
				coder.local_add(tmp);
				coder.local_add(")");
			} else {
				coder.local_add("#define KESO_ALIGN_OBJ(_size_) (_size_)\n");
			}

			coder.local_add("\nobject_t* keso_");
			coder.local_add(alias);
			coder.local_add("_alloc(class_id_t class_id, obj_size_t size, obj_size_t roff) {\n");
			coder.local_add("\tchar* mem;\n\tint i=KESO_ALIGN_OBJ(size);\n\tobject_t* obj;\n\n");
			coder.local_add("\tstruct domain_rds_t* domain = (struct domain_rds_t *) KESO_CURRENT_DOMAIN;\n");
			coder.local_add("\tmem = domain->hd.new_ptr;\n");
			coder.local_add("\tdomain->hd.new_ptr += i;\n");
			coder.local_add("\tif (domain->hd.new_ptr > domain->hd.heap_end)");
			coder.local_add("\t\tKESO_THROW_ERROR(\"out of memory\");\n");
			coder.local_add("\tdomain->heap_free -= i;\n");

			// the heap is in the BSS and should be 0-initialized by the startup code
			if(opts.hasDebugCore()) {
				if (opts.is8BitController()) {
					coder.local_add("\twhile(i-->0) mem[i] = 0;\n");
				} else {
					coder.local_add("\ti /= sizeof(int);\n");
					coder.local_add("\twhile(i-->0) ((int *)mem)[i] = 0;\n");
				}
			}

			coder.local_add("\tobj = (object_t*) ((object_t**)mem + roff);\n");
			coder.local_add("\tobj->class_id = class_id;\n");
			coder.local_add("\n\treturn obj;\n");
			coder.local_add("}\n");

			coder.endClassMethod();

			coder.endHeader();
		}
	}
}
