/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import java.util.*;

final public class IMInlineVisitor implements IMCopyVisitor {

	private IMMethod host_method;
	private IMMethodFrame host_frame;
	private IMNode	 call;
	private IMNode	 current;
	private int bcpos;

	private Map<IMBasicBlock, IMBasicBlock> already_seen
	        = new HashMap<IMBasicBlock, IMBasicBlock>();

	public IMInlineVisitor(IMMethod host_method, IMNode self, IMNode[] args, int bcpos,
			IMBasicBlock next_bb, IMNode current, IMNode call) {
		this.host_method = host_method;
		host_method.getOptions();
		this.host_frame  = host_method.getMethodFrame();
		this.bcpos = bcpos;
		this.call = call;
		this.current = current;
		if (next_bb==null) throw new Error();
	}

	public IMNode visit(IMNode nnode, IMNode onode) throws CompileException {
		if (nnode instanceof IMEpilog) {
			if (call==null) return null;

			IMSlot ret_slot = onode.getMethod().getReturnValue();
			if (ret_slot==null) return null;
			IMReadLocalVariable ret_value = host_method.readLocal(call.getBCPosition(),adjustSlot(ret_slot));
			IMReplaceVisitor repl = new IMReplaceVisitor(host_method,call,ret_value);

			nnode = current.copy(repl);
		}
		nnode.method = host_method;
		nnode.bcPosition = bcpos;
		return nnode;
	}

	public IMSlot adjustSlot(IMSlot foreignSlot) throws CompileException {
		return host_frame.adjustSlot(foreignSlot);
	}

	public IMBasicBlock updateBlock(IMBasicBlock label) throws CompileException {
		if (label==null) return null;

		IMBasicBlock nl = already_seen.get(label);
		if (nl==null) {
			nl = label.createStub(host_method, this);
			nl.markAsCopied();
			nl.method = host_method;
			nl.bcPosition = bcpos;
			already_seen.put(nl, nl);
			already_seen.put(label, nl);
			label.updateStub(nl,this);
		}

		return nl;
	}
}

