/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.classfile.constantpool.*;
import keso.compiler.*;
import keso.util.*;

/** Invoke instruction with a non-unique (virtual) callee. */
public class IMInvokeNonUnique extends IMInvoke {

	protected IMInvokeNonUnique() {
		// Nothing to do
	}

	public IMInvokeNonUnique(IMMethod method, int bc, int bcPos,
			InterfaceMethodRefCPEntry methodRef) {
		super(method, bc, bcPos, methodRef);
	}

	public IMInvokeNonUnique(IMMethod method, int bc, int bcPos,
			MethodRefCPEntry methodRef) {
		super(method, bc, bcPos, methodRef);
	}

	@Override
	protected Iterable<IMMethod> doGetCandidates(IMMethod callee)
	throws CompileException {
		MethodTable table = callee.getMethodTable();
		if (table != null)
			return table.getCandidates();
		else
			return new SingleElementIterable<IMMethod>(callee);
	}

}
