/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import java.util.*;

import keso.compiler.*;

public class IMListHead extends IMNode implements Iterable<IMNode> {

	private final class Itr implements Iterator<IMNode> {

		private IMNode current = next_node;

		@Override
		public boolean hasNext() {
			return (current != null);
		}

		@Override
		public IMNode next() {
			if (hasNext() == false)
				throw new NoSuchElementException();
			IMNode result = current;
			current = result.next();
			return result;
		}

		@Override
		public void remove() {
			throw new UnsupportedOperationException();
		}

	}

	public IMListHead() { super(); }

	public IMNode copy(IMBasicBlock currentBasicBlock,IMCopyVisitor visitor) throws CompileException {
		IMListHead nnode = new IMListHead();
		nnode.copyNodeValues(visitor, this);
		return nnode;
	}

	public boolean emptyList() { return next()==null; }

	public String toReadableString() {
		String s = "";

		IMNode prev = this;
		while (prev!=null && prev.next() != null) {
			IMNode curr = prev.next();
			s = s + "\t" + curr.toReadableString() + ";\n";
			prev = curr;
		}

		return s;
	}

	@Override
	public Iterator<IMNode> iterator() {
		return new Itr();
	}

}
