/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import keso.compiler.backend.Coder;
import keso.classfile.datatypes.BCBasicDatatype;

public class IMMethodNotFound extends IMSystemMethod {

	private String sig;

	public IMMethodNotFound(IMClass clazz, String sig) throws CompileException {
		super(clazz);
		this.sig = sig;
	}

	@Override
	public String getMethodName() {
		return "unknown";
	}

	@Override
	public String getArgString() throws CompileException {
		return "...";
	}

	@Override
	public String getReturnType() {
		int pos = sig.indexOf(")");
		int return_type = BCBasicDatatype.toBasicDatatype(sig.charAt(pos+1));
		return BuilderOptions.typeToString(return_type);
	}

	@Override
	public String getMethodNameAndType() {
		return "unknown: "+sig;
	}

	@Override
	public void translate(Coder coder) throws CompileException {
		opts.warn("method not found! "+this.getClass().getName()+"."+sig);
	}

	@Override
	public <R> R accept(IMMethodVisitor<R> visitor) throws CompileException {
		return visitor.visit(this);
	}

}
