/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import keso.compiler.backend.*;

public final class IMNullCheck extends IMConditionalBranch {

	private String  reason;
	private IMField field;

	public IMNullCheck(IMNode checkedNode, IMNode operand,
			IMBasicBlock successor, String reason) throws CompileException {
		super(checkedNode.getMethod(), -1, checkedNode.getBCPosition(), null,
				null);
		lOpr = operand.copy(null);
		init(successor, reason, null);
	}

	public IMNullCheck(IMNode checkedNode, IMNode operand,
			IMBasicBlock successor, String reason, IMField field)
	throws CompileException {
		super(checkedNode.getMethod(), -1, checkedNode.getBCPosition(), null,
				null);
		lOpr = operand.copy(null);
		init(successor, reason, field);
	}

	private IMNullCheck() {
		// Nothing to do
	}

	@Override
	public IMNode copy(IMCopyVisitor visitor) throws CompileException {
		IMNullCheck nnode = new IMNullCheck();
		nnode.copyNodeValues(visitor, this);
		IMBasicBlock target = targets[0];
		if (visitor != null)
			target = visitor.updateBlock(target);
		nnode.lOpr = lOpr.copy(visitor);
		nnode.init(target, reason, field);
		return visitor.visit(nnode, this);
	}

	@Override
	public void visitNodes(IMVisitor visitor) throws CompileException {
		if (visitor.skipNode(this) == true)
			return;
		visitor.visit(this);
		rOpr.visitNodes(visitor);
		lOpr.visitNodes(visitor);
	}

	@Override
	public IMBasicBlock[] getUniqTargets() {
		return targets;
	}

	@Override
	public int costs() throws CompileException {
		return 1;
	}

	@Override
	public void translate(Coder coder) throws CompileException {
		coder.chk_ref(lOpr, method, bcPosition, reason, field);
	}

	@Override
	public String toReadableString() {
		return "if (" + lOpr.toReadableString() + " == "
		+ rOpr.toReadableString() +") throw new NullPointerException()";
	}

	private void init(IMBasicBlock successor, String reason, IMField field)
			throws CompileException {
		targets = new IMBasicBlock[] {successor};
		rOpr    = new IMNullConstant(method, -1, bcPosition);
		this.reason = reason;
		this.field  = field;
	}

	@Override
	public void accept(IMTypedVisitor visitor) throws CompileException {
		visitor.visit(this);
	}

}
