/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import keso.util.MarkableStack;
import java.util.*;

final public class IMPhiUpdateVisitor extends IMBasicVisitor {

	private BuilderOptions opts;
	private Map<IMSlot, MarkableStack<IMSlot>> stacks;
	private IMBasicBlock bb;
	private int pb_nr;

	public IMPhiUpdateVisitor(IMMethod method, IMBasicBlock bb, IMBasicBlock pb,
	        Map<IMSlot, MarkableStack<IMSlot>> stacks) throws CompileException {
		this.opts=method.getOptions();
		this.stacks = stacks;
		this.bb = bb;
		pb_nr=-1;
		IMBasicBlockList list = bb.getPred();
		for (int i=0;i<list.length();i++) {
			if (list.at(i)==pb) {
				pb_nr = i;
				break;
			}
		}

		if (pb_nr==-1) throw new CompileException("unknown pred. basic block "+pb+" of "+bb);
	}

	public boolean skipNode(IMNode node) {
		if (node instanceof IMPhi) return false;
		if (node instanceof IMStoreLocalVariable) return false;
		return true;
	}

	public void visit(IMNode node) throws CompileException {
		if (node instanceof IMPhi) {
			IMPhi phi = (IMPhi)node;
			//opts.verbose("update o "+phi.toReadableString()+" "+pb_nr+":"+phi.getSourceVar(pb_nr));
			IMNode old = phi.getSource(pb_nr);

			IMSlot ovar = old.getIMSlot();
			if (ovar==null) {
				opts.warn("bad update "+phi.toReadableString()+" "+pb_nr+":"+phi.getSourceVar(pb_nr));
			}
			MarkableStack<IMSlot> stack = stacks.get(ovar);
			if (stack==null) {
				opts.warn("bad update "+ovar+" "+phi.toReadableString()+" "+pb_nr+":"+phi.getSourceVar(pb_nr));
			}
			IMSlot slot = stack.peek();
			// slot may be null if the variable is not defined!
			phi.setSourceVar(pb_nr, slot);

			//if (old!=null) slot.delete_use(old);
			if (slot!=null) slot.use(bb, phi.getSource(pb_nr));
			//if (slot!=null) slot.use(bb, phi);
			//opts.verbose("update n "+phi.toReadableString());
		}
	}
}
