/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.imcode;

import keso.compiler.*;
import keso.util.*;

import java.util.*;

final public class IMRenameVisitor extends IMBasicVisitor {

	private IMMethod method;
	private IMMethodFrame frame;
	private BuilderOptions opts;
	private Map<IMSlot, MarkableStack<IMSlot>> stacks;
	private IMBasicBlock bb;

	public IMRenameVisitor(IMMethod method, IMBasicBlock bb,
			Map<IMSlot, MarkableStack<IMSlot>> stacks) throws CompileException {
		this.method = method;
		this.frame = method.getMethodFrame();
		this.opts = method.getOptions();
		this.stacks = stacks;
		this.bb = bb;
	}

	public boolean skipNode(IMNode node) {
		if (node instanceof IMPhi) return true;
		return false;
	}

	private void dumpStacks() {
		for (IMSlot var : stacks.keySet()) {
			System.err.print(var+" => ");
			System.err.println(stacks.get(var));
		}
	}

	public void visit(IMNode node) throws CompileException {

		if (node instanceof IMPhi)
			throw new CompileException("unexpected node type");

		if (node.usesSlot()) {
			IMSlot var = node.getIMSlot();
			if (var != null) {
				MarkableStack<IMSlot> stack = stacks.get(var);
				if (stack == null) {
					opts.warn("no definition for " + var + " in "
							+ method.getAlias() + " bb " + bb.getDFNum()
							+ " node " + node);
					//dumpStacks();
					node.setIMSlot(null);
				} else {
					IMSlot slot = stack.peek();
					if (slot!=null) {
						slot.use(bb, node);
						node.setIMSlot(slot);
					} else {
						opts.warn("undefined variable " + var + " in bb"
								+ bb.getDFNum());
					}
				}
			}
		}

		if (node instanceof IMStoreLocalVariable) {
			IMSlot ovar = node.getIMSlot();
			MarkableStack<IMSlot> stack = stacks.get(ovar);
			IMSlot nvar = frame.createNewVariableSSA(ovar);
			node.setIMSlot(nvar);
			nvar.define(bb, node);
			stack.push(nvar);
		}
	}

}
