/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.compiler.imcode;

import keso.compiler.CompileException;

/* THIS FILE IS GENERATED FROM createNodes.pl */
/**
 * Interface of the visitor pattern implementing a way to call different methods
 * for different node types without using large cascades of instanceof. You
 * should probably not implement this interface directly, but rather subclass
 * one of its implementing adapter classes, e.g. IMTypedVisitorSimpleAdapter,
 * which provides all required methods without implementation. This allows you
 * to override the methods you need to specialize without cluttering your
 * visitor class with all the other methods required by this interface.
 */
public interface IMTypedVisitor {
	public void visit(IMAllocation n) throws CompileException;
	public void visit(IMNullCheck n) throws CompileException;
	public void visit(IMLRem n) throws CompileException;
	public void visit(IMStoreLocalVariable n) throws CompileException;
	public void visit(IMCastL2D n) throws CompileException;
	public void visit(IMBStoreArray n) throws CompileException;
	public void visit(IMStackOperationSWAP n) throws CompileException;
	public void visit(IMNew n) throws CompileException;
	public void visit(IMShiftLeft n) throws CompileException;
	public void visit(IMAStoreLocalVariable n) throws CompileException;
	public void visit(IMCastD2F n) throws CompileException;
	public void visit(IMFAdd n) throws CompileException;
	public void visit(IMGoto n) throws CompileException;
	public void visit(IMNEConditionalBranch n) throws CompileException;
	public void visit(IMStackOperationDUP2_X1 n) throws CompileException;
	public void visit(IMBitXor n) throws CompileException;
	public void visit(IMCaughtException n) throws CompileException;
	public void visit(IMCastF2I n) throws CompileException;
	public void visit(IMLShiftRight2 n) throws CompileException;
	public void visit(IMDReadLocalVariable n) throws CompileException;
	public void visit(IMRem n) throws CompileException;
	public void visit(IMMul n) throws CompileException;
	public void visit(IMEpilog n) throws CompileException;
	public void visit(IMCastI2S n) throws CompileException;
	public void visit(IMIStoreLocalVariable n) throws CompileException;
	public void visit(IMDStoreLocalVariable n) throws CompileException;
	public void visit(IMIReadLocalVariable n) throws CompileException;
	public void visit(IMBitAnd n) throws CompileException;
	public void visit(IMStackOperationDUP_X2 n) throws CompileException;
	public void visit(IMStackOperationDUP2_X2 n) throws CompileException;
	public void visit(IMDRem n) throws CompileException;
	public void visit(IMLBitOr n) throws CompileException;
	public void visit(IMCastL2F n) throws CompileException;
	public void visit(IMIShiftRight2 n) throws CompileException;
	public void visit(IMLStoreLocalVariable n) throws CompileException;
	public void visit(IMLNeg n) throws CompileException;
	public void visit(IMCReadArray n) throws CompileException;
	public void visit(IMCastI2C n) throws CompileException;
	public void visit(IMIDiv n) throws CompileException;
	public void visit(IMCStoreArray n) throws CompileException;
	public void visit(IMFReadLocalVariable n) throws CompileException;
	public void visit(IMLEConditionalBranch n) throws CompileException;
	public void visit(IMNewArray n) throws CompileException;
	public void visit(IMEQConditionalBranch n) throws CompileException;
	public void visit(IMIStoreArray n) throws CompileException;
	public void visit(IMIConstant n) throws CompileException;
	public void visit(IMFNeg n) throws CompileException;
	public void visit(IMSub n) throws CompileException;
	public void visit(IMInstanceOf n) throws CompileException;
	public void visit(IMAReadLocalVariable n) throws CompileException;
	public void visit(IMInvokeStatic n) throws CompileException;
	public void visit(IMCastI2L n) throws CompileException;
	public void visit(IMGTConditionalBranch n) throws CompileException;
	public void visit(IMFStoreLocalVariable n) throws CompileException;
	public void visit(IMCastI2D n) throws CompileException;
	public void visit(IMLStoreArray n) throws CompileException;
	public void visit(IMCastD2I n) throws CompileException;
	public void visit(IMSStoreArray n) throws CompileException;
	public void visit(IMIBitXor n) throws CompileException;
	public void visit(IMLBitXor n) throws CompileException;
	public void visit(IMLShiftRight n) throws CompileException;
	public void visit(IMNeg n) throws CompileException;
	public void visit(IMLShiftLeft n) throws CompileException;
	public void visit(IMShiftRight n) throws CompileException;
	public void visit(IMINeg n) throws CompileException;
	public void visit(IMStackOperationDUP2 n) throws CompileException;
	public void visit(IMCheckCast n) throws CompileException;
	public void visit(IMConstant n) throws CompileException;
	public void visit(IMShiftRight2 n) throws CompileException;
	public void visit(IMDGCompare n) throws CompileException;
	public void visit(IMIBitOr n) throws CompileException;
	public void visit(IMDStoreArray n) throws CompileException;
	public void visit(IMDMul n) throws CompileException;
	public void visit(IMDNeg n) throws CompileException;
	public void visit(IMIRem n) throws CompileException;
	public void visit(IMPutStatic n) throws CompileException;
	public void visit(IMCastI2F n) throws CompileException;
	public void visit(IMCastD2L n) throws CompileException;
	public void visit(IMIMul n) throws CompileException;
	public void visit(IMCall n) throws CompileException;
	public void visit(IMCastF2D n) throws CompileException;
	public void visit(IMAObjConstant n) throws CompileException;
	public void visit(IMLConstant n) throws CompileException;
	public void visit(IMFConstant n) throws CompileException;
	public void visit(IMAReadArray n) throws CompileException;
	public void visit(IMReadLocalVariable n) throws CompileException;
	public void visit(IMDConstant n) throws CompileException;
	public void visit(IMFSub n) throws CompileException;
	public void visit(IMNewMultiArray n) throws CompileException;
	public void visit(IMThrow n) throws CompileException;
	public void visit(IMInvokeSpecial n) throws CompileException;
	public void visit(IMCastL2I n) throws CompileException;
	public void visit(IMGetField n) throws CompileException;
	public void visit(IMInvokeInterface n) throws CompileException;
	public void visit(IMAStoreArray n) throws CompileException;
	public void visit(IMStackOperationDUP n) throws CompileException;
	public void visit(IMReadArray n) throws CompileException;
	public void visit(IMLookupSwitch n) throws CompileException;
	public void visit(IMNewObjArray n) throws CompileException;
	public void visit(IMDSub n) throws CompileException;
	public void visit(IMIReadArray n) throws CompileException;
	public void visit(IMReturnSubroutine n) throws CompileException;
	public void visit(IMFStoreArray n) throws CompileException;
	public void visit(IMBReadArray n) throws CompileException;
	public void visit(IMDReadArray n) throws CompileException;
	public void visit(IMPutField n) throws CompileException;
	public void visit(IMAConstant n) throws CompileException;
	public void visit(IMNullConstant n) throws CompileException;
	public void visit(IMCompare n) throws CompileException;
	public void visit(IMBitOr n) throws CompileException;
	public void visit(IMSReadArray n) throws CompileException;
	public void visit(IMInvoke n) throws CompileException;
	public void visit(IMStackOperationDUP_X1 n) throws CompileException;
	public void visit(IMStoreArray n) throws CompileException;
	public void visit(IMPhi n) throws CompileException;
	public void visit(IMInvokeVirtual n) throws CompileException;
	public void visit(IMLReadLocalVariable n) throws CompileException;
	public void visit(IMStackOperationPOP n) throws CompileException;
	public void visit(IMDiv n) throws CompileException;
	public void visit(IMFDiv n) throws CompileException;
	public void visit(IMLMul n) throws CompileException;
	public void visit(IMFReadArray n) throws CompileException;
	public void visit(IMDDiv n) throws CompileException;
	public void visit(IMGetStatic n) throws CompileException;
	public void visit(IMTableSwitch n) throws CompileException;
	public void visit(IMAMTConstantStatic n) throws CompileException;
	public void visit(IMISub n) throws CompileException;
	public void visit(IMGEConditionalBranch n) throws CompileException;
	public void visit(IMLTConditionalBranch n) throws CompileException;
	public void visit(IMIBitAnd n) throws CompileException;
	public void visit(IMLDiv n) throws CompileException;
	public void visit(IMFLCompare n) throws CompileException;
	public void visit(IMAdd n) throws CompileException;
	public void visit(IMDLCompare n) throws CompileException;
	public void visit(IMFRem n) throws CompileException;
	public void visit(IMArrayLength n) throws CompileException;
	public void visit(IMDAdd n) throws CompileException;
	public void visit(IMIAdd n) throws CompileException;
	public void visit(IMLBitAnd n) throws CompileException;
	public void visit(IMLReadArray n) throws CompileException;
	public void visit(IMCastI2B n) throws CompileException;
	public void visit(IMCastF2L n) throws CompileException;
	public void visit(IMCast n) throws CompileException;
	public void visit(IMFMul n) throws CompileException;
	public void visit(IMIShiftLeft n) throws CompileException;
	public void visit(IMFGCompare n) throws CompileException;
	public void visit(IMMonitor n) throws CompileException;
	public void visit(IMAMemConstant n) throws CompileException;
	public void visit(IMLSub n) throws CompileException;
	public void visit(IMPopReturnAddr n) throws CompileException;
	public void visit(IMStackOperationPOP2 n) throws CompileException;
	public void visit(IMIShiftRight n) throws CompileException;
	public void visit(IMLAdd n) throws CompileException;
	public void visit(IMLCompare n) throws CompileException;
	public void visit(IMAMTConstant n) throws CompileException;
}
