/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;
import keso.compiler.config.*;
import keso.compiler.config.domain.*;

import java.util.*;

public class AlarmService extends KesoIndexedResource<AlarmDefinition> {

	public AlarmService(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] { "keso/core/AlarmService\\..+" };
	}

	public void require(int domainID, String className, String methodName) {
		repository.requireClass(domainID,"keso/core/Alarm");
		if (methodName.equals("getAlarmBase(Lkeso/core/Alarm;Lkeso/core/AlarmBase;)I"))
			repository.requireClass(domainID,"keso/core/AlarmBase");
	}

	public int checkAttribute(IMMethod self, int attr) {
		if (attr==ATTR_PURE) {
			if(self.termed("getAlarmByName(Ljava/lang/String;)Lkeso/core/Alarm;")
				|| self.termed("getAlarmBase(Lkeso/core/Alarm;Lkeso/core/AlarmBase;)I")
				|| self.termed("getAlarm(Lkeso/core/Alarm;)I")
			) {
				return TRUE;
			}
		}
		return FALSE;
	}

	public boolean affectMethod(IMClass clazz, IMMethod method, Coder coder) throws CompileException {
		this.clazz = clazz;
		this.coder = coder;
		this.callee = method;

		if (method.termed("getAlarmBase(Lkeso/core/Alarm;Lkeso/core/AlarmBase;)I")) {
			getAlarmBase(method);
		} else if (method.termed("getAlarm(Lkeso/core/Alarm;)I")) {
			getAlarm(method);
		} else if (method.termed("setRelAlarm(Lkeso/core/Alarm;II)I")) {
			retIgetAlarmRefmultInts("SetRelAlarm", 2,method);
		} else if (method.termed("setAbsAlarm(Lkeso/core/Alarm;II)I")) {
			retIgetAlarmRefmultInts("SetAbsAlarm", 2,method);
		} else if (method.termed("cancelAlarm(Lkeso/core/Alarm;)I")) {
			retIgetAlarmRefmultInts("CancelAlarm", 0,method);
		} else return false;
		return true;
	}

	public boolean affectInvokeStatic(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode args[], Coder coder) throws CompileException {
		this.node   = node;
		this.caller = caller;
		this.callee = callee;
		this.args   = args;
		this.coder  = coder;

		if(callee.termed("getAlarmByName(Ljava/lang/String;)Lkeso/core/Alarm;")) {
			getIndexedResourceByName();
		} else return false;

		return true;
	}

	private void getAlarm(IMMethod method) throws CompileException {
		IMClass alarm_class = repository.getClass("keso/core/Alarm");
		coder.add_class(alarm_class);

		coder.addln("TickType ticks;\n");
		coder.add("GetAlarm(");

		coder.add_getField(alarm_class, method, method.getMethodArgument(0).toString(), "alarm_id", "BGETFIELD");
		coder.addln(", &ticks);");
		coder.addln("return (jint) ticks;");
	}

	/**
	 * Add code to call an OSEK call fnname that gets an alarm ID as its first parameter
	 * and optionally no_params int paramters.
	 * The call may return anything as long as it is compatible with the return
	 * type of the KESO Java Service.
	 */
	private void retIgetAlarmRefmultInts(String fnname, int no_params, IMMethod method) throws CompileException {
		IMClass alarm_class = repository.getClass("keso/core/Alarm");
		coder.add_class(alarm_class);

		coder.add("return ");
		coder.add(fnname);
		coder.add("(");
		coder.add_getField(alarm_class, method, method.getMethodArgument(0).toString(), "alarm_id", "BGETFIELD");

		for(int i=0; i<no_params;i++) {
			coder.add(", i");
			coder.add(i+1);
		}

		coder.addln(");");
	}

	private void getAlarmBase(IMMethod method) throws CompileException {
		// param 0 is alarmID, param 1 is AlarmBase object
		IMClass alarm_class = repository.getClass("keso/core/Alarm");
		IMClass alarmbase_class = repository.getClass("keso/core/AlarmBase");

		coder.add_class(alarm_class);
		coder.add_class(alarmbase_class);

		coder.addln("AlarmBaseType abase;");
		coder.addln("StatusType retstatus;\n");

		coder.add("retstatus = GetAlarmBase(");
		coder.add_getField(alarm_class, method, method.getMethodArgument(0).toString(), "alarm_id", "BGETFIELD");
		coder.addln(", &abase);");

		coder.add_getField(alarmbase_class, method, method.getMethodArgument(1).toString(), "maxallowedvalue", "IGETFIELD");
		coder.addln(" = abase.maxallowedvalue;");
		// no need to check the reference again
		coder.add_getField_fast(alarmbase_class, method.getMethodArgument(1).toString(), "ticksperbase", "IGETFIELD");
		coder.addln(" = abase.ticksperbase;");
		coder.add_getField_fast(alarmbase_class, method.getMethodArgument(1).toString(), "mincycle", "IGETFIELD");
		coder.addln(" = abase.mincycle;");

		coder.addln("return retstatus;");
	}

	private Map<IMDomain, Map<String, AlarmDefinition>> localResources;

	protected Map<IMDomain, ? extends Map<String,? extends AlarmDefinition>> accessibleResources() throws CompileException {
		if(localResources == null) {
			localResources = new HashMap<IMDomain, Map<String, AlarmDefinition>>();

			for(IMDomain imd: opts.getDomains()) {
				JavaDomain dom = imd.getDef();
				Map<String,AlarmDefinition> lr = new HashMap<String, AlarmDefinition>();

				for(AlarmDefinition rd: dom.getAlarms())
					lr.put(rd.ident, rd);
				for(String resName: dom.getAlarmImports())
					lr.put(resName, opts.getSysDef().getSysAlarm(resName));
				localResources.put(imd, lr);
			}
		}
		return localResources;
	}

	protected String resCategory() { return "alarm"; }
	protected IMClass resClass(AlarmDefinition ad) {
		return repository.getClass("keso/core/Alarm");
	}
}
