/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.imcode.*;
import keso.compiler.*;
import keso.compiler.backend.*;

import java.util.*;

public class CMath extends Weavelet {
	private HashSet<String> nativeFunctions = new HashSet<String>();
	private HashMap<String,String> java2C_mappings = new HashMap<String,String>();

	public CMath(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] {
			"java/lang/Math\\..+D",
			"java/lang/Math\\..+F",
		};

		nativeFunctions.add("acos");
		nativeFunctions.add("asin");
		nativeFunctions.add("atan");  // atan and
		nativeFunctions.add("atan2"); // atan2 don't work in the Java variants
		nativeFunctions.add("log");

		if(opts.hasOption("use_mathlib_all")) {
			nativeFunctions.add("abs");
			java2C_mappings.put("abs","fabs");
			nativeFunctions.add("cos"); // does not pay off
			nativeFunctions.add("exp");
			nativeFunctions.add("floor");
			nativeFunctions.add("pow");
			nativeFunctions.add("rint");
			nativeFunctions.add("sin"); // does not pay off
			nativeFunctions.add("sqrt"); // on par with Java
			nativeFunctions.add("tan"); // does not pay off
		}
	}

	public boolean ignoreMethodBody(IMClass clazz, IMMethod method) throws CompileException {
		if(!opts.hasOption("use_mathlib"))
			return false;

		if(!nativeFunctions.contains(method.getMethodName()))
			return false;

		return true;
	}

	public boolean affectInvokeStatic(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode args[], Coder coder) throws CompileException
	{
		String mname = callee.getMethodName();

		if(!opts.hasOption("use_mathlib"))
			return false;

		if(!nativeFunctions.contains(mname))
			return false;

		if(caller.getClassName().equals("JMath"))
			return false;

		if(java2C_mappings.containsKey(mname))
			mname = java2C_mappings.get(mname);

		coder.add_class("math");
		coder.add(mname);
		coder.add("(");
		for(int i=0; i<args.length; i++) {
			if(i > 0)
				coder.add(",");
			args[i].translate(coder);
		}
		coder.add(")");

		// return value true tells the builder that this invoke has been handled
		return true;
	}
}
