/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;
import keso.compiler.config.domain.*;

public class CiAOIPStack extends Weavelet {

	private IMClass clazz;
	private Coder coder;
	private IMMethod callee;

	public CiAOIPStack(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] {
			"ciao/net/Interface\\.setIPv4Addr\\(II\\)V",
			"ciao/net/Interface\\.setIPv4Subnetmask\\(II\\)V",
			"ciao/net/Interface\\.setIPv4Up\\(IZ\\)V",

			"ciao/net/udp/Socket\\.<CLASS>",
			"ciao/net/udp/Socket\\.set_sport\\(I\\)V",
			"ciao/net/udp/Socket\\.set_dport\\(I\\)V",
			"ciao/net/udp/Socket\\.set_dst_ipv4_addr\\(I\\)V",
			"ciao/net/udp/Socket\\.bind\\(\\)Z",
			"ciao/net/udp/Socket\\.unbind\\(\\)V",
			"ciao/net/udp/Socket\\.send\\(\\[II\\)Z",
			"ciao/net/udp/Socket\\.send\\(\\[BI\\)Z",
			"ciao/net/udp/Socket\\.internal_read\\(Lciao/net/udp/UDPPacket;Lkeso/core/Memory;\\)Z",
			"ciao/net/udp/Socket\\.internal_free\\(Lciao/net/udp/UDPPacket;\\)V",
			"ciao/net/udp/Socket\\.<init>\\(\\)V",

			"ciao/net/udp/UDPPacket\\.<CLASS>",
			"ciao/net/udp/UDPPacket\\.get_length\\(\\)I",
		};
	}

	public boolean addFields(IMClass clazz, Coder coder, StringBuffer raw_fields) throws CompileException {
		if(clazz.getClassName().equals("ciao/net/udp/Socket")) {
			clazz.addRawField("socket", "IP::IPv4_UDP_Socket<>", "ipstack/api/IPv4_UDP_Socket.h");
			raw_fields.append("\tIP::IPv4_UDP_Socket<> _socket;\n\t");
			return true;
		}

		if(clazz.getClassName().equals("ciao/net/udp/UDPPacket")) {
			clazz.addRawField("packet", "ipstack::IPv4_Packet*", "ipstack/ipv4/IPv4.h");
			raw_fields.append("\tipstack::IPv4_Packet *_packet;\n\t");
			return true;
		}

		return false;
	}

	public boolean ignoreMethodBody(IMClass clazz, IMMethod method) throws CompileException {
		return true;
	}

	public boolean affectMethod(IMClass clazz, IMMethod method, Coder coder) throws CompileException {
		this.clazz = clazz;
		this.coder = coder;
		this.callee = method;

		if(clazz.getClassName().equals("ciao/net/Interface") &&
				( method.termed("setIPv4Addr(II)V") ||
					method.termed("setIPv4Subnetmask(II)V") ||
					method.termed("setIPv4Up(IZ)V")	) ) {

			coder.add_class("ipstack/api/Setup");

			coder.add("Interface* interface = IP::getInterface(");
			coder.add(method.getMethodArgument(0).toString());
			coder.addln(");");
			coder.addln("if(interface != NULL)");

			coder.add("\tinterface->");
			coder.add(method.getMethodName());
			coder.add("(");
			for(int i=1; i<method.getNumberOfArguments(); i++) {
				if(i>1) coder.add(", ");
				coder.add(method.getMethodArgument(i).toString());
			}
			coder.addln(");");

		} else if(clazz.getClassName().equals("ciao/net/udp/Socket") &&
				method.termed("internal_read(Lciao/net/udp/UDPPacket;Lkeso/core/Memory;)Z") ) {

			coder.add_class("ipstack/ipv4/IPv4");

			coder.addln("ipstack::IPv4_Packet* packet;");
			coder.addln("if ( " + method.getMethodArgument(1).toString() + " == NULL ) return 0;");

			coder.add("packet = ");
			coder.add(coder.getField(clazz, method.getMethodArgument(0).toString(), "socket"));
			coder.addln(".read();");

			coder.addln("if(packet == NULL) return 0;");

			// initialize _packet field to point to the CiAO IP Packet
			IMClass packet_class = repository.getClass("ciao/net/udp/UDPPacket");
			coder.add_class(packet_class);
			coder.add(coder.getField(packet_class, method.getMethodArgument(1).toString(), "packet"));
			coder.addln(" = packet;");

			// initialize Memory object to access the packet's payload
			IMClass mem_class = repository.getClass("keso/core/Memory");
			coder.add(coder.getField(mem_class, method.getMethodArgument(2).toString(), "addr"));
			coder.addln(" = (char*) packet->get_data() + 8;"); // UDP payload
			coder.add(coder.getField(mem_class, method.getMethodArgument(2).toString(), "size"));
			coder.addln(" = packet->get_total_len() - 8;"); // - 8 bytes UDP header size
			coder.addln("return 1;");

		} else {
			return false;
		}
		return true;
	}

	public boolean affectInvokeSpecial(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode obj, IMNode args[], Coder coder) throws CompileException {

		if(callee.getClassName().equals("ciao/net/udp/Socket") && callee.termed("<init>()V")) {
			coder.add("new( &");
			coder.add_getField(callee.getIMClass(), obj, "socket",
					node.hasToBeChecked());
			coder.add(") IP::IPv4_UDP_Socket<>");
			return true;
		} else return affectInvokeVirtual(node,caller,callee,obj,args,coder);
	}

	public boolean affectInvokeVirtual(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode obj, IMNode args[], Coder coder) throws CompileException {

		if(callee.getClassName().equals("ciao/net/udp/Socket")) {
			coder.add_class(callee.getIMClass());

			if(callee.termed("set_sport(I)V")
					|| callee.termed("set_dport(I)V")
					|| callee.termed("set_dst_ipv4_addr(I)V")
					|| callee.termed("bind()Z")
					|| callee.termed("unbind()V")) {

				coder.add_getField(callee.getIMClass(), obj, "socket",
						node.hasToBeChecked());
				coder.add(".");
				coder.add(callee.getMethodName());
				coder.add("(" );
				for(int i=0; i<args.length; i++) {
					if(i>0) coder.add(", ");
					args[i].translate(coder);
				}
				coder.add(")");

			} else if ( callee.termed("send([II)Z") ) {
				coder.add_getField(callee.getIMClass(), obj, "socket",
						node.hasToBeChecked());
				coder.add(".send( ((int_array_t*)");
				args[0].translate(coder);
				coder.add(")->data, ");
				args[1].translate(coder);
				coder.add("*sizeof(jint))");

			} else if ( callee.termed("send([BI)Z") ) {
				coder.add_getField(callee.getIMClass(), obj, "socket",
						node.hasToBeChecked());
				coder.add(".send( ((byte_array_t*)");
				args[0].translate(coder);
				coder.add(")->data, ");
				args[1].translate(coder);
				coder.add("*sizeof(jbyte))");

			} else if ( callee.termed("internal_free(Lciao/net/udp/UDPPacket;)V")) {
				IMClass packet_class = repository.getClass("ciao/net/udp/UDPPacket");

				coder.add_getField(callee.getIMClass(), obj, "socket",
						node.hasToBeChecked());
				coder.add(".free(");
				coder.add_getField(packet_class, args[0], "packet");
				coder.addln(")");

			} else return false;

		}	else if(callee.getClassName().equals("ciao/net/udp/UDPPacket")) {
			if(callee.termed("get_length()I")) {
				coder.add("(");
				coder.add_getField(callee.getIMClass(), obj, "packet", node.hasToBeChecked());
				coder.add("->get_total_len() - 8)");
			} else return false;
		} else return false;
		return true;
	}
}
