/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;

final public class DebugOutWeavelet extends Weavelet {

	private boolean caller_side = false;

	public DebugOutWeavelet(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		caller_side = opts.hasOption("dbgout_caller");
		joinPoints = new String[] {
			"keso/io/DebugOut\\.raw_print\\(Ljava/lang/StringBuffer;\\)V",
		};
	}

	/**
	 * This protect a method body from analyses and translation process.
	 */
	public boolean ignoreMethodBody(IMClass clazz, IMMethod method) throws CompileException
	{
		if (method.termed("raw_print(Ljava/lang/StringBuffer;)V")) return true;
		return false;
	}

	public void require(int domainID,String className, String methodName) {

	}

	public int checkAttribute(IMMethod method, int attr) throws CompileException {
		if (attr==ATTR_NOESCAPE) {
			/* The argument won't escape the method frame */
			if (method.termed("raw_print(Ljava/lang/StringBuffer;)V")) return TRUE;
		}
		return FALSE;
	}

	public int checkFieldAttribute(String classname, String fieldname, int attr) {
		if (attr==ATTR_DO_NOT_OMIT) {
			if (classname.equals("java/lang/StringBuffer")) {
				return TRUE;
			}
		}
		return FALSE;
	}
	/**
	 * This affect the method body.
	 */
	public boolean affectMethod(IMClass clazz, IMMethod method, Coder coder) throws CompileException {
		if (caller_side) return false;

		IMClass array_class = method.getIMClass("[C");
		if (method.termed("raw_print(Ljava/lang/StringBuffer;)V")) {

			IMClass string_class = method.getIMClass("java/lang/StringBuffer");

			coder.add_class(array_class.getAlias());
			coder.add_class(string_class.getAlias());

			coder.add(array_class.getClassTypeString());
			coder.add("* char_array = (");
			coder.add(array_class.getClassTypeString());
			coder.add("*)");
			coder.add_getField(string_class, method, method.getMethodArgument(0).toString(), "data", "LGETFIELD");
			coder.addln(";");

			coder.add("KESO_WRITE((jchar*) char_array->data, ");
			coder.add_getField_fast(string_class, method.getMethodArgument(0).toString(), "length", "IGETFIELD");
			coder.addln(");");

			coder.addln("return;");

			return true;
		}

		return false;
	}

	public boolean affectInvokeStatic(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode args[], Coder coder) throws CompileException {
		if (!caller_side) return false;

		IMClass array_class = caller.getIMClass("[C");

		if (callee.termed("raw_print(Ljava/lang/StringBuffer;)V")) {

			IMClass string_class = callee.getIMClass("java/lang/StringBuffer");

			coder.add_class(array_class.getAlias());
			coder.add_class(string_class.getAlias());

			coder.addln("{");
			coder.addln("jint len;");
			coder.add(string_class.getClassTypeString());
			coder.addln("* arr;");
			coder.add("arr = ");
			coder.add_getField(string_class, args[0], "data");
			coder.addln(";");
			coder.add("len = ");
			coder.add_getField(string_class, args[0], "length");
			coder.addln(";");

			coder.addln("KESO_WRITE((jchar*)arr->data, len);");

			coder.addln("}");

			return true;
		}

		return false;
	}
}
