/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;

public class DoubleWeavelet extends Weavelet {

	public DoubleWeavelet(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] {
			"java/lang/Double\\.toString\\(D\\)Ljava/lang/String;",
			"java/lang/Double\\.doubleToLongBits\\(D\\)J",
			"java/lang/Double\\.longBitsToDouble\\(J\\)D",
			"java/lang/Double\\.valueOf0\\(Ljava/lang/String;\\)D",
			"java/lang/Float\\.floatToIntBits\\(F\\)I",
			"java/lang/Float\\.intBitsToFloat\\(I\\)F",
		};
	}

	public void require(int domainID, String className, String methodName) {
		repository.requireClass(domainID, "[C");
		repository.requireMethod(domainID, "java/lang/String", "<init>([C)V");
	}

	public java.util.Set<IMMethod> keepMethods() {
		if (keep_methods.isEmpty() == false)
			return keep_methods;
		IMClass string = repository.getClass("java/lang/String");
		if (string == null)
			return keep_methods;
		IMMethod stringInit = string.getMethod("<init>([C)V");
		if (stringInit != null)
			keep_methods.add(stringInit);
		return keep_methods;
	}

	public boolean affectMethod(IMClass clazz, IMMethod method, Coder coder) throws CompileException {
		if (method.termed("toString(D)Ljava/lang/String;")) {
			IMClass string_class = method.getIMClass("java/lang/String");
			IMClass char_array_class = method.getIMClass("[C");

			coder.add_class("stdio");
			coder.add_class(string_class);
			coder.add_class(char_array_class);

			coder.addln("\tchar str[64];");
			if( coder instanceof CoderCiAO) {
				coder.addln("\tint size = 3;");
				coder.addln("\tstr[0] = 'N';");
				coder.addln("\tstr[1] = '/';");
				coder.addln("\tstr[2] = 'A';");
				coder.addln("\tstr[3] = '\\0';");
			} else {
				coder.addln("\tdouble dval = (double) " +method.getMethodArgument(0)+";");
				coder.addln("\tint size = snprintf(str, sizeof(str), \"%f\", dval);");
			}
			coder.add("\tobject_t *jstring = ");
			coder.add_allocObject(string_class);
			coder.addln(";");
			coder.addln("\tchar_array_t *ca = (char_array_t*) keso_alloc_char_array(size);");
			coder.addln("\twhile(size-->0) ca->data[size] = str[size];");

			IMMethod str_init = string_class.getMethod("<init>([C)V");
			coder.invoke(str_init, "jstring, (object_t*) ca");
			coder.addln("\treturn jstring;");

		} else if(method.termed("doubleToLongBits(D)J")) {
   		coder.addln("union  {");
			coder.addln("\tdouble dval;");
 			coder.addln("\tlong   lval;");
 			coder.addln("} convert;");
			coder.addln("convert.dval = " + method.getMethodArgument(0) +";");
			coder.addln("return convert.lval;");

		} else if(method.termed("longBitsToDouble(J)D")) {
   		coder.addln("union  {");
			coder.addln("\tdouble dval;");
 			coder.addln("\tlong   lval;");
 			coder.addln("} convert;");
			coder.addln("convert.lval = " + method.getMethodArgument(0) +";");
			coder.addln("return convert.dval;");

		} else if(method.termed("floatToIntBits(F)I")) {
   		coder.addln("union  {");
			coder.addln("\tjfloat fval;");
 			coder.addln("\tjint   ival;");
 			coder.addln("} convert;");
			coder.addln("convert.fval = " + method.getMethodArgument(0) +";");
			coder.addln("return convert.ival;");

		} else if(method.termed("intBitsToFloat(I)F")) {
   		coder.addln("union  {");
			coder.addln("\tjfloat fval;");
 			coder.addln("\tjint   ival;");
 			coder.addln("} convert;");
			coder.addln("convert.ival = " + method.getMethodArgument(0) +";");
			coder.addln("return convert.fval;");

		} else if(method.termed("valueOf0(Ljava/lang/String;)D")) {
			IMClass string_class = method.getIMClass("java/lang/String");
			IMClass char_array_class = method.getIMClass("[C");

			coder.add_class("stdlib");
			coder.add_class(string_class);
			coder.add_class(char_array_class);

			coder.addln("\tint i;");
			coder.add("\tchar_array_t *ca = (char_array_t*) ");
			coder.add_getField(string_class, method, method.getMethodArgument(0).toString(), "value", "AGETFIELD");
			coder.addln(";");
			coder.addln("\tchar tmp[ca->size + 1];");
			coder.addln("\tfor(i=0; i<=ca->size; i++) tmp[i] = (char) ca->data[i];");
			coder.addln("\ttmp[ca->size] = '\\0';");
			coder.addln("\tjdouble val= (jdouble) atof(tmp);");
			coder.addln("\treturn val;");

		} else return false;

		return true;
	}

	public boolean ignoreMethodBody(IMClass clazz, IMMethod method) throws CompileException {
		return true;
	}
}
