/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;

public class EventService extends Weavelet {

	private Coder coder;

	public EventService(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] { "keso/core/EventService\\..+" };
	}

	public void require(int domainID, String className, String methodName) {
		repository.requireClass(domainID,"keso/core/Events");
	}

	public int checkAttribute(IMMethod self, int attr) {
		if (attr==ATTR_BLOCK) {
			if (self.termed("waitEvent(I)I")) return TRUE;
		} else if (attr == ATTR_PURE) {
			if (self.termed("getEvent(Ljava/lang/Thread;)I")) {
				return TRUE;
			}
		}
		return FALSE;
	}

	public boolean affectMethod(IMClass clazz, IMMethod method, Coder coder) throws CompileException
	{
		this.coder = coder;

		if (method.termed("setEvent(Ljava/lang/Thread;I)I")) {
			getsetEvent("SetEvent", false,method);
			return true;
		} else if (method.termed("getEvent(Ljava/lang/Thread;)I")) {
			getsetEvent("GetEvent", true,method);
			return true;
		} else if (method.termed("waitEvent(I)I") && opts.getGlobalHeap().deferTasksOnDemand()) {
			coder.addln("StatusType s;");
			coder.addln("s = WaitEvent("+method.getMethodArgument(1).toString()+");");
			coder.addln("KESO_GC_DEFER_TASK();");
			coder.addln("return s;");
			return true;
		}

		return false;
	}

	public boolean affectInvokeStatic(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode args[], Coder coder) throws CompileException
	{
		if (callee.termed("clearEvent(I)I")) {
			coder.add("ClearEvent(");
			args[0].translate(coder);
			coder.add(")");
		} else if (callee.termed("waitEvent(I)I")) {
			caller.getMethodFrame().codeEOLL(coder);
			coder.add("WaitEvent(");
			args[0].translate(coder);
			coder.add(")");
		} else return false;

		// return value true tells the builder that this invoke has been handled
		return true;
	}

	private void getsetEvent(String fnname, boolean get, IMMethod method) throws CompileException {
		IMClass task_class = repository.getClass("java/lang/Thread");
		coder.add_class(task_class);

		if(get) coder.addln("EventMaskType ev;\n");
		else coder.add("return ");

		coder.add(fnname);
		coder.add("(");
		coder.add_getField(task_class, method, method.getMethodArgument(0).toString(), "task_id", "BGETFIELD");
		coder.add(", ");

		if(get) coder.add("&ev");
		else coder.add(method.getMethodArgument(1).toString());

		coder.addln(");");

		if(get) coder.addln("return ev;");
	}

	public boolean ignoreMethodBody(IMClass clazz, IMMethod method) throws CompileException {
		return true;
	}
}
