/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;
import keso.compiler.config.*;
import keso.compiler.config.domain.*;

public class InterruptService extends Weavelet {

	public InterruptService(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] { "keso/core/InterruptService\\..+" };
	}

	public int checkAttribute(IMMethod self, int attr) {
		if (attr == ATTR_PURE) {
			if (self.termed("getIRQLevel(Ljava/lang/String;)I")) {
				return TRUE;
			}
		}
		return FALSE;
	}

	// modifies a call to a static method
	public boolean affectInvokeStatic(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode args[], Coder coder) throws CompileException {
		if (callee.termed("disableAll()V")) {
			coder.add("DisableAllInterrupts()");
		} else if (callee.termed("enableAll()V")) {
			coder.add("EnableAllInterrupts()");
		} else if (callee.termed("suspendAll()V")) {
			coder.add("SuspendAllInterrupts()");
		} else if (callee.termed("resumeAll()V")) {
			coder.add("ResumeAllInterrupts()");
		} else if (callee.termed("suspendOS()V")) {
			coder.add("SuspendOSInterrupts()");
		} else if (callee.termed("resumeOS()V")) {
			coder.add("ResumeOSInterrupts()");
		} else if (callee.termed("getIRQLevel(Ljava/lang/String;)I")) {
			final String isrIdentifier = assert_string(args[0], "getIRQLevel(): first argument");
			String isrMakroName = null;

			for(DomainDefinition<?> ddef: opts.getSysDef().getAppDomains()) {
				ISRDefinition isr = ddef.getISR(isrIdentifier);
				if (isr != null) {
					isrMakroName = "RUNPRIO_ISR_" + isr.getIdentifier();
					break;
				}
			}
			if (isrMakroName == null)
				throw new CompileException("getIRQLevel(): no ISR found for identifier '" + isrIdentifier + "'");

			coder.add(isrMakroName);
		} else {
			return false;
		}

		return true;
	}

	public boolean ignoreMethodBody(IMClass clazz, IMMethod method) throws CompileException {
		return true;
	}
}
