/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;

public class MemoryTypes extends Weavelet {

	public MemoryTypes(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] {
			"keso/core/MT_U\\d+(RO)?\\..+",
			"keso/core/MT_U\\d+(RO)?\\.<CLASS>",
			"keso/core/MT_BOOLEAN\\..+",
			"keso/core/MT_BOOLEAN\\.<CLASS>",
			"keso/core/MT_FLOAT\\..+",
			"keso/core/MT_FLOAT\\.<CLASS>",
			"keso/core/MT_SPACE32\\..+",
			"keso/core/MT_SPACE32\\.<CLASS>",
		};
	}

	public int checkFieldAttribute(String classname, String fieldname, int attr) {
		if (attr==ATTR_DO_NOT_OMIT) {
			if (fieldname.equals("self")) return TRUE;
			return FALSE;
		}
		return NOTDEF;
	}

	public boolean ignoreMethodBody(IMClass clazz, IMMethod method)
		throws CompileException {
		return true;
	}

	public boolean affectMethod(IMClass clazz, IMMethod method, Coder coder) throws CompileException
	{
		/* Creates and returns a copy of this object. */
		if (method.termed("clone()Ljava/lang/Object;")) {
		}

		/* Called by the garbage collector on an object when garbage collection
		 * determines that there are no more references to the object. */
		if (method.termed("finalize()V")) {
			coder.addln("return;");
			return true;
		}

		/* Returns the runtime class of an object. */
		if (method.termed("getClass()Ljava/lang/Class;")) {
			opts.warn("getClass() not implemeted yet.");
			coder.addln("return NULL;");
			return true;
		}

		/* Wakes up a single thread that is waiting on this object's monitor. */
		if (method.termed("notify()V")) {
		}

		/* Wakes up all threads that are waiting on this object's monitor. */
		if (method.termed("notifyAll()V")) {
		}

		/* Causes current thread to wait until another thread invokes the
		 * notify() method or the notifyAll() method for this object. */
		if (method.termed("wait()V")) { }

		/* Causes current thread to wait until either another thread invokes the
		 * notify() method or the notifyAll() method for this object, or a
		 * specified amount of time has elapsed. */
		if (method.termed("wait(J)V")) {}

		/*
		   Causes current thread to wait until another thread invokes the
		   notify() method or the notifyAll() method for this object, or
		   some other thread interrupts the current thread, or a certain
		   amount of real time has elapsed.
		   */
		if (method.termed("wait(JI)V")) {}

		/* Returns a hash code value for the object. */
		if (method.termed("hashCode()I")) {
			coder.addln("/* we use the object address as default hash code */");
			coder.addln("return (jint) " + method.getMethodArgument(0).toString() + ";");
			return true;
		}

		/* Indicates whether some other object is "equal to" this one. */
		if (method.termed("equals")) {
			coder.addln("return (jint)("+ method.getMethodArgument(0).toString() + "==" + method.getMethodArgument(1).toString() +");");
			return true;
		}

		/*
		   String toString()
		   Returns a string representation of the object.
		   */

		return false;
	}

	public boolean affectInvokeVirtual(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode obj, IMNode args[], Coder coder) throws CompileException {

		if (callee.termed("set")) {
			if (obj.translate_MT_Ref(coder, "MPUTFIELD")) {
				coder.add('=');
				args[0].translate(coder);
				return true;
			}
			return false;
		}

		if (callee.termed("and")) {
			if (obj.translate_MT_Ref(coder, "MPUTFIELD")) {
				coder.add("&=");
				args[0].translate(coder);
				return true;
			}
			return false;
		}

		if (callee.termed("or")) {
			if (obj.translate_MT_Ref(coder, "MPUTFIELD")) {
				coder.add("|=");
				args[0].translate(coder);
				return true;
			}
			return false;
		}

		if (callee.termed("xor")) {
			if (obj.translate_MT_Ref(coder, "MPUTFIELD")) {
				coder.add("^=");
				args[0].translate(coder);
				return true;
			}
			return false;
		}

		if (callee.termed("setBit")) {
			if (obj.translate_MT_Ref(coder, "MPUTFIELD")) {
				coder.add(" |= (1 << ");
				args[0].translate(coder);
				coder.add(")");
				return true;
			}
			return false;
		}

		if (callee.termed("clearBit")) {
			if (obj.translate_MT_Ref(coder, "MPUTFIELD")) {
				coder.add(" &= ~(1 << ");
				args[0].translate(coder);
				coder.add(")");
				return true;
			}
			return false;
		}

		if (callee.termed("isBitSet")) {
			coder.add("((");
			if (obj.translate_MT_Ref(coder, "MPUTFIELD")) {
				coder.add(" & (1 <<");
				args[0].translate(coder);
				coder.add(")) != 0)");
				return true;
			}
			return false;
		}

		if (callee.termed("isBitClear")) {
			coder.add("((");
			if (obj.translate_MT_Ref(coder, "MPUTFIELD")) {
				coder.add(" & (1 <<");
				args[0].translate(coder);
				coder.add(")) == 0)");
				return true;
			}
			return false;
		}

		if (callee.termed("get")) {
			if (obj.translate_MT_Ref(coder, "MGETFIELD")) return true;
		}

		return false;
	}
}
