/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;
import keso.compiler.config.domain.*;
import keso.compiler.config.messageport.*;

public class MessagePorts extends Weavelet {

	private IMClass clazz;
	private Coder coder;
	private IMMethod callee;

	public MessagePorts(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] {
			"ciao/messageports/MemoryReceiverPort_.+\\..+",
			"ciao/messageports/MemoryReceiverPort_.+\\.<CLASS>",
			"ciao/messageports/MemorySenderPort_.+\\..+",
			"ciao/messageports/MemorySenderPort_.+\\.<CLASS>",
		};
	}

	public void require(int domainID, String className, String methodName) {
		repository.requireClass(domainID, "keso/core/Memory");

		/* why would we require these classes here, they should be used in the
		 * normal code and therefore be auto required?!

		for(JavaDomain jd: opts.getSysDef().getJavaDomains()) {
			for(ReceiverPortDefinition rpd: jd.getReceiverPorts()) {
				repository.requireClass(domainID, "ciao/messageports/MemoryReceiverPort_" + rpd.ident);
			}
			for(SenderPortDefinition spd: jd.getSenderPorts()) {
				repository.requireClass(domainID, "ciao/messageports/MemorySenderPort_" + spd.ident);
			}
		}
		*/
	}

	public boolean addFields(IMClass clazz, Coder coder, StringBuffer raw_fields) throws CompileException {
		coder.add_class(memory_class);

		if(clazz.getClassName().startsWith("ciao/messageports/MemoryReceiverPort_")) {
			clazz.addRawField("recvport", "os::krn::ReceiverPortTypes::"+ portName +"*", "messageport_objects.h");
			raw_fields.append("\tos::krn::ReceiverPortTypes::"+ portName +"*_rp;\n\t");
		} else {
			clazz.addRawField("sndrport", "os::krn::SenderPortTypes::"+ portName + " *", "messageport_objects.h");
			raw_fields.append("\tos::krn::SenderPortTypes::" + portName+ "*_sp;\n\t");
		}
		return true;
	}

	private IMClass memory_class;
	private String portName;
	private boolean isReceiver;
	private ReceiverPortDefinition rpd;
	private SenderPortDefinition spd;

	private void prepareWeavelet(String classname) throws CompileException {
		if(classname.startsWith("ciao/messageports/MemoryReceiverPort_")) {
			portName = classname.substring(37);
			isReceiver = true;
		} else if (classname.startsWith("ciao/messageports/MemorySenderPort_")) {
			portName = classname.substring(35);
			isReceiver = false;
		}
		rpd = opts.getSysDef().getReceiverPort(portName);
		if(rpd == null)
			throw new CompileException("ReceiverPort " + portName + " found in code base but not in configuration file");
		spd = rpd.getSenderPort();
		memory_class = repository.getClass("keso/core/Memory");
	}

	public boolean affectMethod(IMClass clazz, IMMethod method, Coder coder) throws CompileException {
		this.clazz = clazz;
		this.coder = coder;
		this.callee = method;
		prepareWeavelet(clazz.getClassName());

		if(isReceiver) {
			if(method.termed("receiveMessage()Lkeso/core/Memory;")) {
				requestOperation("rp","receiveMessage");
			} else if(method.termed("releaseMessage()V")) {
				releaseOperation("rp","releaseMessage");
			} else if(method.termed("lookup()L" + clazz.getClassName() + ";")) {
				lookup("receiver", rpd.parent);
			} else if(method.termed("peek()Z")) {
				coder.add("return ");
				coder.add_getField_fast(clazz, method.getMethodArgument(0).toString(), "rp", "AGETFIELD");
				coder.addln("->peek();");
			} else {
				return false;
			}
		} else {
			if(method.termed("lookup()L" + clazz.getClassName() + ";")) {
				lookup("sender", spd.parent);
			} else if(method.termed("allocateMessage()Lkeso/core/Memory;")) {
				requestOperation("sp","allocateMessage");
			} else if(method.termed("sendMessage()V")) {
				releaseOperation("sp","sendMessage");
			} else {
				return false;
			}
		}
		return true;
	}

	public boolean ignoreMethodBody(IMClass clazz, IMMethod method) throws CompileException {
		return true;
	}

	private void requestOperation(String portType, String opname) throws CompileException {
		coder.add_class(memory_class);

		coder.add(memory_class.getClassTypeString());
		coder.addln(" *_window;");

		coder.add("void *msg = ");
		coder.add_getField_fast(clazz,callee.getMethodArgument(0).toString(), portType, "AGETFIELD");
		coder.addln("->"+opname+"();");

		coder.addln("if(msg == NULL) return NULL;");

		coder.add("_window = (");
		coder.add(memory_class.getClassTypeString());
		coder.add("*) ");
		coder.add_getField_fast(clazz,callee.getMethodArgument(0).toString(), "_msgwindow", "AGETFIELD");
		coder.addln(";");
		coder.addln("_window->_addr = (void*) msg;");
		coder.add("_window->_size = sizeof(");
		coder.add(spd.getMessageType());
		coder.addln(");");
		coder.addln("return (object_t*) _window;");
	}

	private void releaseOperation(String portType, String opname) throws CompileException {
		coder.add_class(memory_class);

		coder.add(memory_class.getClassTypeString());
		coder.add(" *_window = (");
		coder.add(memory_class.getClassTypeString());
		coder.add("*) ");
		coder.add_getField_fast(clazz,callee.getMethodArgument(0).toString(), "_msgwindow", "AGETFIELD");
		coder.addln(";");

		coder.add_getField_fast(clazz,callee.getMethodArgument(0).toString(), portType, "AGETFIELD");
		coder.add("->"+opname+"((");
		coder.add(spd.getMessageType());
		coder.addln(" *) _window->_addr);");

		coder.addln("_window->_addr = 0;");
		coder.addln("_window->_size = 0;");
	}

	private void lookup(String portType, DomainDefinition<?> dom) throws CompileException {
		if(dom instanceof NativeDomain) {
			opts.warn(portType + " messageport " + portName + " not accessible from a Java Domain");
			coder.addln("\treturn NULL;");
			return;
		}
		
		String prefix = coder.getDomainSymbolPrefix(dom) + "msgport_"+portType;
		String window = prefix+"_window_"+portName;
		String proxy = prefix+"_proxy_"+portName;

		coder.local_add("static ");
		coder.local_add(memory_class.getClassTypeString());
		coder.local_add(" ALIGN4 ");
		coder.local_add(window);
		coder.local_add(" = { \n");
		coder.local_add(coder.add_initObject(memory_class,true));
		coder.local_add("\t0 /* addr */, 0 /* size */,\n};\n");

		coder.add("\treturn (KESO_CURRENT_DOMAIN != ");
		coder.add(coder.getDDescPtr(dom));
		coder.add(") ? NULL : ");
		coder.add_allocConstObject(false,clazz, proxy, "(object_t*) &"+window+", ");
		coder.add_init_field(null, "_rp", "&"+portName+"_"+portType);
		coder.add_init_end();
		coder.addln(";");
	}
}

