/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;
import keso.compiler.config.*;
import keso.compiler.config.domain.*;

import java.util.*;

public class PortalServiceWeavelet extends KesoIndexedResource<ImportDefinition> {

	public PortalServiceWeavelet(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] { "keso/core/PortalService\\..+" };
	}

	public int checkAttribute(IMMethod method, int attr) {
		if (method.termed("keso/core/handlePackets(Ljava/lang/String;Ljava/lang/String;)V")) {
			if (attr==ATTR_BLOCK) return TRUE;
		} else if(method.termed("getService(Ljava/lang/String;)Lkeso/core/Service;")) {
			if (attr==ATTR_OMIT_METHOD) return TRUE;
			if (attr==ATTR_PURE) return TRUE;
		} else if(method.termed("lookup(Ljava/lang/String;)Lkeso/core/Portal;")) {
			if (attr==ATTR_OMIT_METHOD) return TRUE;
			if (attr==ATTR_PURE) return TRUE;
		}
		return FALSE;
	}

	public boolean affectInvokeStatic(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode args[], Coder coder) throws CompileException {

		this.node   = node;
		this.caller = caller;
		this.callee = callee;
		this.args   = args;
		this.coder  = coder;

		if (callee.termed("getService(Ljava/lang/String;)Lkeso/core/Service;")) {
			String srv_name = assert_string(args[0],"Service name");
			ServiceDefinition srv = ServiceManager.instance.lookupService(srv_name);

			coder.add("(KESO_CURRENT_DOMAIN==");
			coder.add(coder.getDDescPtr(srv.parent));
			coder.add("? (");
			coder.add(IMPortalService.getGetMacro(srv_name));
			coder.add(") : NULL)");
			return true;

		} else if (callee.termed("lookup(Ljava/lang/String;)Lkeso/core/Portal;")) {
			getIndexedResourceByName();
			return true;

		} else if (callee.termed("handlePackets(Ljava/lang/String;Ljava/lang/String;)V")) {
			String srv_name = assert_string(args[0],"Service name");
			String net_name = assert_string(args[1],"Network");
			ServiceDefinition srv = ServiceManager.instance.lookupService(srv_name);
			NetworkDefinition net = ServiceManager.instance.lookupNetwork(net_name);

			if (srv==null || net==null) {
				opts.warn("no network of service defined! in "+caller.getIMClass().getSourceFile());
				coder.add("/* no service found */");
				return true;
			}
			coder.add(IMPortalService.getPacketHandlerFunction(srv, net));
			coder.add("()");
			return true;
		}

		return false;
	}

	private Map<IMDomain, Map<String, ImportDefinition>> localResources;

	protected Map<IMDomain, ? extends Map<String,? extends ImportDefinition>> accessibleResources() throws CompileException {
		if(localResources == null) {
			localResources = new HashMap<IMDomain, Map<String, ImportDefinition>>();

			for(IMDomain imd: opts.getDomains()) {
				JavaDomain dom = imd.getDef();
				Map<String,ImportDefinition> lr = new HashMap<String, ImportDefinition>();

				for(ImportDefinition rd: dom.getImports())
					lr.put(rd.ident, rd);
				localResources.put(imd, lr);
			}
		}
		return localResources;
	}

	protected String resCategory() { return "portal"; }

	protected IMClass resClass(ImportDefinition id) {
		return repository.getClass(id.getResourceClass());
	}
}
