/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;
import keso.compiler.config.*;
import keso.compiler.config.domain.*;

import java.util.*;

public class ResourceService extends KesoIndexedResource<ResourceDefinition> {

	public ResourceService(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] { "keso/core/ResourceService\\..+" };
	}

	public void require(int domainID, String className, String methodName) {
		repository.requireClass(domainID, "keso/core/Resource");
	}

	public int checkAttribute(IMMethod self, int attr) {
		if (attr==ATTR_PURE) {
			if(self.termed("getResourceByName(Ljava/lang/String;)Lkeso/core/Resource;")) {
				return TRUE;
			}
		}
		return FALSE;
	}

	public boolean affectMethod(IMClass clazz, IMMethod method, Coder coder) throws CompileException {
		this.coder = coder;
		this.clazz = clazz;
		this.callee = method;

		if (method.termed("getResource(Lkeso/core/Resource;)I")) {
			retIgetResRef("GetResource", "calls OSEK GetResource", "keso/core/Resource", "resource_id",method);
		} else if (method.termed("releaseResource(Lkeso/core/Resource;)I")) {
			retIgetResRef("ReleaseResource", "calls OSEK ReleaseResource", "keso/core/Resource", "resource_id",method);
		} else return false;

		return true;
	}

	public boolean affectInvokeStatic(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode args[], Coder coder) throws CompileException {
		this.node   = node;
		this.caller = caller;
		this.callee = callee;
		this.args   = args;
		this.coder  = coder;

		if(callee.termed("getResourceByName(Ljava/lang/String;)Lkeso/core/Resource;")) {
			getIndexedResourceByName();
		} else if(callee.termed("getScheduler()I")) {
			coder.add("GetResource(RES_SCHEDULER)");
		} else if(callee.termed("releaseScheduler()I")) {
			coder.add("ReleaseResource(RES_SCHEDULER)");
		} else return false;

		return true;
	}

	private Map<IMDomain, Map<String, ResourceDefinition>> localResources;

	protected Map<IMDomain, ? extends Map<String,? extends ResourceDefinition>> accessibleResources() throws CompileException {
		if(localResources == null) {
			localResources = new HashMap<IMDomain, Map<String, ResourceDefinition>>();

			for(IMDomain imd: opts.getDomains()) {
				JavaDomain dom = imd.getDef();
				Map<String,ResourceDefinition> lr = new HashMap<String, ResourceDefinition>();

				for(ResourceDefinition rd: dom.getResources())
					lr.put(rd.ident, rd);
				for(String resName: dom.getResourceImports())
					lr.put(resName, opts.getSysDef().getSysResource(resName));
				localResources.put(imd, lr);
			}
		}
		return localResources;
	}

	public boolean ignoreMethodBody(IMClass clazz, IMMethod method) throws CompileException {
		return true;
	}

	protected String resCategory() { return "resource"; }
	protected IMClass resClass(ResourceDefinition rd) {
		return repository.getClass("keso/core/Resource");
	}
}
