/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;
import keso.compiler.config.*;
import keso.compiler.config.domain.*;
import java.util.*;

public class SharedMemory extends KesoIndexedResource<SharedMemoryDefinition> {

	public SharedMemory(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] { "ciao/sharedmemory/SharedMemoryService\\..+" };
	}

	public boolean affectInvokeStatic(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode args[], Coder coder) throws CompileException {
		this.node   = node;
		this.caller = caller;
		this.callee = callee;
		this.args   = args;
		this.coder  = coder;

		if(callee.termed("getSharedMemoryByName(Ljava/lang/String;)Lkeso/core/Memory;")) {
			getIndexedResourceByName();
			return true;
		}

		return false;
	}

	private Map<IMDomain, Map<String, SharedMemoryDefinition>> localResources;

	protected Map<IMDomain, ? extends Map<String,? extends SharedMemoryDefinition>> accessibleResources() throws CompileException {
		if(localResources == null) {
			localResources = new HashMap<IMDomain, Map<String, SharedMemoryDefinition>>();

			for(IMDomain imd: opts.getDomains()) {
				JavaDomain dom = imd.getDef();
				Map<String,SharedMemoryDefinition> lr = new HashMap<String, SharedMemoryDefinition>();

				for(SharedMemoryDefinition rd: dom.getSharedMemories())
					lr.put(rd.ident, rd);

				localResources.put(imd, lr);
			}
		}
		return localResources;
	}


	protected String resCategory() { return "SharedMemory"; }

	protected IMClass resClass(SharedMemoryDefinition sd) {
		return repository.getClass("keso/core/Memory");
	}
}
