/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.kni;

import keso.compiler.*;
import keso.compiler.imcode.*;
import keso.compiler.backend.*;


public class Task extends Weavelet {

	public Task(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] { "java/lang/Thread\\.<CLASS>" };
	}

	public boolean addFields(IMClass clazz, Coder coder, StringBuffer raw_fields) throws CompileException {
		/* XXX if you change the layout of a task object's fields, be sure to
		 * also adjust the initialization code. Otherwise, C++ code generation
		 * will break.
		 */
		if(!opts.isSingleDomainSystem()) {
			clazz.addRawField("domain", "struct DOMAINDESC *");
			raw_fields.append("\tstruct DOMAINDESC *KESO_CONST _domain;\n");
			if(opts.hasLocalPortalUse()) {
				clazz.addRawField("e_domain", "struct DOMAINDESC *");
				raw_fields.append("\tstruct DOMAINDESC *_e_domain;\n");
			}
		}

		if(opts.hasLinkedListOfLocalReferences()) {
			clazz.addRawField("stack", "keso_stack_t *");
			raw_fields.append("\tKESO_CONST keso_stack_t *_stack;\n");
		}

		// reference to the runnable; we use a raw field for this to hide it from
		// the garbage collector
		clazz.addRawField("user_object", "object_t *");
		clazz.addRawField("task_id", "TaskType");
		if(opts.hasOption("init_systemobjects_on_startup")) {
			raw_fields.append("\tobject_t* _user_object;\n");
			raw_fields.append("\tTaskType _task_id;\n");
		} else {
			raw_fields.append("\tobject_t* KESO_CONST _user_object;\n");
			raw_fields.append("\tKESO_CONST TaskType _task_id;\n");
		}

		return true;
	}
}
