/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/


package keso.compiler.kni;

import keso.compiler.imcode.*;
import keso.compiler.*;
import keso.compiler.backend.Coder;

public class Tricore extends Weavelet {

	public Tricore(BuilderOptions opts, ClassStore repository) {
		super(opts, repository);
		joinPoints = new String[] {
			"keso/driver/tricore/tc1796b/TricorePWR\\..+",
			"keso/driver/tricore/tc1796b/Tricore\\..+",
		};
	}

	// NOTE: opts.getTarget() is not valid in this method!
	public boolean ignoreMethodBody(IMClass clazz, IMMethod method) throws CompileException {
		if (
				method.termed("unlockSystemRegisters") ||
				method.termed("lockSystemRegisters")  ||
				method.termed("dsync") ||
				method.termed("isync") ) {

			return true;
		}

		return false;
	}

	public boolean affectInvokeStatic(IMInvoke node, IMMethod caller, IMMethod callee,
			IMNode args[], Coder coder) throws CompileException {

		if (callee.termed("unlockSystemRegisters")) {
			coder.add(coder.unlock_sysreg());

		} else if (callee.termed("lockSystemRegisters")) {
			coder.add(coder.lock_sysreg());

		} else if (callee.termed("isync")) {
			coder.add("__asm__(\"isync\")");

		} else if (callee.termed("dsync")) {
			coder.add("__asm__(\"dsync\")");

		} else return false;

		return true;
	}
}
