/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.transform;

import keso.compiler.*;
import keso.compiler.analysis.*;
import keso.compiler.imcode.*;

public final class CheckInsertion extends MethodPass {

	@Override
	public String getName() {
		return "Runtime-check insertion";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.hasOption("imcode_checks");
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		info.addDependency(ConstantFolding.class);
		info.addAntiDependency(FlowAnalysis.class);
		info.addInvalidated(DomTreeComputation.class);
		info.addInvalidated(LivenessAnalysis.class);
		info.setDoDump(opts.hasOption("dump_check_insertion"));
	}

	@Override
	public boolean runOnMethod(IMMethod method) throws CompileException {

		if (method.isEligibleForOptimization() == false)
			return false;

		for (IMBasicBlock nfBB : method) {

			IMBasicBlock bb = nfBB;
			IMNode prevRoot = null;

			for (IMNode root : bb) {

				for (IMNode node : IMCodeFlattener.getFlattenedCode(root)) {

					if (node.hasToBeChecked() == false)
						continue;

					// Split the block
					IMBasicBlock postBB = bb.split(prevRoot);
					postBB.markAsCopied();

					// Add the check at the end of the first block
					IMListHead checkList = new IMListHead();
					node.emitCheck(checkList, postBB);
					bb.appendBCList(checkList);

					// Re-add the current root at the beginning of the second block
					IMListHead rootList = new IMListHead();
					rootList.append(root);
					postBB.insertBCBefore(rootList);

					bb = postBB;
					prevRoot = null;
				}

				prevRoot = root;
			}
		}

		// Fix uses and definitions
		// This is necessary because they may have been copied and/or moved to
		// another basic block.
		for (IMBasicBlock bb : method) {
			bb.visitNodes(new IMDefUseUpdater(bb));
		}
		return true;
	}

}
