/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.transform;

import keso.compiler.*;
import keso.compiler.analysis.*;
import keso.compiler.imcode.*;

public final class ConstantFolding extends MethodPass {

	@Override
	public String getName() {
		return "Constant folding";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return !opts.hasOption("ssa_dbg");
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		info.addDependency(ConstantParameterPropagation.class);
		info.addDependency(DomTreeComputation.class);
		info.addDependency(SSAConstruction.class);
		info.addDependency(CallGraphAnalysis.class, true);
		info.addDependency(FieldAccessReset.class,  true);
		info.addAntiDependency(SSADeconstruction.class);
		info.addInvalidated(FlowAnalysis.class);
		info.addInvalidated(LivenessAnalysis.class);
		info.addInvalidated(SimpleReachabilityAnalysis.class);
		info.setDoLoop(true);
		info.setDoDump(opts.hasOption("dump_constfold"));
	}

	@Override
	public boolean runOnClass(IMClass imClass) throws CompileException {
		if (imClass instanceof IMArrayClass)
			return false;
		BuilderOptions opts = imClass.getOptions();
		opts.verbose("###### constant folding " + imClass.getClassName());
		opts.setCurrentClass(imClass.getClassName(), imClass.getClassTypeID(),
				"ssa");
		boolean modified = super.runOnClass(imClass);
		opts.setCurrentClass("xxx", 0, null);
		return modified;
	}

	@Override
	public boolean runOnMethod(IMMethod method) throws CompileException {
		if (method.isEligibleForOptimization() == false)
			return false;
		return method.doConstantFolding();
	}

}
