/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.transform;

import keso.compiler.*;
import keso.compiler.analysis.*;
import keso.compiler.imcode.*;

public final class ConstantParameterPropagation extends MethodPass {

	@Override
	public String getName() {
		return "Constant-parameter propagation";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.hasOption("const_arg_prop");
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		info.addDependency(CallGraphAnalysis.class);
		info.addDependency(MethodInlining.class);
		info.addAntiDependency(SSAConstruction.class);
	}

	@Override
	public boolean runOnMethod(IMMethod method) throws CompileException {
		if (method.isEligibleForOptimization() == false)
			return false;
		IMMethodFrame frame = method.getMethodFrame();
		method.insertBCBefore(frame.constant_parameter_propagation());
		return true;
	}

	@Override
	protected boolean skipClass(IMClass imClass) {
		return (imClass instanceof IMArrayClass);
	}

}
