/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.transform;

import keso.compiler.*;
import keso.compiler.analysis.*;
import keso.compiler.imcode.*;

/**
 * Unused-method removal pass.
 * Unlike DeadInstructionRemoval, this pass eliminates entire methods which have
 * been marked as unreachable.
 */
public final class UnusedMethodRemoval extends ClassPass {

	@Override
	public String getName() {
		return "Unused-method removal";
	}

	@Override
	public boolean isEnabled(BuilderOptions opts) {
		return opts.hasOption("whole_program_opt");
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		info.addDependency(FlowBasedOptimization.class);
		info.addAntiDependency(EscapeAnalysis.class);
		info.addAntiDependency(ExtendedEscapeAnalysis.class);
		info.addAntiDependency(SSADeconstruction.class);
		info.addInvalidated(CallGraphAnalysis.class);
		info.addInvalidated(VTableComputation.class);
	}

	@Override
	public boolean runOnClass(IMClass imClass) throws CompileException {
		return imClass.omitUnusedMethods();
	}

}
