/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.compiler.transform;

import keso.compiler.*;
import keso.compiler.analysis.*;

public final class VTableComputation extends Pass {

	private MethodTableFactory table;

	@Override
	public String getName() {
		return "VTable computation";
	}

	@Override
	public void getPassInfo(PassInfo info, BuilderOptions opts) {
		if (opts.hasOption("iface_no_omit_unused") == false)
			info.addDependency(CallGraphAnalysis.class);
		info.addDependency(ClassTypeInfoComputation.class);
		info.addDependency(VTableReset.class);
		info.addAntiDependency(MethodInlining.class);
		info.addAntiDependency(ConstantFolding.class);
		info.addInvalidated(VTableReset.class);
	}

	@Override
	public boolean runOnRepository(ClassStore repository)
			throws CompileException {
		table = new MethodTableFactory(repository.getOptions(), repository);
		table.computeTables();
		return true;
	}

	public MethodTableFactory getTable() {
		return table;
	}

}
