/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.test;

import java.util.*;
import keso.util.*;
import org.junit.*;
import static org.junit.Assert.*;

public class DirectedGraphTest {

	private DirectedGraph<String> graph = new DirectedGraph<String>();

	@Before
	public void setUp() {
		graph.clear();
		graph.addNode("A");
		graph.addNode("B");
		graph.addNode("C");
		graph.addEdge("B", "C");
		graph.addEdge("C", "A");
		graph.addEdge("C", "E");
		graph.addEdge("D", "E");
		graph.addEdge("D", "A");
		graph.addEdge("E", "A");
		graph.addNode("F");
	}

	@Test
	public void testSuccessorCount() {
		assertTrue(graph.getSuccessors("A").isEmpty());
		assertEquals(1, graph.getSuccessors("B").size());
		assertEquals(2, graph.getSuccessors("C").size());
		assertEquals(2, graph.getSuccessors("D").size());
		assertEquals(1, graph.getSuccessors("E").size());
		assertTrue(graph.getSuccessors("F").isEmpty());
	}

	@Test
	public void testTopSort() throws GraphException {
		List<String> sorted = graph.topSort();
		System.out.println(sorted);
		assertEquals(6, sorted.size());
		assertTrue(sorted.get(0).equals("B") || sorted.get(1).equals("B")
				|| sorted.get(2).equals("B"));
		assertTrue(sorted.get(0).equals("D") || sorted.get(1).equals("D")
				|| sorted.get(2).equals("D"));
		assertTrue(sorted.get(0).equals("F") || sorted.get(1).equals("F")
				|| sorted.get(2).equals("F"));
		assertEquals("C", sorted.get(3));
		assertEquals("E", sorted.get(4));
		assertEquals("A", sorted.get(5));
	}

	@Test(expected = GraphException.class)
	public void testTopSortCircular() throws GraphException {
		graph.addEdge("E", "B");
		graph.topSort();
	}

}
