/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.util;

final public class Bitmap {

	private int size;
	private int asize;
	private int bitmap[];

	public Bitmap(int size) {
		this.size = size;
		this.asize = (size+31)/32;
		this.bitmap = new int[asize];
	}

	public void mark(int pos) {
		if (pos<0 || pos>=size) throw new IndexOutOfBoundsException("pos="+pos+" size="+size);
		int p = pos / 32;
		int b = pos % 32;
		bitmap[p] |= (1<<b);
	}

	public boolean test(int pos) {
		if (pos<0 || pos>=size) throw new IndexOutOfBoundsException("pos="+pos+" size="+size);
		int p = pos / 32;
		int b = pos % 32;
		return ((bitmap[p] & (1<<b))!=0);
	}

	public void clear() {
		bitmap = new int[asize];
	}

	public int getSize() {
		return asize;
	}

	public int value(int pos) {
		return bitmap[pos];
	}

	public String emit_flat() {
		String ret="";
		for (int i=0;i<bitmap.length;i++) {
			ret+="0x"+Integer.toHexString(bitmap[i])+",";
		}
		return ret;
	}

	public String emit() {
		String ret = "{";
		for (int i=0;i<bitmap.length;i++) {
			if (i>0) ret+=",";
			ret+=bitmap[i];
		}
		ret+="}";
		return ret;
	}

	public boolean equals(Object obj) {
		if (obj==null || !(obj instanceof Bitmap)) return false;
		Bitmap bit = (Bitmap) obj;
		if (bit.bitmap.length!=bitmap.length) return false;
		for (int i=0;i<bitmap.length;i++)
			if (bit.bitmap[i]!=bitmap[i]) return false;
		return true;
	}

	public int hashCode() {
		int value=0;
		for (int i=0;i<bitmap.length;i++) value += bitmap[i];
		return value;
	}
}
