/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.util;

import java.util.*;

public final class MarkableStack<T> {

	private List<T> stack;
	private int ptr;
	private List<Integer> marks;
	private int mptr;

	public MarkableStack() {
		this(64);
	}

	public MarkableStack(int size) {
		stack = new ArrayList<T>(size);
	}

	public void push(T slot) {
		if (slot == null)
			throw new Error();
		if (ptr >= stack.size())
			stack.add(slot);
		else
			stack.set(ptr, slot);
		++ptr;
	}

	public T peek() {
		if (ptr <= 0)
			return null;
		return stack.get(ptr - 1);
	}

	public T pop() {
		if (ptr <= 0)
			return null;
		return stack.get(--ptr);
	}

	public void mark() {
		if (marks == null)
			marks = new ArrayList<Integer>(32);
		if (mptr >= marks.size())
			marks.add(ptr);
		else
			marks.set(mptr, ptr);
		++mptr;
	}

	public void reset() {
		ptr = marks.get(--mptr);
	}

	public void clear() {
		stack = new ArrayList<T>(stack.size());
		ptr   = 0;
		marks = null;
		mptr  = 0;
	}

	public String toString() {
		String ret = "[";
		for (int i = 0; i < ptr; ++i) {
			if (i > 0)
				ret += ", ";
			ret += stack.get(i);
		}
		ret += ']';
		return ret;
	}

}
