/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.util;

import java.util.*;

public abstract class Multimap<K, V> {

	private Map<K, Collection<V>> map = createMap();

	public int size() {
		return map.size();
	}

	public boolean isEmpty() {
		return map.isEmpty();
	}

	public boolean containsKey(Object key) {
		return map.containsKey(key);
	}

	public boolean contains(Object key, Object value) {
		return map.containsKey(key) && map.get(key).contains(value);
	}

	public Collection<V> get(Object key) {
		Collection<V> entries = map.get(key);
		if (entries == null)
			entries = Collections.emptyList();
		return Collections.unmodifiableCollection(entries);
	}

	public void add(K key, V value) {
		Collection<V> entries = map.get(key);
		if (entries == null)
			entries = createCollection();
		entries.add(value);
		map.put(key, entries);
	}

	public void addAll(K key, Collection<V> values) {
		for (V value : values) {
			add(key, value);
		}
	}

	public boolean remove(K key, V value) {
		Collection<V> entries = map.get(key);
		if (entries == null)
			return false;
		if (entries.remove(value)) {
			// if this was the last entry in this list, remove the list to prevent a key with no values from showing up
			// in {@code keySet}.
			if (entries.isEmpty()) {
				map.remove(key);
			}
			return true;
		}
		return false;
	}

	public Collection<V> removeAll(K key) {
		return map.remove(key);
	}

	public void clear() {
		map.clear();
	}

	public Set<K> keySet() {
		return map.keySet();
	}
	
	public Set<Map.Entry<K, Collection<V>>> entrySet() {
		return map.entrySet();
	}

	@Override
	public String toString() {
		return map.toString();
	}

	protected abstract Map<K, Collection<V>> createMap();

	protected abstract Collection<V> createCollection();

}
