/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.util;

import java.util.*;

/**
 * Single-element iterable class.
 * This is a utility class to get an iterable wrapper for a single object.
 * @param <E> Element type.
 */
public final class SingleElementIterable<E> implements Iterable<E> {

	private final class Itr implements Iterator<E> {

		boolean hasNext = true;

		@Override
		public boolean hasNext() {
			return hasNext;
		}

		@Override
		public E next() {
			if (hasNext() == false)
				throw new NoSuchElementException();
			hasNext = false;
			return value;
		}

		@Override
		public void remove() {
			throw new UnsupportedOperationException();
		}

	}

	private E value;

	public SingleElementIterable(E value) {
		this.value = value;
	}

	@Override
	public Iterator<E> iterator() {
		return new Itr();
	}

}
