/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.util;

import java.util.*;

public abstract class SplitKeyMap<K1, K2, V> {

	private Map<K1, Map<K2, V>> map = createLevel1Map();

	public int size() {
		int result = 0;
		for (Map<K2, V> subMap : map.values()) {
			result += subMap.size();
		}
		return result;
	}

	public boolean isEmpty() {
		for (Map<K2, V> subMap : map.values()) {
			if (subMap.isEmpty() == false)
				return false;
		}
		return true;
	}

	public boolean containsKey(Object key1, Object key2) {
		Map<K2, V> subMap = map.get(key1);
		if (subMap == null)
			return false;
		return subMap.containsKey(key2);
	}

	public boolean containsValue(Object value) {
		for (Map<K2, V> subMap : map.values()) {
			if (subMap.containsValue(value))
				return true;
		}
		return false;
	}

	public V get(Object key1, Object key2) {
		Map<K2, V> subMap = map.get(key1);
		if (subMap == null)
			return null;
		return subMap.get(key2);
	}

	public V put(K1 key1, K2 key2, V value) {
		Map<K2, V> subMap = map.get(key1);
		if (subMap == null) {
			subMap = createLevel2Map();
			map.put(key1, subMap);
		}
		return subMap.put(key2, value);
	}

	public V remove(Object key1, Object key2) {
		Map<K2, V> subMap = map.get(key1);
		if (subMap == null)
			return null;
		return subMap.remove(key2);
	}

	public void clear() {
		map.clear();
	}

	protected abstract Map<K1, Map<K2, V>> createLevel1Map();

	protected abstract Map<K2, V> createLevel2Map();

}
