/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.util;

import java.util.*;

public abstract class SplitKeyMultimap<K1, K2, V> {

	private Map<K1, Map<K2, Collection<V>>> map = createLevel1Map();

	public int size() {
		int result = 0;
		for (Map<K2, Collection<V>> subMap : map.values()) {
			for (Collection<V> collection : subMap.values()) {
				result += collection.size();
			}
		}
		return result;
	}

	public boolean isEmpty() {
		for (Map<K2, Collection<V>> subMap : map.values()) {
			for (Collection<V> collection : subMap.values()) {
				if (!collection.isEmpty()) {
					return false;
				}
			}
		}
		return true;
	}

	public boolean containsKey(K1 key1, K2 key2) {
		Map<K2, Collection<V>> subMap = map.get(key1);
		if (subMap == null) {
			return false;
		}
		return subMap.containsKey(key2);
	}

	public Collection<V> get(K1 key1, K2 key2) {
		Map<K2, Collection<V>> subMap = map.get(key1);
		if (subMap == null) {
			return null;
		}
		return subMap.get(key2);
	}

	public boolean put(K1 key1, K2 key2, V value) {
		Map<K2, Collection<V>> subMap = map.get(key1);
		if (subMap == null) {
			subMap = createLevel2Map();
			map.put(key1, subMap);
		}
		Collection<V> collection = subMap.get(key2);
		if (collection == null) {
			collection = createCollection();
			subMap.put(key2, collection);
		}
		return collection.add(value);
	}

	public boolean remove(K1 key1, K2 key2, V value) {
		Map<K2, Collection<V>> subMap = map.get(key1);
		if (subMap == null) {
			return false;
		}
		Collection<V> collection = subMap.get(key2);
		if (collection == null) {
			return false;
		}
		return collection.remove(value);
	}

	public void clear() {
		map.clear();
	}

	protected abstract Map<K1, Map<K2, Collection<V>>> createLevel1Map();
	protected abstract Map<K2, Collection<V>> createLevel2Map();
	protected abstract Collection<V> createCollection();
}
