/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package keso.util;

/**
 * Split-key set class.
 * A "split-key set" works like a normal set, with the exception that it does
 * not hold single values, but pairs of values.
 * @param <E1> First element type.
 * @param <E2> Second element type.
 */
public abstract class SplitKeySet<E1, E2> {

	private Multimap<E1, E2> map = createMap();

	public int size() {
		return map.size();
	}

	public boolean isEmpty() {
		return map.isEmpty();
	}

	public boolean contains(Object o1, Object o2) {
		return map.get(o1).contains(o2);
	}

	public void add(E1 e1, E2 e2) {
		if (contains(e1, e2) == false)
			map.add(e1, e2);
	}

	public boolean remove(E1 e1, E2 e2) {
		return map.remove(e1, e2);
	}

	public void clear() {
		map.clear();
	}

	protected abstract Multimap<E1, E2> createMap();

}
