/*KESO--CFILE--KESO*/
/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

#include "keso_types.h"

#ifndef KESO_OMIT_SAFECHECKS
#ifndef KESO_PRODUCTION
void keso_throw_index_out_of_bounds(const char* method, int bcpos) {
	KESO_PRINTF("index out of bounds exception (%s, BCP %d)\n", method, bcpos);
	ShutdownOS(E_NOT_OK);
}
#endif
#endif

/*KESO--HEADER--KESO*/

#ifdef KESO_OMIT_SAFECHECKS
#  define KESO_CHECK_ARRAY_NOTNULL(_obj_,_index_,_method_,_bcpos_, _chkid_)
#  define KESO_CHECK_ARRAY_KNOWNBOUNDS(_m_size_, _a_size_, _method_, _bcpos_, _chkid_)
#  define KESO_CHECK_BOUNDS(_clazz_, _mem_, _a_size_, _method_, _bcpos_, _chkid_)

#else

#ifdef KESO_ENABLE_CODED_REFS
/* bounds check for non-null array object */

#  define KESO_CHECK_ARRAY_NOTNULL(_obj_,_index_,_method_,_bcpos_, _chkid_) \
	KESO_COUNT_CHECK(_chkid_); \
	if (unlikely(((array_t*) (NATIVE_ADDR(_obj_)))->size <= (array_size_t) _index_)) \
		keso_throw_index_out_of_bounds(_method_, _bcpos_);
// normal refs
#else
/* bounds check for non-null array object */
#  define KESO_CHECK_ARRAY_NOTNULL(_obj_,_index_,_method_,_bcpos_, _chkid_) \
	KESO_COUNT_CHECK(_chkid_); \
	if (unlikely(((array_t*) _obj_)->size <= (array_size_t) _index_)) \
		keso_throw_index_out_of_bounds(_method_, _bcpos_);

// KESO_ENABLE_CODED_REFS
#endif

/* constant bounds check for known sizes */
#  define KESO_CHECK_ARRAY_KNOWNBOUNDS(_m_size_, _a_size_, _method_, _bcpos_, _chkid_)\
	KESO_COUNT_CHECK(_chkid_); \
	if (unlikely((array_size_t)(_a_size_)>=(array_size_t)(_m_size_)))\
		keso_throw_index_out_of_bounds(_method_,_bcpos_)

/* bounds check for non-null _mem_ memory object */
#  define KESO_CHECK_BOUNDS(_clazz_, _mem_, _a_size_, _method_, _bcpos_, _chkid_)\
	KESO_COUNT_CHECK(_chkid_); \
	if (unlikely((_a_size_)>=((_clazz_*)_mem_)->_size)) \
		keso_throw_index_out_of_bounds(_method_,_bcpos_)

#endif

#ifdef KESO_PRODUCTION
#define keso_throw_index_out_of_bounds(_method_,_bcpos_) keso_throw_error()
#else
void 	keso_throw_index_out_of_bounds(const char* method, int bcpos);
#endif

/********   instruction specific array check macros   ********/
#ifdef KESO_OMIT_CHECK_ARRAYLENGTH_ARRAY
	#define KESO_CHECK_ARRAYLENGTH_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_ARRAYLENGTH_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_ARRAYLENGTH_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_ARRAYLENGTH_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_ARRAYLENGTH_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_ARRAYLENGTH_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAYLENGTH_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_ARRAYLENGTH_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_LALOAD_ARRAY
	#define KESO_CHECK_LALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_LALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_LALOAD_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_LALOAD_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_LALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_LALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_LALOAD_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_LALOAD_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_LASTORE_ARRAY
	#define KESO_CHECK_LASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_LASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_LASTORE_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_LASTORE_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_LASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_LASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_LASTORE_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_LASTORE_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_BALOAD_ARRAY
	#define KESO_CHECK_BALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_BALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_BALOAD_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_BALOAD_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_BALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_BALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_BALOAD_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_BALOAD_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_BASTORE_ARRAY
	#define KESO_CHECK_BASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_BASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_BASTORE_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_BASTORE_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_BASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_BASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_BASTORE_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_BASTORE_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_CALOAD_ARRAY
	#define KESO_CHECK_CALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_CALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_CALOAD_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_CALOAD_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_CALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_CALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_CALOAD_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_CALOAD_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_CASTORE_ARRAY
	#define KESO_CHECK_CASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_CASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_CASTORE_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_CASTORE_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_CASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_CASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_CASTORE_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_CASTORE_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_DALOAD_ARRAY
	#define KESO_CHECK_DALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_DALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_DALOAD_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_DALOAD_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_DALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_DALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_DALOAD_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_DALOAD_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_DASTORE_ARRAY
	#define KESO_CHECK_DASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_DASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_DASTORE_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_DASTORE_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_DASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_DASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_DASTORE_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_DASTORE_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_FALOAD_ARRAY
	#define KESO_CHECK_FALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_FALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_FALOAD_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_FALOAD_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_FALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_FALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_FALOAD_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_FALOAD_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_FASTORE_ARRAY
	#define KESO_CHECK_FASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_FASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_FASTORE_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_FASTORE_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_FASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_FASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_FASTORE_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_FASTORE_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_IALOAD_ARRAY
	#define KESO_CHECK_IALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_IALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_IALOAD_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_IALOAD_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_IALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_IALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_IALOAD_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_IALOAD_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_IASTORE_ARRAY
	#define KESO_CHECK_IASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_IASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_IASTORE_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_IASTORE_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_IASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_IASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_IASTORE_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_IASTORE_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_JALOAD_ARRAY
	#define KESO_CHECK_JALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_JALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_JALOAD_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_JALOAD_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_JALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_JALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_JALOAD_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_JALOAD_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_JASTORE_ARRAY
	#define KESO_CHECK_JASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_JASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_JASTORE_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_JASTORE_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_JASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_JASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_JASTORE_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_JASTORE_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_SALOAD_ARRAY
	#define KESO_CHECK_SALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_SALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_SALOAD_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_SALOAD_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_SALOAD_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_SALOAD_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_SALOAD_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_SALOAD_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

#ifdef KESO_OMIT_CHECK_SASTORE_ARRAY
	#define KESO_CHECK_SASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_SASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_)
	#define KESO_CHECK_SASTORE_ARRAY_KNOWNBOUNDS(_bound_,_index_, _method_,_bcpos_, _chkid_)
#else

#define KESO_CHECK_SASTORE_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_KNOWNBOUNDS(_bound_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_SASTORE_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_ARRAY_NOTNULL(_arr_, _index_, _method_, _bcpos_, _chkid_)

#define KESO_CHECK_SASTORE_ARRAY(_arr_, _index_, _method_, _bcpos_, _chkid_) \
	KESO_CHECK_SASTORE_NULLPOINTER(_arr_, _method_, _bcpos_, _chkid_); \
	KESO_CHECK_SASTORE_ARRAY_NOTNULL(_arr_,_index_,_method_,_bcpos_, _chkid_);
#endif

