/*KESO--CFILE--KESO*/
/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
#include "keso_types.h"

#if !(KESO_OMIT_SAFECHECKS || KESO_OMIT_CHECK_CAST)

#ifdef KESO_ENABLE_CODED_REFS
object_t *keso_checkcast_parity(class_id_t class_id, class_id_t type_range, object_t* obj, unsigned int chkid) {
	register class_id_t obj_type;
	/*KESO_COUNT_CHECK(chkid);*/
	if (NATIVE_ADDR(obj)==NULL) return obj;

	obj_type=(NATIVE_ADDR(obj))->class_id;
	if (obj_type<class_id || obj_type>type_range)
		KESO_THROW_ERROR("checkcast");

	return obj;
}

#else
object_t *keso_checkcast(class_id_t class_id, class_id_t type_range, object_t* obj, unsigned int chkid) {
	register class_id_t obj_type;
	/*KESO_COUNT_CHECK(chkid);*/
	if (obj==NULL) return obj;

	obj_type=obj->class_id;
	if (obj_type<class_id || obj_type>type_range)
		KESO_THROW_ERROR("checkcast");

	return obj;
}
//KESO_ENABLE_CODED_REFS
#endif
#endif

/*KESO--HEADER--KESO*/

#ifdef KESO_ENABLE_CODED_REFS

#if KESO_OMIT_SAFECHECKS || KESO_OMIT_CHECK_CAST

// coded address should be used here, since the parity check is peformed after that
#define KESO_CHECK_CAST(_class_id_, _type_range_, _obj_, _chkid_) _obj
#define KESO_CHECK_CAST_FAST(_class_id_, _obj_, _chkid_) _obj

#else

object_t *keso_checkcast_parity(class_id_t class_id, class_id_t type_range, object_t* obj, unsigned int chkid);

#define KESO_CHECK_CAST_FAST(_class_id_, _obj_, _chkid_) \
	(object_t*)(_class_id_!=((object_t*)(NATIVE_ADDR(_obj_)))->class_id ? KESO_THROW_ERROR("checkcast"),_obj_ : _obj_)

#define KESO_CHECK_CAST(_class_id_, _type_range_, _obj_, _chkid_) \
	keso_checkcast_parity(_class_id_, _type_range_, _obj_, _chkid_)

#endif

//normal refs
#else 
#if KESO_OMIT_SAFECHECKS || KESO_OMIT_CHECK_CAST

#define KESO_CHECK_CAST(_class_id_, _type_range_, _obj_, _chkid_) _obj_
#define KESO_CHECK_CAST_FAST(_class_id_, _obj_, _chkid_) _obj_

#else

object_t *keso_checkcast(class_id_t class_id, class_id_t type_range, object_t* obj, unsigned int chkid);

#define KESO_CHECK_CAST_FAST(_class_id_, _obj_, _chkid_) \
	(object_t*)(_class_id_!=(_obj_)->class_id ? KESO_THROW_ERROR("checkcast"),_obj_ : _obj_)

#define KESO_CHECK_CAST(_class_id_, _type_range_, _obj_, _chkid_) \
	keso_checkcast(_class_id_, _type_range_, _obj_, _chkid_)

#endif
//KESO_ENABLE_CODED_REFS
#endif

