/*KESO--CFILE--KESO*/
/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
#include "keso_types.h"

#ifndef KESO_OMIT_SAFECHECKS
#ifndef KESO_PRODUCTION

void 	keso_throw_nullpointer(const char* method, int bcpos) {
	KESO_PRINTF("null pointer exception (%s, BCP %d)\n", method, bcpos);
	ShutdownOS(E_NOT_OK);
}

#endif
#endif

/*KESO--HEADER--KESO*/

#define KESO_ASSERT_NULLPOINTER(_obj,_method_,_bcpos_, _chkid_) KESO_ASSERT((_obj_ != (object_t*)0))

#ifdef KESO_OMIT_SAFECHECKS
#  define KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else

#ifdef KESO_ENABLE_CODED_REFS

#define KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) KESO_COUNT_CHECK(_chkid_); if (unlikely(((object_t*)0)==(NATIVE_ADDR(_obj_)))) keso_throw_nullpointer(_method_,_bcpos_);

#else 

#define KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) KESO_COUNT_CHECK(_chkid_); if (unlikely(((object_t*)0)==(_obj_))) keso_throw_nullpointer(_method_,_bcpos_);

// KESO_ENABLE_CODED_REFS
#endif

#ifdef KESO_PRODUCTION
#define keso_throw_nullpointer(_method_,_bcpos_) keso_throw_error()
#else
void 	keso_throw_nullpointer(const char* method, int bcpos);
#endif
#endif

/********   pseudo instruction specific null check macros   ********/
#ifndef KESO_OMIT_CHECK_MEMBOUNDS_NULLPOINTER
#define KESO_CHECK_MEMBOUNDS_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_MEMBOUNDS_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_MEMLOAD_NULLPOINTER
#define KESO_CHECK_MEMLOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_MEMLOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_MEMSTORE_NULLPOINTER
#define KESO_CHECK_MEMSTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_MEMSTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_MGETFIELD_NULLPOINTER
#define KESO_CHECK_MGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_MGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_MPUTFIELD_NULLPOINTER
#define KESO_CHECK_MPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_MPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

/********   instruction specific null check macros   ********/
#ifndef KESO_OMIT_CHECK_ARRAYLENGTH_NULLPOINTER
#define KESO_CHECK_ARRAYLENGTH_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_ARRAYLENGTH_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_LALOAD_NULLPOINTER
#define KESO_CHECK_LALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_LALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_LASTORE_NULLPOINTER
#define KESO_CHECK_LASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_LASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_AGETFIELD_NULLPOINTER
#define KESO_CHECK_AGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_AGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_APUTFIELD_NULLPOINTER
#define KESO_CHECK_APUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_APUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_BALOAD_NULLPOINTER
#define KESO_CHECK_BALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_BALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_BASTORE_NULLPOINTER
#define KESO_CHECK_BASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_BASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_BGETFIELD_NULLPOINTER
#define KESO_CHECK_BGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_BGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_BPUTFIELD_NULLPOINTER
#define KESO_CHECK_BPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_BPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_CALOAD_NULLPOINTER
#define KESO_CHECK_CALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_CALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_CASTORE_NULLPOINTER
#define KESO_CHECK_CASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_CASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_CGETFIELD_NULLPOINTER
#define KESO_CHECK_CGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_CGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_CPUTFIELD_NULLPOINTER
#define KESO_CHECK_CPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_CPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_DALOAD_NULLPOINTER
#define KESO_CHECK_DALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_DALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_DASTORE_NULLPOINTER
#define KESO_CHECK_DASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_DASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_DGETFIELD_NULLPOINTER
#define KESO_CHECK_DGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_DGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_DPUTFIELD_NULLPOINTER
#define KESO_CHECK_DPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_DPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_FALOAD_NULLPOINTER
#define KESO_CHECK_FALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_FALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_FASTORE_NULLPOINTER
#define KESO_CHECK_FASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_FASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_FGETFIELD_NULLPOINTER
#define KESO_CHECK_FGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_FGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_FPUTFIELD_NULLPOINTER
#define KESO_CHECK_FPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_FPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_JGETFIELD_NULLPOINTER
#define KESO_CHECK_JGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_JGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_JPUTFIELD_NULLPOINTER
#define KESO_CHECK_JPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_JPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_IALOAD_NULLPOINTER
#define KESO_CHECK_IALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_IALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_IASTORE_NULLPOINTER
#define KESO_CHECK_IASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_IASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_IGETFIELD_NULLPOINTER
#define KESO_CHECK_IGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_IGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_IPUTFIELD_NULLPOINTER
#define KESO_CHECK_IPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_IPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_ZGETFIELD_NULLPOINTER
#define KESO_CHECK_ZGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_ZGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_ZPUTFIELD_NULLPOINTER
#define KESO_CHECK_ZPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_ZPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_JALOAD_NULLPOINTER
#define KESO_CHECK_JALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_JALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_JASTORE_NULLPOINTER
#define KESO_CHECK_JASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_JASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_LGETFIELD_NULLPOINTER
#define KESO_CHECK_LGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_LGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_LPUTFIELD_NULLPOINTER
#define KESO_CHECK_LPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_LPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_SALOAD_NULLPOINTER
#define KESO_CHECK_SALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_SALOAD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_SASTORE_NULLPOINTER
#define KESO_CHECK_SASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_SASTORE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_SGETFIELD_NULLPOINTER
#define KESO_CHECK_SGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_SGETFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_SPUTFIELD_NULLPOINTER
#define KESO_CHECK_SPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_SPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_ZPUTFIELD_NULLPOINTER
#define KESO_CHECK_ZPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_ZPUTFIELD_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_INVOKEINTERFACE_NULLPOINTER
#define KESO_CHECK_INVOKEINTERFACE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_INVOKEINTERFACE_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_INVOKESPECIAL_NULLPOINTER
#define KESO_CHECK_INVOKESPECIAL_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_INVOKESPECIAL_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_INVOKEVIRTUAL_NULLPOINTER
#define KESO_CHECK_INVOKEVIRTUAL_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_INVOKEVIRTUAL_NULLPOINTER(_obj_,_method_,_bcpos_, _chkid_)
#endif


