/*KESO--CFILE--KESO*/
/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

#include "keso_types.h"

#ifndef KESO_OMIT_SAFECHECKS
#ifndef KESO_PRODUCTION
void  keso_throw_parity(const char* method, int bcpos, unsigned char parity, unsigned char parity_bit, object_t* obj) {
	KESO_PRINTF("parity check exception (%s, BCP %d, PARITY %d, PARITY_BIT %d, ADDRESS 0x%x)\n", method, bcpos, parity, parity_bit, obj);
	//ShutdownOS(E_NOT_OK);
}

#endif
#endif

#ifndef PARITY_CHECKER
#define PARITY_CHECKER

void parity_check(object_t* _obj, const char* _method_, int _bcpos_, int _chkid_) {
	unsigned char bit_cnt=0;
	unsigned char _parity_=0;
	unsigned char _parity_bit_=0;
	/* compute parity of a reference */
	bit_cnt=BITCOUNT((unsigned int)_obj);
	_parity_=bit_cnt;
	/* determine odd or even parity */
	bit_cnt=bit_cnt%2;
	/* extract and check parity bit from object reference */
	//_parity_bit_ = (_obj->gcinfo)&2;
	_parity_bit_ = ((unsigned int) (_obj)&2)>>1;
	if(unlikely(_parity_bit_)!=bit_cnt) {
		//KESO_PRINTF("FALSE 0x%x\t", _obj);
		keso_throw_parity(_method_, _bcpos_, _parity_, _parity_bit_, _obj);
	}
}

void set_parity(object_t** _obj) {
	unsigned char parity_bit=0;
	KESO_PRINTF("SETTING PARITY ON 0x%x\t", *_obj);
	parity_bit=BITCOUNT((unsigned int)(*_obj));
	/* set parity bit */
	if((parity_bit%2)==1)
		*_obj=(object_t*)((unsigned int)(*_obj)|2);
	KESO_PRINTF("PARITY IS NOW 0x%x\n", *_obj);
}
#endif

/*KESO--HEADER--KESO*/

#define NATIVE_ADDR(_obj) (object_t*)((unsigned int)(_obj)&~(1<<1))

#define BITCOUNT(x)     (((BX_(x)+(BX_(x)>>4)) & 0x0F0F0F0F) % 255)

#define BITCOUNT_32(x)     (((BX_32(x)+(BX_32(x)>>4)) & 0x0F0F0F0F) % 255)

#define  BX_(x)         ((x&0xfffffffd) - (((x)>>1)&0x77777776)   \
		- (((x)>>2)&0x33333333)        \
		- (((x)>>3)&0x11111111))

#define BX_32(x)         ((x) - (((x)>>2)&0x77777777)   \
		- (((x)>>2)&0x33333333)        \
		- (((x)>>3)&0x11111111))


#define KESO_CHECK_PARITY(_obj, _method_, _bcpos_, _chkid_) parity_check(_obj, _method_, _bcpos_, _chkid_)
#define SET_PARITY(_obj) set_parity((object_t**)(&_obj))

#ifdef KESO_PRODUCTION
#define keso_throw_parity(_method_,_bcpos_, _parity_) keso_throw_error()
#else
void 	keso_throw_parity(const char* method, int bcpos, unsigned char parity, unsigned char parity_bit, object_t* obj);
#endif

#ifdef KESO_PRODUCTION
#define keso_throw_parity(_method_,_bcpos_, _parity_) keso_throw_error()
#else
void parity_check(object_t* _obj, const char* _method_, int _bcpos_, int _chkid_);
void set_parity(object_t** _obj);
#endif



/********   instruction parity check macros   ********/
#ifndef KESO_OMIT_CHECK_ARRAYLENGTH_PARITY
#define KESO_CHECK_ARRAYLENGTH_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_ARRAYLENGTH_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_LALOAD_PARITY
#define KESO_CHECK_LALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_LALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_LASTORE_PARITY
#define KESO_CHECK_LASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_LASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_AGETFIELD_PARITY
#define KESO_CHECK_AGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_AGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_APUTFIELD_PARITY
#define KESO_CHECK_APUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_APUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_BALOAD_PARITY
#define KESO_CHECK_BALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_BALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_BASTORE_PARITY
#define KESO_CHECK_BASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_BASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_BGETFIELD_PARITY
#define KESO_CHECK_BGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_BGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_BPUTFIELD_PARITY
#define KESO_CHECK_BPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_BPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_CALOAD_PARITY
#define KESO_CHECK_CALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_CALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_CASTORE_PARITY
#define KESO_CHECK_CASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_CASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_CGETFIELD_PARITY
#define KESO_CHECK_CGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_CGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_CPUTFIELD_PARITY
#define KESO_CHECK_CPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_CPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_DALOAD_PARITY
#define KESO_CHECK_DALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_DALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_DASTORE_PARITY
#define KESO_CHECK_DASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_DASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_DGETFIELD_PARITY
#define KESO_CHECK_DGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_DGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_DPUTFIELD_PARITY
#define KESO_CHECK_DPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_DPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_FALOAD_PARITY
#define KESO_CHECK_FALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_FALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_FASTORE_PARITY
#define KESO_CHECK_FASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_FASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_FGETFIELD_PARITY
#define KESO_CHECK_FGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_FGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_FPUTFIELD_PARITY
#define KESO_CHECK_FPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_FPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_JGETFIELD_PARITY
#define KESO_CHECK_JGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_JGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_JPUTFIELD_PARITY
#define KESO_CHECK_JPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_JPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_IALOAD_PARITY
#define KESO_CHECK_IALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_IALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_IASTORE_PARITY
#define KESO_CHECK_IASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_IASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_IGETFIELD_PARITY
#define KESO_CHECK_IGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_IGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_IPUTFIELD_PARITY
#define KESO_CHECK_IPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_IPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_ZGETFIELD_PARITY
#define KESO_CHECK_ZGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_ZGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_ZPUTFIELD_PARITY
#define KESO_CHECK_ZPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_ZPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_JALOAD_PARITY
#define KESO_CHECK_JALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_JALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_JASTORE_PARITY
#define KESO_CHECK_JASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_JASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_LGETFIELD_PARITY
#define KESO_CHECK_LGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_LGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_LPUTFIELD_PARITY
#define KESO_CHECK_LPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_LPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_SALOAD_PARITY
#define KESO_CHECK_SALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_SALOAD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_SASTORE_PARITY
#define KESO_CHECK_SASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_SASTORE_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_SGETFIELD_PARITY
#define KESO_CHECK_SGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_SGETFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_SPUTFIELD_PARITY
#define KESO_CHECK_SPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_SPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_ZPUTFIELD_PARITY
#define KESO_CHECK_ZPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_ZPUTFIELD_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif

#ifndef KESO_OMIT_CHECK_INVOKEINTERFACE_PARITY
#define KESO_CHECK_INVOKEINTERFACE_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_INVOKEINTERFACE_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_INVOKESPECIAL_PARITY
#define KESO_CHECK_INVOKESPECIAL_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_INVOKESPECIAL_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif


#ifndef KESO_OMIT_CHECK_INVOKEVIRTUAL_PARITY
#define KESO_CHECK_INVOKEVIRTUAL_PARITY(_obj_,_method_,_bcpos_, _chkid_) \
	KESO_CHECK_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#else
#define KESO_CHECK_INVOKEVIRTUAL_PARITY(_obj_,_method_,_bcpos_, _chkid_)
#endif
