/*KESO--HEADER--KESO*/
/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

#define GC_LOG_INFO_PARAMETER
/* #define GC_LOG_ALLOC */

/*KESO--CFILE--KESO*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>

#define GC_LOG_INFO (0)

#ifdef GC_LOG_ALLOC
#define GC_LOG_ALLOC_EVENT(_tid_, _aid_, _class_id_, _freespace_, _size_) gc_log_alloc(_tid_, _aid_, _class_id_, _freespace_, _size_)
#else
#define GC_LOG_ALLOC_EVENT(_tid_, _aid_, _class_id_, _freespace_, _size_)
#endif

#define GC_LOG_ACTIVATION() gc_periode++
#define GC_LOG_START_EVENT(_did_, _freespace_) gc_log('s', _did_, _freespace_)
#define GC_LOG_MARK_END_EVENT(_did_, _freespace_) gc_log('m', _did_, _freespace_)
#define GC_LOG_SWEEP_END_EVENT(_did_, _freespace_) gc_log('e', _did_, _freespace_)
#define GC_LOG_OUT_OF_MEMORY(_did_, _freespace_) gc_log('o', _did_, _freespace_)

static FILE *lfd = 0;
static unsigned long long time_stamp_s =0;
static unsigned long total_alloc = 0;
static unsigned long gc_periode = 0;

static unsigned long rd_time() {
	unsigned long long cts;
	__asm__ __volatile__ ("rdtsc": "=A" (cts) );
	return (unsigned long)(cts>>10);
}

static void gc_log_init() {
	lfd = fopen("gc.log", "w");
	time_stamp_s = rd_time();
}

void gc_log_alloc(int tid, int aid, class_id_t class_id, unsigned int free_space, obj_size_t size) {
	unsigned long ts;

	if (lfd<0) return;
	if (!lfd) gc_log_init();

	ts = rd_time() - time_stamp_s;

	total_alloc += size;
	/* format "event:periode:time:total:free:size:domain id:thread id:position:class id" */
	fprintf(lfd, "a:%lu:%lu:%lu:%u:%u:", gc_periode, ts, total_alloc, free_space, size);
	fprintf(lfd, "0:%u:%u:%u\n", tid, aid, class_id);
}

void gc_log(char mark, int did, unsigned int free_space) {
	unsigned long ts;

	if (lfd<0) return;
	if (!lfd) gc_log_init();

	ts = rd_time() - time_stamp_s;

	/* format "event:periode:time:total:free:size:domain id:thread id:position:class id" */
	fprintf(lfd, "%c:%lu:%lu:%lu:%u:0:", mark, gc_periode, ts, total_alloc, free_space);
	fprintf(lfd, "%u:0:0:0\n", did);

	if (mark=='e') {
		fflush(lfd);
		fsync(fileno(lfd));
	}
}

