/*KESO--HEADER--KESO*/
/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
jfloat keso_fmodf(jfloat x, jfloat y) ATTR_CONST;

/*KESO--CFILE--KESO*/

#include <stdint.h>

/* A union which permits us to convert between a float and a 32 bit
   int.  */

typedef union
{
  float value;
  uint32_t word;
} ieee_float_shape_type;

/* Get a 32 bit int from a float.  */

#define GET_FLOAT_WORD(i,d)					\
do {								\
  ieee_float_shape_type gf_u;					\
  gf_u.value = (d);						\
  (i) = gf_u.word;						\
} while (0)

/* Set a float from a 32 bit int.  */

#define SET_FLOAT_WORD(d,i)					\
do {								\
  ieee_float_shape_type sf_u;					\
  sf_u.word = (i);						\
  (d) = sf_u.value;						\
} while (0)
static const float one_flt = 1.0, Zero_flt[] = {0.0, -0.0,};

#define FLT_UWORD_IS_ZERO(x) ((x)==0)
#define FLT_UWORD_IS_FINITE(x) ((x)<0x7f800000L)
#define FLT_UWORD_IS_NAN(x) ((x)>0x7f800000L)
#define FLT_UWORD_IS_SUBNORMAL(x) ((x)<0x00800000L)

jfloat keso_fmodf(jfloat x, jfloat y) {
	int32_t n,hx,hy,hz,ix,iy,sx,i;

	GET_FLOAT_WORD(hx,x);
	GET_FLOAT_WORD(hy,y);
	sx = hx&0x80000000;		/* sign of x */
	hx ^=sx;		/* |x| */
	hy &= 0x7fffffff;	/* |y| */

	/* purge off exception values */
	if(FLT_UWORD_IS_ZERO(hy)||
			!FLT_UWORD_IS_FINITE(hx)||
			FLT_UWORD_IS_NAN(hy))
		return (x*y)/(x*y);
	if(hx<hy) return x;			/* |x|<|y| return x */
	if(hx==hy)
		return Zero_flt[(uint32_t)sx>>31];	/* |x|=|y| return x*0*/

	/* Note: y cannot be zero if we reach here. */

	/* determine ix = ilogb(x) */
	if(FLT_UWORD_IS_SUBNORMAL(hx)) {	/* subnormal x */
		for (ix = -126,i=(hx<<8); i>0; i<<=1) ix -=1;
	} else ix = (hx>>23)-127;

	/* determine iy = ilogb(y) */
	if(FLT_UWORD_IS_SUBNORMAL(hy)) {	/* subnormal y */
		for (iy = -126,i=(hy<<8); i>=0; i<<=1) iy -=1;
	} else iy = (hy>>23)-127;

	/* set up {hx,lx}, {hy,ly} and align y to x */
	if(ix >= -126)
		hx = 0x00800000|(0x007fffff&hx);
	else {		/* subnormal x, shift x to normal */
		n = -126-ix;
		hx = hx<<n;
	}
	if(iy >= -126)
		hy = 0x00800000|(0x007fffff&hy);
	else {		/* subnormal y, shift y to normal */
		n = -126-iy;
		hy = hy<<n;
	}

	/* fix point fmod */
	n = ix - iy;
	while(n--) {
		hz=hx-hy;
		if(hz<0){hx = hx+hx;}
		else {
			if(hz==0) 		/* return sign(x)*0 */
				return Zero_flt[(uint32_t)sx>>31];
			hx = hz+hz;
		}
	}
	hz=hx-hy;
	if(hz>=0) {hx=hz;}

	/* convert back to floating value and restore the sign */
	if(hx==0) 			/* return sign(x)*0 */
		return Zero_flt[(uint32_t)sx>>31];
	while(hx<0x00800000) {		/* normalize x */
		hx = hx+hx;
		iy -= 1;
	}
	if(iy>= -126) {		/* normalize output */
		hx = ((hx-0x00800000)|((iy+127)<<23));
		SET_FLOAT_WORD(x,hx|sx);
	} else {		/* subnormal output */
		/* If denormals are not supported, this code will generate a
			 zero representation.  */
		n = -126 - iy;
		hx >>= n;
		SET_FLOAT_WORD(x,hx|sx);
		x *= one_flt;		/* create necessary signal */
	}
	return x;		/* exact output */
}

