/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.driver.avr.atmega128;

import keso.driver.avr.mica2.Mica2;

public final class Adc {

	private static byte channel = 0;//the current channel
	private static final byte channelmask = (byte) 0x07;

	private static byte ON   = (byte) 0;
	private static byte OFF  = (byte) 1;

	private static byte MODE;

	static {
		MODE = ON;
		on();
	}

	/**
	 * switches on the ADC
	 */
	public static void on() {
		//ATMega128.registers.ADCSRA.set((1<<ATMega128.ADEN) | (1<<ATMega128.ADPS2) | (1<<ATMega128.ADPS1) | (1<<ATMega128.ADPS0));
		ATMega128.registers.ADCSRA.set((1<<ATMega128.ADEN) | (1<<ATMega128.ADPS2) | (1<<ATMega128.ADPS1));
		ATMega128.registers.ADMUX.set((1<<ATMega128.REFS1) | (1<<ATMega128.REFS0));
		irq_on();
		MODE = ON;
	}
	/**
	 * switches off the ADC
	 */
	public static void off() {
		ATMega128.registers.ADCSRA.clearBit(ATMega128.ADEN);
		irq_off();
		MODE = OFF;
	}
	/**
	 * enables the interrupt for the ADC
	 */
	public static void irq_on() {
		ATMega128.registers.ADCSRA.setBit(ATMega128.ADIE);
	}
	/**
	 * disables the interrupt for the ADC
	 */
	public static void irq_off() {
		ATMega128.registers.ADCSRA.clearBit(ATMega128.ADIE);
	}
	/**
	 * polls the adc
	 * @return the received adc value after the conversion is completed
	 */
	public static short poll() {
		short adc_value = 0;

		if(MODE == OFF) {
			on();
		}
		irq_off();//we don't need irq because we poll
		//ATMega128.registers.ADCSRA.set( (1<<ATMega128.ADIE) | (1<<ATMega128.ADIF) | (1<<ATMega128.ADSC));
		ATMega128.registers.ADCSRA.set((1<<ATMega128.ADSC));
		//wait for conversion to finish
		while((ATMega128.registers.ADCSRA.get() & (1<<ATMega128.ADIF)) == 0);
		//assemble values;
		adc_value |= (short) (ATMega128.registers.ADCL.get() & 0xff);
		adc_value |= ( ( ((short) ATMega128.registers.ADCH.get()) << 8) & 0xff00);

		return adc_value;
	}
	/**
	 * sets a new Channel as an input to the ADC
	 * @param ch the new Channel
	 */
	public static void setChannel(byte ch) {
		channel = (byte) (ch & channelmask);
		ATMega128.registers.ADMUX.set((ATMega128.registers.ADMUX.get() & 0xf8) | (channel & 0x07));
	}

	public static void startConversion(byte channel) {
		if(MODE == OFF) {
			on();
		}
		//Mica2.led_toggle(Mica2.LED_GREEN);
		setChannel(channel);
		ATMega128.registers.ADCSRA.setBit(ATMega128.ADSC);
	}
}
