/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.driver.avr.atmega32;

import keso.core.*;

public class ATMega32 implements MemoryMappedObject {
    public static final ATMega32 registers = (ATMega32) MemoryService.mapStaticDeviceMemory(0x20, "keso/driver/avr/atmega32/ATMega32");

/* TWI stands for "Two Wire Interface" or "TWI Was I2C(tm)" */
	public MT_U8 TWBR;
	public MT_U8 TWSR;
	public MT_U8 TWAR;
	public MT_U8 TWDR;

	public MT_U8 ADCL;
	public MT_U8 ADCH;
	//public MT_U16 ADC;
	public MT_U8 ADCSRA;
	public MT_U8 ADMUX;

/* analog comparator */
	public MT_U8 ACSR;

/* USART */
	public MT_U8 UBRRL;
	public MT_U8 UCSRB;
	public MT_U8 UCSRA;
	public MT_U8 UDR;

/* SPI */
	public MT_U8 SPCR;
	public MT_U8 SPSR;
	public MT_U8 SPDR;

/* Port D */
	public MT_U8 PIND;
	public MT_U8 DDRD;
	public MT_U8 PORTD;

/* Port C */
	public MT_U8 PINC;
	public MT_U8 DDRC;
	public MT_U8 PORTC;

/* Port B */
	public MT_U8 PINB;
	public MT_U8 DDRB;
	public MT_U8 PORTB;

/* Port A */
	public MT_U8 PINA;
	public MT_U8 DDRA;
	public MT_U8 PORTA;

/* EEPROM Control Register */
	public MT_U8 EECR;

/* EEPROM Data Register */
	public MT_U8 EEDR;

/* EEPROM Address Register */
	public MT_U8 EEARL;
	public MT_U8 EEARH;
	//public MT_U16 EEAR;

	public MT_U8 UBRRH;
	public MT_U8 WDTCR;
	public MT_U8 ASSR;

/* Timer 2 */
	public MT_U8 OCR2;
	public MT_U8 TCNT2;
	public MT_U8 TCCR2;

/* Timer 1 */
	public MT_U8 ICR1L;
	public MT_U8 ICR1H;
	//public MT_U16 ICR1;
	public MT_U8 OCR1BL;
	public MT_U8 OCR1BH;
	//public MT_U16 OCR1B;
	public MT_U8 OCR1AL;
	public MT_U8 OCR1AH;
	//public MT_U16 OCR1A;
	public MT_U8 TCNT1L;
	public MT_U8 TCNT1H;
	//public MT_U16 TCNT1;
	public MT_U8 TCCR1B;
	public MT_U8 TCCR1A;

	public MT_U8 SFIOR;

	public MT_U8 OSCCAL;

/* Timer 0 */
	public MT_U8 TCNT0;
	public MT_U8 TCCR0;

	public MT_U8 MCUSR;
	public MT_U8 MCUCR;

	public MT_U8 TWCR;

	public MT_U8 SPMCR;

	public MT_U8 TIFR;
	public MT_U8 TIMSK;

	public MT_U8 GIFR;
	public MT_U8 GICR;
	public MT_U8 OCR0;

/* Bit numbers */

/* GICR */
	public static final byte INT1=7;
	public static final byte INT0=6;
	public static final byte INT2=5;
	public static final byte IVSEL=1;
	public static final byte IVCE=0;

/* GIFR */
	public static final byte INTF1=7;
	public static final byte INTF0=6;
	public static final byte INTF2=5;

/* TIMSK */
	public static final byte OCIE2=7;
	public static final byte TOIE2=6;
	public static final byte TICIE1=5;
	public static final byte OCIE1A=4;
	public static final byte OCIE1B=3;
	public static final byte TOIE1=2;
	public static final byte OCIE0=1;
	public static final byte TOIE0=0;

/* TIFR */
	public static final byte OCF2=7;
	public static final byte TOV2=6;
	public static final byte ICF1=5;
	public static final byte OCF1A=4;
	public static final byte OCF1B=3;
	public static final byte TOV1=2;
	public static final byte OCF0=1;
	public static final byte TOV0=0;

/* SPMCR */
	public static final byte SPMIE=7;
	public static final byte RWWSB=6;
/* bit 5 reserved */
	public static final byte RWWSRE=4;
	public static final byte BLBSET=3;
	public static final byte PGWRT=2;
	public static final byte PGERS=1;
	public static final byte SPMEN=0;

/* TWCR */
	public static final byte TWINT=7;
	public static final byte TWEA=6;
	public static final byte TWSTA=5;
	public static final byte TWSTO=4;
	public static final byte TWWC=3;
	public static final byte TWEN=2;
/* bit 1 reserved */
	public static final byte TWIE=0;

/* TWAR */
	public static final byte TWA6=7;
	public static final byte TWA5=6;
	public static final byte TWA4=5;
	public static final byte TWA3=4;
	public static final byte TWA2=3;
	public static final byte TWA1=2;
	public static final byte TWA0=1;
	public static final byte TWGCE=0;

/* TWSR */
	public static final byte TWS7=7;
	public static final byte TWS6=6;
	public static final byte TWS5=5;
	public static final byte TWS4=4;
	public static final byte TWS3=3;
/* bit 2 reserved */
	public static final byte TWPS1=1;
	public static final byte TWPS0=0;

/* MCUCR */
	public static final byte SE=7;
	public static final byte SM2=6;
	public static final byte SM1=5;
	public static final byte SM0=4;
	public static final byte ISC11=3;
	public static final byte ISC10=2;
	public static final byte ISC01=1;
	public static final byte ISC00=0;

/* MCUCSR */
	public static final byte JTD=7;
	public static final byte ISC2=6;
/* bit 5 reserved */
	public static final byte JTRF=4;
	public static final byte WDRF=3;
	public static final byte BORF=2;
	public static final byte EXTRF=1;
	public static final byte PORF=0;

/* SFIOR */
	public static final byte ADTS2=7;
	public static final byte ADTS1=6;
	public static final byte ADTS0=5;
/* bit 4 reserved */
	public static final byte ACME=3;
	public static final byte PUD=2;
	public static final byte PSR2=1;
	public static final byte PSR10=0;

/* TCCR0 */
	public static final byte FOC0=7;
	public static final byte WGM00=6;
	public static final byte COM01=5;
	public static final byte COM00=4;
	public static final byte WGM01=3;
	public static final byte CS02=2;
	public static final byte CS01=1;
	public static final byte CS00=0;

/* TCCR2 */
	public static final byte FOC2=7;
	public static final byte WGM20=6;
	public static final byte COM21=5;
	public static final byte COM20=4;
	public static final byte WGM21=3;
	public static final byte CS22=2;
	public static final byte CS21=1;
	public static final byte CS20=0;

/* ASSR */
/* bits 7-4 reserved */
	public static final byte AS2=3;
	public static final byte TCN2UB=2;
	public static final byte OCR2UB=1;
	public static final byte TCR2UB=0;

/* TCCR1A */
	public static final byte COM1A1=7;
	public static final byte COM1A0=6;
	public static final byte COM1B1=5;
	public static final byte COM1B0=4;
	public static final byte FOC1A=3;
	public static final byte FOC1B=2;
	public static final byte WGM11=1;
	public static final byte WGM10=0;

/* TCCR1B */
	public static final byte ICNC1=7;
	public static final byte ICES1=6;
/* bit 5 reserved */
	public static final byte WGM13=4;
	public static final byte WGM12=3;
	public static final byte CS12=2;
	public static final byte CS11=1;
	public static final byte CS10=0;

/* WDTCR */
/* bits 7-5 reserved */
	public static final byte WDTOE=4;
	public static final byte WDE=3;
	public static final byte WDP2=2;
	public static final byte WDP1=1;
	public static final byte WDP0=0;

/* PA7-PA0 = ADC7-ADC0 */
/* PORTA */
	public static final byte PA7=7;
	public static final byte PA6=6;
	public static final byte PA5=5;
	public static final byte PA4=4;
	public static final byte PA3=3;
	public static final byte PA2=2;
	public static final byte PA1=1;
	public static final byte PA0=0;

/* DDRA */
	public static final byte DDA7=7;
	public static final byte DDA6=6;
	public static final byte DDA5=5;
	public static final byte DDA4=4;
	public static final byte DDA3=3;
	public static final byte DDA2=2;
	public static final byte DDA1=1;
	public static final byte DDA0=0;

/* PINA */
	public static final byte PINA7=7;
	public static final byte PINA6=6;
	public static final byte PINA5=5;
	public static final byte PINA4=4;
	public static final byte PINA3=3;
	public static final byte PINA2=2;
	public static final byte PINA1=1;
	public static final byte PINA0=0;

/*
   PB7 = SCK
   PB6 = MISO
   PB5 = MOSI
   PB4 = SS#
   PB3 = OC0/AIN1
   PB2 = INT2/AIN0
   PB1 = T1
   PB0 = XCK/T0
 */

/* PORTB */
	public static final byte PB7=7;
	public static final byte PB6=6;
	public static final byte PB5=5;
	public static final byte PB4=4;
	public static final byte PB3=3;
	public static final byte PB2=2;
	public static final byte PB1=1;
	public static final byte PB0=0;

/* DDRB */
	public static final byte DDB7=7;
	public static final byte DDB6=6;
	public static final byte DDB5=5;
	public static final byte DDB4=4;
	public static final byte DDB3=3;
	public static final byte DDB2=2;
	public static final byte DDB1=1;
	public static final byte DDB0=0;

/* PINB */
	public static final byte PINB7=7;
	public static final byte PINB6=6;
	public static final byte PINB5=5;
	public static final byte PINB4=4;
	public static final byte PINB3=3;
	public static final byte PINB2=2;
	public static final byte PINB1=1;
	public static final byte PINB0=0;

/*
   PC7 = TOSC2
   PC6 = TOSC1
   PC1 = SDA
   PC0 = SCL
 */
/* PORTC */
	public static final byte PC7=7;
	public static final byte PC6=6;
	public static final byte PC5=5;
	public static final byte PC4=4;
	public static final byte PC3=3;
	public static final byte PC2=2;
	public static final byte PC1=1;
	public static final byte PC0=0;

/* DDRC */
	public static final byte DDC7=7;
	public static final byte DDC6=6;
	public static final byte DDC5=5;
	public static final byte DDC4=4;
	public static final byte DDC3=3;
	public static final byte DDC2=2;
	public static final byte DDC1=1;
	public static final byte DDC0=0;

/* PINC */
	public static final byte PINC7=7;
	public static final byte PINC6=6;
	public static final byte PINC5=5;
	public static final byte PINC4=4;
	public static final byte PINC3=3;
	public static final byte PINC2=2;
	public static final byte PINC1=1;
	public static final byte PINC0=0;

/*
   PD7 = OC2
   PD6 = ICP
   PD5 = OC1A
   PD4 = OC1B
   PD3 = INT1
   PD2 = INT0
   PD1 = TXD
   PD0 = RXD
 */

/* PORTD */
	public static final byte PD7=7;
	public static final byte PD6=6;
	public static final byte PD5=5;
	public static final byte PD4=4;
	public static final byte PD3=3;
	public static final byte PD2=2;
	public static final byte PD1=1;
	public static final byte PD0=0;

/* DDRD */
	public static final byte DDD7=7;
	public static final byte DDD6=6;
	public static final byte DDD5=5;
	public static final byte DDD4=4;
	public static final byte DDD3=3;
	public static final byte DDD2=2;
	public static final byte DDD1=1;
	public static final byte DDD0=0;

/* PIND */
	public static final byte PIND7=7;
	public static final byte PIND6=6;
	public static final byte PIND5=5;
	public static final byte PIND4=4;
	public static final byte PIND3=3;
	public static final byte PIND2=2;
	public static final byte PIND1=1;
	public static final byte PIND0=0;

/* SPSR */
	public static final byte SPIF=7;
	public static final byte WCOL=6;
/* bits 5-1 reserved */
	public static final byte SPI2X=0;

/* SPCR */
	public static final byte SPIE=7;
	public static final byte SPE=6;
	public static final byte DORD=5;
	public static final byte MSTR=4;
	public static final byte CPOL=3;
	public static final byte CPHA=2;
	public static final byte SPR1=1;
	public static final byte SPR0=0;

/* UCSRA */
	public static final byte RXC=7;
	public static final byte TXC=6;
	public static final byte UDRE=5;
	public static final byte FE=4;
	public static final byte DOR=3;
	public static final byte PE=2;
	public static final byte U2X=1;
	public static final byte MPCM=0;

/* UCSRB */
	public static final byte RXCIE=7;
	public static final byte TXCIE=6;
	public static final byte UDRIE=5;
	public static final byte RXEN=4;
	public static final byte TXEN=3;
	public static final byte UCSZ2=2;
	public static final byte RXB8=1;
	public static final byte TXB8=0;

/* UCSRC */
	public static final byte URSEL=7;
	public static final byte UMSEL=6;
	public static final byte UPM1=5;
	public static final byte UPM0=4;
	public static final byte USBS=3;
	public static final byte UCSZ1=2;
	public static final byte UCSZ0=1;
	public static final byte UCPOL=0;

/* ACSR */
	public static final byte ACD=7;
	public static final byte ACBG=6;
	public static final byte ACO=5;
	public static final byte ACI=4;
	public static final byte ACIE=3;
	public static final byte ACIC=2;
	public static final byte ACIS1=1;
	public static final byte ACIS0=0;

/* ADCSRA */
	public static final byte ADEN=7;
	public static final byte ADSC=6;
	public static final byte ADATE=5;
	public static final byte ADIF=4;
	public static final byte ADIE=3;
	public static final byte ADPS2=2;
	public static final byte ADPS1=1;
	public static final byte ADPS0=0;

/* ADMUX */
	public static final byte REFS1=7;
	public static final byte REFS0=6;
	public static final byte ADLAR=5;
	public static final byte MUX4=4;
	public static final byte MUX3=3;
	public static final byte MUX2=2;
	public static final byte MUX1=1;
	public static final byte MUX0=0;

/* EEPROM Control Register */
	public static final byte EERIE=3;
	public static final byte EEMWE=2;
	public static final byte EEWE=1;
	public static final byte EERE=0;

/* Constants */
	public static final short SPM_PAGESIZE=128;
	public static final short RAMEND=0x85F;
	public static final short XRAMEND=0x85F;
	public static final short E2END=0x3FF;
	public static final short FLASHEND=0x7FFF;
}
