/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package keso.driver.avr.mica2;

import keso.driver.avr.atmega128.ATMega128;

public class Mica2 {
	public static final int LED_YELLOW = 1;
	public static final int LED_GREEN = 2;
	public static final int LED_RED = 4;

	static {
		// configure LED pins as output
		ATMega128.registers.DDRA.or(LED_YELLOW|LED_GREEN|LED_RED);
		// reset LEDS
		led_off(LED_YELLOW|LED_GREEN|LED_RED);
	};

	public static final void led_toggle(int led_mask) {
		ATMega128.registers.PORTA.xor(led_mask);
	}

	public static final void led_on(int led_mask) {
		ATMega128.registers.PORTA.or(led_mask);
	}

	public static final void led_off(int led_mask) {
		ATMega128.registers.PORTA.and(~led_mask);
	}

	// TCNTn       Timer/Counter 16b
	// OCRnA/B/C   Output Compare Registers 16b
	// TCCRn/A/B/C Timer/Counter Control Registers 8b
	// TIFR        Timer Interrupt Flag Register
	// ETIFR       Extended Timer Interrupt Flag Register
	// TIMSK       Timer Interrupt Mask Register
	// ETIMSK      Extended Timer Interrupt Mask Register

	/* Prescaler Bits 0:2 in TCCRnB
	 *  stopped, 1, 1/8, 1/64, 1/256, 1/1024, eclock falling edge, ecloc rising edge
	 */
// select clock source (CSn2:0 in TCCRnB, CSn=0 - timer stopped)
// output compare interrupt OCIEnx=1, OCFnx auto-cleared

}
