/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package ciao.net.udp;

import keso.core.Memory;

public final class Socket {
	private static UDPPacket free_packet_buffers;

	public void set_sport(int port) { /* KNI */ }
	public void set_dport(int port) { /* KNI */ }
	public void set_dst_ipv4_addr(int addr) { /* KNI */ }
	public boolean bind() { /* KNI */ return false; }
	public void unbind() { /* KNI */ }

	/**
	 * Transmits a data packet via the underlying CiAO UDP socket.
	 *
	 * @param data int array containing the data payload
	 * @param size number of integers that are occupied by the payload
	 * @return true if the packet has successfully been sent, false on error
	 */
	public boolean send(int[] data, int size) { /* KNI */ return false; }

	/**
	 * Transmits a data packet via the underlying CiAO UDP socket.
	 *
	 * @param data byte array containing the data payload
	 * @param size number of bytes that are occupied by the payload
	 * @return true if the packet has successfully been sent, false on error
	 */
	public boolean send(byte[] data, int size) { /* KNI */ return false; }

	/**
	 * Read a packet from the underlying socket and adjusts the provided
	 * {@link keso.core.Memory} object to point to the IP packet's raw
	 * data.
	 *
	 * @return true if a packet was read, false otherwise
	 */
	private boolean internal_read(UDPPacket p, Memory target) { /* KNI */ return false; }

	public void set_dst_ipv4_addr(byte a, byte b, byte c, byte d) {
		set_dst_ipv4_addr( convert_ipv4_addr(a,b,c,d) );
	}

	public static int convert_ipv4_addr(byte a, byte b, byte c, byte d) {
		int addr =  (a&0xff)
			|        ((b&0xff) << 8 )
			|        ((c&0xff) << 16)
			|        ((d&0xff) << 24);
		return addr;
	}

	public UDPPacket read() {
		UDPPacket p;
		if(free_packet_buffers == null) {
			p = new UDPPacket();
		} else {
			p = free_packet_buffers;
			free_packet_buffers = p.next;
		}

		if(! internal_read(p,p.get_data()) ) {
			// read failed, enqueue UDP packet
			p.next = free_packet_buffers;
			free_packet_buffers = p;
			return null;
		}

		return p;
	}

	private void internal_free(UDPPacket p) { /* KNI */	}

	public void free(UDPPacket p) {
		if(p != null) {
			internal_free(p);
			p.unmap();

			p.next = free_packet_buffers;
			free_packet_buffers = p;
		}
	}
}
