/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package ciao.net.udp;

import keso.core.*;

public class UDPPacket {
	public static final int UDP_HEADER_SIZE = 8;
	public static final int IPV4_TYPE_UDP   = 17;
	public static final int UNUSED_PORT     = 0;

	// UDP header
	private static final int OFS_sport=0;
	private static final int OFS_dport=2;
	private static final int OFS_length=4;
	private static final int OFS_checksum=6;

	// link pointer to manage unused packets in a linked list
	UDPPacket next;

	// payload data
	private final Memory data = MemoryService.allocMemory(0);

	/**
	 * Returns the length of the UDP payload.
	 *
	 * @return size of the UDP payload in bytes
	 */
	public int get_length() { return 0; /* KNI */ }

	/** Provides a {@link keso.core.Memory} object pointing to the raw data of
	 * the packet.
	 *
	 */
	public Memory get_data() {
		return data;
	}

	/**
	 * Clears the mapping of the {@link UDPPacket} object to the underlying CiAO
	 * IPStack packet.
	 *
	 * The backing {@link Memory} object is readjusted to an area of size 0 to detect
	 * further use and signal an error.
	 */
	void unmap() {
		MemoryService.adjustMemory(data,0, data,0);
	}
}
