/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
/**
 * LCD Treiber fuer HD44780 Displays
 *
 * @author Benedikt Dremel, Martin Hoffmann
 *
 * */

package keso.driver.lcdHD44780;
import keso.driver.avr.atmega32.*;


public class LCD{


   private static final byte  DATA   =  (byte) 0x04; //  PB2,  Data(1) or Command(0)
   private static final byte  READ   =  (byte) 0x02; //  PB1, Read(1)  or Write(0) data
   private static final byte  ENABLE     =(byte) 0x01; //  PB0
   private static final byte  CONTROLPINS = (byte)(0x07);//PB0, PB1, PB2
   private static final byte  DATABUS = (byte)(0x0F); //PA0 bis PA3

private static volatile int counter;
   private static  void wait_active(int ms){
      counter = ms<<2;
      while(counter-- > 0);
    }

static {
//  public static void init(){
  wait_active(500);
      write_nibble('c',(byte) 0x3);
	wait_active(5); //5 ms warten

	write_nibble('c',(byte) 0x3);
      wait_active(5); //5 ms warten

	write_nibble('c',(byte) 0x3);
      wait_active(5); //5 ms warten

	//3. Auf 4-bit Modus schalten
	write_nibble('c',(byte) 0x2);
      wait_active(5); //5 ms warten


	// 4. Display waehlen
	write_byte('c',(byte) 0x28);
	wait_bf();

	// 5. Aktivieren, Cursor aus, blinken aus
	write_byte('c',(byte) 0x0c);
	wait_bf();

	//Clear Display
	write_byte('c',(byte) 0x1);
//  wait_active(10);
}


   private static void write_nibble(char  mode, byte nibble){
       ATMega32.registers.DDRB.or(CONTROLPINS); //Port output
       //Daten oder Befehl?...
       if(mode == 'd'){
           ATMega32.registers.PORTB.or(DATA);
       }else if(mode == 'c'){
           ATMega32.registers.PORTB.and(~DATA);
       }else{
           return;
       }

       ATMega32.registers.PORTB.and(~READ);//RW  low -> Write Data
       ATMega32.registers.DDRA.or(DATABUS);   // Port Output
       ATMega32.registers.PORTA.and(~DATABUS);   // Clear Databus

       ATMega32.registers.PORTA.or(nibble & 0x0F); //send lower nibble
       ATMega32.registers.PORTB.or(ENABLE);  //Enable line high
       wait_active(1); //wait ca. 230ns
       ATMega32.registers.PORTB.and(~ENABLE);  //Enable line low


   }

   private static void write_byte(char  mode, byte my_byte){
    ATMega32.registers.DDRB.or(CONTROLPINS); //Port output
       //Daten oder Befehl?...
       if(mode == 'd'){ // Send data
           ATMega32.registers.PORTB.or(DATA);
       }else if(mode == 'c'){ // Send command
           ATMega32.registers.PORTB.and(~DATA);
       }else{
           return;
       }

       ATMega32.registers.PORTB.and(~READ);// RW  low -> Write Data
       ATMega32.registers.DDRA.or(DATABUS);      // Port Output
       ATMega32.registers.PORTA.and(~DATABUS);   // Clear Databus

       ATMega32.registers.PORTA.or((my_byte & 0xF0)>>4); //send high nibble
       ATMega32.registers.PORTB.or(ENABLE);  //Enable line high
       wait_active(1); //wait ca. 230ns
       ATMega32.registers.PORTB.and(~ENABLE);  //Enable line low

       ATMega32.registers.PORTA.and(~DATABUS);   // Clear Databus

       ATMega32.registers.PORTA.or(my_byte & 0x0F); //send lower nibble
       ATMega32.registers.PORTB.or(ENABLE);  //Enable line high
       wait_active(1); //wait ca. 230ns
       ATMega32.registers.PORTB.and(~ENABLE);  //Enable line low


   }

  private static byte read(char  mode){
      byte nibble = 0;
    ATMega32.registers.DDRB.or(CONTROLPINS); //Port output
    if(mode == 'd'){
        ATMega32.registers.PORTB.or(DATA);
    }else if(mode == 'c'){
        ATMega32.registers.PORTB.and(~DATA);
    }else{
        return -1;
    }

    ATMega32.registers.DDRA.and(~DATABUS); //Databus input direction

    ATMega32.registers.PORTB.or(READ); //Read Data

    //Read higher nibble:
    ATMega32.registers.PORTB.or(ENABLE);
    wait_active(1);
    nibble = (byte) (ATMega32.registers.PINA.get() << 4);
    ATMega32.registers.PORTB.and(~ENABLE);

    //Read lower nibble
    ATMega32.registers.PORTB.or(ENABLE);
    wait_active(1);
    nibble |= ((byte)ATMega32.registers.PINA.get() & 0x0F);
    ATMega32.registers.PORTB.and(~ENABLE);

    return nibble;

  }

  private static void wait_bf(){
    byte bf = 1;
    while(bf != 0){
        bf = read('c');
        bf &= 0x80;
    }
  }

 public static int put(int  c){
    switch(c){
   case 0xe4: write_byte('d', (byte)0xe1); break;
    case 0xc4: write_byte('d', (byte)0xe1); break;
    case 0xd6: write_byte('d', (byte)0xef); break;
    case 0xf6: write_byte('d', (byte)0xef); break;
    case 0xfc: write_byte('d', (byte)0xf5); break;
    case 0xdc: write_byte('d', (byte)0xf5); break;
    case 0xdf: write_byte('d', (byte)0xe2); break;
    case 0xb0: write_byte('d', (byte)0xdf); break;
    case 0x19: write_byte('d', (byte)0xE3); break;
    default: write_byte('d', (byte)c);
   }
   wait_bf();
   return c;
  }

public static void puts(String s){
    for(int i = 0 ; i < s.length(); i++){
        put(s.charAt(i));
    }
}

public static void putcarr(char[] s){
    for(int i = 0 ; i < s.length; i++){
        put(s[i]);
    }
}

 public  static void locate(int  row, int col){
    if(row>0) row = 1;
    byte c = (byte)0x80;
    c += 0x40*row + col;
    write_byte('c',(byte) c);
    wait_bf();
 }

 public  static void set_cursor(byte c){
    byte cmd = read('c');

    switch(c){
        case 'l': cmd--; cmd |= 0x80; break;
        case 'r': cmd++; cmd |= 0x80; break;
        case 'u': cmd &= ~0x80; if(cmd >= 0x40){cmd |= 0x80; cmd &= 0xbf;}; break;
        case 'd': cmd &= ~0x80; if(cmd < 0x40){cmd |= 0x80; cmd |= 0xc0;};  break;
        default: return;
    }
    write_byte('c', cmd);
    wait_bf();
 }

 public static  void clear_display(){
    write_byte('c', (byte)1);
    wait_bf();
 }

}
