/**(c)

  Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt

  This file is part of the KESO Java Runtime Environment.

  KESO is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

  KESO is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package trafficlight;

import keso.core.*;

import keso.driver.avr.atmega128.*;

public class RemoteControlTask implements Runnable {
	private static Usart usart;

	public RemoteControlTask() {
		usart = new Usart(38400);
		usart.setRXInterruptEnable(true);

		usart.puts("================================\r\n");
		usart.puts("* TrafficLights Remote Control *\r\n");
		usart.puts("================================\r\n");
	}


	/**
	 * ISR Handling routine for the rx_complete event.
	 *
	 * This interrupt is triggered whenever a byte has been
	 * received on the serial port.
	 *
	 * (the tx_isr is located in the serial driver Usart)
	 */
	private static void rx_isr() {
		byte input = usart.read();

		// ignore everything except S which initiates a signal change
		if(input == 'S') {
			// retrieve the system object of the rcTask task by using the name service
			Thread rct = TaskService.getTaskByName("rcTask");

			// activate the rcTask task
			TaskService.activate(rct);
		}
	}

	/**
	 * Entry function of the rcTask task.
	 *
	 * This task is activated whenever a valid command has been received via the
	 * serial port. It uses a portal to trigger the signal switch of the traffic
	 * light in the trafficLightDomain.
	 */
	public void run() {
		// retrieve the portal object via the name service
		TrafficLightServiceInterface srv = (TrafficLightServiceInterface) PortalService.lookup("TrafficLightService");

		// signalSwitch will be called on the service object in the
		// context of the trafficLightDomain
		if(srv.signalSwitch()) {
			usart.puts("signal switch accepted\r\n");
		} else {
			// if the traffic light is not red, the command is rejected
			usart.puts("signal switch rejected\r\n");
		}

		// terminate the task
		TaskService.terminate();
	}
}

