# /**(c)
# 
#   Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt
# 
#   This file is part of the KESO Java Runtime Environment.
# 
#   KESO is free software: you can redistribute it and/or modify it under the
#   terms of the Lesser GNU General Public License as published by the Free
#   Software Foundation, either version 3 of the License, or (at your option)
#   any later version.
# 
#   KESO is distributed in the hope that it will be useful, but WITHOUT ANY
#   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#   FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
#   more details. You should have received a copy of the GNU Lesser General
#   Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.
# 
#   Please contact keso@cs.fau.de for more info.
# 
#   (c)**/

$ext_purenode = "
	/**
	 * Nodes which only read but not change the global state
	 * are pure.
	 */
	\@Override
	public boolean isPureNode() { return true; }\n\n";
$ext_no_pure = "
	\@Override
	public void analyseCall() throws CompileException {
		method.isPure=false;
		method.isConst=false;
	}
";
$ext_sideeffect = "\t\@Override\n\tpublic boolean hasSideEffect() { return false; }\n\n";
$ext_const = "\t\@Override\n\tpublic boolean isConstant() { return true; }\n\n";
$ext_const .= "\t\@Override\n\tpublic boolean hasConstant() { return true; }\n\n";
$ext_const .= "\t\@Override\n\tpublic IMConstant nodeToConstant() throws CompileException { return this; }\n\n";
$ext_const .= "\tpublic int getIntValue() { throw new Error(); }\n\n";
$ext_const .= "\tpublic long getLongValue() { return getIntValue(); }\n\n";
$ext_const .= "\tpublic float getFloatValue() { throw new Error(); }\n\n";
$ext_const .= "\tpublic double getDoubleValue() { return getFloatValue(); }\n\n";
$ext_const .= "\tpublic void setIntValue(int value) { throw new Error(); }\n\n";
$ext_const .= "\tpublic void setLongValue(long value) { throw new Error(); }\n\n";
$ext_const .= "\tpublic void setFloatValue(float value) { throw new Error(); }\n\n";
$ext_const .= "\tpublic void setDoubleValue(double value) { throw new Error(); }\n\n";
$ext_const .= "\tpublic boolean equalValue(IMConstant node) { throw new Error(); }\n\n";
$ext_aconst = "$ext_const\tpublic String getString() { return ((StringCPEntry)value).getDescription(); }\n\n
\t\@Override\n\tpublic ConstantPoolEntry getCPEntry() { return value; }\n\n";
$is_const_obj = "\t\@Override\n\tpublic boolean isConstObject() throws CompileException { return true; }\n
\t\@Override\n\tpublic boolean isChecked(IMBasicBlock bb) throws CompileException { return true; }\n
";
$ext_add = "\t\@Override\n\tpublic boolean isAdd() {return true;}\n\n";
$ext_sub = "\t\@Override\n\tpublic boolean isSub() {return true;}\n\n";
$ext_mul = "\t\@Override\n\tpublic boolean isMul() {return true;}\n\n";
$ext_bit = "\t\@Override\n\tpublic boolean isBitOpr() {return true;}\n\n";
$ext_valid_ref = "\n\t\@Override\n\tpublic boolean isChecked(IMBasicBlock bb) throws CompileException { return true; }\n";
$ext_new_block = '
	/**
	 * The new operation will be removed, if the result is
	 * unused.
	 */
	@Override
	public boolean removeIfUnUsed() {
		return true;
	}

	@Override
	public void analyseCall() throws CompileException {
		method.isPure = false;
		method.isConst = false;
		if (method.isClassInit() == false &&
				opts.getGlobalHeap().allocationCanBlock()) method.blockInNew();
	}
';
$ext_valid_ref2 = '
	@Override
	public boolean isChecked(IMBasicBlock bb) throws CompileException {
		IMMethod callee = null;
		try { callee = method.findMethod(cpEntry); } catch (Exception ex) { return false; }
		return joinPoints.checkAttribute(callee,Weavelet.ATTR_NOTNULL)==Weavelet.TRUE;
	}
';
$ext_field_inline = "
	\@Override
	public IMBasicBlock inlineMethodCalls(IMBasicBlock current, IMNode prev) throws CompileException {
		if (!opts.doInlineRValues()) return null;
		return rvalue.inlineMethodCalls(current, prev);
	}
";
$ext_new_escape = "
	private boolean does_not_escape = false;
	private IMNode escape_path = null;
	private IMNode why_escape = null;
	\@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) {
			if (doesEscape(this, false)) {
				why_escape=whyEscape(this, false);
				if (why_escape instanceof IMEpilog && !doesEscape(this, true)) {
					opts.verbose(this.lineInfo()+\" escaping object thru \"+why_escape.toReadableString()+\" only!\");
				}
				does_not_escape = false;
			} else {
				does_not_escape = true;
			}
		}
	}

	\@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		if (escape_path==null) {
			opts.warn(this.lineInfo()+\": new \"+toReadableString()+\" with unknown escape path!\");
			return false;
		}
		return escape_path.doesEscape(node, isArgument);
	}

	\@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		if (escape_path==null) {
			return null;
		}
		return escape_path.whyEscape(this, isArgument);
	}

	\@Override
	public void setEscapePath(IMNode node) throws CompileException {
		escape_path = node;
	}

	\@Override
	public IMNode getEscapePath() throws CompileException {
		return escape_path;
	}
";
$ext_new_escape_arr = $ext_new_escape;
$ext_invoke_virtual = "
	\@Override
	public void analyseCall() throws CompileException {
		IMDataFlow[] argsInfo = new IMDataFlow[args.length + 1];
		IMBasicBlock bb       = method.getCurrentBasicBlock();
		argsInfo[0] = IMDataFlow.createValidDataFlowObj(obj, bb);
		for (int i = 0; i < args.length; ++i) {
			if (args[i] != null)
				argsInfo[i + 1] = IMDataFlow.createDataFlowObj(args[i], bb);
		}
		for (IMMethod candidate : getCandidates()) {
			opts.vverbose(\"++ \" + candidate.getAlias() + \" called from \" + method.getAlias());
			method.call(candidate.getClassName(), candidate.getMethodNameAndType());
			candidate.calledFrom(method, argsInfo);
			CallGraphAnalysis.graph.addEdge(method, candidate);
		}
	}

	\@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		if (opts.hasOption(\"_X_ssa_less_astack\"))
			return true;
		for (IMMethod candidate : getCandidates()) {
			if (obj == node && candidate.doArgumentEscape(0))
				return true;
			for (int i = 0; i < args.length; ++i) {
				if (node == args[i] && candidate.doArgumentEscape(i + 1))
					return true;
			}
		}
		return false;
	}

	public void showEscape(Coder coder) throws CompileException {
		String   result = \"\";
		for (IMMethod candidate : getCandidates()) {
			result += \"\t/* \" + candidate.getClassName() + \".\" + candidate.getMethodName() + \"(\";
			if (candidate.doArgumentEscape(0))
				result += \" esc> \";
			result += obj.toReadableString();
			for (int i = 0; i < args.length; ++i) {
				result += \",\";
				if (args[i].isReference() && candidate.doArgumentEscape(i + 1))
					result += \" esc> \";
				result += args[i].toReadableString();
			}
			result += \") */\\n\";
		}
		coder.add_before(result);
	}

	\@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		if (doesEscape(node,isArgument)) return this;
		return null;
	}

	\@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		obj.setEscapePath(this);
		for (int i=0;i<args.length;i++) {
			if (args[i]==null) continue;
			args[i].setEscapePath(this);
		}
	}

	\@Override
	public IMMethod getCallee() throws CompileException {
		return method.findMethod(cpEntry);
	}

	\@Override
	public IMClass getCalleeClass() throws CompileException {
		return method.getIMClass(cpEntry.getClassName());
	}
";

$ext_invoke_static = "
	\@Override
	public void analyseCall() throws CompileException {
		IMDataFlow argsInfo[] = new IMDataFlow[args.length];
		for (int i = 0; i < args.length; ++i) {
			if (args[i] != null)
				argsInfo[i] = IMDataFlow.createDataFlowObj(args[i],
						method.getCurrentBasicBlock());
		}
		IMMethod callee = method.findMethod(cpEntry);
		opts.vverbose(\"++ \" + callee.getAlias() + \" called from \" + method.getAlias());
		method.call(callee.getClassName(), callee.getMethodNameAndType());
		callee.calledFrom(method, argsInfo);
		CallGraphAnalysis.graph.addEdge(method, callee);
	}

	\@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		if (opts.hasOption(\"_X_ssa_less_astack\"))
			return true;
		IMMethod callee = method.findMethod(cpEntry);
		for (int i = 0; i < args.length; ++i) {
			if (node == args[i] && callee.doArgumentEscape(i))
				return true;
		}
		return false;
	}

	public void showEscape(Coder coder) throws CompileException {
		IMMethod callee = method.findMethod(cpEntry);
		StringBuffer buf = new StringBuffer();
		buf.append(\"\t/* \");
		buf.append(callee.getClassName());
		buf.append(\".\");
		buf.append(callee.getMethodName());
		buf.append(\"(\");
		for (int i=0;i<args.length;i++) {
			if (i>0) buf.append(\",\");
			if (args[i].isReference() && callee.doArgumentEscape(i))
				 buf.append(\" esc> \");
			buf.append(args[i].toReadableString());
		}
		buf.append(\") */\\n\");
		coder.add_before(buf.toString());
	}

	\@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		return this;
	}

	\@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		for (int i=0;i<args.length;i++) {
			if (args[i]==null) continue;
			args[i].setEscapePath(this);
		}
	}

	\@Override
	public IMMethod getCallee() throws CompileException {
		return method.findMethod(cpEntry);
	}

	\@Override
	public IMClass getCalleeClass() throws CompileException {
		return method.getIMClass(cpEntry.getClassName());
	}
";

my $ext_epilog = '	@Override
	public boolean usesSlot() {
		return (var != null);
	}
';

%extra = (

	'IMConstant' 	=> "$ext_const
	public ConstantPoolEntry getCPEntry() { throw new Error(); }
$ext_sideeffect
	public int compareTo(IMConstant o) {
		throw new Error();
	}
",
	'IMIConstant' 	=> "

	\@Override
	public boolean isNullValue() {
		return (value==0);
	}

	\@Override
	public int getDatatype() {
		if (opts.hasOption(\"stack8\") ||
			opts.hasOption(\"stack16\")) {
			if (value>=-128 && 127>=value) return BCBasicDatatype.BYTE;
			if (value>=-32768 && 32767>=value) return BCBasicDatatype.SHORT;
		}
		return datatype;
	}
",
	'IMLConstant' 	=> "
	\@Override
	public boolean isNullValue() {
		return (value==0L);
	}
",
	'IMFConstant' 	=> "
	\@Override
	public boolean isNullValue() {
		return (value==0.0f);
	}
",
	'IMDConstant' 	=> "
	\@Override
	public boolean isNullValue() {
		return (value==0.0);
	}
",
	'IMAConstant' 	=> "$ext_aconst $ext_sideeffect $is_const_obj
	\@Override
	public void analyseCall() throws CompileException {
		if (value instanceof StringCPEntry) {
			ClassStore repository = method.getClassStore();
			IMClass field_class = repository.getClass(\"java/lang/String\");
			IMField imfield = field_class.getField(\"value\");
			repository.registerIMPutField(imfield);
		}
	}
",
	'IMNullConstant'	=> "

	\@Override
	public boolean isNullValue() {
		return true;
	}

\tpublic int getIntValue() { return 0; }\n\n",
	'IMAMemConstant' 	=> "$ext_const $ext_sideeffect $is_const_obj
	\@Override
	public void translate_global(Coder coder) throws CompileException {
		coder.global_add_allocConstObject(memory_class, alias);
		coder.global_add_init_field_hex(\"_\",\"addr\", \"(void*)\", addr);
		coder.global_add_init_field_hex(\"_\",\"size\", \"\", size);
		coder.global_add_init_end();
	}
",
	'IMAMTConstant' 	=> "$ext_const $ext_sideeffect $is_const_obj",
	'IMAMTConstantStatic' 	=> "
	\@Override
	public void translate_global(Coder coder) throws CompileException {
		coder.global_add_allocConstObject(memory_type_class, alias);
		coder.global_add_init_field_hex(\"\", \"base\", \"(void*)\", base);
		coder.global_add_init_end();
	}
",
	'IMAObjConstant' 	=> "$ext_const $ext_sideeffect $is_const_obj
	\@Override
	public void translate_global(Coder coder) throws CompileException {
		coder.global_add_allocConstObject(obj_class, alias);
		coder.global_add_init_end();
	}
",
	'IMAdd' 	=> "$ext_add $ext_sideeffect",
	'IMSub' 	=> "$ext_sub $ext_sideeffect",
	'IMMul' 	=> "$ext_mul $ext_sideeffect",
	'IMDiv' 	=> "$ext_sideeffect",
	'IMBitAnd' 	=> "$ext_bit $ext_sideeffect",
	'IMBitOr' 	=> "$ext_bit $ext_sideeffect",
	'IMBitXor' 	=> "$ext_bit $ext_sideeffect",
	'IMShiftLeft' 	=> "$ext_sideeffect",
	'IMShiftRight' 	=> "$ext_sideeffect",
	'IMShiftRight2' => "$ext_sideeffect",
	'IMMul' 	=> "$ext_sideeffect",
	'IMNeg' 	=> "$ext_sideeffect",
	'IMRem' 	=> "$ext_sideeffect",
	'IMMonitor'	=> '
	@Override
	final public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		operand.setEscapePath(IMNode.NONE);
	}
',
	'IMInstanceOf' => '
	@Override
	public void analyseCall() throws CompileException {
		ClassStore rep = method.getClassStore();
		IMClass clazz = method.getIMClass(cpEntry.getClassName());
		opts.vverbose("# need InstanceOf "+clazz.getAlias());
		rep.needInstanceOf(clazz);
	}

	@Override
	final public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		rOpr.setEscapePath(IMNode.NONE);
	}

',
	'IMEpilog' => "
	\@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		return !isArgument;
	}

	\@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		if (!isArgument) return this;
		return null;
	}

	$ext_epilog
",
	'IMInvoke' => "
	\@Override
	public boolean isInvoke() {
		return true;
	}

	\@Override
	public void analyseCall() throws CompileException {
		opts.warn(this+\" has no analyse call method!\");
	}

	public IMMethod getCallee() throws CompileException {
		ONI();
		return null;
	}

	public IMClass getCalleeClass() throws CompileException {
		ONI();
		return null;
	}

	public Iterable<IMMethod> getCandidates() throws CompileException {
		return doGetCandidates(getCallee());
	}

	public IMNode[] getArguments() {
		return args;
	}

	public IMNode getObject() {
		return obj;
	}

	public IMClass getReturnType() {
		String typeStr = getReturnTypeString();
		return method.getIMClass(typeStr.substring(1, typeStr.length() - 1));
	}

	protected Iterable<IMMethod> doGetCandidates(IMMethod callee)
	throws CompileException {
		ONI();
		return null;
	}

	protected String getReturnTypeString() {
		return null;
	}

",
	'IMInvokeSpecial' => "
$ext_valid_ref2
$ext_invoke_virtual
	\@Override
	public IMBasicBlock inlineMethodCalls(IMBasicBlock current, IMNode prev) throws CompileException {
		IMMethod callee = method.findMethod(cpEntry);

		if (!callee.isInlineCandidate()) return null;

		return current.insertCode(prev, this, callee, obj, args);
	}
",
	'IMInvokeVirtual' => "
$ext_valid_ref2
$ext_invoke_virtual
	\@Override
	public IMBasicBlock inlineMethodCalls(IMBasicBlock current, IMNode prev) throws CompileException {
		IMMethod callee = method.findMethod(cpEntry);

		if (!callee.isFinal()) return null;

		if (!callee.isInlineCandidate()) return null;

		return current.insertCode(prev, this, callee, obj, args);
	}
",
	'IMInvokeStatic' => "
$ext_valid_ref2
$ext_invoke_static
	\@Override
	public IMBasicBlock inlineMethodCalls(IMBasicBlock current, IMNode prev) throws CompileException {
		IMMethod callee = method.findMethod(cpEntry);

		if (!callee.isInlineCandidate()) return null;

		return current.insertCode(prev, this, callee, null, args);
	}

",
	'IMInvokeInterface' => "
$ext_valid_ref2
$ext_invoke_virtual
	\@Override
	public IMBasicBlock inlineMethodCalls(IMBasicBlock current, IMNode prev) throws CompileException {
		IMMethod callee = method.findMethod(cpEntry);

		MethodTable table = callee.getMethodTable();
		if (table==null) return null;

		if (table.hasDispatchTable()) return null;
		callee = table.getFirstCandidate();

		if (callee==null) return null;
		if (!callee.isFinal()) return null;

		if (!callee.isInlineCandidate()) return null;
		return current.insertCode(prev, this, callee, obj, args);
	}
",
	'IMGoto'	=> "
	public boolean hasShortcut() {
		IMBasicBlock basicBlock = method.getCurrentBasicBlock();
		if (basicBlock.getSucc()==null) return false;
		return (basicBlock.next()==basicBlock.getSucc().at(0));
	}

	\@Override
	public boolean needSemicolon() throws CompileException {
		return false;
	}

",
	'IMLookupSwitch' => "
	\@Override
	final public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		operand.setEscapePath(IMNode.NONE);
	}
",
	'IMTableSwitch' => "
	\@Override
	final public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		operand.setEscapePath(IMNode.NONE);
	}
",
	'IMThrow' => "
	\@Override
	final public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		exception.setEscapePath(this);
	}

	\@Override
	final public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		return true;
	}

	\@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		return this;
	}
",
	'IMArrayLength' => "$ext_purenode
	\@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		array.setEscapePath(IMNode.NONE);
	}
",
	'IMReadArray' => "$ext_purenode $ext_check_arr
	\@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		aOpr.setEscapePath(IMNode.NONE);
	}
",
	'IMNew' => "
$ext_new_block$ext_valid_ref$ext_valide_ref$ext_new_escape
	\@Override
	public void translate_global(Coder coder) throws CompileException {
		if (opts.hasOption(\"no_inline_alloc\")) {
			coder.global_header_add(\"object_t* keso_alloc_stack(object_t* obj, class_id_t cid);\\n\");
			coder.global_add(\"object_t* keso_alloc_stack(object_t* obj, class_id_t cid) {\\n\");
			coder.global_add(\"\tobj_size_t size,roff;\\n\");
			coder.global_add(\"\tsize = CLASS(cid).size;\\n\");
			coder.global_add(\"\troff = CLS_ROFF(cid);\\n\");
		} else {
			coder.global_header_add(\"object_t* keso_alloc_stack(object_t* obj, class_id_t cid, obj_size_t size, obj_size_t roff);\\n\");
			coder.global_add(\"object_t* keso_alloc_stack(object_t* obj, class_id_t cid, obj_size_t size, obj_size_t roff) {\\n\");
		}
		coder.global_add_class(\"string\");
		coder.global_add(\"\\tmemset(obj, 0, size);\\n\");
		coder.global_add(\"\\tobj = (object_t*) (((object_t**)obj)+roff);\\n\");

		coder.global_add(\"\\tobj->class_id = cid;\\n\");
		if (opts.getGlobalHeap().needGCInfo())
			coder.global_add(\"\\tobj->gcinfo = KESO_CURRENT_DOMAIN->colorbit;\\n\");
		coder.global_add(\"\treturn (object_t*)obj;\\n\");
		coder.global_add(\"}\\n\");
	}
",
	'IMNewArray' => "
$ext_valid_ref$ext_new_block$ext_new_escape_arr
",
	'IMNewMultiArray' => "$ext_valid_ref$ext_new_block$ext_new_escape_arr",
	'IMNewObjArray' => "$ext_valid_ref$ext_new_block$ext_new_escape_arr",
	'IMGetField' => "$ext_purenode

	\@Override
	public IMField getField() throws CompileException {
		return method.getIMClass(cpEntry.getClassName()).getField(cpEntry);
	}

	\@Override
	public void escapeAnalyses(int mode) throws CompileException {
		rOpr.setEscapePath(IMNode.NONE);
	}

	\@Override
	public void analyseCall() throws CompileException {
		ClassStore repository = method.getClassStore();
		IMClass field_class = repository.getClass(cpEntry.getClassName());
		IMField imfield = field_class.getField(cpEntry);
		repository.register\%CLASS\%(imfield);
	}

	\@Override
	public boolean isMemoryType() throws CompileException {
		if (datatype!=BCBasicDatatype.REFERENCE) return false;
		String fieldType = cpEntry.getMemberTypeDesc();
		if (fieldType.charAt(0)=='[') return false;
		IMClass fieldClass = method.getIMClass(fieldType.substring(1,fieldType.length()-1));
		if (fieldClass!=null) return fieldClass.isMemoryType();
		opts.warn(\"Cannot find field type \"+fieldType.substring(1,fieldType.length()-1));
		return false;
	}

	\@Override
	public boolean isVolatile() throws CompileException {
		IMClass clazz = method.getIMClass(cpEntry.getClassName());
		IMField field = clazz.getField(cpEntry);
		return field.isVolatile();
	}

	/**
	 * This method is the right place to add a piece of code to global.c if needed.
	 * The imnode must be registered with
	 * repository.registerGlobalTranslationCallback(\"mt_escape\", this);
	 */
	\@Override
	public void translate_global(Coder coder) throws CompileException {
		IMClass mtclazz = method.getIMClass(\"keso/core/MT\");
		if (mtclazz!=null) {
			coder.global_add_fkt(\"mt_escape.c\");
		}
	}

	\@Override
	public boolean translate_MT_Ref(Coder coder, String reason) throws CompileException {
		if (isMemoryType()) {
			IMClass clazz = method.getIMClass(cpEntry.getClassName());
			coder.add_class(clazz);
			if (rOpr instanceof IMAMTConstant) {
				coder.add(\"((\");
				coder.add(clazz.getAlias());
				coder.add(\"_mt*)\");
				coder.add_hex(((IMAMTConstant)rOpr).getAddr());
				coder.add(\")->\");
				coder.add(clazz.emitField(cpEntry.getMemberName()));
			} else {
				coder.chk_ref(rOpr,method,bcPosition,reason);
				coder.add(\"(\");
				coder.add(clazz.getAlias().toUpperCase());
				coder.add(\"_MT(\");
				rOpr.translate(coder);
				coder.add(\")->\");
				coder.add(clazz.emitField(cpEntry.getMemberName()));
				coder.add(\")\");
			}
			return true;
		}
		return false;
	}
",
	'IMGetStatic' => "$ext_sideeffect $ext_purenode
	\@Override
	public IMField getField() throws CompileException {
		return method.getIMClass(cpEntry.getClassName()).getField(cpEntry);
	}

	/**
	 * \@return the entry in the class constant pool for the base of this operation
	 */
	public ClassCPEntry getClassCPEntry() {
		return cpEntry.getClassCPEntry();
	}

	\@Override
	public boolean isFinalStatic() throws CompileException {
		IMClass field_class = method.getIMClass(cpEntry.getClassName());
		IMField field = field_class.getField(cpEntry);
		return field.isFinal();
	}

	public String emit() throws CompileException {
		return imfield.getMacroName();
	}

	\@Override
	public boolean isVolatile() throws CompileException {
		IMClass clazz = method.getIMClass(cpEntry.getClassName());
		IMField field = clazz.getField(cpEntry);
		return field.isVolatile();
	}

	\@Override
	public void analyseCall() throws CompileException {
		ClassStore repository = method.getClassStore();
		IMClass field_class = repository.getClass(cpEntry.getClassName());
		imfield = field_class.getField(cpEntry);

		repository.registerIMGetStatic(imfield);

		if (imfield==null) {
			opts.warn(\"Field \"+cpEntry.getMemberName()+\" in class \"+field_class.getClassName()+\" not found!\");
			return;
		}
		repository.registerStaticRefField(imfield);
	}
",
	'IMReadLocalVariable' => "
$ext_sideeffect

	\@Override
	public boolean isVariable() {
		return true;
	}

	\@Override
	public boolean usesSlot() {
		return true;
	}

	/*
	public boolean translate_MT_Ref(Coder coder, String reason) throws CompileException {
		if (datatype!=BCBasicDatatype.REFERENCE) return false;
		IMNode alias = var.getAlias();
		if (alias!=null && alias.translate_MT_Ref(coder, reason)) {
			opts.warn(\"mt forwarded by alias\");
			return true;
		}
		return false;
	}
	*/

	\@Override
	public boolean isStackVariable(int stack_pos) {
		return var.isStackVariable(stack_pos);
	}

	\@Override
	public boolean isChecked(IMBasicBlock bb) throws CompileException {
		if (var==null) return false;
		return var.isChecked(bb);
	}

	\@Override
	public void setChecked(IMBasicBlock bb) throws CompileException {
		var.setChecked(bb);
	}

	\@Override
	public boolean isConstant() {
		if (var==null) return false;
		IMBasicBlock basicBlock = method.getCurrentBasicBlock();
		return var.varToConstant(basicBlock)!=null;
	}

	\@Override
	public IMConstant nodeToConstant() throws CompileException {
		IMBasicBlock basicBlock = method.getCurrentBasicBlock();
		IMConstant const_node = var.varToConstant(basicBlock);
		if (const_node==null) throw new CompileException(\"not constant!\");
		return const_node;
	}
",
	'IMStoreLocalVariable' => '

	public void setOperand(IMNode opr) {
		this.operand = opr;
	}

	public IMNode getOperand() {
		return operand;
	}

	public IMReadLocalVariable getReadOperation() throws CompileException {
		throw new CompileException();
	}

	@Override
	public void setChecked(IMBasicBlock bb) throws CompileException {
		var.setChecked(bb);
	}

	public IMBasicBlock inlineMethodCalls(IMBasicBlock current, IMNode prev) throws CompileException {
		if (!opts.doInlineRValues()) return null;
		return operand.inlineMethodCalls(current, prev);
	}

	@Override
	public boolean needSemicolon() throws CompileException {
		if (operand instanceof IMPhi) return false;
		return true;
	}

',
	'IMCheckCast' => '
	@Override
	public void setEscapePath(IMNode node) throws CompileException {
		rOpr.setEscapePath(node);
	}

	@Override
	public IMNode getEscapePath() throws CompileException {
		return rOpr.getEscapePath();
	}

	@Override
	public void analyseCall() throws CompileException {
		ClassStore rep = method.getClassStore();
		IMClass clazz = getCheckedClass(cpEntry.getClassName()); // FIXME
		opts.vverbose("# need check cast "+clazz.getAlias());
		rep.needCheckCast(clazz);
	}

	@Override
	public boolean isChecked(IMBasicBlock bb) throws CompileException {
		return rOpr.isChecked(bb);
	}

	@Override
	public boolean hasSideEffect() {
		return true;
	}

	@Override
	public IMBasicBlock inlineMethodCalls(IMBasicBlock current, IMNode prev) throws CompileException {
		if (!opts.doInlineRValues()) return null;
		return rOpr.inlineMethodCalls(current, prev);
	}
',
	'IMStoreArray' => $ext_check_arr.'
	@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		return true;
	}

	@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		return this;
	}

	@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		aOpr.setEscapePath(IMNode.NONE);
		rvalue.setEscapePath(this);
	}

	@Override
	public IMBasicBlock inlineMethodCalls(IMBasicBlock current, IMNode prev) throws CompileException {
		if (!opts.doInlineRValues()) return null;
		return rvalue.inlineMethodCalls(current, prev);
	}

	public String toMemAlias() {
		return aOpr.toReadableString()+"["+iOpr.toReadableString()+"]";
	}
',
	'IMPhi' => "$ext_sideeffect $ext_purenode

	\@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		for (int i=0;i<slots.length;i++) slots[i].setEscapePath(this);
	}

	private IMNode escape_path = null;
	private int escape_state = 0;
	\@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {

		/* cycle */
		if (escape_state==1) return true;
		/* dominate */
		if (escape_state==2) return true;
		/* do escape */
		if (escape_state==3) return true;
		/* no escape */
		if (escape_state==4) return false;

		IMBasicBlock phi_bb = escape_path.getIMSlot().getDefiningBB();
		IMSlot var_in = node.getIMSlot();
		if (var_in!=null) {
			DominatorTree domtree = method.getDomTree();
			IMBasicBlock in_bb = var_in.getDefiningBB();
			if (domtree.dom(phi_bb,in_bb)) {
				opts.warn(\"In \" + node.getMethod() + ':');
				opts.warn(\"    \" + escape_path.toReadableString() +
						\"dominates the definition of \"
						+ node.toReadableString());
				escape_state=2;
				return true;
			}
		} else {
			opts.warn(method.getAlias()+\": phi-fkt no slot \"+node.toReadableString());
		}

		if (escape_path==null) {
			opts.warn(this.lineInfo()+\": phi-fkt \"+toReadableString()+\" with unknown escape path!\");
			return false;
		}

		escape_state = 1;
		//boolean esc =  escape_path.doesEscape(this, isArgument);
		boolean esc =  escape_path.doesEscape(node, isArgument);
		escape_state = ( esc ? 3 : 4 );
		return esc;
	}

	\@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {

		if (doesEscape(node, isArgument) && escape_state<4) {
			return escape_path;
		}

		return escape_path.whyEscape(this, isArgument);
	}

	\@Override
	public void setEscapePath(IMNode node) throws CompileException {
		escape_path = node;
	}

	\@Override
	public IMNode getEscapePath() throws CompileException {
		return escape_path;
	}

",
	'IMPutField' => "
	private boolean initial_write = false;
$ext_field_inline

	\@Override
	public IMField getField() throws CompileException {
		return method.getIMClass(cpEntry.getClassName()).getField(cpEntry);
	}

	\@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		return true;
		/*
		 This is more complex!

		if (opts.hasOption(\"_X_ssa_astack_less\")) return true;
		if (obj.getIMSlot().doesEscape(false)) return true;
		return false;
		*/
	}

	\@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		if (!doesEscape(this,false)) return null;
		return this;
	}

	\@Override
	public void analyseCall() throws CompileException {
		ClassStore repository = method.getClassStore();
		IMClass field_class = repository.getClass(cpEntry.getClassName());
		IMField imfield = field_class.getField(cpEntry);
		repository.registerIMPutField(imfield);
	}

	\@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		obj.setEscapePath(IMNode.NONE);
		rvalue.setEscapePath(this);
	}

	\@Override
	public String toMemAlias() {
		return obj.toReadableString()+\".\"+cpEntry.getMemberName();
	}
",
	'IMPutStatic' => "$ext_field_inline
	\@Override
	public IMField getField() throws CompileException {
		return method.getIMClass(cpEntry.getClassName()).getField(cpEntry);
	}

	/**
	 * \@return the entry in the class constant pool for the base of this operation
	 */
	public ClassCPEntry getClassCPEntry() {
		return cpEntry.getClassCPEntry();
	}

	\@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		return true;
	}

	\@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		return this;
	}

	\@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		rvalue.setEscapePath(this);
	}

	\@Override
	public void analyseCall() throws CompileException {
		method.isPure  = false;
		method.isConst = false;

		ClassStore repository = method.getClassStore();

		IMField fld = method.getIMClass(cpEntry.getClassName()).getField(cpEntry);

		if (fld==null) throw new CompileException (\"field \"+
				method.getIMClass().getClassName()+\":\"+cpEntry.getMemberName()+\" not found!\");

		if (method.isClassInit() && fld.isFinal()) {
			repository.registerIMPutStatic(fld, rvalue);
		} else {
			repository.registerIMPutStatic(fld, null);
		}

		IMClass field_class = repository.getClass(cpEntry.getClassName());
		imfield = field_class.getField(cpEntry);
		repository.registerStaticRefField(imfield);
	}

	\@Override
	public String toMemAlias() {
		return cpEntry.getClassName()+\".\"+cpEntry.getMemberName();
	}
",
	'IMIStoreLocalVariable' => '
	@Override
	public IMReadLocalVariable getReadOperation() throws CompileException {
		return new IMIReadLocalVariable(method,21,bcPosition,var);
	}

',
	'IMLStoreLocalVariable' => '
	@Override
	public IMReadLocalVariable getReadOperation() throws CompileException {
		return new IMLReadLocalVariable(method,21,bcPosition,var);
	}

',
	'IMFStoreLocalVariable' => '
	@Override
	public IMReadLocalVariable getReadOperation() throws CompileException {
		return new IMFReadLocalVariable(method,21,bcPosition,var);
	}

',
	'IMDStoreLocalVariable' => '
	@Override
	public IMReadLocalVariable getReadOperation() throws CompileException {
		return new IMDReadLocalVariable(method,21,bcPosition,var);
	}

',
	'IMAStoreLocalVariable' => '

	@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		return var.doesEscape(isArgument);
	}

	@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		return var.whyEscape(isArgument);
	}

	@Override
	public void escapeAnalyses(int mode) throws CompileException {
		if (mode!=IMEscapeVisitor.PREPARE) return;
		operand.setEscapePath(this);
	}

	@Override
	public IMReadLocalVariable getReadOperation() throws CompileException {
		return new IMAReadLocalVariable(method,21,bcPosition,var);
	}

	@Override
	public boolean needSemicolon() throws CompileException {
		if (operand instanceof IMPhi) return false;
		return true;
	}

',
	'IMAReadLocalVariable' => '
	private IMNode escape_path = null;
	@Override
	public boolean doesEscape(IMNode node, boolean isArgument) throws CompileException {
		if (escape_path==null) {
			opts.warn(this.lineInfo()+": read "+toReadableString()+" with unknown escape path!");
			return false;
		}
		return escape_path.doesEscape(this, isArgument);
	}

	@Override
	final public IMNode whyEscape(IMNode node, boolean isArgument) throws CompileException {
		if (escape_path==null) {
			return this;
		}
		return escape_path.whyEscape(this, isArgument);
	}

	@Override
	public void setEscapePath(IMNode node) throws CompileException {
		if (node == null ) throw new Error("upps");
		escape_path = node;
	}

	@Override
	public IMNode getEscapePath() throws CompileException {
		return escape_path;
	}

	public String emit() throws CompileException {
		return var.toString();
	}

',
	'IMReturnSubroutine' => "$ext_epilog
",
); # end %extra
