# /**(c)
# 
#   Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt
# 
#   This file is part of the KESO Java Runtime Environment.
# 
#   KESO is free software: you can redistribute it and/or modify it under the
#   terms of the Lesser GNU General Public License as published by the Free
#   Software Foundation, either version 3 of the License, or (at your option)
#   any later version.
# 
#   KESO is distributed in the hope that it will be useful, but WITHOUT ANY
#   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#   FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
#   more details. You should have received a copy of the GNU Lesser General
#   Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.
# 
#   Please contact keso@cs.fau.de for more info.
# 
#   (c)**/
$init_imnode = "
	public \%CLASS\%(IMMethod method, int bc, int type, int bcpos) {
		super(method,bc,type,bcpos);\%DOMID\%
	}
";

$init_super = "\t\tsuper(method,bc,\%BCTYPE\%,bcpos);\%DOMID\%";

$init_default = "
	public \%CLASS\%(IMMethod method, int bc, int bcpos) {
$init_super
	}

";

$init_const_value = "
	protected \%JTYPE\% value;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, \%JTYPE\% value) {
$init_super
		this.value = value;
	}

	public \%JTYPE\% getValue() {
		return value;
	}

	\@Override
	public \%JTYPE\% get\%UJTYPE\%Value() {
		return value;
	}

	\@Override
	public void set\%UJTYPE\%Value(\%JTYPE\% value) {
		this.value = value;
	}

	\@Override
	public int compareTo(IMConstant o) {
		\%JTYPE\% rhs = o.get\%UJTYPE\%Value();
		if (value < rhs)
			return -1;
		else if (value == rhs)
			return 0;
		else
			return 1;
	}

";

$init_aconst = "
	protected ConstantPoolEntry value;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, ConstantPoolEntry value) {
$init_super
		this.value=value;
	}

	public ConstantPoolEntry getValue() {
		return value;
	}

	\@Override
	public IMClass determineType() {
		return method.getIMClass(\"java/lang/\" + value.getTypeString());
	}

	\@Override
	public int compareTo(IMConstant o) {
		if (o instanceof \%CLASS\% == false)
			return 1;
		return (value == ((\%CLASS\%) o).value) ? 0 : 1;
	}
";

$init_var_super = "
	protected IMSlot var;

	public \%CLASS\%(IMMethod method, int bc, int type, int bcpos, IMSlot var) {
		super(method,bc,type,bcpos);
		if (var==null) throw new NullPointerException();
		this.var=var;
	}

	public IMSlot getIMSlot() {
		return var;
	}

	public void setIMSlot(IMSlot slot) {
		//if (slot==null) throw new NullPointerException();
		var = slot;
	}
";

$init_var = "
	public \%CLASS\%(IMMethod method, int bc, int bcpos, int var) {
		this(method,bc,bcpos,
			method.getMethodFrame().getLocalVariable(\%BCTYPE\%,var));
	}

	public \%CLASS\%(IMMethod method, int bc, int bcpos, IMSlot var) {
		super(method,bc,\%BCTYPE\%,bcpos, var);
	}

	public \%CLASS\%(IMMethod method, int bc, int type, int bcpos, IMSlot var) {
		super(method,bc,type,bcpos, var);
	}
";
$init_var_ref = "
	private int alength = -1;

$init_var

	public int getArrayLength() throws CompileException {
		return alength;
	}
";
$init_field = "
	protected FieldRefCPEntry cpEntry;
	protected boolean nukeInit = false;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, FieldRefCPEntry field) {
		super(method, bc, BCBasicDatatype.toBasicDatatype(field.getMemberTypeDesc()), bcpos);\%DOMID\%
		this.cpEntry=field;
		method.requireClass(domainID, cpEntry.getClassName());
		String fieldType = cpEntry.getMemberTypeDesc();
		if (fieldType.charAt(0)=='L') {
			method.requireClass(domainID, fieldType.substring(1,fieldType.length()-1));
		}
	}

	\@Override
	public IMClass determineType() {
		String typeDesc = cpEntry.getMemberTypeDesc();
		return method.getIMClass(typeDesc.replaceFirst(\"^L(.*);\$\", \"\$1\"));
	}
";
$init_static_field = "
	protected FieldRefCPEntry cpEntry;
	protected IMField imfield=null;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, FieldRefCPEntry field) {
		super(method, bc, BCBasicDatatype.toBasicDatatype(field.getMemberTypeDesc()), bcpos);\%DOMID\%
		this.cpEntry=field;
		method.requireClass(domainID, cpEntry.getClassName());
		String fieldType = cpEntry.getMemberTypeDesc();
		if (fieldType.charAt(0)=='L') {
			method.requireClass(domainID, fieldType.substring(1,fieldType.length()-1));
		}
	}

	\@Override
	public IMClass determineType() {
		String typeDesc = cpEntry.getMemberTypeDesc();
		return method.getIMClass(typeDesc.replaceFirst(\"^L(.*);\$\", \"\$1\"));
	}
";

$init_method = "
	protected MethodRefCPEntry cpEntry;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, MethodRefCPEntry methodRef) {
		super(method,bc,bcpos,methodRef);\%DOMID\%
		this.cpEntry=methodRef;
		method.requireMethod(domainID, cpEntry);
	}

	\@Override
	public IMClass determineType() {
		if (datatype == BCBasicDatatype.VOID)
			return null;
		IMClass clazz = getReturnType();
		if (clazz == null || clazz.hasSubClasses() || clazz.isInterface() || clazz.isAbstract())
			return null;
		opts.vverbose(clazz+\" has no sub-classes. \"+lineInfo());
		return clazz;
	}

	public void setCPEntry(MethodRefCPEntry cpEntry) {
		this.cpEntry = cpEntry;
	}

	\@Override
	protected String getReturnTypeString() {
		String memberDesc = cpEntry.getMemberTypeDesc();
		return MethodTypeDescriptor.byName(memberDesc).getReturnTypeDesc();
	}
";

$init_method_interface = "
	protected InterfaceMethodRefCPEntry cpEntry;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, InterfaceMethodRefCPEntry methodRef) {
		super(method,bc,bcpos,methodRef);\%DOMID\%
		this.cpEntry=methodRef;
		method.requireMethod(domainID, cpEntry);
	}

	\@Override
	protected String getReturnTypeString() {
		String memberDesc = cpEntry.getMemberTypeDesc();
		return MethodTypeDescriptor.byName(memberDesc).getReturnTypeDesc();
	}
";

$init_class = "
	protected ClassCPEntry cpEntry;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, ClassCPEntry clazz) {
$init_super
		this.cpEntry=clazz;
		String className = cpEntry.getClassName();
		if(className.indexOf(\"[L\") >= 0) { // FIXME
			method.requireClass(domainID, \"[Ljava/lang/Object;\");
		} else {
			method.requireClass(domainID, className);
		}
	}

	\@Override
	public IMClass determineType() {
		return getCheckedClass(cpEntry.getClassName());
	}

	private IMClass getCheckedClass(String className) {
		int typeIndex = className.lastIndexOf('[');

		if(typeIndex != 0 || className.charAt(typeIndex+1) != 'L' ) {
			return method.getIMClass(cpEntry.getClassName());
		}

		// FIXME
		return method.getIMClass(\"[Ljava/lang/Object;\");
	}
";

$init_instanceof = "$init_class

	\@Override
	public IMNode getRightNode() {
		return rOpr;
	}
";

$init_new = "
	protected ClassCPEntry cpEntry;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, ClassCPEntry clazz) {
$init_super
		this.cpEntry=clazz;
		String className = cpEntry.getClassName();
		method.requireClass(domainID, className);
	}

	\@Override
	public IMClass determineType() {
		return method.getIMClass(cpEntry.getClassName());
	}

";

$init_marray = "
	protected ClassCPEntry cpEntry;
	protected IMArrayClass aclass;
	protected int dim;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, ClassCPEntry clazz, int dim) {
$init_super
		this.cpEntry=clazz;
		this.dim=dim;

		// if array of primitive, require that's array type, otherwise require the element class
		method.requireClass(domainID, ClassStore.getArrayElementType(cpEntry.getClassName()));
		method.requireClass(domainID, \"[Ljava/lang/Object;\");
		aclass=(IMArrayClass)method.getIMClass(\"[Ljava/lang/Object;\");
		aclass.requireMultiAlloc();
	}

	@Override
	public IMClass determineType() {
		return aclass;
	}

";

$init_obj_array = "
	protected ClassCPEntry cpEntry;
	protected IMArrayClass aclass;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, ClassCPEntry clazz) {
$init_super
		this.cpEntry=clazz;

		method.requireClass(domainID, ClassStore.getArrayElementType(cpEntry.getClassName()));
		method.requireClass(domainID, \"[Ljava/lang/Object;\");
		aclass=(IMArrayClass)method.getIMClass(\"[Ljava/lang/Object;\");
		aclass.requireAlloc();
	}

	@Override
	public IMClass determineType() {
		return aclass;
	}

	public IMNode getSize() {
		return size;
	}

	public int getArrayLength() throws CompileException {
		if (size.isConstant()) return ((IMConstant)size).getIntValue();
		return -1;
	}
";

$init_array = "
	protected IMArrayClass aclass;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, int type) {
$init_super
		String className=\"unknown\";
		switch (type) {
			case BCBasicDatatype.INT: className=\"[I\"; break;
			case BCBasicDatatype.LONG: className=\"[J\"; break;
			case BCBasicDatatype.FLOAT: className=\"[F\"; break;
			case BCBasicDatatype.DOUBLE: className=\"[D\"; break;
			case BCBasicDatatype.BOOLEAN: //className=\"[Z\"; break;
			case BCBasicDatatype.BYTE: className=\"[B\"; break;
			case BCBasicDatatype.CHAR: className=\"[C\"; break;
			case BCBasicDatatype.SHORT: className=\"[S\"; break;
			default:
				throw new Error(\"unkown type!\");
		}
		method.requireClass(domainID, className);
		aclass=(IMArrayClass)method.getIMClass(className);
		aclass.requireAlloc();
	}

	\@Override
	public IMClass determineType() {
		return aclass;
	}

	public IMNode getSize() {
		return size;
	}

	public int getArrayLength() throws CompileException {
		if (size.isConstant()) return ((IMConstant)size).getIntValue();
		return -1;
	}
";

$init_arraylength = "$init_default
	public IMNode getArray() {
		return array;
	}
";

$init_label = "
	public \%CLASS\%(IMMethod method, int bc, int bcpos, IMBasicBlock label) {
		super(method,bc,-1,bcpos);
		targets = new IMBasicBlock[1];
		targets[0]=label;
	}

";

$init_call = "
	private int index;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, IMBasicBlock label, IMBasicBlock ret_label) {
		super(method,bc,-1,bcpos);
		targets = new IMBasicBlock[1];
		targets[0]=label;
		index = method.registerReturnAddr(ret_label);
	}

";

$init_conditional = "
	public \%CLASS\%(IMMethod method, int bc, int bcpos, IMBasicBlock label, IMBasicBlock next) {
		super(method,bc,bcpos,label,next);
	}


";
$init_const_null = "
	public \%CLASS\%(IMMethod method, int bc, int bcpos) {
$init_super
	}

	\@Override
	public int compareTo(IMConstant o) {
		return o.isNullValue() ? 0 : 1;
	}
";
$init_tswitch = "
	protected int high;
	protected int low;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, int low, int high, IMBasicBlock[] table) {
$init_super
		this.low=low;
		this.high=high;
		this.targets=table;
	}

";
$init_lswitch = "
	protected int[] keys;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, int[] keys, IMBasicBlock[] table) {
$init_super
		this.keys=keys;
		this.targets=table;
	}

";

%init = (
   'IMPhi' => "
   	protected IMNode slots[];

	/**
	 * \@return the array of slots this {\@code IMPhi} node can choose from
	 */
	public IMNode[] getSlots() {
		return slots;
	}

	protected IMSlot orig_slot;
	protected IMBasicBlock bb;

	public \%CLASS\%(IMMethod method, IMBasicBlock bb, IMSlot orig_slot, int type) throws CompileException {
		super(method, -1, type, -1);\%DOMID\%
		this.orig_slot = orig_slot;
		this.bb = bb;

		IMBasicBlockList bl = bb.getPred();

		slots = new IMNode[bl.length()];
		for (int i=0;i<slots.length;i++) slots[i]=method.readLocal(-1,orig_slot);
	}

	public void setSource(int pos, IMNode slot) throws CompileException {
		slots[pos]=slot;
	}

	public IMNode getSource(int pos) throws CompileException {
		return slots[pos];
	}

	public void setSourceVar(int pos, IMSlot slot) throws CompileException {
		slots[pos].setIMSlot(slot);
	}

	public IMSlot getSourceVar(int pos) throws CompileException {
		return slots[pos].getIMSlot();
	}

	public void removeSource(int pos) throws CompileException {
		int      length   = slots.length;
		IMNode[] newSlots = new IMNode[length - 1];
		System.arraycopy(slots, 0,       newSlots, 0,   pos);
		System.arraycopy(slots, pos + 1, newSlots, pos, length - 1 - pos);
		IMSlot slot = getSourceVar(pos);
		if (slot != null)
			slot.deleteUse(getSource(pos));
		slots = newSlots;
	}

",
    'IMReadLocalVariable' => $init_var_super,
    'IMStoreLocalVariable' => $init_var_super,
    'IMArrayLength' => $init_arraylength,
    'IMCall' => $init_call,
    'IMStackOperationPOP' => $init_default,
    'IMStackOperationPOP2' => $init_default,
    'IMStackOperationDUP' => $init_default,
    'IMStackOperationDUP_X1' => $init_default,
    'IMStackOperationDUP_X2' => $init_default,
    'IMStackOperationDUP2' => $init_default,
    'IMStackOperationDUP2_X1' => $init_default,
    'IMStackOperationDUP2_X2' => $init_default,
    'IMStackOperationSWAP' => $init_default,
#    'IMCast' => 'IMUnaryNode',
    'IMCastD2F' => $init_default,
    'IMCastD2I' => $init_default,
    'IMCastD2L' => $init_default,
    'IMCastF2D' => $init_default,
    'IMCastF2I' => $init_default,
    'IMCastF2L' => $init_default,
    'IMCastI2B' => $init_default,
    'IMCastI2C' => $init_default,
    'IMCastI2D' => $init_default,
    'IMCastI2F' => $init_default,
    'IMCastI2L' => $init_default,
    'IMCastI2S' => $init_default,
    'IMCastL2D' => $init_default,
    'IMCastL2F' => $init_default,
    'IMCastL2I' => $init_default,
    'IMCaughtException' => '
	private IMExceptionHandler exphandler;

	public %CLASS%(IMMethod method, IMExceptionHandler exphandler, int bcpos) {
		super(method,-1,BCBasicDatatype.REFERENCE,bcpos);
		this.exphandler=exphandler;
	}
',
    'IMCheckCast' => $init_class,
    'IMDGCompare' => $init_default,
    'IMDLCompare' => $init_default,
    'IMFGCompare' => $init_default,
    'IMFLCompare' => $init_default,
    'IMLCompare' => $init_default,
#    'IMAdd' => 'IMBinaryNode',
#    'IMDiv' => 'IMBinaryNode',
#    'IMCompare' => 'IMBinaryNode',
#    'IMMul' => 'IMBinaryNode',
#    'IMNeg' => 'IMUnaryNode',
#    'IMRem' => 'IMBinaryNode',
#    'IMReturn' => 'IMNode',
    'IMEpilog' => '
    	private IMSlot var;

	public %CLASS%(IMMethod method, int bcpos) {
		super(method, -1, BCBasicDatatype.VOID, bcpos);
		var = method.getReturnValue();
	}

	public IMSlot getIMSlot() {
		return var;
	}

	public void setIMSlot(IMSlot slot) {
		var = slot;
	}
    ',
#    'IMSub' => 'IMBinaryNode',
#    'IMBitAnd' => 'IMBinaryNode',
#    'IMBitOr' => 'IMBinaryNode',
#    'IMBitXor' => 'IMBinaryNode',
#    'IMShiftLeft' => 'IMBinaryNode',
#    'IMShiftRight' => 'IMBinaryNode',
#    'IMShiftRight2' => 'IMBinaryNode',
    'IMEQConditionalBranch' => $init_conditional,
    'IMGEConditionalBranch' => $init_conditional,
    'IMGTConditionalBranch' => $init_conditional,
    'IMLEConditionalBranch' => $init_conditional,
    'IMLTConditionalBranch' => $init_conditional,
    'IMNEConditionalBranch' => $init_conditional,
    'IMGetField' => $init_field,
    'IMGetStatic' => $init_static_field,
    'IMGoto' => $init_label,
    'IMInstanceOf' => $init_instanceof,
    'IMInvoke' => "
	protected MethodTypeDescriptor	typeDesc;
	protected JoinPointChecker 	joinPoints;

	public static int nonVirtualCalls;
	public static int nonVirtualCallsInitial;
	public static int virtualCalls;
	public static int virtualCallsInitial;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, ClassMemberCPEntry cpEntry) {
		super(method,bc,-1,bcpos);
		this.joinPoints= method.getJoinPoints();
		this.typeDesc  = new MethodTypeDescriptor(cpEntry.getMemberTypeDesc());
		this.datatype  = typeDesc.getBasicReturnType();
	}
",
    'IMInvokeInterface' => $init_method_interface,
    'IMInvokeSpecial' => $init_method,
    'IMInvokeStatic' => $init_method,
    'IMInvokeVirtual' => $init_method."

	public IMInvokeSpecial convertToSpecial(IMClass clazz) {
		IMInvokeSpecial nnode = new IMInvokeSpecial();

		/* IMNode */
		nnode.method=method;
		nnode.src_method=src_method;
		nnode.opts=opts;
		nnode.bytecode=bytecode;
		nnode.datatype=datatype;
		nnode.bcPosition=bcPosition;
		nnode.src_bcPosition = src_bcPosition;
		// nnode.regId=regId; is private :-(

		/* IMInvoke */
		nnode.typeDesc = typeDesc;
                nnode.joinPoints = joinPoints;
                nnode.obj = obj;
                nnode.args = args;

                nnode.cpEntry=cpEntry;

                return nnode;
	}
",
    'IMLookupSwitch' => $init_lswitch,
    'IMMonitor' => $init_default,
    'IMNew' => $init_new,
    'IMNewArray' => $init_array,
    'IMNewMultiArray' => $init_marray,
    'IMNewObjArray' => $init_obj_array,
    'IMPutField' => $init_field,
    'IMPutStatic' => $init_static_field,
    'IMReturnSubroutine' => "
	protected IMSlot var;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, int var) {
		super(method,bc,\%BCTYPE\%,bcpos);
		this.var = method.getMethodFrame().getLocalVariable(\%BCTYPE\%,var);
	}

	public IMSlot getIMSlot() {
		return var;
	}

	public void setIMSlot(IMSlot slot) {
		var = slot;
	}
",
    'IMTableSwitch' => $init_tswitch,
    'IMThrow' => $init_default,
    'IMAConstant' => $init_aconst,
    'IMAMTConstant' => "
	protected IMClass memory_type_class;
	protected int base;
	protected String alias;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, IMClass type, int base) {
$init_super
		this.memory_type_class = type;
		this.base = base;
		IMClass clazz = method.getIMClass();
		this.alias = clazz.getAlias()+\"_mt_0x\"+Integer.toHexString(base);
	}

	public int getAddr() {
		return base;
	}

	\@Override
	public IMClass determineType() {
		return memory_type_class;
	}

	\@Override
	public boolean isTypeOf(IMClass type) {
		return memory_type_class.typeOf(type);
	}

	\@Override
	public int compareTo(IMConstant o) {
		if (o instanceof IMAMTConstant == false)
			return 1;
		IMAMTConstant mem = (IMAMTConstant) o;
		if (memory_type_class != mem.memory_type_class)
			return 1;
		return base - mem.base;
	}
",
    'IMAMTConstantStatic' => "
	public \%CLASS\%(IMMethod method, int bc, int bcpos, IMClass type, int base) {
		super(method,bc,bcpos,type,base);
	}
",
    'IMAMemConstant' => "
	private IMClass memory_class;
	private int addr;
	private int size;
	private String alias;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, int addr, int size) {
$init_super
		this.addr = addr;
		this.size = size;
		this.memory_class = method.getIMClass(\"keso/core/Memory\");
		IMClass clazz = method.getIMClass();
		this.alias = clazz.getAlias()+\"_mem_0x\"+Integer.toHexString(addr);
	}

	public int getAddr() {
		return addr;
	}

	public int getSize() {
		return size;
	}

	\@Override
	public IMClass determineType() {
		return memory_class;
	}

	\@Override
	public boolean isTypeOf(IMClass type) {
		return memory_class.typeOf(type);
	}

	\@Override
	public int compareTo(IMConstant o) {
		if (o instanceof IMAMemConstant == false)
			return 1;
		IMAMemConstant mem = (IMAMemConstant) o;
		if (addr != mem.addr)
			return addr - mem.addr;
		return size - mem.size;
	}
",
    'IMAObjConstant' => "
	private String alias;
	private IMClass obj_class;

	public \%CLASS\%(IMMethod method, int bc, int bcpos, String alias, IMClass obj_class) {
$init_super
		this.alias = alias;
		this.obj_class = obj_class;
	}

	\@Override
	public IMClass determineType() {
		return obj_class;
	}

	\@Override
	public boolean isTypeOf(IMClass type) {
		return obj_class.typeOf(type);
	}

	\@Override
	public int compareTo(IMConstant o) {
		if (o instanceof IMAObjConstant == false)
			return 1;
		IMAObjConstant obj = (IMAObjConstant) o;
		if (obj_class != obj.obj_class)
			return 1;
		return alias.compareTo(obj.alias);
	}
",
    'IMAReadArray' => $init_default,
    'IMAReadLocalVariable' => $init_var_ref."
	\@Override
	public IMClass determineType() {
		IMNode def = var.getDefiningStatement();
		if (def==null) return null;
		return def.determineType();
    	}
",
    'IMAStoreArray' => $init_default,
    'IMAStoreLocalVariable' => $init_var."
	\@Override
	public IMClass determineType() {
		return operand.determineType();
    	}",
    'IMBReadArray' => $init_default,
    'IMBStoreArray' => $init_default,
    'IMCReadArray' => $init_default,
    'IMCStoreArray' => $init_default,
    'IMDAdd' => $init_default,
    'IMDConstant' => $init_const_value,
    'IMDDiv' => $init_default,
    'IMDMul' => $init_default,
    'IMDNeg' => $init_default,
    'IMDReadArray' => $init_default,
    'IMDReadLocalVariable' => $init_var,
    'IMDRem' => $init_default,
    'IMDReturn' => $init_default,
    'IMDStoreArray' => $init_default,
    'IMDStoreLocalVariable' => $init_var,
    'IMDSub' => $init_default,
    'IMFAdd' => $init_default,
    'IMFConstant' => $init_const_value,
    'IMFDiv' => $init_default,
    'IMFMul' => $init_default,
    'IMFNeg' => $init_default,
    'IMFReadArray' => $init_default,
    'IMFReadLocalVariable' => $init_var,
    'IMFRem' => $init_default,
    'IMFReturn' => $init_default,
    'IMFStoreArray' => $init_default,
    'IMFStoreLocalVariable' => $init_var,
    'IMFSub' => $init_default,
    'IMIAdd' => $init_default,
    'IMIBitAnd' => $init_default,
    'IMIBitOr' => $init_default,
    'IMIBitXor' => $init_default,
    'IMIConstant' => $init_const_value,
    'IMIDiv' => $init_default,
    'IMIMul' => $init_default,
    'IMINeg' => $init_default,
    'IMIReadArray' => $init_default,
    'IMIReadLocalVariable' => $init_var,
    'IMIRem' => $init_default,
    'IMIReturn' => $init_default,
    'IMAReturn' => $init_default,
    'IMIShiftLeft' => $init_default,
    'IMIShiftRight' => $init_default,
    'IMIShiftRight2' => $init_default,
    'IMIStoreArray' => $init_default,
    'IMIStoreLocalVariable' => $init_var,
    'IMISub' => $init_default,
    'IMLAdd' => $init_default,
    'IMLBitAnd' => $init_default,
    'IMLBitOr' => $init_default,
    'IMLBitXor' => $init_default,
    'IMLConstant' => $init_const_value,
    'IMLDiv' => $init_default,
    'IMLMul' => $init_default,
    'IMLNeg' => $init_default,
    'IMLReadArray' => $init_default,
    'IMLReadLocalVariable' => $init_var,
    'IMLRem' => $init_default,
    'IMLReturn' => $init_default,
    'IMLShiftLeft' => $init_default,
    'IMLShiftRight' => $init_default,
    'IMLShiftRight2' => $init_default,
    'IMLStoreArray' => $init_default,
    'IMLStoreLocalVariable' => $init_var,
    'IMLSub' => $init_default,
    'IMNullConstant' => $init_const_null,
    'IMSReadArray' => $init_default,
    'IMSStoreArray' => $init_default,
    'IMVReturn' => $init_default,
);
