# /**(c)
# 
#   Copyright (C) 2006-2012 Christian Wawersich, Michael Stilkerich, Christoph Erhardt
# 
#   This file is part of the KESO Java Runtime Environment.
# 
#   KESO is free software: you can redistribute it and/or modify it under the
#   terms of the Lesser GNU General Public License as published by the Free
#   Software Foundation, either version 3 of the License, or (at your option)
#   any later version.
# 
#   KESO is distributed in the hope that it will be useful, but WITHOUT ANY
#   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#   FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
#   more details. You should have received a copy of the GNU Lesser General
#   Public License along with KESO. If not, see <http://www.gnu.org/licenses/>.
# 
#   Please contact keso@cs.fau.de for more info.
# 
#   (c)**/
%translate=(
'IMPhi' => '
		coder.add(toReadableString());
',
'IMAMTConstant' => '
		IMMethod constructor = memory_type_class.getMethod("<init>()V");
		if(constructor == null)
			throw new CompileException("the memory-mapped class " + memory_type_class + " needs to provide a default constructor!");

		coder.global_add_fkt("mt_init.c");
		coder.add("keso_mt_init(");
		coder.add(memory_type_class.getClassID());
		coder.add(", (void*) ");
		coder.add(base);
		coder.add(", NULL, ");
		if(constructor.getCosts()==0)
			coder.add("NULL");
		else
			coder.add(constructor.getIdentifier());
		coder.add(")");
',
'IMAMTConstantStatic' => '
		final java.util.Set<IMDomain> doms = coder.getCurrentDomains();

		/*
		 * If the object has heap fields, we need a separate instance in each
		 * domain in that the object is used.
		 *
		 * TODO Currently, support for multiple domains is not implemented.
		 */
		if(	doms.size() > 0 && memory_type_class.hasHeapFields() ) {
			if(doms.size() > 1)
				throw new CompileException("statically allocated memory mapped objects using heap fields or constructors are currently not supported in code shared among domains");

			IMDomain dom = doms.iterator().next();
			alias = coder.getDomainSymbolPrefix(dom.getDef()) + "_" + alias;
		}

		/*
		 * If the mapped class has a non-trivial constructor, we need to call in
		 * on startup in every domain in that the object is used.
		 *
		 * In the case where the object does not have heap fields by the class
		 * still defines a non-trivial constructor, we can share the object in
		 * the different domains but need to call the constructor in each of them
		 * on startup.
		 *
		 * NOTE: the code below only works because currently there is at most one
		 * domain, thus the constructor is actually only called in one domain. For
		 * multiple domains, we would also need multiple objects!
		 */
		if(doms.size() > 0) {
			coder.register_static_object_constructor(alias,memory_type_class,doms);
		}

		coder.add_allocGlobalConstObject(memory_type_class, alias, this);
',
'IMAMemConstant' => '
		coder.add_allocGlobalConstObject(memory_class, alias, this);
',
'IMAObjConstant' => '
		coder.add(alias);
		/* coder.add_allocGlobalConstObject(obj_class, alias, this); */
',
'IMCall' => '
		coder.add("ret_addr = ");
		coder.add(index);
		coder.add("; goto ");
		coder.add(targets[0].toLabel());
',
'IMPopReturnAddr' => '
		coder.add("((object_t*)ret_addr)");
',
'IMReturnSubroutine' => '
		coder.add("switch ((jshort)");
		coder.add(var.toString());
		coder.add(") {\n");
		IntegerHashtable<IMBasicBlock> ret_index = method.getReturnTargets();
		int keys[] = ret_index.sortedKeys();
		for (int i=0;i<keys.length;i++) {
			IMBasicBlock retbb = ret_index.get(keys[i]);
			coder.add("\t\tcase ");
			coder.add(keys[i]);
			coder.add(": goto ");
			coder.add(retbb.toLabel());
			coder.add(";\n");
		}
		coder.add("\t}");
',
'IMAConstant' => '
		if (value instanceof StringCPEntry) {
			ClassStore repository = method.getClassStore();
			coder.add(repository.allocString(coder, this, value.getDescription()));
		} else {
			coder.add(value.getDescription());
		}
',
'off_IMAConstant' => '
		if (value instanceof StringCPEntry) {
			IMClass clazz = method.getIMClass("java/lang/String");
			coder.add_class(clazz.getAlias());

			IMClass cclazz = method.getIMClass("[C");
			coder.add_class(cclazz.getAlias());

			String static_string  = method.getAlias()+"_str"+getBCPosition();
			String static_chararr = method.getAlias()+"_arr"+getBCPosition();
			String static_struct  = method.getAlias()+"_s"+getBCPosition();

			String str = value.getDescription();

			coder.local_add("struct ");
			coder.local_add(static_struct);
			coder.local_add(" {\nARRAY_HEADER\njchar data[");
			coder.local_add(str.length()+1);
			coder.local_add("];\n};\n");

			//coder.local_add(opts.declareConst());
			coder.local_add("static struct ");
			coder.local_add(static_struct);
			coder.local_add(" ");
			coder.local_add(static_chararr);
			coder.local_add(" HEAP_SECTION = {\n.class_id = CHAR_ARRAY_ID, .gcinfo=1,\n");
			coder.local_add(".size = ");
			coder.local_add(str.length());
			coder.local_add(",\n.data = {");
			for (int i=0;i<str.length();i++) {
				coder.local_add((int)str.charAt(i));
				coder.local_add(",");
			}
			coder.local_add("0}\n};\n");

	 		coder.add_allocConstObject(clazz, static_string);
	 		coder.add_init_field(clazz,"value","(object_t*)&"+static_chararr);
	 		coder.add_init_end();
		} else {
			coder.add(value.getDescription());
		}
',
'IMAReadArray' => '
		IMClass clazz = method.getIMClass("[Ljava/lang/Object;");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"LALOAD");
		coder.add("((object_array_t*)");
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
			} else {
			coder.add(")->data[");
		}
		iOpr.translate(coder);
		coder.add(\']\');
',
'IMReadLocalVariable' => '
		if (var==null) {
			opts.warn("local variable is null in "+method.getAlias());
			coder.add(0);
		} else {
			coder.add(var.toString());
		}
',
'IMAStoreArray' => '
		IMClass clazz = method.getIMClass("[Ljava/lang/Object;");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"LASTORE");

		if (opts.needGuardedMemoryAccess())
			coder.add("KESO_WR_GUARD(");

		if (opts.getGlobalHeap().needWriteBarriers() && !coder.omitWriteBarrier(BCBasicDatatype.REFERENCE)) {
			coder.add("KESO_WRBR(");
			coder.add("((object_array_t*)");
			
			if(opts.hasOption("use_coded_refs")) {
				coder.add("(NATIVE_ADDR(");
			}
			aOpr.translate(coder);
			if(opts.hasOption("use_coded_refs")) {
				coder.add(")))->data[");
			} else {
			coder.add(")->data[");
			}
			
			iOpr.translate(coder);
			coder.add("],");
			rvalue.translate(coder);
			coder.add(")");
		} else {
			coder.add("((object_array_t*)");
			if(opts.hasOption("use_coded_refs")) {
				coder.add("(NATIVE_ADDR(");
			}
			aOpr.translate(coder);
			if(opts.hasOption("use_coded_refs")) {
				coder.add(")))->data[");
			} else {
			coder.add(")->data[");
			}
			iOpr.translate(coder);
			coder.add("]=");
			rvalue.translate(coder);
		}

		if (opts.needGuardedMemoryAccess())
			coder.add(")");
',
'IMCStoreArray' => '
		IMClass clazz = method.getIMClass("[C");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"CASTORE");
		coder.add("((char_array_t*)");
		
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
		coder.add(")->data[");
		}
		
		iOpr.translate(coder);
		coder.add("]=");
		rvalue.translate(coder);
',
'IMAStoreLocalVariable' => '
		if (operand instanceof IMPhi) coder.add("/* ");

		if (opts.needGuardedMemoryAccess() && operand.isFieldAccess())
			coder.add("KESO_RD_GUARD(");

		coder.add(var.toString());
		coder.add(" = ");
		operand.translate(coder);

		if (opts.needGuardedMemoryAccess() && operand.isFieldAccess())
			coder.add(")");

		if (operand instanceof IMConstant) {
			var.propagateConstant(basicBlock, operand.nodeToConstant());
		} else {
			var.propagateConstant(basicBlock, null);
		}

		if (operand.isChecked(coder)) {
			if (opts.hasVerbose()) coder.addln_before("/* "+var+" = !null */");
			var.setChecked(coder.getCurrentBasicBlock());
		} else {
			if (opts.hasVerbose()) coder.addln_before("/* "+var+" = ?null? */");
			var.setChecked(null);
		}
		if (operand instanceof IMPhi) coder.addln(" */");
',
'IMStoreLocalVariable' => '
		if (operand instanceof IMPhi) coder.add("/* ");
		coder.add(var.toString());
		coder.add(" = ");
		operand.translate(coder);
		if (operand instanceof IMPhi) coder.addln(" */");
',
'IMArrayLength' => '
		if (hasToBeChecked())
			coder.chk_ref(array,method,bcPosition,"ARRAYLENGTH");
		coder.add("(jint) ((array_t*)");
		
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		array.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->size");
		} else {
			coder.add(")->size");
		}
',
'IMBReadArray' => '
		IMClass clazz = method.getIMClass("[B");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"BALOAD");
		coder.add("((byte_array_t*)");
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		iOpr.translate(coder);
		coder.add(\']\');
',
'IMBStoreArray' => '
		IMClass clazz = method.getIMClass("[B");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"BASTORE");
		coder.add("((byte_array_t*)");
		
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		
		iOpr.translate(coder);
		coder.add("]=");
		rvalue.translate(coder);
',
'IMCReadArray' => '
		IMClass clazz = method.getIMClass("[C");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"CALOAD");
		coder.add("((char_array_t*)");
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		iOpr.translate(coder);
		coder.add(\']\');
',
'IMCastD2F' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_D2F.c");
			coder.add("keso_fp_D2F(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.FLOAT_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastD2I' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_D2I.c");
			coder.add("keso_fp_D2I(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.INT_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastD2L' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_D2L.c");
			coder.add("keso_fp_D2L(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.LONG_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastF2D' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_F2D.c");
			coder.add("keso_fp_F2D(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.DOUBLE_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastF2I' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_F2I.c");
			coder.add("keso_fp_F2I(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.INT_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastF2L' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_F2L.c");
			coder.add("keso_fp_F2L(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.LONG_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastI2B' => '
		coder.add("((");
		coder.add(BuilderOptions.BYTE_T);
		coder.add(\')\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMCastI2C' => '
		coder.add("((");
		coder.add(BuilderOptions.CHAR_T);
		coder.add(\')\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMCastI2D' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_I2D.c");
			coder.add("keso_fp_I2D(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.DOUBLE_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastI2F' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_I2F.c");
			coder.add("keso_fp_I2F(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.FLOAT_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastI2L' => '
		coder.add("((");
		coder.add(BuilderOptions.LONG_T);
		coder.add(\')\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMCastI2S' => '
		coder.add("((");
		coder.add(BuilderOptions.SHORT_T);
		coder.add(\')\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMCastL2D' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_L2D.c");
			coder.add("keso_fp_L2D(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.DOUBLE_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastL2F' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_L2F.c");
			coder.add("keso_fp_L2F(");
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add("((");
			coder.add(BuilderOptions.FLOAT_T);
			coder.add(\')\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMCastL2I' => '
		coder.add("((");
		coder.add(BuilderOptions.INT_T);
		coder.add(\')\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMCaughtException' => '
	throw new CompileException("Generic Exceptions are not supported");
',
'IMCheckCast' => '
		IMClass clazz = getCheckedClass(cpEntry.getClassName()); // FIXME
		coder.add_class(clazz.getAlias());

		if (clazz.isInterface()) {
			coder.add("CHK_IFACE_ID");
			coder.add(clazz.getClassTypeID());
			coder.add(\'(\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			ClassTypeInfo type = clazz.getClassTypeInfo();
			if (rOpr.isChecked(coder) && !type.hasSubTypes()) {
				coder.add("(KESO_CHECK_CAST_FAST(");
				coder.add(clazz.getClassID());
				coder.add(\',\');
			} else {
				coder.add("(KESO_CHECK_CAST(");
				coder.add(clazz.getClassID());
				coder.add(\',\');
				coder.add(clazz.getAlias().toUpperCase());
				coder.add("_RANGE,");
			}
			coder.add("(object_t*)");
			rOpr.translate(coder);
			coder.add(", 0 /* fake check id */))");
		}
',
'IMAdd' => '
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(\'+\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMDConstant' => '
		if (opts.useFixPointMath()) {
			int fixpoint = (int)Math.rint(value*(1<<opts.fractionBitsDouble()));
			double test = (double)fixpoint/(1<<opts.fractionBitsDouble());
			double error = value-test;
			if (error<0) error = -error;
			if (error>0.001) {
				coder.add_before("\t/* "+value+" = "+fixpoint+" = "+test+" fp-error:"+error+" */\n");
				if (error>0.1) {
					opts.warn(" "+value+" = "+fixpoint+" = "+test+" fp-error:"+error);
				}
			}
			coder.add(Integer.toString(fixpoint));
		} else {
			coder.global_add_fkt("math/keso_math_double.c");
   		// toString() adds ".0" correctly
   		// NaN and Infinity should be NAN and INFINITY for C => toUpperCase()
			coder.add(\'(\');
			coder.add(Double.toString(value).toUpperCase());
			coder.add(\')\');
		}
',
'IMDiv' => '
		// TODO: div zero test
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(\'/\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMDFiv' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_fdiv.c");
			coder.add("keso_fp_fdiv(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add(\'(\');
			lOpr.translate(coder);
			coder.add(\'/\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMDDiv' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_ddiv.c");
			coder.add("keso_fp_ddiv(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add(\'(\');
			lOpr.translate(coder);
			coder.add(\'/\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMDGCompare' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_dcmp.c");
			coder.add("keso_fp_dcmp(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.global_add_fkt("math/keso_math_double.c");
			coder.add("keso_dcmpg(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMDLCompare' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_dcmp.c");
			coder.add("keso_fp_dcmp(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.global_add_fkt("math/keso_math_double.c");
			coder.add("keso_dcmpl(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMMul' => '
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(\'*\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMDMul' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_dmul.c");
			coder.add("keso_fp_dmul(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add(\'(\');
			lOpr.translate(coder);
			coder.add(\'*\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMFMul' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_fmul.c");
			coder.add("keso_fp_fmul(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.add(\'(\');
			lOpr.translate(coder);
			coder.add(\'*\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMNeg' => '
		coder.add(\'(\');
		rOpr.translate(coder);
		coder.add("* -1)");
',
'IMDReadArray' => '
		IMClass clazz = method.getIMClass("[D");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"DALOAD");
		coder.add("((double_array_t*)");
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		iOpr.translate(coder);
		coder.add(\']\');
',
'IMDRem' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_ddiv.c");
			coder.global_add_fkt("fixpoint/keso_fp_dmul.c");
			coder.global_add_fkt("fixpoint/keso_fp_drem.c");
			coder.add("keso_fp_drem(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else if (opts.hasOption("use_mathlib")) {
			coder.local_add("#include <math.h>\n");
			coder.add("fmod(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.global_add_fkt("math/keso_math_fmod.c");
			coder.add("keso_fmod(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMDReturn' => '
		coder.add("return ");
		rvalue.translate(coder);
',
'IMDStoreArray' => '
		IMClass clazz = method.getIMClass("[D");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"DASTORE");
		coder.add("((double_array_t*)");
		
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		
		iOpr.translate(coder);
		coder.add("]=");
		rvalue.translate(coder);
',
'IMSub' => '
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(\'-\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMEQConditionalBranch' => '
		coder.add("if (");
		lOpr.translate(coder);
		coder.add("==");
		rOpr.translate(coder);
		coder.add(") goto ");
		coder.add(targets[1].toLabel());
',
'IMFConstant' => '
		if (opts.useFixPointMath()) {
			int fixpoint = (int)Math.rint(value*(1<<opts.fractionBitsFloat()));
			coder.add(Integer.toString(fixpoint));
		} else {
   		// toString() adds ".0" correctly
   		// NaN and Infinity should be NAN and INFINITY for C => toUpperCase()
			coder.add(\'(\');
    			coder.add(Float.toString(value).toUpperCase());
			coder.add("f)");
		}
',
'IMFGCompare' => '
/*
   Notes The fcmpg and fcmpl instructions differ only in their treatment of a comparison
   involving NaN. NaN is unordered, so any float comparison fails if either or both
   of its operands are NaN. With both fcmpg and fcmpl available, any float comparison
   may be compiled to push the same result onto the operand stack whether the comparison
   fails on non-NaN values or fails because it encountered a NaN. For more information,
   see Section 7.5, "More Control Examples."
 */
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_fcmp.c");
			coder.add("keso_fp_fcmp(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			opts.vwarn("FCMPG may not be conform to java specification!");
			coder.global_add_fkt("math/keso_math_fcmpg.c");
			coder.add("keso_fcmpg(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMFLCompare' => '
/*
   Notes The fcmpg and fcmpl instructions differ only in their treatment of a comparison
   involving NaN. NaN is unordered, so any float comparison fails if either or both
   of its operands are NaN. With both fcmpg and fcmpl available, any float comparison
   may be compiled to push the same result onto the operand stack whether the comparison
   fails on non-NaN values or fails because it encountered a NaN. For more information,
   see Section 7.5, "More Control Examples."
 */
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_fcmp.c");
			coder.add("keso_fp_fcmp(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			opts.vwarn("FCMPL may not be conform to java specification!");
			coder.global_add_fkt("math/keso_math_fcmpl.c");
			coder.add("keso_fcmpl(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMFReadArray' => '
		IMClass clazz = method.getIMClass("[F");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"FALOAD");
		coder.add("((float_array_t*)");
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		iOpr.translate(coder);
		coder.add(\']\');
',
'IMFRem' => '
		if (opts.useFixPointMath()) {
			coder.global_add_fkt("fixpoint/keso_fp_fmul.c");
			coder.global_add_fkt("fixpoint/keso_fp_fdiv.c");
			coder.global_add_fkt("fixpoint/keso_fp_frem.c");
			coder.add("keso_fp_drem(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else if (opts.hasOption("use_mathlib")) {
			coder.local_add("#include <math.h>\n");
			coder.add("fmodf(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			coder.global_add_fkt("math/keso_math_fmodf.c");
			coder.add("keso_fmodf(");
			lOpr.translate(coder);
			coder.add(\',\');
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMFReturn' => '
		coder.add("return ");
		rvalue.translate(coder);
',
'IMFStoreArray' => '
		IMClass clazz = method.getIMClass("[F");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"FASTORE");
		coder.add("((float_array_t*)");
		
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		
		iOpr.translate(coder);
		coder.add("]=");
		rvalue.translate(coder);
',
'IMGEConditionalBranch' => '
		coder.add("if (");
		lOpr.translate(coder);
		coder.add(">=");
		rOpr.translate(coder);
		coder.add(") goto ");
		coder.add(targets[1].toLabel());
',
'IMGTConditionalBranch' => '
		coder.add("if (");
		lOpr.translate(coder);
		coder.add(\'>\');
		rOpr.translate(coder);
		coder.add(") goto ");
		coder.add(targets[1].toLabel());
',
'IMGetField' => '
		IMClass clazz = method.getIMClass(cpEntry.getClassName());
		coder.add_class(clazz.getAlias());
		if (hasToBeChecked())
			coder.chk_ref(rOpr,method,bcPosition, BCBasicDatatype.toChar(datatype)+"GETFIELD",getField());
		if (datatype==BCBasicDatatype.REFERENCE) {
			String fieldType = cpEntry.getMemberTypeDesc();
			IMClass fieldClass = method.getIMClass(fieldType.substring(1,fieldType.length()-1));
			if (fieldClass!=null && fieldClass.isMemoryType()) {
				coder.add_class(fieldClass.getAlias());
				ClassStore repository = method.getClassStore();
				repository.registerGlobalTranslationCallback("$mt_escape$", this);
				opts.warn("MT field escape! (not tested yet)");
				coder.add("keso_mt_escape(");
				coder.add(fieldClass.getClassID());
				coder.add(",");
				coder.add("(volatile void*)&(");
				coder.add(clazz.getAlias().toUpperCase());
				coder.add("_MT(");
				rOpr.translate(coder);
				coder.add(")->");
				coder.add(clazz.emitField(cpEntry.getMemberName()));
				coder.add("))");
				return;
			}
		}
		coder.add_getField(clazz, rOpr, cpEntry.getMemberName(),
				hasToBeChecked());
',
'IMGetStatic' => '
		IMClass clazz = method.getIMClass(cpEntry.getClassName());
		coder.add_class(clazz.getAlias());
		coder.add_getStatic(this,imfield, datatype);
',
'IMGoto' => '
		IMBasicBlock target = basicBlock.getSucc().at(0);
		if (target==null) {
			opts.critical("ignore blind goto in "+method.getAlias());
			coder.addln("/* goto null */");
			return;
		}
		if (hasShortcut()) {
			if (opts.hasDbgSymbols()) {
				coder.add("/* goto ");
				coder.add(target);
				coder.addln(" */");
			}
			return;
		}

		coder.add("goto ");
		coder.add(target);
		coder.addln(";");
',
'IMBitAnd' => '
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(\'&\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMBitOr' => '
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(\'|\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMBitXor' => '
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(\'^\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMIConstant' => '
		if (opts.is8BitController() || value<=0) {
			coder.add("(");
			coder.add(value);
			coder.add(")");
		} else {
			coder.add("0x");
			coder.add(Integer.toHexString(value));
		}
',
'IMIReadArray' => '
		IMClass clazz = method.getIMClass("[I");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"IALOAD");
		coder.add("((int_array_t*)");
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		iOpr.translate(coder);
		coder.add(\']\');
',
'IMIRem' => '
		// TODO: div zero test
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(\'%\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMAReturn' => '
		coder.add("return ");
		rvalue.translate(coder);
',
'IMEpilog' => '
		if (method.getBasicReturnType()==BCBasicDatatype.VOID) {
			coder.add("return");
		} else if (method.getBasicReturnType()==BCBasicDatatype.REFERENCE) {
			coder.add("return (object_t*)");
			coder.add(var.toString());
		} else {
			coder.add("return ");
			coder.add(var.toString());
		}
',
'IMIReturn' => '
		coder.add("return ");
		rvalue.translate(coder);
',
'IMIShiftLeft' => '
		coder.add(\'(\');
		if (lOpr.getDatatype()!=BCBasicDatatype.INT) {
			coder.add("((");
			coder.add(BuilderOptions.INT_T);
			coder.add(")");
			lOpr.translate(coder);
			coder.add(")");
		} else {
			lOpr.translate(coder);
		}
		coder.add("<<");
		if (rOpr instanceof IMIConstant) {
			coder.add(((IMIConstant)rOpr).getValue() & 0x1f);
		} else {
			coder.add(\'(\');
			rOpr.translate(coder);
			coder.add(" & 0x1f)"); // Don\'t forget to mask to only 5 bits!
		}
		coder.add(\')\');
',
'IMIShiftRight2' => '

		if (opts.is8BitController() && lOpr.isByte()) {
			// TODO: This is experimental code.
			// better use IMBShiftRight2 etc. in the future.
			coder.add("((jint)((jubyte)");
			lOpr.translate(coder);
			coder.add(">>");
			if (rOpr instanceof IMIConstant) {
				coder.add(((IMIConstant)rOpr).getValue() & 0x1f);
			} else {
				coder.add(\'(\');
				rOpr.translate(coder);
				coder.add(" & 0x1f)"); // Don\'t forget to mask to only 5 bits!
			}
			coder.add("))");
		} else {
			// By casting to unsigned long we make sure a possible sign is ignored
			// unsigned shift right (unter linux mit dem gcc getestet!)
			coder.add("((jint)((juint)");
			lOpr.translate(coder);
			coder.add(">>");
			if (rOpr instanceof IMIConstant) {
				coder.add(((IMIConstant)rOpr).getValue() & 0x1f);
			} else {
				coder.add(\'(\');
				rOpr.translate(coder);
				coder.add(" & 0x1f)"); // Don\'t forget to mask to only 5 bits!
			}
			coder.add("))");
		}
',
'IMIShiftRight' => '
		// TODO Problem, wenn auf Plattform Vorzeichen nicht beachtet!!!
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(">>");
		if (rOpr instanceof IMIConstant) {
			coder.add(((IMIConstant)rOpr).getValue() & 0x1f);
		} else {
			coder.add(\'(\');
			rOpr.translate(coder);
			coder.add(" & 0x1f)"); // Don\'t forget to mask to only 5 bits!
		}
		coder.add(\')\');
',
'IMIStoreArray' => '
		IMClass clazz = method.getIMClass("[I");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"IASTORE");
		coder.add("((int_array_t*)");
	
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}	
		
		iOpr.translate(coder);
		coder.add("]=");
		rvalue.translate(coder);
',
'IMInstanceOf' => '
		IMClass clazz = method.getIMClass(cpEntry.getClassName());
		coder.add_class(clazz.getAlias());

		if (clazz.isInterface()) {
			coder.add("INSTANCEOF_IFACE_ID");
			coder.add(clazz.getClassTypeID());
			coder.add(\'(\');
			rOpr.translate(coder);
			coder.add(\')\');
		} else {
			ClassTypeInfo type = clazz.getClassTypeInfo();
			if (rOpr.isChecked(coder) && !type.hasSubTypes()) {
				coder.add("keso_instanceof_fast(");
				coder.add(clazz.getClassID());
				coder.add(\',\');
			} else {
				coder.add("keso_instanceof(");
				coder.add(clazz.getClassID());
				coder.add(\',\');
				coder.add(clazz.getAlias().toUpperCase());
				coder.add("_RANGE,");
			}
			coder.add("(object_t*)");
			rOpr.translate(coder);
			coder.add(\')\');
		}
',
'IMInvokeInterface' => '
		IMMethod callee = method.findMethod(cpEntry);

		if (opts.hasOption("_X_dbg_astack")) showEscape(coder);

		if (!joinPoints.affectInvokeInterface(this,method,callee,obj,args,coder)) {
			IMClass clazz = method.getIMClass(cpEntry.getClassName());
			coder.add_class(clazz.getAlias());

			if (hasToBeChecked())
				coder.chk_ref(obj,method,bcPosition,"INVOKEINTERFACE");
			coder.vcall(frame, callee, obj, args);
			++IMInvoke.virtualCalls;
		}
',
'IMInvokeSpecial' => '
		IMMethod callee = method.findMethod(cpEntry);

		if (opts.hasOption("_X_dbg_astack")) showEscape(coder);

		if (!joinPoints.affectInvokeSpecial(this,method,callee,obj,args,coder)) {
			IMClass clazz = callee.getIMClass();
			coder.add_class(clazz.getAlias());

			if (hasToBeChecked())
				coder.chk_ref(obj,method,bcPosition,"INVOKESPECIAL");
			coder.add(callee.getIdentifier());
			callee.emitArguments(coder, frame, obj, args);
			++IMInvoke.nonVirtualCalls;
		}
',
'IMInvokeStatic' => '
		IMMethod callee = method.findMethod(cpEntry);

		if (opts.hasOption("_X_dbg_astack")) showEscape(coder);

		if (!joinPoints.affectInvokeStatic(this,method,callee,args,coder)) {
			coder.add_class(callee.getIMClass().getAlias());

			coder.add(callee.getIdentifier());
			callee.emitArguments(coder, frame, null, args);
			++IMInvoke.nonVirtualCalls;
		}
',
'IMInvokeVirtual' => '
		IMMethod callee = method.findMethod(cpEntry);

		if (opts.hasOption("_X_dbg_astack")) showEscape(coder);

		if (!joinPoints.affectInvokeVirtual(this,method,callee,obj,args,coder)) {
			IMClass clazz = method.getIMClass(cpEntry.getClassName());
			coder.add_class(clazz.getAlias());

			if (hasToBeChecked())
				coder.chk_ref(obj,method,bcPosition,"INVOKEVIRTUAL");
			coder.vcall(frame, callee, obj, args);
			++IMInvoke.virtualCalls;
		}
',
'IMLCompare' => '
		//coder.add("keso_lcmp(");
		coder.add("(");
		lOpr.translate(coder);
		//coder.add(\',\');
		coder.add(\'-\');
		rOpr.translate(coder);
		coder.add(\')\');

',
'IMLConstant' => '
		coder.add(\'(\');
    		coder.add(Long.toString(value));
		coder.add("LL");
		coder.add(\')\');
',
'IMLEConditionalBranch' => '
		coder.add("if (");
		lOpr.translate(coder);
		coder.add("<=");
		rOpr.translate(coder);
		coder.add(") goto ");
		coder.add(targets[1].toLabel());
',
'IMLReadArray' => '
		IMClass clazz = method.getIMClass("[J");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"JALOAD");
		coder.add("((long_array_t*)");
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		iOpr.translate(coder);
		coder.add(\']\');
',
'IMLRem' => '
		// TODO: div zero test
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(\'%\');
		rOpr.translate(coder);
		coder.add(\')\');
',
'IMLReturn' => '
		coder.add("return ");
		rvalue.translate(coder);
',
'IMLShiftLeft' => '
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add("<<(");
		rOpr.translate(coder);
		coder.add(" & 0x3f))"); // Don\'t forget to mask to only 6 bits!
',
'IMLShiftRight2' => '
      		// By casting to unsigned long we make sure a possible sign is ignored
		coder.add("((jlong)((julong)");
		lOpr.translate(coder);
		coder.add(">>(");
		rOpr.translate(coder);
		coder.add(" & 0x3f)))"); // Don\'t forget to mask to only 6 bits!
',
'IMLShiftRight' => '
		// TODO Problem, wenn auf Plattform Vorzeichen nicht beachtet!!!
		coder.add(\'(\');
		lOpr.translate(coder);
		coder.add(">>(");
		rOpr.translate(coder);
		coder.add(" & 0x3f))"); // Don\'t forget to mask to only 6 bits!
',
'IMLStoreArray' => '
		IMClass clazz = method.getIMClass("[J");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"JASTORE");
		coder.add("((long_array_t*)");
		
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}

		iOpr.translate(coder);
		coder.add("]=");
		rvalue.translate(coder);
',
'IMLTConditionalBranch' => '
		coder.add("if (");
		lOpr.translate(coder);
		coder.add(\'<\');
		rOpr.translate(coder);
		coder.add(") goto ");
		coder.add(targets[1].toLabel());
',
'IMLookupSwitch' => '
		coder.add("switch (");
   		operand.translate(coder);
		coder.addln(") {");

		for (int i=1;i<targets.length;i++) {
			coder.add("\tcase ");
			coder.add(keys[i-1]);
			coder.add(": goto ");
			coder.add(targets[i].toLabel());
			coder.addln(\';\');
		}
		coder.add("\tdefault: goto ");
		coder.add(targets[0].toLabel());
		coder.addln(\';\');

		coder.add(\'}\');
',
'IMMonitor' => '
		if (bytecode==0xc2) {
			coder.add("KESO_LOCK(");
		} else {
			coder.add("KESO_UNLOCK(");
		}
		operand.translate(coder);
		coder.add(")");
',
'IMNEConditionalBranch' => '
		coder.add("if (");
		lOpr.translate(coder);
		coder.add("!=");
		rOpr.translate(coder);
		coder.add(") goto ");
		coder.add(targets[1].toLabel());
',
'IMNewArray' => '
		coder.add_class(aclass.getAlias());

		if (size.isConstant() && (((IMConstant)size).getIntValue()==0) && aclass.hasZeroLengthArray()) {
			aclass.emit_ZeroLengthArray(coder);
		} else {
			if (method.isClassInit() == false &&
					opts.getGlobalHeap().allocationCanBlock())
				method.getMethodFrame().codeEOLL(coder);

			if (does_not_escape && (size.isConstant() && (((IMConstant)size).getIntValue()<9))) {
				coder.add_before("\t/* no escaping: todo ");
				coder.add_before(toReadableString());
				coder.add_before(" (array) */\n");

				coder.add("keso_alloc_");
				coder.add(aclass.getAlias());
				coder.add("(");
				size.translate(coder);
				coder.add(\')\');
			} else {
				if (why_escape!=null) {
					coder.add_before("\t/* escaping: ");
					coder.add_before(why_escape.toReadableString());
					coder.add_before(" (array) */\n");
				}
				coder.add("keso_alloc_");
				coder.add(aclass.getAlias());
				coder.add("(");
				size.translate(coder);
				coder.add(\')\');
			}
		}
',
'IMNew' => '
		IMClass clazz = method.getIMClass(cpEntry.getClassName());
		coder.add_class(clazz.getAlias());

		if (does_not_escape || (escapeState == EscapeState.Local && !livenessOverlap)) {
			if (clazz.getObjRefFieldCount()==0 && !opts.hasOption("ssa_astack_noinline")) {
				opts.vwarn("Inlined stack allocation used in "+lineInfo()+". In case of trouble use -X:ssa_astack_noinline.");
				coder.add_before("\t/* no escaping (stack allocation inlined) */\n");
				coder.add_allocStackObjectInlined(clazz);
			} else {
				coder.add_before("\t/* no escaping (stack allocation) */\n");
				ClassStore repository = method.getClassStore();
				repository.registerGlobalTranslationCallback("$keso_alloc_stack$", this);
				coder.add_allocStackObject(clazz);
			}
		} else {
			if (why_escape!=null) {
				coder.add_before("\t/* escaping: ");
				coder.add_before(why_escape.toReadableString());
				coder.add_before(" */\n");
			}
			if (opts.getGlobalHeap().allocationCanBlock())
				method.getMethodFrame().codeEOLL(coder);
			coder.add_allocObject(clazz);
		}
',
'IMNewMultiArray' => '
		IMClass clazz = method.getIMClass(ClassStore.getArrayElementType(cpEntry.getClassName()));
		coder.add_class(clazz.getAlias());
		coder.add_class(aclass.getAlias());

		if (method.isClassInit() == false &&
			 opts.getGlobalHeap().allocationCanBlock())
				method.getMethodFrame().codeEOLL(coder);

		if (why_escape!=null) {
			coder.add_before("\t/* escaping: ");
			coder.add_before(why_escape.toReadableString());
			coder.add_before(" (array) */\n");
		}
		if (does_not_escape) {
			coder.add_before("\t/* no escaping: ");
			coder.add_before(toReadableString());
			coder.add_before(" (array) */\n");
		}

		// [[[I => lastIndex==2, dimension=3
		// [[Ltest/Hello; => lastIndex==1, dimension=2
		int real_dimension = cpEntry.getClassName().lastIndexOf(\'[\') + 1;

		coder.add("keso_allocMultiArray(");
		coder.add(clazz.getClassID());
		coder.add(\',\');
		coder.add(real_dimension);


		int len = oprs.length;
		for (int i=1; i<=len; i++) {
			coder.add(\',\');
			oprs[len-i].translate(coder);
		}
		if(len < real_dimension) { // partially initialized array
			 coder.add(",0");
		}

		coder.add(\')\');
',
'IMNewObjArray' => '
		coder.add_class(aclass.getAlias());

		if (method.isClassInit() == false &&
			 opts.getGlobalHeap().allocationCanBlock())
				method.getMethodFrame().codeEOLL(coder);

		if (why_escape!=null) {
			coder.add_before("\t/* escaping: ");
			coder.add_before(why_escape.toReadableString());
			coder.add_before(" (array) */\n");
		}
		if (does_not_escape) {
			coder.add_before("\t/* no escaping: ");
			coder.add_before(toReadableString());
			coder.add_before(" (array) */\n");
		}

		coder.add("keso_alloc_object_array(");

		// regular class -1: => object array <reg_class>
		// primitive array class 0: => object array <parray_class>
		// object array class >0 || (0 && [L): => object array <object array class>
		int typeIndex = cpEntry.getClassName().lastIndexOf(\'[\');
		if(typeIndex == -1 ||
			(typeIndex == 0 && cpEntry.getClassName().charAt(1) != \'L\')) {
			IMClass clazz = method.getIMClass(cpEntry.getClassName());
			if (clazz==null) throw new CompileException("Class not found! "+cpEntry.getClassName());

			coder.add_class(clazz.getAlias());
			coder.add(clazz.getClassID());
		} else {
			coder.add("OBJECT_ARRAY_ID");
		}
		coder.add(", ");
		size.translate(coder);
		coder.add(\')\');
',
'IMNop' => ' return; ',
'IMNullConstant' => '
		coder.add("((object_t*)0)");
',
'IMPutField' => '
		if (nukeInit) {
			coder.add_before("/* no store needed because the object is cleared by initialization! */");
			return;
		}

		IMClass clazz = method.getIMClass(cpEntry.getClassName());
		coder.add_class(clazz.getAlias());
		if (hasToBeChecked())
			coder.chk_ref(obj,method,bcPosition,BCBasicDatatype.toChar(datatype)+"PUTFIELD",getField());
		if (datatype==BCBasicDatatype.REFERENCE) {
			String fieldType = cpEntry.getMemberTypeDesc();
			if (fieldType.charAt(0)==\'L\') {
				IMClass fieldClass = method.getIMClass(fieldType.substring(1,fieldType.length()-1));
				if (fieldClass==null)
					opts.warn("class not found: "+fieldType);
				if (fieldClass!=null && fieldClass.isMemoryType()) {
					opts.warn("memory type store ignored!");
					coder.add_comment("memory type store ignored!");
					obj.translate(coder);
					return;
				}
			}
		}

		if (initial_write) {
			coder.add_before("/* no write barrier needed because "+obj.toReadableString()+"==null */");
			coder.add_putField_NoWrBr(clazz, obj, cpEntry.getMemberName(), datatype, rvalue);
		} else {
			coder.add_putField(clazz, obj, cpEntry.getMemberName(), datatype, rvalue);
		}
',
'IMPutStatic' => '
		IMClass clazz = method.getIMClass(cpEntry.getClassName());
		coder.add_class(clazz.getAlias());
		coder.add_putStatic(this,imfield,datatype,rvalue);
',
'IMSReadArray' => '
		IMClass clazz = method.getIMClass("[S");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"SALOAD");
		coder.add("((short_array_t*)");
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
			} else {
			coder.add(")->data[");
		}
		iOpr.translate(coder);
		coder.add(\']\');
',
'IMSStoreArray' => '
		IMClass clazz = method.getIMClass("[S");
		coder.add_class(clazz.getAlias());
		coder.chk_array(this,aOpr,iOpr,method,bcPosition,"SASTORE");
		coder.add("((short_array_t*)");
		
		if(opts.hasOption("use_coded_refs")) {
			coder.add("(NATIVE_ADDR(");
		}
		aOpr.translate(coder);
		if(opts.hasOption("use_coded_refs")) {
			coder.add(")))->data[");
		} else {
			coder.add(")->data[");
		}
		
		iOpr.translate(coder);
		coder.add("]=");
		rvalue.translate(coder);
',
'IMTableSwitch' => '
		coder.add("switch (");
   		operand.translate(coder);
		coder.addln(") {");

		for (int i=1;i<targets.length;i++) {
			coder.add("\tcase ");
			coder.add(low+(i-1));
			coder.add(": goto ");
			coder.add(targets[i].toLabel());
			coder.addln(\';\');
		}
		coder.add("\tdefault: goto ");
		coder.add(targets[0].toLabel());
		coder.addln(\';\');

		coder.add(\'}\');
',
'IMThrow' => '
		coder.add_before("\t/* throw "+exception.toReadableString()+" */\n");
		coder.add_before("\t/* For now we just halt. */\n");
		if(opts.hasOption("exception_messages")) {
			coder.add("KESO_PRINTF(\"Exception: %s\\\\n\\", \\"");
			coder.add(method.getClassName() + "." + method.getMethodName() + "(" + lineInfo() + ")");
			coder.addln("\\");");
		}
		coder.addln("while (1) { __asm__ __volatile__ (\"nop\"); }");
		if(method.hasReturnValue()) {
			if(method.getBasicReturnType() == BCBasicDatatype.REFERENCE) {
				coder.addln("return (object_t*) 0;");
			} else {
				coder.addln("return 0;");
			}
		}
',
'IMVReturn' => '
		coder.add("return");
',
);
